"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateOptions = exports.pressAnyKeyToContinue = exports.inquirerPrompt = void 0;
var tslib_1 = require("tslib");
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var logger_1 = require("../core/systemManager/logger");
var config_1 = (0, tslib_1.__importDefault)(require("../core/configManager/config"));
var inquirerPrompt = function (params) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var c, msg, type, name;
    var _a;
    var _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        c = config_1.default.getConfig();
        if ((_b = c.program) === null || _b === void 0 ? void 0 : _b.yes)
            return [2 /*return*/, true];
        msg = params.logMessage || params.warningMessage || params.message;
        if ((_c = c.program) === null || _c === void 0 ? void 0 : _c.ci) {
            if (Array.isArray(params.choices) && typeof params.default !== 'undefined' && params.choices.includes(params.default)) {
                (0, logger_1.logDebug)("defaulting to choice '".concat(params.default, "' for prompt '").concat(params.name, "'"));
                return [2 /*return*/, Promise.resolve((_a = {}, _a[params.name] = params.default, _a))];
            }
            return [2 /*return*/, Promise.reject("--ci option does not allow prompts. question: ".concat(msg, "."))];
        }
        if (msg && params.logMessage)
            (0, logger_1.logTask)(msg, (0, logger_1.chalk)().grey);
        if (msg && params.warningMessage)
            (0, logger_1.logWarning)(msg);
        type = params.type, name = params.name;
        if (type === 'confirm' && !name)
            params.name = 'confirm';
        return [2 /*return*/, inquirer_1.default.prompt(params)];
    });
}); };
exports.inquirerPrompt = inquirerPrompt;
var pressAnyKeyToContinue = function () {
    var params = {
        type: 'input',
        name: 'confirm',
        message: 'Press any key to continue'
    };
    return inquirer_1.default.prompt(params);
};
exports.pressAnyKeyToContinue = pressAnyKeyToContinue;
var generateOptions = function (inputData, isMultiChoice, mapping, renderMethod) {
    if (isMultiChoice === void 0) { isMultiChoice = false; }
    (0, logger_1.logDebug)('generateOptions', isMultiChoice);
    var asString = '';
    var valuesAsObject = {};
    var valuesAsArray = [];
    var keysAsObject = {};
    var keysAsArray = [];
    var optionsAsArray = [];
    var isArray = Array.isArray(inputData);
    var output = {};
    var renderer = renderMethod || _generateOptionString;
    if (isArray) {
        inputData.forEach(function (v, i) {
            var rn = renderer(i, v, mapping, v);
            asString += rn;
            optionsAsArray.push(rn);
            valuesAsArray.push(v);
            if (!mapping)
                keysAsArray.push(v);
            if (!mapping)
                valuesAsObject[v] = v;
        });
    }
    else {
        var i_1 = 0;
        Object.keys(inputData).forEach(function (k) {
            var v = inputData[k];
            var rn = renderer(i_1, v, mapping, k);
            asString += rn;
            optionsAsArray.push(rn);
            keysAsArray.push(k);
            keysAsObject[k] = true;
            valuesAsObject[k] = v;
            valuesAsArray.push(v);
            i_1++;
        });
    }
    output.keysAsArray = keysAsArray.sort(_sort);
    output.valuesAsArray = valuesAsArray.sort(_sort);
    output.keysAsObject = keysAsObject;
    output.valuesAsObject = valuesAsObject;
    output.asString = asString;
    output.optionsAsArray = optionsAsArray;
    return output;
};
exports.generateOptions = generateOptions;
var _sort = function (a, b) {
    var aStr = '';
    var bStr = '';
    if (typeof a === 'string') {
        // TODO: temp fix for weird issue when a/b are marked as string
        // but toLowerCase() is undefined. need to investigate
        aStr = a.toLowerCase ? a.toLowerCase() : a;
        bStr = b.toLowerCase ? b.toLowerCase() : b;
    }
    else {
        if (a && a.name)
            aStr = a.name.toLowerCase();
        if (b && b.name)
            bStr = b.name.toLowerCase();
    }
    var com = 0;
    if (aStr > bStr) {
        com = 1;
    }
    else if (aStr < bStr) {
        com = -1;
    }
    return com;
};
var _generateOptionString = function (i, obj, mapping, defaultVal) { return " [".concat((0, logger_1.chalk)().bold.grey(i + 1), "]> ").concat((0, logger_1.chalk)().bold.grey(mapping ? '' : defaultVal), " \n"); };
//# sourceMappingURL=prompt.js.map