"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBuildFilePath = exports.getFlavouredProp = exports.checkPortInUse = exports.getIP = exports.getBuildsFolder = exports.getMonorepoRoot = exports.isMonorepo = exports.getAppVersionCode = exports.getAppVersion = exports.getAppDescription = exports.getGetJsBundleFile = exports.getEntryFile = exports.getAppLicense = exports.getAppAuthor = exports.getAppTitle = exports.getAppId = exports.getConfigPropArray = exports._getConfigProp = exports.getConfigProp = exports.getAppTemplateFolder = exports.getAppFolder = exports.getTemplateProjectDir = exports.getTemplateDir = exports.getPlatformServerDir = exports.getPlatformProjectDir = exports.getPlatformOutputDir = exports.getPlatformBuildDir = exports.confirmActiveBundler = exports.existBuildsOverrideForTargetPathSync = exports.waitForHost = exports.getDevServerHost = exports.sanitizeColor = exports.addSystemInjects = exports.getCliArguments = exports.getTimestampPathsConfig = void 0;
var tslib_1 = require("tslib");
var color_string_1 = (0, tslib_1.__importDefault)(require("color-string"));
var detect_port_1 = (0, tslib_1.__importDefault)(require("detect-port"));
var ip_1 = (0, tslib_1.__importDefault)(require("ip"));
var kill_port_1 = (0, tslib_1.__importDefault)(require("kill-port"));
var axios_1 = (0, tslib_1.__importDefault)(require("axios"));
var lodash_get_1 = (0, tslib_1.__importDefault)(require("lodash.get"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var prompt_1 = require("../cli/prompt");
var constants_1 = require("./constants");
var fileutils_1 = require("./systemManager/fileutils");
var logger_1 = require("./systemManager/logger");
var utils_1 = require("./systemManager/utils");
var getTimestampPathsConfig = function (c, platform) {
    var timestampBuildFiles;
    var pPath = path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(platform));
    if (platform === 'web') {
        timestampBuildFiles = (0, exports.getConfigProp)(c, platform, 'timestampBuildFiles', []).map((function (v) { return path_1.default.join(pPath, v); }));
    }
    if (timestampBuildFiles === null || timestampBuildFiles === void 0 ? void 0 : timestampBuildFiles.length) {
        return { paths: timestampBuildFiles, timestamp: c.runtime.timestamp };
    }
    return null;
};
exports.getTimestampPathsConfig = getTimestampPathsConfig;
var getCliArguments = function (c) {
    var _a = c.program, args = _a.args, rawArgs = _a.rawArgs;
    var argsCopy = (0, tslib_1.__spreadArray)([], args, true);
    var missingArg = rawArgs[rawArgs.indexOf(argsCopy[1]) + 1];
    if ((missingArg === null || missingArg === void 0 ? void 0 : missingArg[0]) === '-') {
        if (rawArgs[rawArgs.indexOf(argsCopy[1]) + 2]) {
            missingArg = rawArgs[rawArgs.indexOf(argsCopy[1]) + 2];
        }
        else {
            missingArg = undefined;
        }
    }
    if (rawArgs.length === 3)
        missingArg = undefined;
    argsCopy[2] = missingArg;
    return argsCopy.filter(function (arg) { return !!arg; });
};
exports.getCliArguments = getCliArguments;
var addSystemInjects = function (c, injects) {
    if (!c.systemPropsInjects)
        c.systemPropsInjects = [];
    if (injects) {
        injects.forEach(function (item) {
            c.systemPropsInjects.push(item);
        });
    }
};
exports.addSystemInjects = addSystemInjects;
var sanitizeColor = function (val, key) {
    if (!val) {
        (0, logger_1.logWarning)("You are missing ".concat((0, logger_1.chalk)().white(key), " in your renative config. will use default #FFFFFF instead"));
        return {
            rgb: [255, 255, 255, 1],
            rgbDecimal: [1, 1, 1, 1],
            hex: '#FFFFFF'
        };
    }
    var rgb = color_string_1.default.get.rgb(val);
    var hex = color_string_1.default.to.hex(rgb);
    return {
        rgb: rgb,
        rgbDecimal: rgb.map(function (v) { return (v > 1 ? Math.round((v / 255) * 10) / 10 : v); }),
        hex: hex
    };
};
exports.sanitizeColor = sanitizeColor;
var getDevServerHost = function (c) {
    var devServerHostOrig = (0, exports.getConfigProp)(c, c.platform, 'devServerHost');
    if (!devServerHostOrig) {
        devServerHostOrig = (0, exports.getConfigProp)(c, c.platform, 'webpack', {}).devServerHost;
        if (devServerHostOrig) {
            (0, logger_1.logWarning)('DEPRECATED: webpack.devServerHost. use devServerHost directly instead');
        }
    }
    if (!devServerHostOrig) {
        if (!devServerHostOrig) {
            devServerHostOrig = (0, exports.getConfigProp)(c, c.platform, 'webpackConfig', {}).devServerHost;
            if (devServerHostOrig) {
                (0, logger_1.logWarning)('DEPRECATED: webpackConfig.devServerHost. use devServerHost directly instead');
            }
        }
    }
    var devServerHostFixed = (0, utils_1.getValidLocalhost)(devServerHostOrig, c.runtime.localhost);
    return devServerHostFixed;
};
exports.getDevServerHost = getDevServerHost;
var waitForHost = function (c, suffix) {
    if (suffix === void 0) { suffix = 'assets/bundle.js'; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var attempts, maxAttempts, CHECK_INTEVAL, devServerHost, url;
        return (0, tslib_1.__generator)(this, function (_a) {
            (0, logger_1.logTask)('waitForHost', "port:".concat(c.runtime.port));
            attempts = 0;
            maxAttempts = 10;
            CHECK_INTEVAL = 2000;
            devServerHost = (0, exports.getDevServerHost)(c);
            url = "http://".concat(devServerHost, ":").concat(c.runtime.port, "/").concat(suffix);
            return [2 /*return*/, new Promise(function (resolve, reject) {
                    var interval = setInterval(function () {
                        axios_1.default
                            .get(url)
                            .then(function (res) {
                            if (res.status === 200) {
                                clearInterval(interval);
                                // spinner.succeed();
                                return resolve(true);
                            }
                            attempts++;
                            if (attempts === maxAttempts) {
                                clearInterval(interval);
                                // spinner.fail('Can\'t connect to webpack. Try restarting it.');
                                return reject("Can't connect to host ".concat(url, ". Try restarting it."));
                            }
                        })
                            .catch(function () {
                            attempts++;
                            if (attempts > maxAttempts) {
                                clearInterval(interval);
                                // spinner.fail('Can\'t connect to webpack. Try restarting it.');
                                return reject("Can't connect to host ".concat(url, ". Try restarting it."));
                            }
                        });
                    }, CHECK_INTEVAL);
                })];
        });
    });
};
exports.waitForHost = waitForHost;
var existBuildsOverrideForTargetPathSync = function (c, destPath) {
    var appFolder = (0, exports.getAppFolder)(c);
    var relativePath = path_1.default.relative(appFolder, destPath);
    var result = false;
    var pathsToCheck = [];
    if (c.paths.appConfig.dirs) {
        c.paths.appConfig.dirs.forEach(function (v) {
            pathsToCheck.push((0, exports.getBuildsFolder)(c, c.platform, v));
        });
    }
    for (var i = 0; i < pathsToCheck.length; i++) {
        if ((0, fileutils_1.fsExistsSync)(path_1.default.join(pathsToCheck[i], relativePath))) {
            result = true;
            break;
        }
    }
    return result;
};
exports.existBuildsOverrideForTargetPathSync = existBuildsOverrideForTargetPathSync;
var confirmActiveBundler = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var choices, selectedOption;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                if (c.runtime.skipActiveServerCheck)
                    return [2 /*return*/, true];
                if (c.program.ci) {
                    return [2 /*return*/, (0, kill_port_1.default)(c.runtime.port)];
                }
                choices = ['Restart the server (recommended)', 'Use existing session'];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'selectedOption',
                        type: 'list',
                        choices: choices,
                        warningMessage: "Another ".concat(c.platform, " server at port ").concat((0, logger_1.chalk)().white(c.runtime.port), " already running")
                    })];
            case 1:
                selectedOption = (_a.sent()).selectedOption;
                if (!(choices[0] === selectedOption)) return [3 /*break*/, 3];
                return [4 /*yield*/, (0, kill_port_1.default)(c.runtime.port)];
            case 2:
                _a.sent();
                return [3 /*break*/, 4];
            case 3: return [2 /*return*/, false];
            case 4: return [2 /*return*/, true];
        }
    });
}); };
exports.confirmActiveBundler = confirmActiveBundler;
var getPlatformBuildDir = function (c, isRelativePath) {
    if (!c.runtime.engine) {
        (0, logger_1.logError)('getPlatformBuildDir not available without specific engine');
        return null;
    }
    return (0, exports.getAppFolder)(c, isRelativePath);
};
exports.getPlatformBuildDir = getPlatformBuildDir;
var getPlatformOutputDir = function (c) {
    if (!c.runtime.engine) {
        (0, logger_1.logError)('getPlatformOutputDir not available without specific engine');
        return null;
    }
    return path_1.default.join((0, exports.getAppFolder)(c), c.runtime.engine.outputDirName || '');
};
exports.getPlatformOutputDir = getPlatformOutputDir;
var getPlatformProjectDir = function (c) {
    if (!c.runtime.engine) {
        (0, logger_1.logError)('getPlatformProjectDir not available without specific engine');
        return null;
    }
    return path_1.default.join((0, exports.getAppFolder)(c), c.runtime.engine.projectDirName || '');
};
exports.getPlatformProjectDir = getPlatformProjectDir;
var getPlatformServerDir = function (c) {
    if (!c.runtime.engine) {
        (0, logger_1.logError)('getPlatformProjectDir not available without specific engine');
        return null;
    }
    return path_1.default.join((0, exports.getAppFolder)(c), c.runtime.engine.serverDirName || '');
};
exports.getPlatformServerDir = getPlatformServerDir;
var getTemplateDir = function (c) { return path_1.default.join(c.paths.project.platformTemplatesDirs[c.platform], "".concat(c.platform)); };
exports.getTemplateDir = getTemplateDir;
var getTemplateProjectDir = function (c) {
    if (!c.runtime.engine) {
        (0, logger_1.logError)('getTemplateProjectDir not available without specific engine');
        return null;
    }
    return path_1.default.join((0, exports.getTemplateDir)(c), c.runtime.engine.projectDirName);
};
exports.getTemplateProjectDir = getTemplateProjectDir;
// DEPRECATED
var getAppFolder = function (c, isRelativePath) {
    if (isRelativePath) {
        return "platformBuilds/".concat(c.runtime.appId, "_").concat(c.platform).concat(c.runtime._platformBuildsSuffix || '');
    }
    return path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(c.platform).concat(c.runtime._platformBuildsSuffix || ''));
};
exports.getAppFolder = getAppFolder;
// DEPRECATED
var getAppTemplateFolder = function (c, platform) { return path_1.default.join(c.paths.project.platformTemplatesDirs[platform], "".concat(platform)); };
exports.getAppTemplateFolder = getAppTemplateFolder;
var _getValueOrMergedObject = function (resultCli, resultScheme, resultPlatforms, resultCommon) {
    if (resultCli !== undefined) {
        return resultCli;
    }
    if (resultScheme !== undefined) {
        if (Array.isArray(resultScheme) || typeof resultScheme !== 'object') {
            return resultScheme;
        }
        var val = Object.assign(resultCommon || {}, resultPlatforms || {}, resultScheme);
        return val;
    }
    if (resultPlatforms !== undefined) {
        if (Array.isArray(resultPlatforms)
            || typeof resultPlatforms !== 'object') {
            return resultPlatforms;
        }
        return Object.assign(resultCommon || {}, resultPlatforms);
    }
    if (resultPlatforms === null)
        return null;
    return resultCommon;
};
var getConfigProp = function (c, platform, key, defaultVal) {
    if (!c.buildConfig) {
        (0, logger_1.logError)('getConfigProp: c.buildConfig is undefined!');
        return null;
    }
    return (0, exports._getConfigProp)(c, platform, key, defaultVal, c.buildConfig);
};
exports.getConfigProp = getConfigProp;
var _getConfigProp = function (c, platform, key, defaultVal, sourceObj) {
    var _a, _b, _c;
    if (!sourceObj)
        return null;
    if (!key || !key.split) {
        (0, logger_1.logError)('getConfigProp: invalid key!');
        return null;
    }
    var p = (_a = sourceObj.platforms) === null || _a === void 0 ? void 0 : _a[platform];
    var ps = c.runtime.scheme;
    var keyArr = key.split('.');
    var baseKey = keyArr.shift();
    var subKey = keyArr.join('.');
    var resultPlatforms;
    var scheme;
    if (p) {
        scheme = p.buildSchemes ? p.buildSchemes[ps] : undefined;
        resultPlatforms = (0, exports.getFlavouredProp)(c, sourceObj.platforms[platform], baseKey);
    }
    scheme = scheme || {};
    var resultCli = constants_1.CLI_PROPS.includes(baseKey) ? c.program[baseKey] : undefined;
    var resultScheme = scheme[baseKey];
    var resultCommonRoot = (0, exports.getFlavouredProp)(c, sourceObj.common, baseKey);
    var resultCommonScheme = (0, exports.getFlavouredProp)(c, ((_c = (_b = sourceObj.common) === null || _b === void 0 ? void 0 : _b.buildSchemes) === null || _c === void 0 ? void 0 : _c[c.runtime.scheme]) || {}, baseKey);
    var resultCommon = resultCommonScheme || resultCommonRoot;
    var result = _getValueOrMergedObject(resultCli, resultScheme, resultPlatforms, resultCommon);
    if (result === undefined || result === null) {
        result = (0, exports.getFlavouredProp)(c, sourceObj, baseKey);
    }
    if (result === undefined || result === null)
        result = defaultVal; // default the value only if it's not specified in any of the files. i.e. undefined
    if (typeof result === 'object' && subKey.length) {
        return (0, lodash_get_1.default)(result, subKey);
    }
    return result;
};
exports._getConfigProp = _getConfigProp;
var getConfigPropArray = function (c, platform, key) {
    var result = [];
    var configArr = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([
        c.files.defaultWorkspace.config,
        c.files.rnv.projectTemplates.config,
        // { plugins: extraPlugins },
        // { pluginTemplates },
        c.files.workspace.config,
        c.files.workspace.configPrivate,
        c.files.workspace.configLocal,
        c.files.workspace.project.config,
        c.files.workspace.project.configPrivate,
        c.files.workspace.project.configLocal
    ], c.files.workspace.appConfig.configs, true), c.files.workspace.appConfig.configsPrivate, true), c.files.workspace.appConfig.configsLocal, true), [
        c.files.project.config,
        c.files.project.configPrivate,
        c.files.project.configLocal
    ], false), c.files.appConfig.configs, true), c.files.appConfig.configsPrivate, true), c.files.appConfig.configsLocal, true);
    configArr.forEach(function (config) {
        var val = (0, exports._getConfigProp)(c, platform, key, null, config);
        if (val) {
            result.push(val);
        }
    });
    return result;
};
exports.getConfigPropArray = getConfigPropArray;
var getAppId = function (c, platform) {
    var id = (0, exports.getConfigProp)(c, platform, 'id');
    var idSuffix = (0, exports.getConfigProp)(c, platform, 'idSuffix');
    return idSuffix ? "".concat(id).concat(idSuffix) : id;
};
exports.getAppId = getAppId;
var getAppTitle = function (c, platform) { return (0, exports.getConfigProp)(c, platform, 'title'); };
exports.getAppTitle = getAppTitle;
var getAppAuthor = function (c, platform) { var _a; return (0, exports.getConfigProp)(c, platform, 'author') || ((_a = c.files.project.package) === null || _a === void 0 ? void 0 : _a.author); };
exports.getAppAuthor = getAppAuthor;
var getAppLicense = function (c, platform) { var _a; return (0, exports.getConfigProp)(c, platform, 'license') || ((_a = c.files.project.package) === null || _a === void 0 ? void 0 : _a.license); };
exports.getAppLicense = getAppLicense;
var getEntryFile = function (c, platform) { var _a, _b; return (_b = (_a = c.buildConfig.platforms) === null || _a === void 0 ? void 0 : _a[platform]) === null || _b === void 0 ? void 0 : _b.entryFile; };
exports.getEntryFile = getEntryFile;
var getGetJsBundleFile = function (c, platform) { return (0, exports.getConfigProp)(c, platform, 'getJsBundleFile'); };
exports.getGetJsBundleFile = getGetJsBundleFile;
var getAppDescription = function (c, platform) {
    var _a;
    return (0, exports.getConfigProp)(c, platform, 'description')
        || ((_a = c.files.project.package) === null || _a === void 0 ? void 0 : _a.description);
};
exports.getAppDescription = getAppDescription;
var getAppVersion = function (c, platform) {
    var _a;
    var version = (0, exports.getConfigProp)(c, platform, 'version') || ((_a = c.files.project.package) === null || _a === void 0 ? void 0 : _a.version);
    if (!version) {
        (0, logger_1.logWarning)('You are missing version prop in your config. will default to 0');
        return '0';
    }
    var versionFormat = (0, exports.getConfigProp)(c, platform, 'versionFormat');
    if (!versionFormat)
        return version;
    var versionCodeArr = versionFormat.split('.');
    var dotLength = versionCodeArr.length;
    var isNumArr = versionCodeArr.map(function (v) { return !Number.isNaN(Number(v)); });
    var verArr = [];
    var i = 0;
    version.split('.').map(function (v) { return v.split('-').map(function (v2) { return v2.split('+').forEach(function (v3) {
        var isNum = !Number.isNaN(Number(v3));
        if (isNumArr[i] && isNum) {
            verArr.push(v3);
        }
        else if (!isNumArr[i]) {
            verArr.push(v3);
        }
        i++;
    }); }); });
    if (verArr.length > dotLength) {
        verArr.length = dotLength;
    }
    var output = verArr.join('.');
    // console.log(`IN: ${version}\nOUT: ${output}`);
    return output;
};
exports.getAppVersion = getAppVersion;
var getAppVersionCode = function (c, platform) {
    var _a;
    var versionCode = (0, exports.getConfigProp)(c, platform, 'versionCode');
    if (versionCode)
        return versionCode;
    var version = (0, exports.getConfigProp)(c, platform, 'version') || ((_a = c.files.project.package) === null || _a === void 0 ? void 0 : _a.version);
    if (!version || typeof version !== 'string') {
        (0, logger_1.logWarning)('You are missing version prop in your config. will default to 0');
        return '0';
    }
    var versionCodeFormat = (0, exports.getConfigProp)(c, platform, 'versionCodeFormat', '00.00.00');
    var vFormatArr = versionCodeFormat.split('.').map(function (v) { return v.length; });
    var versionCodeMaxCount = vFormatArr.length;
    var verArr = [];
    version.split('.').map(function (v) { return v.split('-').map(function (v2) { return v2.split('+').forEach(function (v3) {
        var asNumber = Number(v3);
        if (!Number.isNaN(asNumber)) {
            var val = v3;
            var maxDigits = vFormatArr[verArr.length] || 2;
            if (v3.length > maxDigits) {
                val = v3.substr(0, maxDigits);
            }
            else if (v3.length < maxDigits) {
                var toAdd = maxDigits - v3.length;
                val = v3;
                while (toAdd > 0) {
                    val = "0".concat(val);
                    toAdd--;
                }
            }
            verArr.push(val);
        }
    }); }); });
    var verCountDiff = verArr.length - versionCodeMaxCount;
    if (verCountDiff < 0) {
        while (verCountDiff < 0) {
            var extraVersionLen = vFormatArr[versionCodeMaxCount + verCountDiff];
            var num = '';
            while (extraVersionLen) {
                num += '0';
                extraVersionLen--;
            }
            verArr.push(num);
            verCountDiff++;
        }
    }
    var output = Number(verArr.join('')).toString();
    // console.log(`IN: ${version}\nOUT: ${output}`);
    return output;
};
exports.getAppVersionCode = getAppVersionCode;
var isMonorepo = function () {
    try {
        (0, fileutils_1.fsExistsSync)(path_1.default.resolve(__dirname, '../../../../lerna.json'));
        return true;
    }
    catch (_err) {
        return false;
    }
};
exports.isMonorepo = isMonorepo;
var getMonorepoRoot = function () {
    if ((0, exports.isMonorepo)()) {
        return path_1.default.resolve(__dirname, '../../../..');
    }
};
exports.getMonorepoRoot = getMonorepoRoot;
var getBuildsFolder = function (c, platform, customPath) {
    var pp = customPath || c.paths.appConfig.dir;
    // if (!fsExistsSync(pp)) {
    //     logWarning(`Path ${chalk().white(pp)} does not exist! creating one for you..`);
    // }
    if (!pp) {
        (0, logger_1.logWarning)("getBuildsFolder: Path ".concat((0, logger_1.chalk)().white('c.paths.appConfig.dir'), " not defined! can't return path. You might not be in renative project"));
        return null;
    }
    var p = path_1.default.join(pp, "builds/".concat(platform, "@").concat(c.runtime.scheme));
    if ((0, fileutils_1.fsExistsSync)(p))
        return p;
    return path_1.default.join(pp, "builds/".concat(platform));
};
exports.getBuildsFolder = getBuildsFolder;
var getIP = function () { return ip_1.default.address(); };
exports.getIP = getIP;
var checkPortInUse = function (c, platform, port) { return new Promise(function (resolve, reject) {
    if (port === undefined || port === null) {
        resolve(false);
        return;
    }
    (0, detect_port_1.default)(port, function (err, availablePort) {
        if (err) {
            reject(err);
            return;
        }
        var result = parseInt(port, 10) !== parseInt(availablePort, 10);
        resolve(result);
    });
}); };
exports.checkPortInUse = checkPortInUse;
var getFlavouredProp = function (c, obj, key) {
    if (!key || !obj)
        return null;
    var val1 = obj["".concat(key, "@").concat(c.runtime.scheme)];
    if (val1)
        return val1;
    return obj[key];
};
exports.getFlavouredProp = getFlavouredProp;
var getBuildFilePath = function (c, platform, filePath) {
    // P1 => platformTemplates
    var sp = path_1.default.join((0, exports.getAppTemplateFolder)(c, platform), filePath);
    // P2 => appConfigs/base + @buildSchemes
    var sp2 = path_1.default.join((0, exports.getBuildsFolder)(c, platform, c.paths.project.appConfigBase.dir), filePath);
    if ((0, fileutils_1.fsExistsSync)(sp2))
        sp = sp2;
    // P3 => appConfigs + @buildSchemes
    var sp3 = path_1.default.join((0, exports.getBuildsFolder)(c, platform), filePath);
    if ((0, fileutils_1.fsExistsSync)(sp3))
        sp = sp3;
    return sp;
};
exports.getBuildFilePath = getBuildFilePath;
exports.default = {
    getBuildFilePath: exports.getBuildFilePath,
    getBuildsFolder: exports.getBuildsFolder,
    getAppFolder: exports.getAppFolder,
    getAppTemplateFolder: exports.getAppTemplateFolder,
    getAppId: exports.getAppId,
    getAppTitle: exports.getAppTitle,
    getAppVersion: exports.getAppVersion,
    getAppVersionCode: exports.getAppVersionCode,
    writeCleanFile: fileutils_1.writeCleanFile,
    getEntryFile: exports.getEntryFile,
    getGetJsBundleFile: exports.getGetJsBundleFile,
    getAppDescription: exports.getAppDescription,
    getAppAuthor: exports.getAppAuthor,
    getAppLicense: exports.getAppLicense,
    getConfigProp: exports.getConfigProp,
    getIP: exports.getIP,
    checkPortInUse: exports.checkPortInUse,
    logTask: function (val) {
        (0, logger_1.logError)('DEPRECATED: Common.logTask() has been removed. use Logger.logTask() instead');
        (0, logger_1.logTask)(val);
    },
    logWarning: function (val) {
        (0, logger_1.logError)('DEPRECATED: Common.logWarning() has been removed. use Logger.logWarning() instead');
        (0, logger_1.logWarning)(val);
    },
    logError: function (val) {
        (0, logger_1.logError)('DEPRECATED: Common.logError() has been removed. use Logger.logError() instead');
        (0, logger_1.logError)(val);
    },
    logSuccess: function (val) {
        (0, logger_1.logError)('DEPRECATED: Common.logError() has been removed. use Logger.logError() instead');
        (0, logger_1.logSuccess)(val);
    },
    logDebug: function (val) {
        (0, logger_1.logError)('DEPRECATED: Common.logDebug() has been removed. use Logger.logDebug() instead');
        (0, logger_1.logDebug)(val);
    }
};
//# sourceMappingURL=common.js.map