"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable global-require, import/no-dynamic-require, valid-typeof */
var fileutils_1 = require("../systemManager/fileutils");
var logger_1 = require("../systemManager/logger");
var constants_1 = require("../constants");
var Config = /** @class */ (function () {
    function Config() {
        this.config = {};
    }
    Config.prototype.initializeConfig = function (c) {
        this.config = c;
        return c;
    };
    Config.prototype.getConfig = function () {
        return this.config;
    };
    // RNV CONFIG
    Config.prototype.getConfigValueSeparate = function (key, global) {
        if (global === void 0) { global = false; }
        var paths = this.config.paths;
        if (!global && !(0, fileutils_1.fsExistsSync)(paths.project.config))
            return 'N/A'; // string because there might be a setting where we will use null
        var cfg = global
            ? require(paths.GLOBAL_RNV_CONFIG)
            : require(paths.project.config);
        var value = cfg[constants_1.configSchema[key].key];
        if (value === undefined)
            return 'N/A';
        return value;
    };
    Config.prototype.getMergedConfigValue = function (key) {
        var _a;
        var value = (_a = this.config.buildConfig) === null || _a === void 0 ? void 0 : _a[constants_1.configSchema[key].key];
        if (value === undefined && constants_1.configSchema[key].default) {
            value = constants_1.configSchema[key].default;
        }
        return value;
    };
    Config.prototype.listConfigValue = function (key) {
        var localVal = this.getConfigValueSeparate(key).toString();
        var globalVal = this.getConfigValueSeparate(key, true).toString();
        if (globalVal === 'N/A' && constants_1.configSchema[key].default) {
            globalVal = constants_1.configSchema[key].default;
        }
        if (localVal === 'N/A')
            localVal = globalVal;
        var table = [
            {
                Key: key,
                'Global Value': globalVal
            }
        ];
        if (localVal !== 'N/A') {
            table[0]['Project Value'] = localVal;
        }
        return table;
    };
    Config.prototype.isConfigValueValid = function (key, value) {
        var keySchema = constants_1.configSchema[key];
        if (!keySchema) {
            (0, logger_1.logWarning)("Unknown config param ".concat(key));
            return false;
        }
        if (keySchema.values && !keySchema.values.includes(value)) {
            (0, logger_1.logWarning)("Unsupported value provided for ".concat(key, ". Correct values are ").concat(keySchema.values.join(', ')));
            return false;
        }
        return true;
    };
    Config.prototype.setConfigValue = function (key, value) {
        var _a = this.config, global = _a.program.global, paths = _a.paths;
        if (this.isConfigValueValid(key, value)) {
            var isValid = value;
            var configPath = global
                ? paths.GLOBAL_RNV_CONFIG
                : paths.project.config;
            var config = require(configPath);
            if (['true', 'false'].includes(isValid))
                isValid = isValid === 'true'; // convert string to bool if it matches a bool value
            config[constants_1.configSchema[key].key] = isValid;
            (0, fileutils_1.writeFileSync)(configPath, config);
            return true;
        }
        return false;
    };
    Object.defineProperty(Config.prototype, "isAnalyticsEnabled", {
        get: function () {
            return this.getMergedConfigValue('analytics');
        },
        enumerable: false,
        configurable: true
    });
    return Config;
}());
var Conf = new Config();
exports.default = Conf;
//# sourceMappingURL=config.js.map