"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRnvConfig = exports.parseRenativeConfigs = exports.listAppConfigsFoldersSync = exports.generateLocalConfig = exports.generateRuntimeConfig = exports.loadFileExtended = exports.generateBuildConfig = exports.writeRenativeConfigFile = exports.checkIsRenativeProject = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var deepmerge_1 = (0, tslib_1.__importDefault)(require("deepmerge"));
var constants_1 = require("../constants");
var fileutils_1 = require("../systemManager/fileutils");
var common_1 = require("../common");
var workspace_1 = require("../projectManager/workspace");
var logger_1 = require("../systemManager/logger");
var resolve_1 = require("../systemManager/resolve");
// import { loadPluginTemplates } from '../pluginManager';
var IGNORE_FOLDERS = ['.git'];
var checkIsRenativeProject = function (c) { return new Promise(function (resolve, reject) {
    if (!c.paths.project.configExists) {
        return reject("This directory is not ReNative project. Project config ".concat((0, logger_1.chalk)().white(c.paths.project.config), " is missing!. You can create new project with ").concat((0, logger_1.chalk)().white('rnv new')));
    }
    return resolve();
}); };
exports.checkIsRenativeProject = checkIsRenativeProject;
var _generateConfigPaths = function (pathObj, dir, configName) {
    pathObj.dir = dir;
    pathObj.config = path_1.default.join(dir, configName || constants_1.RENATIVE_CONFIG_NAME);
    pathObj.configLocal = path_1.default.join(dir, constants_1.RENATIVE_CONFIG_LOCAL_NAME);
    pathObj.configPrivate = path_1.default.join(dir, constants_1.RENATIVE_CONFIG_PRIVATE_NAME);
    pathObj.appConfigsDir = path_1.default.join(dir, '..');
};
var _arrayMergeOverride = function (destinationArray, sourceArray) { return sourceArray; };
var getEnginesPluginDelta = function (c) {
    var _a;
    (0, logger_1.logDebug)('getEnginesPluginDelta');
    if (!c.buildConfig)
        return;
    var enginePlugins = {};
    var missingEnginePlugins = {};
    var engineConfig = (_a = c.runtime.enginesByPlatform[c.platform]) === null || _a === void 0 ? void 0 : _a.config;
    if (engineConfig === null || engineConfig === void 0 ? void 0 : engineConfig.plugins) {
        var ePlugins = Object.keys(engineConfig.plugins);
        if (ePlugins === null || ePlugins === void 0 ? void 0 : ePlugins.length) {
            ePlugins.forEach(function (pluginKey) {
                var _a, _b, _c;
                if (!((_c = (_b = (_a = c.files) === null || _a === void 0 ? void 0 : _a.project) === null || _b === void 0 ? void 0 : _b.config) === null || _c === void 0 ? void 0 : _c[pluginKey])) {
                    missingEnginePlugins[pluginKey] = engineConfig.plugins[pluginKey];
                }
                enginePlugins[pluginKey] = engineConfig.plugins[pluginKey];
            });
        }
    }
    c.runtime.missingEnginePlugins = missingEnginePlugins;
    return enginePlugins;
};
var writeRenativeConfigFile = function (c, configPath, configData) {
    (0, logger_1.logDebug)("writeRenativeConfigFile:".concat(configPath));
    (0, fileutils_1.writeFileSync)(configPath, configData);
    (0, exports.generateBuildConfig)(c);
};
exports.writeRenativeConfigFile = writeRenativeConfigFile;
var generateBuildConfig = function (c) {
    (0, logger_1.logDebug)('generateBuildConfig');
    var mergeOrder = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([
        c.paths.defaultWorkspace.config,
        c.paths.rnv.projectTemplates.config,
        c.paths.rnv.pluginTemplates.config,
        // c.paths.rnv.platformTemplates.config,
        c.paths.workspace.config,
        c.paths.workspace.configPrivate,
        c.paths.workspace.configLocal,
        c.paths.workspace.project.config,
        c.paths.workspace.project.configPrivate,
        c.paths.workspace.project.configLocal
    ], c.paths.workspace.appConfig.configs, true), c.paths.workspace.appConfig.configsPrivate, true), c.paths.workspace.appConfig.configsLocal, true), [
        c.paths.project.config,
        c.paths.project.configPrivate,
        c.paths.project.configLocal
    ], false), c.paths.appConfig.configs, true), c.paths.appConfig.configsPrivate, true), c.paths.appConfig.configsLocal, true);
    var cleanPaths = mergeOrder.filter(function (v) { return v; });
    var existsPaths = cleanPaths.filter(function (v) {
        var exists = (0, fileutils_1.fsExistsSync)(v);
        if (exists) {
            (0, logger_1.logDebug)("Merged: ".concat(v));
        }
        else {
            // console.log(chalk().red(v));
        }
        return exists;
    });
    var pluginTemplates = {};
    if (c.files.rnv.pluginTemplates.configs) {
        Object.keys(c.files.rnv.pluginTemplates.configs).forEach(function (v) {
            var plgs = c.files.rnv.pluginTemplates.configs[v];
            pluginTemplates[v] = plgs;
        });
    }
    var extraPlugins = getEnginesPluginDelta(c);
    var mergeFiles = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([
        c.files.defaultWorkspace.config,
        c.files.rnv.projectTemplates.config,
        { plugins: extraPlugins },
        // { pluginTemplates },
        c.files.workspace.config,
        c.files.workspace.configPrivate,
        c.files.workspace.configLocal,
        c.files.workspace.project.config,
        c.files.workspace.project.configPrivate,
        c.files.workspace.project.configLocal
    ], c.files.workspace.appConfig.configs, true), c.files.workspace.appConfig.configsPrivate, true), c.files.workspace.appConfig.configsLocal, true), [
        c.files.project.config,
        c.files.project.configPrivate,
        c.files.project.configLocal
    ], false), c.files.appConfig.configs, true), c.files.appConfig.configsPrivate, true), c.files.appConfig.configsLocal, true);
    // mergeFiles.forEach((mergeFile, i) => {
    //     console.log(`MERGEDIAGNOSTICS ${i}`, Object.keys(mergeFile?.plugins || {}));
    // });
    var meta = [
        {
            _meta: {
                generated: new Date().getTime(),
                mergedConfigs: existsPaths
            }
        }
    ];
    var existsFiles = mergeFiles.filter(function (v) { return v; });
    (0, logger_1.logDebug)("generateBuildConfig:mergeOrder.length:".concat(mergeOrder.length, ",cleanPaths.length:").concat(cleanPaths.length, ",existsPaths.length:").concat(existsPaths.length, ",existsFiles.length:").concat(existsFiles.length));
    var out = deepmerge_1.default.all((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], meta, true), existsFiles, true), {
        arrayMerge: _arrayMergeOverride
    });
    out = (0, deepmerge_1.default)({}, out);
    out.pluginTemplates = pluginTemplates;
    c.buildConfig = (0, fileutils_1.sanitizeDynamicRefs)(c, out);
    var propConfig = {
        files: c.files,
        runtimeProps: c.runtime,
        props: c.buildConfig._refs,
        configProps: c.configPropsInjects
    };
    c.buildConfig = (0, fileutils_1.sanitizeDynamicProps)(c.buildConfig, propConfig);
    (0, logger_1.logDebug)('BUILD_CONFIG', Object.keys(c.buildConfig));
    if (c.runtime.appId) {
        c.paths.project.builds.config = path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(c.platform, ".json"));
        (0, logger_1.logDebug)("generateBuildConfig: will sanitize file at: ".concat(c.paths.project.builds.config));
        if (c.paths.project.builds.dir) {
            if (!(0, fileutils_1.fsExistsSync)(c.paths.project.builds.dir)) {
                (0, fileutils_1.mkdirSync)(c.paths.project.builds.dir);
            }
            var result = (0, fileutils_1.writeFileSync)(c.paths.project.builds.config, c.buildConfig);
            if (result) {
                var size = (0, fileutils_1.formatBytes)(Buffer.byteLength(result || '', 'utf8'));
                (0, logger_1.logTask)((0, logger_1.chalk)().grey('generateBuildConfig'), "size:".concat(size));
            }
            else {
                (0, logger_1.logDebug)("generateBuildConfig NOT SAVED: ".concat(c.paths.project.builds.config));
            }
        }
        else {
            (0, logger_1.logWarning)('Cannot save buildConfig as c.paths.project.builds.dir is not defined');
        }
    }
};
exports.generateBuildConfig = generateBuildConfig;
var loadFileExtended = function (c, fileObj, pathObj, key) {
    var _a;
    var result = (0, fileutils_1.loadFile)(fileObj, pathObj, key);
    if (fileObj[key]) {
        fileObj["".concat(key, "_original")] = (0, tslib_1.__assign)({}, fileObj[key]);
    }
    var extendsTemplate = (_a = fileObj[key]) === null || _a === void 0 ? void 0 : _a.extendsTemplate;
    if (extendsTemplate) {
        var currTemplate = c.files.project[key].currentTemplate || fileObj[key].currentTemplate;
        if (!currTemplate) {
            if (extendsTemplate.startsWith('@')) {
                currTemplate = extendsTemplate.split('/').slice(0, 2).join('/');
            }
            else {
                currTemplate = extendsTemplate.split('/').slice(0, 1);
            }
        }
        if (currTemplate) {
            var extendsPath = void 0;
            if (extendsTemplate.startsWith(currTemplate)) {
                extendsPath = path_1.default.join((0, resolve_1.doResolve)(currTemplate), extendsTemplate.replace(currTemplate, ''));
            }
            else {
                extendsPath = path_1.default.join((0, resolve_1.doResolve)(currTemplate), extendsTemplate);
            }
            if ((0, fileutils_1.fsExistsSync)(extendsPath)) {
                var extendsFile = (0, fileutils_1.readObjectSync)(extendsPath);
                fileObj[key] = (0, fileutils_1.mergeObjects)(c, extendsFile, fileObj[key], false, true);
                // CLEAN props which should not be inherited
                delete fileObj[key].isTemplate;
                delete fileObj[key].tasks;
            }
            else {
                (0, logger_1.logWarning)("You are trying to extend config file with ".concat(extendsPath, " does not exists. SKIPPING."));
            }
        }
    }
    return result;
};
exports.loadFileExtended = loadFileExtended;
var _loadConfigFiles = function (c, fileObj, pathObj, parseAppConfigs) {
    var result = false;
    var extendAppId;
    if ((0, exports.loadFileExtended)(c, fileObj, pathObj, 'config', true)) {
        extendAppId = fileObj.config.extend || extendAppId;
        result = true;
    }
    if ((0, exports.loadFileExtended)(c, fileObj, pathObj, 'configLocal')) {
        extendAppId = fileObj.configLocal.extend || extendAppId;
        result = true;
    }
    if ((0, exports.loadFileExtended)(c, fileObj, pathObj, 'configPrivate')) {
        extendAppId = fileObj.configPrivate.extend || extendAppId;
        result = true;
    }
    if (parseAppConfigs) {
        pathObj.dirs = [];
        pathObj.fontsDirs = [];
        pathObj.pluginDirs = [];
        pathObj.configs = [];
        pathObj.configsLocal = [];
        pathObj.configsPrivate = [];
        fileObj.configs = [];
        fileObj.configsLocal = [];
        fileObj.configsPrivate = [];
        var fileObj1 = {};
        // PATH1: appConfigs/base
        var path1 = path_1.default.join(pathObj.appConfigsDir, 'base');
        var pathObj1 = {
            config: path_1.default.join(path1, constants_1.RENATIVE_CONFIG_NAME),
            configLocal: path_1.default.join(path1, constants_1.RENATIVE_CONFIG_LOCAL_NAME),
            configPrivate: path_1.default.join(path1, constants_1.RENATIVE_CONFIG_PRIVATE_NAME),
        };
        pathObj.dirs.push(path1);
        pathObj.fontsDirs.push(path_1.default.join(path1, 'fonts'));
        pathObj.pluginDirs.push(path_1.default.join(path1, 'plugins'));
        pathObj.configs.push(pathObj1.config);
        pathObj.configsPrivate.push(pathObj1.configPrivate);
        pathObj.configsLocal.push(pathObj1.configLocal);
        // FILE1: appConfigs/base
        (0, exports.loadFileExtended)(c, fileObj1, pathObj1, 'config');
        (0, exports.loadFileExtended)(c, fileObj1, pathObj1, 'configPrivate');
        (0, exports.loadFileExtended)(c, fileObj1, pathObj1, 'configLocal');
        if (fileObj1.config)
            fileObj.configs.push(fileObj1.config);
        if (fileObj1.configPrivate)
            fileObj.configsPrivate.push(fileObj1.configPrivate);
        if (fileObj1.configLocal)
            fileObj.configsLocal.push(fileObj1.configLocal);
        if ((0, fileutils_1.fsExistsSync)(pathObj.appConfigsDir)) {
            var appConfigsDirNames = (0, fileutils_1.fsReaddirSync)(pathObj.appConfigsDir);
            if (parseAppConfigs && extendAppId && appConfigsDirNames.includes(extendAppId)) {
                var path2 = path_1.default.join(pathObj.appConfigsDir, extendAppId);
                var pathObj2 = {
                    config: path_1.default.join(path2, constants_1.RENATIVE_CONFIG_NAME),
                    configLocal: path_1.default.join(path2, constants_1.RENATIVE_CONFIG_LOCAL_NAME),
                    configPrivate: path_1.default.join(path2, constants_1.RENATIVE_CONFIG_PRIVATE_NAME),
                };
                var fileObj2 = {};
                // PATH2: appConfigs/<extendConfig>
                pathObj.dirs.push(path2);
                pathObj.fontsDirs.push(path_1.default.join(path2, 'fonts'));
                pathObj.pluginDirs.push(path_1.default.join(path2, 'plugins'));
                pathObj.configs.push(pathObj2.config);
                pathObj.configsLocal.push(pathObj2.configLocal);
                pathObj.configsPrivate.push(pathObj2.configPrivate);
                // FILE2: appConfigs/<extendConfig>
                (0, exports.loadFileExtended)(c, fileObj2, pathObj2, 'config');
                (0, exports.loadFileExtended)(c, fileObj2, pathObj2, 'configPrivate');
                (0, exports.loadFileExtended)(c, fileObj2, pathObj2, 'configLocal');
                if (fileObj2.config)
                    fileObj.configs.push(fileObj2.config);
                if (fileObj2.configLocal)
                    fileObj.configsLocal.push(fileObj2.configLocal);
                if (fileObj2.configPrivate)
                    fileObj.configsPrivate.push(fileObj2.configPrivate);
            }
        }
        // PATH2: appConfigs/<appId>
        var path3 = pathObj.dir;
        pathObj.dirs.push(path3);
        pathObj.fontsDirs.push(path_1.default.join(path3, 'fonts'));
        pathObj.pluginDirs.push(path_1.default.join(path3, 'plugins'));
        pathObj.configs.push(path_1.default.join(path3, constants_1.RENATIVE_CONFIG_NAME));
        pathObj.configsLocal.push(path_1.default.join(path3, constants_1.RENATIVE_CONFIG_LOCAL_NAME));
        pathObj.configsPrivate.push(path_1.default.join(path3, constants_1.RENATIVE_CONFIG_PRIVATE_NAME));
        // FILE3: appConfigs/<appId>
        (0, exports.loadFileExtended)(c, fileObj, pathObj, 'config');
        (0, exports.loadFileExtended)(c, fileObj, pathObj, 'configPrivate');
        (0, exports.loadFileExtended)(c, fileObj, pathObj, 'configLocal');
        if (fileObj.config)
            fileObj.configs.push(fileObj.config);
        if (fileObj.configPrivate)
            fileObj.configsPrivate.push(fileObj.configPrivate);
        if (fileObj.configLocal)
            fileObj.configsLocal.push(fileObj.configLocal);
    }
    (0, exports.generateBuildConfig)(c);
    return result;
};
var generateRuntimeConfig = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var sanitizedConfig;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        (0, logger_1.logTask)('generateRuntimeConfig');
        // c.assetConfig = {
        //     common: c.buildConfig.common,
        //     runtime: c.buildConfig.runtime
        // };
        c.assetConfig = (0, fileutils_1.mergeObjects)(c, c.assetConfig, c.buildConfig.runtime || {});
        c.assetConfig = (0, fileutils_1.mergeObjects)(c, c.assetConfig, ((_a = c.buildConfig.common) === null || _a === void 0 ? void 0 : _a.runtime) || {});
        c.assetConfig = (0, fileutils_1.mergeObjects)(c, c.assetConfig, ((_c = (_b = c.buildConfig.platforms) === null || _b === void 0 ? void 0 : _b[c.platform]) === null || _c === void 0 ? void 0 : _c.runtime) || {});
        c.assetConfig = (0, fileutils_1.mergeObjects)(c, c.assetConfig, (0, common_1.getConfigProp)(c, c.platform, 'runtime') || {});
        if ((0, fileutils_1.fsExistsSync)(c.paths.project.assets.dir)) {
            sanitizedConfig = (0, fileutils_1.sanitizeDynamicProps)(c.assetConfig, {
                files: c.files,
                runtimeProps: c.runtime,
                props: {},
                configProps: c.configPropsInjects
            });
            (0, fileutils_1.writeFileSync)(c.paths.project.assets.config, sanitizedConfig);
            c.files.project.assets.config = sanitizedConfig;
        }
        return [2 /*return*/, true];
    });
}); };
exports.generateRuntimeConfig = generateRuntimeConfig;
var generateLocalConfig = function (c, resetAppId) {
    (0, logger_1.logTask)('generateLocalConfig', "resetAppId:".concat(!!resetAppId));
    var configLocal = c.files.project.configLocal || {};
    configLocal._meta = configLocal._meta || {};
    if (resetAppId) {
        delete configLocal._meta.currentAppConfigId;
    }
    else {
        configLocal._meta.currentAppConfigId = c.runtime.appId;
    }
    c.files.project.configLocal = configLocal;
    (0, fileutils_1.writeFileSync)(c.paths.project.configLocal, configLocal);
};
exports.generateLocalConfig = generateLocalConfig;
var _generatePlatformTemplatePaths = function (c) {
    var _a, _b, _c, _d;
    (0, logger_1.logTask)('_generatePlatformTemplatePaths');
    if (!c.buildConfig.paths) {
        (0, logger_1.logWarning)("You're missing paths object in your ".concat((0, logger_1.chalk)().red(c.paths.project.config)));
        c.buildConfig.paths = {};
    }
    if ((_a = c.files.config) === null || _a === void 0 ? void 0 : _a.platformTemplatesDirs) {
        (0, logger_1.logWarning)("DEPRECATED: platformTemplatesDirs in ".concat((0, logger_1.chalk)().white(c.paths.project.config), " has been moved to engine config"));
    }
    if ((_c = (_b = c.files.config) === null || _b === void 0 ? void 0 : _b.paths) === null || _c === void 0 ? void 0 : _c.platformTemplatesDir) {
        (0, logger_1.logWarning)("DEPRECATED: paths.platformTemplatesDir in ".concat((0, logger_1.chalk)().white(c.paths.project.config), " has been moved to engine config"));
    }
    if ((_d = c.files.config) === null || _d === void 0 ? void 0 : _d.platformTemplatesDir) {
        (0, logger_1.logWarning)("DEPRECATED: platformTemplatesDir in ".concat((0, logger_1.chalk)().white(c.paths.project.config), " has been moved to engine config"));
    }
    var pt = c.buildConfig.paths.platformTemplatesDirs || c.buildConfig.platformTemplatesDirs || {};
    var result = {};
    if (c.buildConfig.defaults) {
        c.buildConfig.defaults.supportedPlatforms.forEach(function (platform) {
            var engine = c.runtime.enginesByPlatform[platform];
            if (engine) {
                var originalPath = engine.originalTemplatePlatformsDir;
                if (originalPath) {
                    if (!pt[platform]) {
                        result[platform] = (0, fileutils_1.getRealPath)(c, originalPath, 'platformTemplatesDir', originalPath);
                    }
                    else {
                        result[platform] = (0, fileutils_1.getRealPath)(c, pt[platform], 'platformTemplatesDir', originalPath);
                    }
                }
                else {
                    (0, logger_1.logWarning)("Platform ".concat((0, logger_1.chalk)().red(platform), " not supported by any registered engine. SKIPPING..."));
                }
            }
        });
    }
    else {
        (0, logger_1.logWarning)("Your renative.json is missing property: ".concat((0, logger_1.chalk)().red('defaults.supportedPlatforms'), " "));
    }
    return result;
};
var listAppConfigsFoldersSync = function (c, ignoreHiddenConfigs, appConfigsDirPath) {
    var _a;
    (0, logger_1.logTask)('listAppConfigsFoldersSync', "ignoreHiddenConfigs:".concat(!!ignoreHiddenConfigs));
    if (!((_a = c.paths) === null || _a === void 0 ? void 0 : _a.project))
        return [];
    var dirPath = appConfigsDirPath || c.paths.project.appConfigsDir;
    if (!(0, fileutils_1.fsExistsSync)(dirPath))
        return [];
    var appConfigsDirs = [];
    (0, fileutils_1.fsReaddirSync)(dirPath).forEach(function (dir) {
        var appConfigDir = path_1.default.join(dirPath, dir);
        if (!IGNORE_FOLDERS.includes(dir)
            && (0, fileutils_1.fsLstatSync)(appConfigDir).isDirectory()) {
            if (ignoreHiddenConfigs) {
                var appConfig = path_1.default.join(appConfigDir, constants_1.RENATIVE_CONFIG_NAME);
                if ((0, fileutils_1.fsExistsSync)(appConfig)) {
                    try {
                        var config = (0, fileutils_1.readObjectSync)(appConfig);
                        if ((config === null || config === void 0 ? void 0 : config.hidden) !== true) {
                            appConfigsDirs.push(dir);
                        }
                    }
                    catch (e) {
                        (0, logger_1.logWarning)("_listAppConfigsFoldersSync: ".concat(e));
                    }
                }
            }
            else {
                appConfigsDirs.push(dir);
            }
        }
    });
    return appConfigsDirs;
};
exports.listAppConfigsFoldersSync = listAppConfigsFoldersSync;
var _loadWorkspacesSync = function (c) {
    var _a;
    // CHECK WORKSPACES
    if ((0, fileutils_1.fsExistsSync)(c.paths.rnv.configWorkspaces)) {
        (0, logger_1.logDebug)("".concat(c.paths.rnv.configWorkspaces, " file exists!"));
        c.files.rnv.configWorkspaces = (0, fileutils_1.readObjectSync)(c.paths.rnv.configWorkspaces);
        if (!c.files.rnv.configWorkspaces)
            c.files.rnv.configWorkspaces = {};
        if (!((_a = c.files.rnv.configWorkspaces) === null || _a === void 0 ? void 0 : _a.workspaces)) {
            c.files.rnv.configWorkspaces.workspaces = {};
        }
        if (Object.keys(c.files.rnv.configWorkspaces.workspaces).length === 0) {
            (0, logger_1.logWarning)("No workspace found in ".concat(c.paths.rnv.configWorkspaces, ". Creating default rnv one for you"));
            c.files.rnv.configWorkspaces.workspaces = {
                rnv: {
                    path: c.paths.workspace.dir
                }
            };
            (0, fileutils_1.writeFileSync)(c.paths.rnv.configWorkspaces, c.files.rnv.configWorkspaces);
        }
    }
    else {
        (0, logger_1.logWarning)("Cannot find ".concat(c.paths.rnv.configWorkspaces, ". creating one.."));
        c.files.rnv.configWorkspaces = {
            workspaces: {
                rnv: {
                    path: c.paths.workspace.dir
                }
            }
        };
        (0, fileutils_1.writeFileSync)(c.paths.rnv.configWorkspaces, c.files.rnv.configWorkspaces);
    }
};
var parseRenativeConfigs = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _a, _b, _c, _d, workspaceAppConfigsDir, _e, _f, _g, _h;
    return (0, tslib_1.__generator)(this, function (_j) {
        switch (_j.label) {
            case 0:
                (0, logger_1.logTask)('parseRenativeConfigs');
                // LOAD ./package.json
                (0, fileutils_1.loadFile)(c.files.project, c.paths.project, 'package');
                // LOAD ./RENATIVE.*.JSON
                _loadConfigFiles(c, c.files.project, c.paths.project);
                if (c.runtime.appId) {
                    c.paths.project.builds.config = path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(c.platform, ".json"));
                }
                else {
                    c.paths.project.builds.config = path_1.default.join(c.paths.project.builds.dir, "<TBC>_".concat(c.platform, ".json"));
                }
                // LOAD ./platformBuilds/RENATIVE.BUILLD.JSON
                (0, fileutils_1.loadFile)(c.files.project.builds, c.paths.project.builds, 'config');
                // LOAD WORKSPACE /RENATIVE.*.JSON
                _a = _generateConfigPaths;
                _b = [c.paths.workspace];
                _c = fileutils_1.getRealPath;
                _d = [c];
                return [4 /*yield*/, (0, workspace_1.getWorkspaceDirPath)(c)];
            case 1:
                // LOAD WORKSPACE /RENATIVE.*.JSON
                _a.apply(void 0, _b.concat([_c.apply(void 0, _d.concat([_j.sent()]))]));
                _loadConfigFiles(c, c.files.workspace, c.paths.workspace);
                // LOAD DEFAULT WORKSPACE
                _generateConfigPaths(c.paths.defaultWorkspace, c.paths.GLOBAL_RNV_DIR);
                _loadConfigFiles(c, c.files.defaultWorkspace, c.paths.defaultWorkspace);
                // LOAD PROJECT TEMPLATES
                c.files.rnv.projectTemplates.config = (0, fileutils_1.readObjectSync)(c.paths.rnv.projectTemplates.config);
                // // LOAD PLUGIN TEMPLATES
                // await loadPluginTemplates(c);
                if (!c.files.project.config) {
                    (0, logger_1.logDebug)("BUILD_CONFIG: c.files.project.config does not exists. path: ".concat(c.paths.project.config));
                    return [2 /*return*/];
                }
                // LOAD WORKSPACE /[PROJECT_NAME]/RENATIVE.*.JSON
                if (!c.files.project.config.projectName) {
                    return [2 /*return*/, Promise.reject('Your renative.json is missing required property: projectName ')];
                }
                _generateConfigPaths(c.paths.workspace.project, path_1.default.join(c.paths.workspace.dir, c.files.project.config.projectName));
                _loadConfigFiles(c, c.files.workspace.project, c.paths.workspace.project);
                c.paths.workspace.project.appConfigBase.dir = path_1.default.join(c.paths.workspace.project.dir, 'appConfigs', 'base');
                c.paths.workspace.project.appConfigBase.dir_LEGACY = path_1.default.join(c.paths.workspace.project.dir, 'projectConfig');
                c.paths.project.platformTemplatesDirs = _generatePlatformTemplatePaths(c);
                if (!c.runtime.appId) return [3 /*break*/, 3];
                if (!c.files.appConfig.config) {
                    // _generateConfigPaths(
                    //     c.paths.appConfig,
                    //     path.join(c.paths.project.appConfigsDir, c.runtime.appId)
                    // );
                    _generateConfigPaths(c.paths.appConfig, c.runtime.appConfigDir);
                    _loadConfigFiles(c, c.files.appConfig, c.paths.appConfig, true);
                }
                workspaceAppConfigsDir = (0, fileutils_1.getRealPath)(c, c.buildConfig.workspaceAppConfigsDir);
                c.paths.workspace.project.appConfigsDir = workspaceAppConfigsDir
                    || path_1.default.join(c.paths.workspace.project.dir, 'appConfigs');
                _generateConfigPaths(c.paths.workspace.appConfig, path_1.default.join(c.paths.workspace.project.appConfigsDir, c.runtime.appId));
                _loadConfigFiles(c, c.files.workspace.appConfig, c.paths.workspace.appConfig, true);
                (0, fileutils_1.loadFile)(c.files.project.assets, c.paths.project.assets, 'config');
                // LOAD WORKSPACE /RENATIVE.*.JSON
                _e = _generateConfigPaths;
                _f = [c.paths.workspace];
                _g = fileutils_1.getRealPath;
                _h = [c];
                return [4 /*yield*/, (0, workspace_1.getWorkspaceDirPath)(c)];
            case 2:
                // LOAD WORKSPACE /RENATIVE.*.JSON
                _e.apply(void 0, _f.concat([_g.apply(void 0, _h.concat([_j.sent()]))]));
                _loadConfigFiles(c, c.files.workspace, c.paths.workspace);
                (0, exports.generateLocalConfig)(c);
                (0, exports.generateBuildConfig)(c);
                _j.label = 3;
            case 3: return [2 /*return*/];
        }
    });
}); };
exports.parseRenativeConfigs = parseRenativeConfigs;
var createRnvConfig = function (program, process, cmd, subCmd, _a) {
    var _b = _a === void 0 ? {} : _a, projectRoot = _b.projectRoot;
    var c = {
        _renativePluginCache: {},
        cli: {},
        api: {
            fsExistsSync: fileutils_1.fsExistsSync,
            fsReadFileSync: fileutils_1.fsReadFileSync,
            fsReaddirSync: fileutils_1.fsReaddirSync,
            fsWriteFileSync: fileutils_1.fsWriteFileSync,
            path: path_1.default
        },
        configPropsInjects: {},
        runtime: {
            enginesByPlatform: {},
            enginesByIndex: [],
            enginesById: {}
        },
        paths: {
            rnv: {
                pluginTemplates: {
                    configs: {}
                },
                platformTemplates: {},
                projectTemplates: {},
                platformTemplate: {},
                plugins: {},
                engines: {},
                projectTemplate: {}
            },
            workspace: {
                project: {
                    appConfigBase: {},
                    builds: {},
                    assets: {},
                    platformTemplates: {},
                    appConfigsDirs: [],
                    appConfigsDirNames: []
                },
                appConfig: {
                    configs: [],
                    configsPrivate: [],
                    configsLocal: []
                }
            },
            defaultWorkspace: {
                project: {
                    appConfigBase: {},
                    builds: {},
                    assets: {},
                    platformTemplates: {},
                    appConfigsDirs: [],
                    appConfigsDirNames: []
                },
                appConfig: {
                    configs: [],
                    configsPrivate: [],
                    configsLocal: []
                }
            },
            project: {
                appConfigBase: {},
                builds: {},
                assets: {},
                platformTemplates: {},
                appConfigsDirs: [],
                appConfigsDirNames: []
            },
            appConfig: {
                configs: [],
                configsPrivate: [],
                configsLocal: []
            },
            // EXTRA
            GLOBAL_RNV_DIR: '',
            buildHooks: {
                dist: {}
            },
            home: {},
            template: {
                appConfigBase: {},
                builds: {},
                assets: {},
                platformTemplates: {}
            },
        },
        files: {
            rnv: {
                pluginTemplates: {},
                platformTemplates: {},
                projectTemplates: {},
                platformTemplate: {},
                plugins: {},
                engines: {},
                projectTemplate: {}
            },
            workspace: {
                project: {
                    appConfigBase: {},
                    builds: {},
                    assets: {},
                    platformTemplates: {}
                },
                appConfig: {
                    configs: [],
                    configsPrivate: [],
                    configsLocal: []
                }
            },
            defaultWorkspace: {
                project: {
                    appConfigBase: {},
                    builds: {},
                    assets: {},
                    platformTemplates: {}
                },
                appConfig: {
                    configs: [],
                    configsPrivate: [],
                    configsLocal: []
                }
            },
            project: {
                appConfigBase: {},
                builds: {},
                assets: {},
                platformTemplates: {}
            },
            appConfig: {
                configs: [],
                configsPrivate: [],
                configsLocal: []
            }
        }
    };
    global.RNV_CONFIG = c;
    c.program = program;
    c.process = process;
    c.command = cmd;
    c.subCommand = subCmd;
    // c.platformDefaults = PLATFORMS;
    c.paths.rnv.dir = constants_1.RNV_HOME_DIR;
    c.paths.rnv.engines.dir = path_1.default.join(c.paths.rnv.dir, 'engineTemplates');
    c.paths.rnv.pluginTemplates.dir = path_1.default.join(c.paths.rnv.dir, 'pluginTemplates');
    c.paths.rnv.pluginTemplates.config = path_1.default.join(c.paths.rnv.pluginTemplates.dir, constants_1.RENATIVE_CONFIG_PLUGINS_NAME);
    c.paths.rnv.projectTemplates.dir = path_1.default.join(c.paths.rnv.dir, 'coreTemplateFiles');
    c.paths.rnv.projectTemplates.config = path_1.default.join(c.paths.rnv.projectTemplates.dir, constants_1.RENATIVE_CONFIG_TEMPLATES_NAME);
    c.paths.rnv.package = path_1.default.join(c.paths.rnv.dir, 'package.json');
    c.paths.rnv.projectTemplate.dir = path_1.default.join(c.paths.rnv.dir, 'coreTemplateFiles');
    c.files.rnv.package = JSON.parse((0, fileutils_1.fsReadFileSync)(c.paths.rnv.package).toString());
    c.platform = c.program.platform;
    c.paths.home.dir = constants_1.USER_HOME_DIR;
    c.paths.GLOBAL_RNV_DIR = path_1.default.join(c.paths.home.dir, '.rnv');
    c.paths.GLOBAL_RNV_CONFIG = path_1.default.join(c.paths.GLOBAL_RNV_DIR, constants_1.RENATIVE_CONFIG_NAME);
    c.paths.rnv.configWorkspaces = path_1.default.join(c.paths.GLOBAL_RNV_DIR, constants_1.RENATIVE_CONFIG_WORKSPACES_NAME);
    if (!(0, fileutils_1.fsExistsSync)(c.paths.GLOBAL_RNV_DIR)) {
        (0, fileutils_1.mkdirSync)(c.paths.GLOBAL_RNV_DIR);
    }
    _generateConfigPaths(c.paths.project, projectRoot || constants_1.CURRENT_DIR, c.program.configName);
    c.paths.buildHooks.dir = path_1.default.join(c.paths.project.dir, 'buildHooks/src');
    c.paths.buildHooks.dist.dir = path_1.default.join(c.paths.project.dir, 'buildHooks/dist');
    c.paths.buildHooks.index = path_1.default.join(c.paths.buildHooks.dir, 'index.js');
    c.paths.buildHooks.dist.index = path_1.default.join(c.paths.buildHooks.dist.dir, 'index.js');
    c.paths.project.nodeModulesDir = path_1.default.join(c.paths.project.dir, 'node_modules');
    c.paths.project.srcDir = path_1.default.join(c.paths.project.dir, 'src');
    c.paths.project.appConfigsDir = path_1.default.join(c.paths.project.dir, 'appConfigs');
    c.paths.project.package = path_1.default.join(c.paths.project.dir, 'package.json');
    c.paths.project.rnCliConfig = path_1.default.join(c.paths.project.dir, constants_1.RN_CLI_CONFIG_NAME);
    c.paths.project.babelConfig = path_1.default.join(c.paths.project.dir, constants_1.RN_BABEL_CONFIG_NAME);
    // c.paths.project.npmLinkPolyfill = path.join(
    //     c.paths.project.dir,
    //     'npm_link_polyfill.json'
    // );
    c.paths.project.appConfigBase.dir = path_1.default.join(c.paths.project.dir, 'appConfigs', 'base');
    c.paths.project.appConfigBase.pluginsDir = path_1.default.join(c.paths.project.appConfigBase.dir, 'plugins');
    c.paths.project.appConfigBase.fontsDir = path_1.default.join(c.paths.project.appConfigBase.dir, 'fonts');
    c.paths.project.appConfigBase.fontsDirs = [
        c.paths.project.appConfigBase.fontsDir
    ];
    c.paths.project.assets.dir = path_1.default.join(c.paths.project.dir, 'platformAssets');
    c.paths.project.assets.runtimeDir = path_1.default.join(c.paths.project.assets.dir, 'runtime');
    c.paths.project.assets.config = path_1.default.join(c.paths.project.assets.dir, constants_1.RENATIVE_CONFIG_RUNTIME_NAME);
    c.paths.project.builds.dir = path_1.default.join(c.paths.project.dir, 'platformBuilds');
    _generateConfigPaths(c.paths.workspace, c.paths.GLOBAL_RNV_DIR);
    // LOAD WORKSPACES
    try {
        _loadWorkspacesSync(c);
    }
    catch (e) {
        (0, logger_1.logError)(e);
    }
    return c;
};
exports.createRnvConfig = createRnvConfig;
//# sourceMappingURL=index.js.map