"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.injectPlatformDependencies = exports.checkRequiredPackage = void 0;
var tslib_1 = require("tslib");
var semver_1 = (0, tslib_1.__importDefault)(require("semver"));
var deepmerge_1 = (0, tslib_1.__importDefault)(require("deepmerge"));
var exec_1 = require("../systemManager/exec");
var npmUtils_1 = require("../systemManager/npmUtils");
var logger_1 = require("../systemManager/logger");
var prompt_1 = require("../../cli/prompt");
var engineManager_1 = require("../engineManager");
var index_1 = require("./index");
var pluginManager_1 = require("../pluginManager");
var projectManager_1 = require("../projectManager");
var common_1 = require("../common");
var injectProjectDependency = function (c, dependency, version, type, skipInstall) {
    if (skipInstall === void 0) { skipInstall = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var currentPackage, existingPath;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0:
                    (0, logger_1.logTask)('injectProjectDependency');
                    currentPackage = c.files.project.package;
                    existingPath = c.paths.project.package;
                    if (!currentPackage[type])
                        currentPackage[type] = {};
                    currentPackage[type][dependency] = version;
                    (0, index_1.writeRenativeConfigFile)(c, existingPath, currentPackage);
                    if (!!skipInstall) return [3 /*break*/, 4];
                    return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, (0, pluginManager_1.overrideTemplatePlugins)(c)];
                case 2:
                    _a.sent();
                    return [4 /*yield*/, (0, projectManager_1.configureFonts)(c)];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4: return [2 /*return*/, true];
            }
        });
    });
};
var checkRequiredPackage = function (c, pkg, version, type, skipAsking, skipInstall, skipVersionCheck) {
    if (version === void 0) { version = false; }
    if (skipAsking === void 0) { skipAsking = false; }
    if (skipInstall === void 0) { skipInstall = false; }
    if (skipVersionCheck === void 0) { skipVersionCheck = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var projectConfig, confirm_1, resp, latestVersion, e_1, currentVersion, latestVersion, e_2, updateAvailable, confirm_2, resp;
        var _a;
        return (0, tslib_1.__generator)(this, function (_b) {
            switch (_b.label) {
                case 0:
                    (0, logger_1.logDebug)('checkRequiredPackage');
                    if (!pkg)
                        return [2 /*return*/, false];
                    projectConfig = c.files.project;
                    if (!!((_a = projectConfig.package[type]) === null || _a === void 0 ? void 0 : _a[pkg])) return [3 /*break*/, 8];
                    confirm_1 = skipAsking;
                    if (!!confirm_1) return [3 /*break*/, 2];
                    return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                            type: 'confirm',
                            message: "You do not have ".concat(pkg, " installed. Do you want to add it now?")
                        })];
                case 1:
                    resp = _b.sent();
                    // eslint-disable-next-line prefer-destructuring
                    confirm_1 = resp.confirm;
                    _b.label = 2;
                case 2:
                    if (!confirm_1) return [3 /*break*/, 7];
                    latestVersion = 'latest';
                    if (!(!version && !skipVersionCheck)) return [3 /*break*/, 6];
                    _b.label = 3;
                case 3:
                    _b.trys.push([3, 5, , 6]);
                    return [4 /*yield*/, (0, exec_1.executeAsync)("npm show ".concat(pkg, " version"))];
                case 4:
                    latestVersion = _b.sent();
                    return [3 /*break*/, 6];
                case 5:
                    e_1 = _b.sent();
                    return [3 /*break*/, 6];
                case 6: return [2 /*return*/, injectProjectDependency(c, pkg, version || latestVersion, type, skipInstall)];
                case 7: return [3 /*break*/, 15];
                case 8:
                    if (!!version) return [3 /*break*/, 15];
                    currentVersion = projectConfig.package[type][pkg];
                    latestVersion = false;
                    _b.label = 9;
                case 9:
                    _b.trys.push([9, 11, , 12]);
                    return [4 /*yield*/, (0, exec_1.executeAsync)("npm show ".concat(pkg, " version"))];
                case 10:
                    latestVersion = _b.sent();
                    return [3 /*break*/, 12];
                case 11:
                    e_2 = _b.sent();
                    return [3 /*break*/, 12];
                case 12:
                    if (!latestVersion) return [3 /*break*/, 15];
                    updateAvailable = false;
                    try {
                        // semver might fail if you have a path instead of a version (like when you are developing)
                        updateAvailable = semver_1.default.lt(currentVersion, latestVersion);
                        // eslint-disable-next-line no-empty
                    }
                    catch (e) { }
                    if (!updateAvailable) return [3 /*break*/, 15];
                    confirm_2 = skipAsking;
                    if (!!confirm_2) return [3 /*break*/, 14];
                    return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                            type: 'confirm',
                            message: "Seems like ".concat(pkg, "@").concat(currentVersion, " is installed while there is a newer version, ").concat(pkg, "@").concat(latestVersion, ". Do you want to upgrade?")
                        })];
                case 13:
                    resp = _b.sent();
                    // eslint-disable-next-line prefer-destructuring
                    confirm_2 = resp.confirm;
                    _b.label = 14;
                case 14:
                    if (confirm_2) {
                        return [2 /*return*/, injectProjectDependency(c, pkg, latestVersion, type, skipInstall)];
                    }
                    _b.label = 15;
                case 15: return [2 /*return*/, false];
            }
        });
    });
};
exports.checkRequiredPackage = checkRequiredPackage;
var injectPlatformDependencies = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var platform, engine, npmDepsBase, npmDepsExt, npmDeps, promises, installed, isMonorepo;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logTask)('injectPlatformDependencies');
                platform = c.platform;
                engine = (0, engineManager_1.getEngineRunnerByPlatform)(c, platform);
                npmDepsBase = ((_a = engine === null || engine === void 0 ? void 0 : engine.config) === null || _a === void 0 ? void 0 : _a.npm) || {};
                npmDepsExt = ((_c = (_b = engine === null || engine === void 0 ? void 0 : engine.config) === null || _b === void 0 ? void 0 : _b.platforms[platform]) === null || _c === void 0 ? void 0 : _c.npm) || {};
                npmDeps = (0, deepmerge_1.default)(npmDepsBase, npmDepsExt);
                if (!npmDeps) return [3 /*break*/, 6];
                promises = Object.keys(npmDeps).reduce(function (acc, type) {
                    // iterate over dependencies, devDepencencies or optionalDependencies
                    Object.keys(npmDeps[type]).forEach(function (dep) {
                        // iterate over deps
                        acc.push((0, exports.checkRequiredPackage)(c, dep, npmDeps[type][dep], type, true, true));
                    });
                    return acc;
                }, []);
                return [4 /*yield*/, Promise.all(promises)];
            case 1:
                installed = _d.sent();
                if (!installed.some(function (i) { return i === true; })) return [3 /*break*/, 6];
                isMonorepo = (0, common_1.getConfigProp)(c, c.platform, 'isMonorepo');
                if (!isMonorepo) return [3 /*break*/, 2];
                (0, logger_1.logInfo)("Found extra npm dependencies required by ".concat((0, logger_1.chalk)().white(engine.config.id), " engine. project marked as monorepo. SKIPPING"));
                return [3 /*break*/, 6];
            case 2:
                // do npm i only if something new is added
                (0, logger_1.logInfo)("Found extra npm dependencies required by ".concat((0, logger_1.chalk)().white(engine.config.id), " engine. ADDING...DONE"));
                return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
            case 3:
                _d.sent();
                return [4 /*yield*/, (0, pluginManager_1.overrideTemplatePlugins)(c)];
            case 4:
                _d.sent();
                return [4 /*yield*/, (0, projectManager_1.configureFonts)(c)];
            case 5:
                _d.sent();
                _d.label = 6;
            case 6: return [2 /*return*/];
        }
    });
}); };
exports.injectPlatformDependencies = injectPlatformDependencies;
//# sourceMappingURL=packageParser.js.map