"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isBuildSchemeSupported = void 0;
var tslib_1 = require("tslib");
var deepmerge_1 = (0, tslib_1.__importDefault)(require("deepmerge"));
var logger_1 = require("../systemManager/logger");
var prompt_1 = require("../../cli/prompt");
var isBuildSchemeSupported = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var scheme, baseBuildSchemes, platformBuildSchemes, buildSchemes, schemeDoesNotExist, schemeOptions_1, schemeVals_1, selectedScheme;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logTask)('isBuildSchemeSupported');
                scheme = c.program.scheme;
                if (!c.buildConfig.platforms[c.platform]) {
                    c.buildConfig.platforms[c.platform] = {};
                }
                baseBuildSchemes = ((_a = c.buildConfig.common) === null || _a === void 0 ? void 0 : _a.buildSchemes) || {};
                platformBuildSchemes = ((_c = (_b = c.buildConfig.platforms) === null || _b === void 0 ? void 0 : _b[c.platform]) === null || _c === void 0 ? void 0 : _c.buildSchemes) || {};
                buildSchemes = (0, deepmerge_1.default)(baseBuildSchemes, platformBuildSchemes);
                if (!buildSchemes) {
                    (0, logger_1.logWarning)("Your appConfig for platform ".concat(c.platform, " has no buildSchemes. Will continue with defaults"));
                    return [2 /*return*/, false];
                }
                schemeDoesNotExist = scheme && !buildSchemes[scheme];
                if (!(scheme === true || schemeDoesNotExist)) return [3 /*break*/, 2];
                if (schemeDoesNotExist && scheme && scheme !== true) {
                    (0, logger_1.logError)('Build scheme you picked does not exists.');
                }
                schemeOptions_1 = [];
                schemeVals_1 = {};
                Object.keys(buildSchemes).forEach(function (k) {
                    var s = buildSchemes[k];
                    var desc = s.description ? (0, logger_1.chalk)().grey(" (".concat(s.description, ")")) : '';
                    var key = "".concat(k).concat(desc);
                    schemeOptions_1.push(key);
                    schemeVals_1[key] = k;
                });
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'selectedScheme',
                        type: 'list',
                        message: 'Pick one of available buildSchemes',
                        choices: schemeOptions_1,
                        logMessage: 'You need to specify scheme'
                    })];
            case 1:
                selectedScheme = (_d.sent()).selectedScheme;
                c.program.scheme = schemeVals_1[selectedScheme];
                c.runtime.scheme = c.program.scheme;
                _d.label = 2;
            case 2:
                (0, logger_1.logInfo)("Current Build Scheme: ".concat((0, logger_1.chalk)().bold.white(c.runtime.scheme)));
                return [2 /*return*/, true];
        }
    });
}); };
exports.isBuildSchemeSupported = isBuildSchemeSupported;
//# sourceMappingURL=schemeParser.js.map