"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRegisteredEngines = exports.getEngineRunner = exports.getEngineSubTasks = exports.hasEngineTask = exports.getEngineTask = exports.getEngineRunnerByPlatform = exports.getPlatformExtensions = exports.generateEnvVars = exports.loadEngines = exports.loadEnginePackageDeps = exports.loadEnginePluginDeps = exports.registerAllPlatformEngines = exports.registerMissingPlatformEngines = exports.configureEngines = exports.generateEngineTasks = exports.generateEngineExtensions = exports.registerEngineExtension = exports.registerEngine = void 0;
var tslib_1 = require("tslib");
/* eslint-disable import/no-dynamic-require, global-require */
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var fileutils_1 = require("../systemManager/fileutils");
var npmUtils_1 = require("../systemManager/npmUtils");
var constants_1 = require("../constants");
var logger_1 = require("../systemManager/logger");
var common_1 = require("../common");
var resolve_1 = require("../systemManager/resolve");
var utils_1 = require("../systemManager/utils");
var configManager_1 = require("../configManager");
var pluginManager_1 = require("../pluginManager");
var prompt_1 = require("../../cli/prompt");
var ENGINE_CORE = 'engine-core';
var registerEngine = function (c, engine, platform, engConfig) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)("registerEngine:".concat(engine.config.id));
        c.runtime.enginesById[engine.config.id] = engine;
        engine.initializeRuntimeConfig(c);
        c.runtime.enginesByIndex.push(engine);
        if (engConfig === null || engConfig === void 0 ? void 0 : engConfig.packageName) {
            engine.rootPath = _resolvePkgPath(c, engConfig.packageName);
            engine.originalTemplatePlatformsDir = path_1.default.join(engine.rootPath, 'templates/platforms');
            engine.originalTemplateAssetsDir = path_1.default.join(engine.rootPath, 'templates/assets');
            engine.originalTemplatePlatformProjectDir = path_1.default.join(engine.originalTemplatePlatformsDir, engine.projectDirName);
        }
        _registerEnginePlatform(c, platform, engine);
        return [2 /*return*/];
    });
}); };
exports.registerEngine = registerEngine;
var _registerEnginePlatform = function (c, platform, engine) {
    if (platform) {
        c.runtime.enginesByPlatform[platform] = engine;
    }
};
var registerEngineExtension = function (ext, eExt, extras) {
    if (extras === void 0) { extras = []; }
    var e1 = ext ? "".concat(ext, ".") : '';
    var e2 = eExt ? "".concat(eExt, ".") : '';
    var extArr;
    if (eExt) {
        extArr = (0, tslib_1.__spreadArray)([
            "".concat(e1).concat(e2, "jsx"),
            "".concat(e1, "jsx"),
            "".concat(e1).concat(e2, "js"),
            "".concat(e1, "js"),
            "".concat(e1).concat(e2, "tsx"),
            "".concat(e1, "tsx"),
            "".concat(e1).concat(e2, "ts"),
            "".concat(e1, "ts")
        ], extras, true);
    }
    else {
        extArr = (0, tslib_1.__spreadArray)([
            "".concat(e1, "jsx"),
            "".concat(e1, "js"),
            "".concat(e1, "tsx"),
            "".concat(e1, "ts")
        ], extras, true);
    }
    return extArr;
};
exports.registerEngineExtension = registerEngineExtension;
var generateEngineExtensions = function (exts, config) {
    var id = config.id, engineExtension = config.engineExtension;
    var extArr = (0, tslib_1.__spreadArray)([], (0, exports.registerEngineExtension)(id), true);
    exts.forEach(function (ext) {
        extArr = extArr.concat((0, exports.registerEngineExtension)(ext, engineExtension));
    });
    extArr = extArr.concat((0, exports.registerEngineExtension)(null, null, ['mjs', 'json', 'cjs', 'wasm']));
    return extArr;
};
exports.generateEngineExtensions = generateEngineExtensions;
var generateEngineTasks = function (taskArr) {
    var tasks = {};
    taskArr.forEach(function (taskInstance) {
        tasks[taskInstance.task] = taskInstance;
    });
    return tasks;
};
exports.generateEngineTasks = generateEngineTasks;
var configureEngines = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var engines, devDependencies, needsPackageUpdate;
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('configureEngines');
        engines = _getFilteredEngines(c);
        devDependencies = c.files.project.package.devDependencies;
        needsPackageUpdate = false;
        if (engines && !c.runtime.skipPackageUpdate && !c.program.skipDependencyCheck && !c.program.skipRnvCheck) {
            Object.keys(engines).forEach(function (k) {
                var _a, _b;
                var engVer = (_b = (_a = c.buildConfig.engineTemplates) === null || _a === void 0 ? void 0 : _a[k]) === null || _b === void 0 ? void 0 : _b.version;
                if (engVer) {
                    if (devDependencies[k]) {
                        if (devDependencies[k] !== engVer) {
                            needsPackageUpdate = true;
                            (0, logger_1.logInfo)("Updating missing engine ".concat(k, " ").concat((0, logger_1.chalk)().red(devDependencies[k]), "=>").concat((0, logger_1.chalk)().green(engVer), " to package.json"));
                            devDependencies[k] = engVer;
                        }
                    }
                    else {
                        needsPackageUpdate = true;
                        (0, logger_1.logInfo)("Adding missing engine ".concat(k, "@").concat(engVer, " to package.json"));
                        devDependencies[k] = engVer;
                    }
                }
            });
            if (needsPackageUpdate) {
                (0, fileutils_1.writeFileSync)(c.paths.project.package, c.files.project.package);
            }
        }
        return [2 /*return*/, true];
    });
}); };
exports.configureEngines = configureEngines;
var registerMissingPlatformEngines = function (c, taskInstance) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var registerEngineList_1;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                (0, logger_1.logTask)('registerMissingPlatformEngines');
                if (!(!taskInstance || (!taskInstance.isGlobalScope && ((_a = taskInstance === null || taskInstance === void 0 ? void 0 : taskInstance.platforms) === null || _a === void 0 ? void 0 : _a.length) === 0) || c.program.platform === 'all')) return [3 /*break*/, 2];
                registerEngineList_1 = [];
                c.buildConfig.defaults.supportedPlatforms.forEach(function (platform) {
                    registerEngineList_1.push(_registerPlatformEngine(c, platform));
                });
                if (!registerEngineList_1.length) return [3 /*break*/, 2];
                return [4 /*yield*/, Promise.all(registerEngineList_1)];
            case 1:
                _b.sent();
                _b.label = 2;
            case 2: return [2 /*return*/, true];
        }
    });
}); };
exports.registerMissingPlatformEngines = registerMissingPlatformEngines;
var registerAllPlatformEngines = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var registerEngineList;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logTask)('registerAllPlatformEngines');
                if (!((_c = (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.defaults) === null || _b === void 0 ? void 0 : _b.supportedPlatforms) === null || _c === void 0 ? void 0 : _c.forEach)) {
                    c.runtime.hasAllEnginesRegistered = true;
                    return [2 /*return*/, true];
                }
                registerEngineList = [];
                c.buildConfig.defaults.supportedPlatforms.forEach(function (platform) {
                    registerEngineList.push(_registerPlatformEngine(c, platform));
                });
                if (!registerEngineList.length) return [3 /*break*/, 2];
                return [4 /*yield*/, Promise.all(registerEngineList)];
            case 1:
                _d.sent();
                _d.label = 2;
            case 2:
                c.runtime.hasAllEnginesRegistered = true;
                return [2 /*return*/, true];
        }
    });
}); };
exports.registerAllPlatformEngines = registerAllPlatformEngines;
var loadEnginePluginDeps = function (c, engineConfigs) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var addedPlugins, hasAddedPlugins, originalProjectPlugins, engines, allPlugins, confirm_1;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('loadEnginePluginDeps');
                if (c.files.project.config.isTemplate)
                    return [2 /*return*/, 0];
                addedPlugins = {};
                hasAddedPlugins = false;
                originalProjectPlugins = c.files.project.config_original.plugins || {};
                engineConfigs.forEach(function (ecf) {
                    var engineConfig = (0, fileutils_1.readObjectSync)(ecf.configPath);
                    if (engineConfig === null || engineConfig === void 0 ? void 0 : engineConfig.plugins) {
                        var projectPlugins_1 = c.files.project.config.plugins;
                        // Comparing original config causes engine think that template is not extended with additional deps
                        // const projectPlugins = c.files.project.config_original.plugins;
                        Object.keys(engineConfig === null || engineConfig === void 0 ? void 0 : engineConfig.plugins).forEach(function (k) {
                            if (!projectPlugins_1[k]) {
                                hasAddedPlugins = true;
                                originalProjectPlugins[k] = engineConfig === null || engineConfig === void 0 ? void 0 : engineConfig.plugins[k];
                                addedPlugins[k] = addedPlugins[k] || [];
                                addedPlugins[k].push(k);
                            }
                        });
                    }
                });
                if (!hasAddedPlugins) return [3 /*break*/, 2];
                engines = Object.keys(addedPlugins);
                allPlugins = Object.keys(originalProjectPlugins);
                (0, logger_1.logInfo)("Engines: ".concat((0, logger_1.chalk)().yellow(engines.join(',')), " require plugins ").concat((0, logger_1.chalk)().white(allPlugins.join(',')), " to be added to ").concat((0, logger_1.chalk)().white(c.paths.project.config)));
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'selectedScheme',
                        type: 'confirm',
                        message: "Continue?.\nIf you don't want to use this dependency make sure you remove platform which requires this engine from supportedPlatforms",
                    })];
            case 1:
                confirm_1 = _a.sent();
                if (confirm_1) {
                    (0, logger_1.logInfo)("Adding ".concat(allPlugins.join(','), ". ...DONE"));
                    // Prepare original file to be decorated (as addon plugins as we can't edit template itself)
                    c.files.project.config_original.plugins = originalProjectPlugins;
                    (0, configManager_1.writeRenativeConfigFile)(c, c.paths.project.config, c.files.project.config_original);
                }
                _a.label = 2;
            case 2: return [2 /*return*/, addedPlugins.length];
        }
    });
}); };
exports.loadEnginePluginDeps = loadEnginePluginDeps;
var loadEnginePackageDeps = function (c, engineConfigs) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var addedDeps;
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('loadEnginePackageDeps');
        if (c.program.skipDependencyCheck || c.files.project.config.isTemplate)
            return [2 /*return*/, 0];
        addedDeps = [];
        engineConfigs.forEach(function (ecf) {
            var engineConfig = (0, fileutils_1.readObjectSync)(ecf.configPath);
            c.buildConfig.defaults.supportedPlatforms.forEach(function (platform) {
                var _a, _b;
                var npm = (_b = (_a = engineConfig === null || engineConfig === void 0 ? void 0 : engineConfig.platforms) === null || _a === void 0 ? void 0 : _a[platform]) === null || _b === void 0 ? void 0 : _b.npm;
                if (npm) {
                    if (npm.devDependencies) {
                        var deps_1 = c.files.project.package.devDependencies || {};
                        Object.keys(npm.devDependencies).forEach(function (k) {
                            if (!deps_1[k]) {
                                var isMonorepo = (0, common_1.getConfigProp)(c, c.platform, 'isMonorepo');
                                if (isMonorepo) {
                                    (0, logger_1.logInfo)("Engine ".concat(ecf.key, " requires npm devDependency ").concat(k, " for platform ").concat(platform, ". project marked as monorepo. SKIPPING"));
                                }
                                else {
                                    (0, logger_1.logInfo)("Engine ".concat(ecf.key, " requires npm devDependency ").concat(k, " for platform ").concat(platform, ". ADDING...DONE"));
                                    deps_1[k] = npm === null || npm === void 0 ? void 0 : npm.devDependencies[k];
                                    addedDeps.push(k);
                                }
                            }
                        });
                        c.files.project.package.devDependencies = deps_1;
                    }
                    if (npm.dependencies) {
                        var deps_2 = c.files.project.package.dependencies || {};
                        Object.keys(npm.dependencies).forEach(function (k) {
                            if (!deps_2[k]) {
                                if (c.files.project.config.isTemplate) {
                                    if (!c.files.project.package.devDependencies[k]) {
                                        (0, logger_1.logWarning)("Engine ".concat(ecf.key, " requires npm dependency ").concat(k, " for platform ").concat(platform, ". which in template project should be placed in devDependencies"));
                                    }
                                }
                                else {
                                    (0, logger_1.logInfo)("Engine ".concat(ecf.key, " requires npm dependency ").concat(k, " for platform ").concat(platform, ". ADDING...DONE"));
                                    deps_2[k] = npm === null || npm === void 0 ? void 0 : npm.dependencies[k];
                                    addedDeps.push(k);
                                }
                            }
                        });
                        c.files.project.package.dependencies = deps_2;
                    }
                    if (npm.optionalDependencies) {
                        var deps_3 = c.files.project.package.optionalDependencies || {};
                        Object.keys(npm.optionalDependencies).forEach(function (k) {
                            if (!deps_3[k]) {
                                (0, logger_1.logInfo)("Engine ".concat(ecf.key, " requires npm optionalDependency ").concat(k, " for platform ").concat(platform, ". ADDING...DONE"));
                                deps_3[k] = npm === null || npm === void 0 ? void 0 : npm.optionalDependencies[k];
                                addedDeps.push(k);
                            }
                        });
                        c.files.project.package.optionalDependencies = deps_3;
                    }
                    if (addedDeps.length > 0) {
                        (0, fileutils_1.writeFileSync)(c.paths.project.package, c.files.project.package);
                    }
                    //
                }
            });
        });
        // if (addedDeps.length > 0) {
        //     await installPackageDependencies(c);
        // }
        return [2 /*return*/, addedDeps.length];
    });
}); };
exports.loadEnginePackageDeps = loadEnginePackageDeps;
var ENGINE_ID_MAP = {
    'engine-lightning': '@rnv/engine-lightning',
    'engine-rn': '@rnv/engine-rn',
    'engine-rn-electron': '@rnv/engine-rn-electron',
    'engine-rn-macos': '@rnv/engine-rn-macos',
    'engine-rn-next': '@rnv/engine-rn-next',
    'engine-rn-tvos': '@rnv/engine-rn-tvos',
    'engine-rn-web': '@rnv/engine-rn-web',
    'engine-rn-windows': '@rnv/engine-rn-windows'
};
var _getFilteredEngines = function (c) {
    var _a, _b, _c, _d;
    var engines = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.engines;
    if (!engines) {
        (0, logger_1.logError)('Engine configs missing in your renative.json. FIXING...DONE');
        return {};
    }
    var rnvPlatforms = (_c = (_b = c.files.rnv.projectTemplates) === null || _b === void 0 ? void 0 : _b.config) === null || _c === void 0 ? void 0 : _c.platforms;
    var supportedPlatforms = ((_d = c.files.project.config.defaults) === null || _d === void 0 ? void 0 : _d.supportedPlatforms) || [];
    var filteredEngines = {};
    supportedPlatforms.forEach(function (v) {
        var _a, _b;
        var platforms = c.files.project.config.platforms || {};
        var engineKey = ((_a = platforms[v]) === null || _a === void 0 ? void 0 : _a.engine) || ((_b = rnvPlatforms[v]) === null || _b === void 0 ? void 0 : _b.engine);
        var engKey = ENGINE_ID_MAP[engineKey] || engineKey;
        if (engineKey) {
            if (engines[engKey]) {
                filteredEngines[engKey] = engines[engKey];
            }
            else {
                (0, logger_1.logWarning)("Platform ".concat(v, " requires engine ").concat(engKey, " which is not available in engines list"));
            }
        }
        else {
            (0, logger_1.logWarning)("Platform ".concat(v, " has no engine configured"));
        }
    });
    return filteredEngines;
};
var loadEngines = function (c, failOnMissingDeps) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var filteredEngines, enginesToInstall, readyEngines, engineConfigs, plugDepsCount, pkgDepsCount;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('loadEngines');
                if (!(0, fileutils_1.fsExistsSync)(c.paths.project.config))
                    return [2 /*return*/, true];
                filteredEngines = _getFilteredEngines(c);
                enginesToInstall = [];
                readyEngines = [];
                engineConfigs = [];
                // if (filteredEngines) {
                Object.keys(filteredEngines).forEach(function (k) {
                    var engineRootPath = (0, resolve_1.doResolve)(k);
                    var configPath = engineRootPath ? path_1.default.join(engineRootPath, 'renative.engine.json') : null;
                    if (!configPath || !(0, fileutils_1.fsExistsSync)(configPath)) {
                        var engVer = (0, utils_1.getScopedVersion)(c, k, filteredEngines[k], 'engineTemplates');
                        if (engVer) {
                            enginesToInstall.push({
                                key: k,
                                version: engVer,
                                engineRootPath: engineRootPath
                            });
                        }
                    }
                    else {
                        readyEngines.push(k);
                        engineConfigs.push({
                            key: k,
                            engineRootPath: engineRootPath,
                            configPath: configPath
                        });
                    }
                });
                if (!enginesToInstall.length) return [3 /*break*/, 3];
                if (failOnMissingDeps) {
                    return [2 /*return*/, Promise.reject("Failed to load some engines:\n".concat(enginesToInstall.map(function (v) { return "> ".concat(v.key, "@").concat(v.version, " path: ").concat(v.engineRootPath); }).join('\n')))];
                }
                (0, logger_1.logInfo)("Some engines not installed in your project:\n".concat(enginesToInstall.map(function (v) { return "> ".concat(v.key, "@").concat(v.version); }).join('\n'), "\n ADDING TO PACKAGE.JSON...DONE"));
                return [4 /*yield*/, (0, npmUtils_1.checkAndCreateProjectPackage)(c)];
            case 1:
                _a.sent();
                enginesToInstall.forEach(function (v) {
                    c.files.project.package.devDependencies[v.key] = v.version;
                    (0, fileutils_1.writeFileSync)(c.paths.project.package, c.files.project.package);
                });
                return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
            case 2:
                _a.sent();
                return [2 /*return*/, (0, exports.loadEngines)(c, true)];
            case 3: return [4 /*yield*/, (0, exports.loadEnginePluginDeps)(c, engineConfigs)];
            case 4:
                plugDepsCount = _a.sent();
                return [4 /*yield*/, (0, exports.loadEnginePackageDeps)(c, engineConfigs)];
            case 5:
                pkgDepsCount = _a.sent();
                if (!(plugDepsCount + pkgDepsCount > 0)) return [3 /*break*/, 8];
                c.runtime._skipPluginScopeWarnings = true;
                return [4 /*yield*/, (0, pluginManager_1.configurePlugins)(c, true)];
            case 6:
                _a.sent(); // TODO: This is too early as scoped plugin have not been installed
                c.runtime._skipPluginScopeWarnings = false;
                return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
            case 7:
                _a.sent();
                _a.label = 8;
            case 8:
                // All engines ready to be registered
                _registerPlatformEngine(c, c.platform);
                // } else if (c.files.project.config_original) {
                //     logError('Engine configs missing in your renative.json. FIXING...DONE');
                // c.files.project.config_original.engines = {
                //     '@rnv/engine-rn': 'source:rnv',
                //     '@rnv/engine-rn-web': 'source:rnv',
                //     '@rnv/engine-rn-next': 'source:rnv',
                //     '@rnv/engine-rn-electron': 'source:rnv',
                //     '@rnv/engine-lightning': 'source:rnv',
                //     '@rnv/engine-rn-macos': 'source:rnv',
                // };
                // // TODO: use parseRenativeConfigs instead
                // c.buildConfig.engines = c.files.project.config_original.engines;
                // writeFileSync(c.paths.project.config, c.files.project.config_original);
                // return loadEngines(c);
                // }
                return [2 /*return*/, true];
        }
    });
}); };
exports.loadEngines = loadEngines;
var _getMergedEngineConfigs = function (c) {
    var _a, _b;
    var engines = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.engines;
    var engineTemplates = (_b = c.buildConfig) === null || _b === void 0 ? void 0 : _b.engineTemplates;
    var mergedEngineConfigs = {};
    Object.keys(engineTemplates).forEach(function (packageName) {
        mergedEngineConfigs[packageName] = (0, tslib_1.__assign)({ packageName: packageName }, engineTemplates[packageName]);
    });
    var engineTemplatesKeys = Object.keys(engineTemplates);
    if (engines) {
        Object.keys(engines).forEach(function (enginePackageName) {
            var engineVal = engines[enginePackageName];
            if (engineVal === 'source:rnv') {
                if (!engineTemplatesKeys.includes(enginePackageName)) {
                    (0, logger_1.logWarning)("Engine ".concat(enginePackageName, " not found in default engineTemplates"));
                }
            }
            else {
                mergedEngineConfigs[enginePackageName] = engineVal;
            }
        });
    }
    return mergedEngineConfigs;
};
var _getEngineConfigByPlatform = function (c, platform) {
    var mergedEngineConfigs = _getMergedEngineConfigs(c);
    var engineId = c.program.engine || (0, common_1.getConfigProp)(c, platform, 'engine');
    var selectedEngineConfig;
    Object.values(mergedEngineConfigs).forEach(function (engineConfig) {
        if (engineConfig.id === engineId) {
            selectedEngineConfig = engineConfig;
        }
    });
    return selectedEngineConfig;
};
var _resolvePkgPath = function (c, packageName) {
    if (constants_1.IS_LINKED && !c.program.unlinked) {
        // In the instances of running linked rnv instead of installed one load local packages
        try {
            var pkgPathLocal = require.resolve(packageName, { paths: [path_1.default.join(constants_1.RNV_HOME_DIR, '..')] });
            pkgPathLocal = pkgPathLocal.replace('/dist/index.js', '').replace('\\dist\\index.js', '');
            return pkgPathLocal;
        }
        catch (_a) {
            (0, logger_1.logInfo)("Running local rnv but did not find linked ".concat(packageName, ". moving on..."));
        }
    }
    var pkgPath = path_1.default.join(c.paths.project.dir, 'node_modules', packageName);
    if ((0, fileutils_1.fsExistsSync)(pkgPath)) {
        return pkgPath;
    }
    var monoRoot = (0, common_1.getConfigProp)(c, c.platform, 'monoRoot');
    pkgPath = path_1.default.join(c.paths.project.dir, monoRoot || '../..', 'node_modules', packageName);
    if ((0, fileutils_1.fsExistsSync)(pkgPath)) {
        return pkgPath;
    }
    pkgPath = require.resolve(packageName);
    return pkgPath;
};
var _registerPlatformEngine = function (c, platform) {
    var _a;
    // Only register active platform engine to be faster
    if (platform === true || !platform)
        return;
    var selectedEngineConfig = _getEngineConfigByPlatform(c, platform);
    if (selectedEngineConfig) {
        var existingEngine = c.runtime.enginesById[selectedEngineConfig.id];
        if (!existingEngine) {
            (0, exports.registerEngine)(c, (_a = require(_resolvePkgPath(c, selectedEngineConfig.packageName))) === null || _a === void 0 ? void 0 : _a.default, platform, selectedEngineConfig);
        }
        else {
            _registerEnginePlatform(c, platform, existingEngine);
        }
    }
    else if (platform !== true && platform) {
        (0, logger_1.logWarning)("Could not find suitable engine for platform ".concat(platform, "\nMaybe you forgot to define platforms.").concat(platform, ".engine in your renative.json?"));
    }
};
var generateEnvVars = function (c, moduleConfig, nextConfig) {
    var isMonorepo = (0, common_1.getConfigProp)(c, c.platform, 'isMonorepo');
    var monoRoot = (0, common_1.getConfigProp)(c, c.platform, 'monoRoot');
    return ({
        RNV_EXTENSIONS: (0, exports.getPlatformExtensions)(c),
        RNV_ENGINE_PATH: c.runtime.engine.rootPath,
        RNV_MODULE_PATHS: (moduleConfig === null || moduleConfig === void 0 ? void 0 : moduleConfig.modulePaths) || [],
        RNV_MODULE_ALIASES: (moduleConfig === null || moduleConfig === void 0 ? void 0 : moduleConfig.moduleAliasesArray) || [],
        RNV_NEXT_TRANSPILE_MODULES: nextConfig,
        RNV_PROJECT_ROOT: c.paths.project.dir,
        RNV_APP_BUILD_DIR: (0, common_1.getAppFolder)(c),
        RNV_IS_MONOREPO: isMonorepo,
        RNV_MONO_ROOT: isMonorepo ? path_1.default.join(c.paths.project.dir, monoRoot || '../..') : c.paths.project.dir,
        RNV_ENGINE: c.runtime.engine.config.id,
        RNV_IS_NATIVE_TV: [constants_1.TVOS, constants_1.ANDROID_TV, constants_1.FIRE_TV].includes(c.platform)
    });
};
exports.generateEnvVars = generateEnvVars;
var getPlatformExtensions = function (c, excludeServer, addDotPrefix) {
    if (excludeServer === void 0) { excludeServer = false; }
    if (addDotPrefix === void 0) { addDotPrefix = false; }
    var engine = c.runtime.engine;
    var output;
    var platforms = engine.platforms;
    if (addDotPrefix) {
        output = platforms[c.platform].extensions.map(function (v) { return ".".concat(v); }).filter(function (ext) { return !excludeServer || !ext.includes('server.'); });
    }
    else {
        output = platforms[c.platform].extensions.filter(function (ext) { return !excludeServer || !ext.includes('server.'); });
    }
    return output;
};
exports.getPlatformExtensions = getPlatformExtensions;
var getEngineRunnerByPlatform = function (c, platform, ignoreMissingError) {
    var selectedEngine = c.runtime.enginesByPlatform[platform];
    if (!selectedEngine && !ignoreMissingError) {
        (0, logger_1.logDebug)("ERROR: Engine for platform: ".concat(platform, " does not exists or is not registered ").concat(new Error()));
        // logRaw(new Error());
    }
    return selectedEngine;
};
exports.getEngineRunnerByPlatform = getEngineRunnerByPlatform;
var getEngineTask = function (task, tasks, customTasks) {
    var customTask = customTasks === null || customTasks === void 0 ? void 0 : customTasks[task];
    if (customTask)
        return customTask;
    var tsk;
    var taskCleaned = task.split(' ')[0];
    tsk = tasks[task];
    if (!tsk) {
        tsk = tasks[taskCleaned];
    }
    return tsk;
};
exports.getEngineTask = getEngineTask;
var hasEngineTask = function (task, tasks, isProjectScope) {
    var _a;
    return (isProjectScope ? !!(0, exports.getEngineTask)(task, tasks) : (_a = (0, exports.getEngineTask)(task, tasks)) === null || _a === void 0 ? void 0 : _a.isGlobalScope);
};
exports.hasEngineTask = hasEngineTask;
var getEngineSubTasks = function (task, tasks, exactMatch) { return Object.values(tasks).filter(function (v) { return (exactMatch ? v.task.split(' ')[0] === task : v.task.startsWith(task)); }); };
exports.getEngineSubTasks = getEngineSubTasks;
var getEngineRunner = function (c, task, customTasks, failOnMissingEngine) {
    if (failOnMissingEngine === void 0) { failOnMissingEngine = true; }
    if (customTasks === null || customTasks === void 0 ? void 0 : customTasks[task]) {
        return c.runtime.enginesById[ENGINE_CORE];
    }
    var configExists = c.paths.project.configExists;
    var engine = c.runtime.enginesByPlatform[c.platform];
    if (!engine) {
        engine = c.runtime.enginesById['engine-core'];
    }
    if (!engine) {
        if ((0, exports.hasEngineTask)(task, c.runtime.enginesById[ENGINE_CORE].tasks, configExists)) {
            return c.runtime.enginesById[ENGINE_CORE];
        }
        if (failOnMissingEngine) {
            throw new Error("Cound not find active engine for platform ".concat(c.platform, ". Available engines:\n        ").concat(c.runtime.enginesByIndex.map(function (v) { return v.config.id; }).join(', ')));
        }
        return null;
    }
    if ((0, exports.hasEngineTask)(task, engine.tasks, configExists))
        return engine;
    if ((0, exports.hasEngineTask)(task, c.runtime.enginesById[ENGINE_CORE].tasks, configExists)) {
        return c.runtime.enginesById[ENGINE_CORE];
    }
    if (failOnMissingEngine)
        throw new Error("Cound not find suitable executor for task ".concat((0, logger_1.chalk)().white(task)));
    return null;
};
exports.getEngineRunner = getEngineRunner;
var getRegisteredEngines = function (c) { return c.runtime.enginesByIndex; };
exports.getRegisteredEngines = getRegisteredEngines;
//# sourceMappingURL=index.js.map