"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ejectPlatform = exports.copySharedPlatforms = exports.isPlatformActive = exports.isPlatformSupportedSync = exports.isPlatformSupported = exports.createPlatformBuild = exports.cleanPlatformBuild = exports.generatePlatformChoices = exports.updateProjectPlatforms = exports.logErrorPlatform = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var logger_1 = require("../systemManager/logger");
var prompt_1 = require("../../cli/prompt");
var fileutils_1 = require("../systemManager/fileutils");
var installer_1 = require("../sdkManager/installer");
var common_1 = require("../common");
var constants_1 = require("../constants");
var logErrorPlatform = function (c) {
    (0, logger_1.logError)("Platform: ".concat((0, logger_1.chalk)().white(c.platform), " doesn't support command: ").concat((0, logger_1.chalk)().white(c.command)), true // kill it if we're not supporting this
    );
    return false;
};
exports.logErrorPlatform = logErrorPlatform;
var updateProjectPlatforms = function (c, platforms) {
    var config = c.paths.project.config;
    var currentConfig = c.files.project.config;
    currentConfig.defaults = currentConfig.defaults || {};
    currentConfig.defaults.supportedPlatforms = platforms;
    (0, fileutils_1.writeFileSync)(config, currentConfig);
};
exports.updateProjectPlatforms = updateProjectPlatforms;
var generatePlatformChoices = function (c) {
    var options = c.runtime.supportedPlatforms.map(function (v) {
        var _a, _b;
        return ({
            name: "".concat(v.platform, " - ").concat(v.isConnected ? (0, logger_1.chalk)().green('(connected)') : (0, logger_1.chalk)().yellow('(ejected)'), " [").concat((0, logger_1.chalk)().cyan((_b = (_a = v.engine) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.id), "]"),
            value: v.platform,
            isConnected: v.isConnected
        });
    });
    return options;
};
exports.generatePlatformChoices = generatePlatformChoices;
var cleanPlatformBuild = function (c, platform) { return new Promise(function (resolve) {
    (0, logger_1.logTask)('cleanPlatformBuild');
    var cleanTasks = [];
    if (platform === 'all') {
        Object.keys(c.buildConfig.platforms).forEach(function (k) {
            if ((0, exports.isPlatformSupportedSync)(c, k)) {
                var pPath = path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(k));
                cleanTasks.push((0, fileutils_1.cleanFolder)(pPath));
            }
        });
    }
    else if ((0, exports.isPlatformSupportedSync)(c, platform)) {
        var pPath = (0, common_1.getAppFolder)(c);
        cleanTasks.push((0, fileutils_1.cleanFolder)(pPath));
    }
    Promise.all(cleanTasks).then(function () {
        resolve();
    });
}); };
exports.cleanPlatformBuild = cleanPlatformBuild;
var createPlatformBuild = function (c, platform) { return new Promise(function (resolve, reject) {
    (0, logger_1.logTask)('createPlatformBuild');
    if (!(0, exports.isPlatformSupportedSync)(c, platform, null, reject))
        return;
    var pPath = (0, common_1.getAppFolder)(c);
    var ptPath = path_1.default.join(c.paths.project.platformTemplatesDirs[platform], "".concat(platform));
    (0, fileutils_1.copyFolderContentsRecursiveSync)(ptPath, pPath, false, [
        path_1.default.join(ptPath, '_privateConfig')
    ], false, {}, (0, common_1.getTimestampPathsConfig)(c, platform), c);
    resolve();
}); };
exports.createPlatformBuild = createPlatformBuild;
var isPlatformSupported = function (c, isGlobalScope) {
    if (isGlobalScope === void 0) { isGlobalScope = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var platformsAsObj, opts, platform, configuredPlatforms, confirm_1, newPlatforms;
        var _a, _b, _c, _d;
        return (0, tslib_1.__generator)(this, function (_e) {
            switch (_e.label) {
                case 0:
                    (0, logger_1.logTask)('isPlatformSupported');
                    if (c.platform && c.platform !== true && isGlobalScope) {
                        return [2 /*return*/, c.platform];
                    }
                    if (isGlobalScope) {
                        platformsAsObj = constants_1.SUPPORTED_PLATFORMS;
                    }
                    else {
                        platformsAsObj = c.buildConfig ? c.buildConfig.platforms : c.supportedPlatforms;
                    }
                    if (!platformsAsObj)
                        platformsAsObj = c.runtime.availablePlatforms;
                    opts = (0, prompt_1.generateOptions)(platformsAsObj);
                    if (!(!c.platform
                        || c.platform === true
                        || !((_b = (_a = c.runtime.availablePlatforms) === null || _a === void 0 ? void 0 : _a.includes) === null || _b === void 0 ? void 0 : _b.call(_a, c.platform)))) return [3 /*break*/, 2];
                    return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                            name: 'platform',
                            type: 'list',
                            message: 'Pick one of available platforms',
                            choices: opts.keysAsArray,
                            logMessage: 'You need to specify platform'
                        })];
                case 1:
                    platform = (_e.sent()).platform;
                    c.platform = platform;
                    _e.label = 2;
                case 2:
                    configuredPlatforms = (_d = (_c = c.files.project.config) === null || _c === void 0 ? void 0 : _c.defaults) === null || _d === void 0 ? void 0 : _d.supportedPlatforms;
                    if (!(Array.isArray(configuredPlatforms)
                        && !configuredPlatforms.includes(c.platform))) return [3 /*break*/, 4];
                    return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                            type: 'confirm',
                            message: "Platform ".concat(c.platform, " is not supported by your project. Would you like to enable it?")
                        })];
                case 3:
                    confirm_1 = (_e.sent()).confirm;
                    if (confirm_1) {
                        newPlatforms = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], configuredPlatforms, true), [c.platform], false);
                        (0, exports.updateProjectPlatforms)(c, newPlatforms);
                        c.buildConfig.defaults.supportedPlatforms = newPlatforms;
                        // await configureEntryPoints(c);
                    }
                    else {
                        throw new Error('User canceled');
                    }
                    _e.label = 4;
                case 4: 
                // Check global SDKs
                return [4 /*yield*/, (0, installer_1.checkAndConfigureSdks)(c)];
                case 5:
                    // Check global SDKs
                    _e.sent();
                    return [2 /*return*/, c.platform];
            }
        });
    });
};
exports.isPlatformSupported = isPlatformSupported;
var isPlatformSupportedSync = function (c, platform, resolve, reject) {
    if (!platform) {
        if (reject) {
            reject((0, logger_1.chalk)().red("You didn't specify platform. make sure you add \"".concat((0, logger_1.chalk)().white.bold('-p <PLATFORM>'), "\" option to your command!")));
        }
        return false;
    }
    if (!c.runtime.availablePlatforms.includes(platform)) {
        if (reject) {
            reject((0, logger_1.chalk)().red("Platform ".concat(platform, " is not supported. Use one of the following: ").concat((0, logger_1.chalk)().white(c.runtime.availablePlatforms.join(', ')), " .")));
        }
        return false;
    }
    if (resolve)
        resolve();
    return true;
};
exports.isPlatformSupportedSync = isPlatformSupportedSync;
var isPlatformActive = function (c, platform, resolve) {
    if (!c.buildConfig || !c.buildConfig.platforms) {
        (0, logger_1.logError)("Your appConfigFile is not configured properly! check ".concat((0, logger_1.chalk)().white(c.paths.appConfig.config), " location."));
        if (resolve)
            resolve();
        return false;
    }
    if (!c.buildConfig.platforms[platform]) {
        (0, logger_1.logWarning)("Platform ".concat(platform, " not configured for ").concat(c.runtime.appId, ". skipping."));
        if (resolve)
            resolve();
        return false;
    }
    return true;
};
exports.isPlatformActive = isPlatformActive;
var copySharedPlatforms = function (c) { return new Promise(function (resolve) {
    (0, logger_1.logTask)('copySharedPlatforms');
    if (c.platform) {
        (0, fileutils_1.copyFolderContentsRecursiveSync)(path_1.default.resolve(c.paths.project.platformTemplatesDirs[c.platform], '_shared'), (0, common_1.getPlatformBuildDir)(c));
    }
    resolve();
}); };
exports.copySharedPlatforms = copySharedPlatforms;
var ejectPlatform = function (c, platform) {
    var engine = c.runtime.enginesByPlatform[platform];
    var destDir = path_1.default.join(c.paths.project.dir, 'platformTemplates', platform);
    var sourcePlatformDir = engine.originalTemplatePlatformsDir;
    (0, fileutils_1.copyFolderContentsRecursiveSync)(path_1.default.join(sourcePlatformDir, platform), destDir);
    // DEPRECATED: only for legacy support
    (0, fileutils_1.copyFolderContentsRecursiveSync)(path_1.default.join(sourcePlatformDir, '_shared'), destDir);
};
exports.ejectPlatform = ejectPlatform;
//# sourceMappingURL=index.js.map