"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getModuleConfigs = exports.getLocalRenativePlugin = exports.includesPluginPath = exports.sanitizePluginPath = exports.copyTemplatePluginsSync = exports.overrideTemplatePlugins = exports.checkForPluginDependencies = exports.installPackageDependenciesAndPlugins = exports.overrideFileContents = exports.loadPluginTemplates = exports.parsePlugins = exports.resolvePluginDependants = exports.configurePlugins = exports.getMergedPlugin = exports.getPluginList = void 0;
var tslib_1 = require("tslib");
/* eslint-disable no-await-in-loop */
var deepmerge_1 = (0, tslib_1.__importDefault)(require("deepmerge"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var lodash_intersection_1 = (0, tslib_1.__importDefault)(require("lodash.intersection"));
var prompt_1 = require("../../cli/prompt");
var common_1 = require("../common");
var configManager_1 = require("../configManager");
var constants_1 = require("../constants");
var projectManager_1 = require("../projectManager");
var fileutils_1 = require("../systemManager/fileutils");
var logger_1 = require("../systemManager/logger");
var npmUtils_1 = require("../systemManager/npmUtils");
var resolve_1 = require("../systemManager/resolve");
var getPluginList = function (c, isUpdate) {
    if (isUpdate === void 0) { isUpdate = false; }
    var output = {
        asString: '',
        asArray: [],
        plugins: [],
        allPlugins: {} // this is used by taskRnvPluginAdd
    };
    var i = 1;
    Object.keys(c.files.rnv.pluginTemplates.configs).forEach(function (pk) {
        var plugins = c.files.rnv.pluginTemplates.configs[pk].pluginTemplates;
        Object.keys(plugins).forEach(function (k) {
            var p = plugins[k];
            var platforms = '';
            var pluginPlatforms = (0, lodash_intersection_1.default)(c.runtime.supportedPlatforms.map(function (pl) { return pl.platform; }), Object.keys(p));
            pluginPlatforms.forEach(function (v) {
                platforms += "".concat(v, ", ");
            });
            if (platforms.length) {
                platforms = platforms.slice(0, platforms.length - 2);
            }
            var installedPlugin = c.buildConfig
                && c.buildConfig.plugins
                && c.buildConfig.plugins[k];
            var installedString = installedPlugin
                ? (0, logger_1.chalk)().yellow('installed')
                : (0, logger_1.chalk)().green('not installed');
            if (isUpdate && installedPlugin) {
                output.plugins.push(k);
                var versionString = void 0;
                if (installedPlugin.version !== p.version) {
                    versionString = "(".concat((0, logger_1.chalk)().yellow(installedPlugin.version), ") => (").concat((0, logger_1.chalk)().green(p.version), ")");
                }
                else {
                    versionString = "(".concat((0, logger_1.chalk)().green(installedPlugin.version), ")");
                }
                output.asString += " [".concat(i, "]> ").concat((0, logger_1.chalk)().bold(k), " ").concat(versionString, "\n");
                output.asArray.push({
                    name: "".concat(k, " ").concat(versionString),
                    value: k
                });
                output.allPlugins[k] = p; // this is used by taskRnvPluginAdd
                i++;
            }
            else if (!isUpdate) {
                output.plugins.push(k);
                output.asString += " [".concat(i, "]> ").concat((0, logger_1.chalk)().bold(k), " (").concat((0, logger_1.chalk)().grey(p['no-npm'] ? 'no-npm' : p.version), ") [").concat(platforms, "] - ").concat(installedString, "\n");
                output.asArray.push({
                    name: "".concat(k, " (").concat((0, logger_1.chalk)().grey(p['no-npm'] ? 'no-npm' : p.version), ") [").concat(platforms, "] - ").concat(installedString),
                    value: k
                });
                output.allPlugins[k] = p; // this is used by taskRnvPluginAdd
                i++;
            }
            output.asArray.sort(function (a, b) {
                var aStr = a.name.toLowerCase();
                var bStr = b.name.toLowerCase();
                var com = 0;
                if (aStr > bStr) {
                    com = 1;
                }
                else if (aStr < bStr) {
                    com = -1;
                }
                return com;
            });
        });
    });
    return output;
};
exports.getPluginList = getPluginList;
var _getPluginScope = function (plugin) {
    if (typeof plugin === 'string' || plugin instanceof String) {
        if (plugin.startsWith('source:')) {
            return { scope: plugin.split(':').pop() };
        }
        return { npmVersion: plugin };
    }
    if (plugin === null || plugin === void 0 ? void 0 : plugin.source) {
        return { scope: plugin === null || plugin === void 0 ? void 0 : plugin.source };
    }
    return { scope: 'rnv' };
};
var getMergedPlugin = function (c, key) {
    var _a;
    (0, logger_1.logDebug)("getMergedPlugin:".concat(key));
    var plugin = (_a = c.buildConfig.plugins) === null || _a === void 0 ? void 0 : _a[key];
    if (!plugin)
        return null;
    var scopes = [];
    var mergedPlugin = _getMergedPlugin(c, plugin, key, null, scopes);
    scopes.reverse();
    // if (!mergedPlugin.version) {
    //     logWarning(`Plugin ${key} has no version`);
    //     // return null;
    // }
    mergedPlugin._scopes = scopes;
    mergedPlugin._id = key;
    return mergedPlugin;
};
exports.getMergedPlugin = getMergedPlugin;
var _getMergedPlugin = function (c, plugin, pluginKey, parentScope, scopes, skipSanitize) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    if (!plugin) {
        return {};
    }
    var _j = _getPluginScope(plugin), scope = _j.scope, npmVersion = _j.npmVersion;
    if (scope === parentScope)
        return plugin;
    if (npmVersion) {
        return {
            version: npmVersion
        };
    }
    if (scope !== '' && !!scope && !((_b = (_a = c.buildConfig.pluginTemplates) === null || _a === void 0 ? void 0 : _a[scope]) === null || _b === void 0 ? void 0 : _b.pluginTemplates) && !c.runtime._skipPluginScopeWarnings) {
        (0, logger_1.logWarning)("Plugin ".concat(pluginKey, " is not recognized plugin in ").concat(scope, " scope"));
    }
    else if (scope) {
        var skipRnvOverrides = (_d = (_c = c.buildConfig.pluginTemplates) === null || _c === void 0 ? void 0 : _c[parentScope]) === null || _d === void 0 ? void 0 : _d.disableRnvDefaultOverrides;
        if (skipRnvOverrides && scope === 'rnv') {
            // Merges down to RNV defaults will be skipped
        }
        else {
            scopes.push(scope);
        }
    }
    var parentPlugin = _getMergedPlugin(c, (_g = (_f = (_e = c.buildConfig.pluginTemplates) === null || _e === void 0 ? void 0 : _e[scope]) === null || _f === void 0 ? void 0 : _f.pluginTemplates) === null || _g === void 0 ? void 0 : _g[pluginKey], pluginKey, scope, scopes, true);
    var currentPlugin = plugin;
    if (typeof plugin === 'string' || plugin instanceof String) {
        currentPlugin = {};
    }
    constants_1.INJECTABLE_CONFIG_PROPS.forEach(function (v) {
        c.configPropsInjects[v] = (0, common_1.getConfigProp)(c, c.platform, v);
    });
    if (currentPlugin.pluginDependencies) {
        Object.keys(currentPlugin.pluginDependencies).forEach(function (plugDepKey) {
            if (currentPlugin.pluginDependencies[plugDepKey] === 'source:self') {
                currentPlugin.pluginDependencies[plugDepKey] = "source:".concat(parentScope);
            }
        });
    }
    var mergedObj = (0, fileutils_1.mergeObjects)(c, parentPlugin, currentPlugin, true, true);
    if (c._renativePluginCache[pluginKey]) {
        mergedObj.config = c._renativePluginCache[pluginKey];
    }
    // IMPORTANT: only final top level merge should be sanitized
    var obj = skipSanitize ? mergedObj : (0, fileutils_1.sanitizeDynamicProps)(mergedObj, {
        files: c.files,
        runtimeProps: c.runtime,
        props: (_h = c.buildConfig) === null || _h === void 0 ? void 0 : _h._refs,
        configProps: c.configPropsInjects
    });
    // IMPORTANT: only final top level merge should be sanitized
    var mergedPlugin = skipSanitize ? obj : (0, fileutils_1.sanitizeDynamicProps)(obj, {
        files: c.files,
        runtimeProps: c.runtime,
        props: obj.props,
        configProps: c.configPropsInjects
    });
    return mergedPlugin;
};
var configurePlugins = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var hasPackageChanged, newDeps, newDevDeps, _a, dependencies, devDependencies;
    var _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        (0, logger_1.logTask)('configurePlugins');
        if (c.program.skipDependencyCheck || c.files.project.config.isTemplate)
            return [2 /*return*/, true];
        if (!c.files.project.package.dependencies) {
            c.files.project.package.dependencies = {};
        }
        hasPackageChanged = false;
        if (!((_b = c.buildConfig) === null || _b === void 0 ? void 0 : _b.plugins)) {
            return [2 /*return*/];
        }
        newDeps = {};
        newDevDeps = {};
        _a = c.files.project.package, dependencies = _a.dependencies, devDependencies = _a.devDependencies;
        Object.keys(c.buildConfig.plugins).forEach(function (k) {
            var _a, _b;
            var plugin = (0, exports.getMergedPlugin)(c, k);
            if (!plugin) {
                if (((_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.plugins) === null || _b === void 0 ? void 0 : _b[k]) === null) {
                    // Skip Warning as this is intentional "plugin":null
                }
                else {
                    (0, logger_1.logWarning)("Plugin with name ".concat((0, logger_1.chalk)().white(k), " does not exists in ReNative source:rnv scope. you need to define it manually here: ").concat((0, logger_1.chalk)().white(c.paths.project.builds.config)));
                }
            }
            else if (dependencies && dependencies[k]) {
                if (plugin['no-active'] !== true && plugin['no-npm'] !== true) {
                    if (!plugin.version) {
                        if (!c.runtime._skipPluginScopeWarnings) {
                            (0, logger_1.logInfo)("Plugin ".concat(k, " not ready yet (waiting for scope ").concat(plugin.scope, "). SKIPPING..."));
                        }
                    }
                    else if (dependencies[k] !== plugin.version) {
                        (0, logger_1.logWarning)("Version mismatch of dependency ".concat((0, logger_1.chalk)().white(k), " between:\n").concat((0, logger_1.chalk)().white(c.paths.project.package), ": v(").concat((0, logger_1.chalk)().red(dependencies[k]), ") and\n").concat((0, logger_1.chalk)().white(c.paths.project.builds.config), ": v(").concat((0, logger_1.chalk)().green(plugin.version), ").\npackage.json will be overriden"));
                        hasPackageChanged = true;
                        newDeps[k] = plugin.version;
                    }
                }
            }
            else if (devDependencies && devDependencies[k]) {
                if (plugin['no-active'] !== true
                    && plugin['no-npm'] !== true) {
                    if (!plugin.version) {
                        if (!c.runtime._skipPluginScopeWarnings) {
                            (0, logger_1.logInfo)("Plugin ".concat(k, " not ready yet (waiting for scope ").concat(plugin.scope, "). SKIPPING..."));
                        }
                    }
                    else if (devDependencies[k] !== plugin.version) {
                        (0, logger_1.logWarning)("Version mismatch of devDependency ".concat((0, logger_1.chalk)().white(k), " between package.json: v(").concat((0, logger_1.chalk)().red(devDependencies[k]), ") and plugins.json: v(").concat((0, logger_1.chalk)().red(plugin.version), "). package.json will be overriden"));
                        hasPackageChanged = true;
                        newDevDeps[k] = plugin.version;
                    }
                }
            }
            else if (plugin['no-active'] !== true
                && plugin['no-npm'] !== true) {
                // Dependency does not exists
                if (plugin.version) {
                    (0, logger_1.logInfo)("Missing dependency ".concat((0, logger_1.chalk)().white(k), " v(").concat((0, logger_1.chalk)().red(plugin.version), ") in package.json. INSTALLING...DONE"));
                    hasPackageChanged = true;
                    newDeps[k] = plugin.version;
                }
            }
            if (plugin && plugin.npm) {
                Object.keys(plugin.npm).forEach(function (npmKey) {
                    var npmDep = plugin.npm[npmKey];
                    // IMPORTANT: Do not override top level override with plugin.npm ones
                    var topLevelPlugin = (0, exports.getMergedPlugin)(c, npmKey);
                    if (topLevelPlugin && (topLevelPlugin === null || topLevelPlugin === void 0 ? void 0 : topLevelPlugin.version) !== npmDep) {
                        (0, logger_1.logInfo)("RNV Detected plugin dependency conflict. ".concat((0, logger_1.chalk)().cyan('RESOLVING...'), "\n- ").concat(npmKey, "@").concat((0, logger_1.chalk)().green(topLevelPlugin === null || topLevelPlugin === void 0 ? void 0 : topLevelPlugin.version), " ").concat((0, logger_1.chalk)().cyan('<='), "\n- ").concat(k, " .npm sub dependencies:\n   |- ").concat(npmKey, "@").concat((0, logger_1.chalk)().red(npmDep)));
                    }
                    else if (!dependencies[npmKey]) {
                        (0, logger_1.logInfo)("Plugin ".concat((0, logger_1.chalk)().white(k), " requires npm dependency ").concat((0, logger_1.chalk)().white(npmKey), ". INSTALLING...DONE"));
                        newDeps[npmKey] = npmDep;
                        hasPackageChanged = true;
                    }
                    else if (dependencies[npmKey] !== npmDep) {
                        (0, logger_1.logWarning)("Plugin ".concat((0, logger_1.chalk)().white(k), " npm dependency ").concat((0, logger_1.chalk)().white(npmKey), " mismatch (").concat((0, logger_1.chalk)().red(dependencies[npmKey]), ") => (").concat((0, logger_1.chalk)().green(npmDep), ") .updating npm dependency in your package.json"));
                        newDeps[npmKey] = npmDep;
                        hasPackageChanged = true;
                    }
                });
            }
        });
        // c.runtime.skipPackageUpdate only reflects rnv version mismatch. should not prevent updating other deps
        if (hasPackageChanged /*! c.runtime.skipPackageUpdate */ && !c.program.skipDependencyCheck) {
            _updatePackage(c, { dependencies: newDeps, devDependencies: newDevDeps });
        }
        return [2 /*return*/, true];
    });
}); };
exports.configurePlugins = configurePlugins;
var _updatePackage = function (c, override) {
    var newPackage = (0, deepmerge_1.default)(c.files.project.package, override);
    (0, configManager_1.writeRenativeConfigFile)(c, c.paths.project.package, newPackage);
    c.files.project.package = newPackage;
    c._requiresNpmInstall = true;
};
var resolvePluginDependants = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var plugins, pluginKeys, i, key;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('resolvePluginDependants');
                plugins = c.buildConfig.plugins;
                if (!plugins) return [3 /*break*/, 4];
                pluginKeys = Object.keys(plugins);
                i = 0;
                _a.label = 1;
            case 1:
                if (!(i < pluginKeys.length)) return [3 /*break*/, 4];
                key = pluginKeys[i];
                return [4 /*yield*/, _resolvePluginDependencies(c, key, plugins[key], null)];
            case 2:
                _a.sent();
                _a.label = 3;
            case 3:
                i++;
                return [3 /*break*/, 1];
            case 4: return [2 /*return*/, true];
        }
    });
}); };
exports.resolvePluginDependants = resolvePluginDependants;
var _resolvePluginDependencies = function (c, key, keyScope, parentKey) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var pluginTemplates, plugin, scope, depPlugin, confirm_1, deps, depsKeys, i, depKey, depScope;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        switch (_c.label) {
            case 0:
                // IMPORTANT: Do not cache this valuse as they need to be refreshed every
                // round in case new plugin has been installed and c.buildConfig generated
                if (keyScope === null) {
                    return [2 /*return*/, true];
                }
                pluginTemplates = c.buildConfig.pluginTemplates;
                plugin = (0, exports.getMergedPlugin)(c, key);
                scope = _getPluginScope(keyScope).scope;
                if (!!plugin) return [3 /*break*/, 4];
                depPlugin = (_b = (_a = pluginTemplates[scope]) === null || _a === void 0 ? void 0 : _a.pluginTemplates) === null || _b === void 0 ? void 0 : _b[key];
                if (!depPlugin) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'confirm',
                        message: "Install ".concat(key, "?"),
                        warningMessage: "Plugin ".concat((0, logger_1.chalk)().white(key), " source:").concat((0, logger_1.chalk)().white(scope), " required by ").concat((0, logger_1.chalk)().red(parentKey), " is not installed")
                    })];
            case 1:
                confirm_1 = (_c.sent()).confirm;
                if (confirm_1) {
                    c.files.project.config_original.plugins[key] = "source:".concat(scope);
                    (0, configManager_1.writeRenativeConfigFile)(c, c.paths.project.config, c.files.project.config_original);
                    (0, logger_1.logSuccess)("Plugin ".concat(key, " sucessfully installed"));
                    c._requiresNpmInstall = true;
                }
                return [3 /*break*/, 3];
            case 2:
                (0, logger_1.logWarning)("Plugin ".concat((0, logger_1.chalk)().white(parentKey), " requires ").concat((0, logger_1.chalk)().red(key), " which is not available in your system"));
                _c.label = 3;
            case 3: return [3 /*break*/, 4];
            case 4:
                deps = plugin === null || plugin === void 0 ? void 0 : plugin.pluginDependencies;
                if (!deps) return [3 /*break*/, 8];
                depsKeys = Object.keys(deps);
                i = 0;
                _c.label = 5;
            case 5:
                if (!(i < depsKeys.length)) return [3 /*break*/, 8];
                depKey = depsKeys[i];
                depScope = deps[depKey];
                return [4 /*yield*/, _resolvePluginDependencies(c, depKey, depScope, key)];
            case 6:
                _c.sent();
                _c.label = 7;
            case 7:
                i++;
                return [3 /*break*/, 5];
            case 8: return [2 /*return*/, true];
        }
    });
}); };
var parsePlugins = function (c, platform, pluginCallback, ignorePlatformObjectCheck) {
    (0, logger_1.logTask)('parsePlugins');
    if (c.buildConfig) {
        var includedPlugins_1 = (0, common_1.getConfigProp)(c, platform, 'includedPlugins', []);
        var excludedPlugins_1 = (0, common_1.getConfigProp)(c, platform, 'excludedPlugins', []);
        if (includedPlugins_1) {
            var plugins = c.buildConfig.plugins;
            if (plugins) {
                // let totalIncludedPlugins = 0;
                Object.keys(plugins).forEach(function (key) {
                    if ((includedPlugins_1.includes('*')
                        || includedPlugins_1.includes(key))
                        && !excludedPlugins_1.includes(key)) {
                        var plugin = (0, exports.getMergedPlugin)(c, key);
                        if (plugin) {
                            var pluginPlat = plugin[platform];
                            if (ignorePlatformObjectCheck) {
                                // totalIncludedPlugins++;
                                pluginCallback(plugin, pluginPlat, key);
                            }
                            else if (pluginPlat) {
                                if (plugin['no-active'] !== true
                                    && plugin.enabled !== false
                                    && pluginPlat.enabled !== false) {
                                    if (plugin.deprecated) {
                                        (0, logger_1.logWarning)(plugin.deprecated);
                                    }
                                    if (pluginCallback) {
                                        // totalIncludedPlugins++;
                                        pluginCallback(plugin, pluginPlat, key);
                                    }
                                }
                                else {
                                    (0, logger_1.logWarning)("Plugin ".concat(key, " is marked disabled. skipping."));
                                }
                            }
                        }
                    }
                });
                // Not valid warning as web based plugins might not need web definition object to work
                // if (totalIncludedPlugins === 0) {
                //     logWarning(
                //         `Found plugins in your app but non are included. are you sure you added ${chalk().white('includedPlugins')} in your renative.json config?`
                //     );
                // }
            }
            else {
                (0, logger_1.logError)("You have no plugins defined in ".concat((0, logger_1.chalk)().white(c.paths.project.builds.config)));
            }
        }
        else {
            (0, logger_1.logWarning)("You haven't included any ".concat((0, logger_1.chalk)().white('{ common: { includedPlugins: [] }}'), " in your ").concat((0, logger_1.chalk)().white(c.paths.appConfig.config), ". Your app might not work correctly"));
        }
    }
};
exports.parsePlugins = parsePlugins;
var loadPluginTemplates = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var customPluginTemplates, missingDeps, dependencies_1, hasPackageChanged_1;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        switch (_c.label) {
            case 0:
                (0, logger_1.logTask)('loadPluginTemplates');
                c.files.rnv.pluginTemplates.config = (0, fileutils_1.readObjectSync)(c.paths.rnv.pluginTemplates.config);
                c.files.rnv.pluginTemplates.configs = {
                    rnv: c.files.rnv.pluginTemplates.config
                };
                c.paths.rnv.pluginTemplates.dirs = { rnv: c.paths.rnv.pluginTemplates.dir };
                customPluginTemplates = (_b = (_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.paths) === null || _b === void 0 ? void 0 : _b.pluginTemplates;
                missingDeps = _parsePluginTemplateDependencies(c, customPluginTemplates);
                if (!missingDeps.length) return [3 /*break*/, 4];
                dependencies_1 = c.files.project.package.dependencies;
                hasPackageChanged_1 = false;
                missingDeps.forEach(function (dep) {
                    var plugin = (0, exports.getMergedPlugin)(c, dep);
                    if (plugin) {
                        hasPackageChanged_1 = true;
                        dependencies_1[dep] = plugin.version;
                    }
                    else {
                        // Unresolved Plugin
                    }
                });
                if (!hasPackageChanged_1) return [3 /*break*/, 3];
                _updatePackage(c, { dependencies: dependencies_1 });
                (0, logger_1.logInfo)('Found missing dependency scopes. INSTALLING...');
                return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
            case 1:
                _c.sent();
                return [4 /*yield*/, (0, exports.loadPluginTemplates)(c)];
            case 2:
                _c.sent();
                return [3 /*break*/, 4];
            case 3:
                missingDeps.forEach(function (npmDep) {
                    (0, logger_1.logWarning)("Plugin scope ".concat(npmDep, " does not exists in package.json."));
                });
                _c.label = 4;
            case 4: return [2 /*return*/, true];
        }
    });
}); };
exports.loadPluginTemplates = loadPluginTemplates;
var _parsePluginTemplateDependencies = function (c, customPluginTemplates, scope) {
    if (scope === void 0) { scope = 'root'; }
    (0, logger_1.logTask)('_parsePluginTemplateDependencies', "scope:".concat(scope));
    var missingDeps = [];
    if (customPluginTemplates) {
        Object.keys(customPluginTemplates).forEach(function (k) {
            var _a, _b, _c;
            var val = customPluginTemplates[k];
            if (val.npm) {
                var npmDep = ((_b = (_a = c.files.project.package) === null || _a === void 0 ? void 0 : _a.dependencies) === null || _b === void 0 ? void 0 : _b[val.npm])
                    || ((_c = c.files.project.package) === null || _c === void 0 ? void 0 : _c.devDependencies[val.npm]);
                if (npmDep) {
                    var ptPath = void 0;
                    if (npmDep.startsWith('file:')) {
                        ptPath = path_1.default.join(c.paths.project.dir, npmDep.replace('file:', ''), val.path || '');
                    }
                    else {
                        // ptPath = path.join(c.paths.project.nodeModulesDir, val.npm, val.path || '');
                        ptPath = "".concat((0, resolve_1.doResolve)(val.npm), "/").concat(val.path);
                    }
                    var ptConfig = path_1.default.join(ptPath, constants_1.RENATIVE_CONFIG_PLUGINS_NAME);
                    c.paths.rnv.pluginTemplates.dirs[k] = ptPath;
                    if ((0, fileutils_1.fsExistsSync)(ptConfig)) {
                        c.files.rnv.pluginTemplates.configs[k] = (0, fileutils_1.readObjectSync)(ptConfig);
                        _parsePluginTemplateDependencies(c, c.files.rnv.pluginTemplates.configs[k].pluginTemplateDependencies, k);
                    }
                    else {
                        (0, logger_1.logWarning)("Plugin scope ".concat(val.npm, " is not installed yet."));
                    }
                }
                else {
                    // logWarning(`Plugin scope ${val.npm} does not exists in package.json.`);
                    missingDeps.push(val.npm);
                }
            }
        });
    }
    return missingDeps;
};
// const getCleanRegExString = str => str
//     .replace(/\(/g, '\\(')
//     .replace(/\)/g, '\\)')
//     .replace(/\^/g, '\\^')
//     .replace(/\?/g, '\\?')
//     .replace(/\|/g, '\\|')
//     .replace(/\*/g, '\\*')
//     .replace(/\|/g, '\\|')
//     .replace(/\[/g, '\\[')
//     .replace(/\]/g, '\\]')
//     .replace(/\{/g, '\\{')
//     .replace(/\}/g, '\\}')
//     .replace(/\+/g, '\\+')
//     .replace(/ /g, ' {1,}');
// Alternative Regex seem more accurate
var getCleanRegExString = function (str) { return str.replace(/[-\\.,_*+?^$[\](){}!=|`]/ig, '\\$&'); };
var _overridePlugin = function (c, pluginsPath, dir) {
    var source = path_1.default.resolve(pluginsPath, dir, 'overrides');
    var dest = (0, resolve_1.doResolve)(dir, false);
    if (!dest)
        return;
    var plugin = (0, exports.getMergedPlugin)(c, dir);
    var flavourSource;
    if (plugin) {
        flavourSource = path_1.default.resolve(pluginsPath, dir, "overrides@".concat(plugin.version));
    }
    if (flavourSource && (0, fileutils_1.fsExistsSync)(flavourSource)) {
        (0, logger_1.logInfo)("".concat((0, logger_1.chalk)().white(dest.split('node_modules').pop()), " overriden by: ").concat((0, logger_1.chalk)().white(flavourSource.split('node_modules').pop())));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(flavourSource, dest, false);
    }
    else if ((0, fileutils_1.fsExistsSync)(source)) {
        (0, logger_1.logInfo)("".concat((0, logger_1.chalk)().white(dest.split('node_modules').pop()), " overriden by: ").concat((0, logger_1.chalk)().white(source.split('node_modules').pop())));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(source, dest, false);
        // fsReaddirSync(pp).forEach((dir) => {
        //     copyFileSync(path.resolve(pp, file), path.resolve(c.paths.project.dir, 'node_modules', dir));
        // });
    }
    else {
        (0, logger_1.logDebug)("Your plugin configuration has no override path ".concat((0, logger_1.chalk)().white(source), ". skipping folder override action"));
    }
    var overridePath;
    if (plugin.version) {
        var pluginVerArr = plugin.version.split('.');
        var pluginVersions_1 = [];
        var prevVersion_1;
        pluginVerArr.forEach(function (v) {
            if (prevVersion_1) {
                prevVersion_1 = "".concat(prevVersion_1, ".").concat(v);
            }
            else {
                prevVersion_1 = "".concat(v);
            }
            pluginVersions_1.push(prevVersion_1);
        });
        pluginVersions_1.reverse();
        for (var i = 0; i < pluginVersions_1.length; i++) {
            overridePath = path_1.default.resolve(pluginsPath, dir, "overrides@".concat(pluginVersions_1[i], ".json"));
            if ((0, fileutils_1.fsExistsSync)(overridePath)) {
                break;
            }
        }
    }
    if (!(0, fileutils_1.fsExistsSync)(overridePath)) {
        overridePath = path_1.default.resolve(pluginsPath, dir, 'overrides.json');
    }
    var overrideConfig = (0, fileutils_1.readObjectSync)(overridePath);
    var overrides = overrideConfig === null || overrideConfig === void 0 ? void 0 : overrideConfig.overrides;
    if (overrides) {
        Object.keys(overrides).forEach(function (k) {
            var ovDir = path_1.default.join(dest, k);
            var override = overrides[k];
            if ((0, fileutils_1.fsExistsSync)(ovDir)) {
                if ((0, fileutils_1.fsLstatSync)(ovDir).isDirectory()) {
                    (0, logger_1.logWarning)('overrides.json: Directories not supported yet. specify path to actual file');
                }
                else {
                    (0, exports.overrideFileContents)(ovDir, override, overridePath);
                }
            }
        });
    }
};
var overrideFileContents = function (dest, override, overridePath) {
    if (overridePath === void 0) { overridePath = ''; }
    if ((0, fileutils_1.fsExistsSync)(dest)) {
        var fileToFix_1 = (0, fileutils_1.fsReadFileSync)(dest).toString();
        var foundRegEx_1 = false;
        var failTerms_1 = [];
        Object.keys(override).forEach(function (fk) {
            var regEx = new RegExp("".concat(getCleanRegExString(fk)), 'g');
            var count = (fileToFix_1.match(regEx) || []).length;
            if (!count) {
                var overrided = override[fk];
                var regEx2 = new RegExp(getCleanRegExString(overrided), 'g');
                var count2 = (fileToFix_1.match(regEx2) || []).length;
                if (!count2) {
                    failTerms_1.push(fk);
                }
                else {
                    foundRegEx_1 = true;
                    (0, logger_1.logInfo)("".concat((0, logger_1.chalk)().white(dest.split('node_modules').pop()), " overriden by: ").concat((0, logger_1.chalk)().white(overridePath.split('node_modules').pop())));
                }
            }
            else {
                foundRegEx_1 = true;
                fileToFix_1 = fileToFix_1.replace(regEx, override[fk]);
                (0, logger_1.logSuccess)("".concat((0, logger_1.chalk)().white(dest.split('node_modules').pop()), " requires override by: ").concat((0, logger_1.chalk)().white(overridePath.split('node_modules').pop()), ". FIXING...DONE"));
            }
        });
        if (!foundRegEx_1) {
            failTerms_1.forEach(function (term) {
                (0, logger_1.logWarning)("No Match found in ".concat((0, logger_1.chalk)().red(dest.split('node_modules').pop()), " for expression: ").concat((0, logger_1.chalk)().gray(term), ". source: ").concat((0, logger_1.chalk)().white(overridePath.split('node_modules').pop())));
            });
        }
        (0, fileutils_1.fsWriteFileSync)(dest, fileToFix_1);
    }
    else {
        (0, logger_1.logDebug)("overrideFileContents Warning: path does not exists ".concat(dest));
    }
};
exports.overrideFileContents = overrideFileContents;
var installPackageDependenciesAndPlugins = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('installPackageDependenciesAndPlugins');
                return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
            case 1:
                _a.sent();
                return [4 /*yield*/, (0, exports.overrideTemplatePlugins)(c)];
            case 2:
                _a.sent();
                return [4 /*yield*/, (0, projectManager_1.configureFonts)(c)];
            case 3:
                _a.sent();
                return [4 /*yield*/, (0, exports.checkForPluginDependencies)(c)];
            case 4:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
exports.installPackageDependenciesAndPlugins = installPackageDependenciesAndPlugins;
var _getPluginConfiguration = function (c, pluginName) {
    var renativePluginPath;
    try {
        renativePluginPath = require.resolve("".concat(pluginName, "/renative.plugin.json"), { paths: [c.paths.project.dir] });
    }
    catch (_a) {
        //
    }
    if (renativePluginPath) {
        return (0, fileutils_1.readObjectSync)(renativePluginPath);
    }
    return null;
};
var checkForPluginDependencies = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var toAdd, install, answer;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                toAdd = {};
                Object.keys(c.buildConfig.plugins).forEach(function (pluginName) {
                    var renativePluginConfig = _getPluginConfiguration(c, pluginName);
                    if (renativePluginConfig) {
                        c._renativePluginCache[pluginName] = renativePluginConfig;
                    }
                    if (renativePluginConfig === null || renativePluginConfig === void 0 ? void 0 : renativePluginConfig.plugins) {
                        // we have dependencies for this plugin
                        Object.keys(renativePluginConfig.plugins).forEach(function (p) {
                            var _a;
                            if (!c.buildConfig.plugins[p] && ((_a = c.buildConfig.plugins[pluginName].plugins) === null || _a === void 0 ? void 0 : _a[p]) !== null) {
                                (0, logger_1.logWarning)("Plugin ".concat(p, " is not installed yet."));
                                toAdd[p] = renativePluginConfig.plugins[p];
                                c.buildConfig.plugins[p] = renativePluginConfig.plugins[p];
                            }
                        });
                    }
                });
                if (!Object.keys(toAdd).length) return [3 /*break*/, 7];
                install = false;
                if (!!c.program.ci) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'confirm',
                        message: "Install ".concat(Object.keys(toAdd).join(', '), "?"),
                        warningMessage: "One or more dependencies are not installed: ".concat((0, logger_1.chalk)().white(Object.keys(toAdd).join(', ')))
                    })];
            case 1:
                answer = _a.sent();
                install = answer.confirm;
                return [3 /*break*/, 3];
            case 2:
                (0, logger_1.logWarning)('CI detected. Automatically installing dependencies');
                install = true;
                _a.label = 3;
            case 3:
                if (!install) return [3 /*break*/, 7];
                c.files.project.config_original.plugins = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, c.files.project.config_original.plugins), toAdd);
                (0, configManager_1.writeRenativeConfigFile)(c, c.paths.project.config, c.files.project.config_original);
                // Need to reload merged files
                return [4 /*yield*/, (0, configManager_1.parseRenativeConfigs)(c)];
            case 4:
                // Need to reload merged files
                _a.sent();
                return [4 /*yield*/, (0, exports.configurePlugins)(c)];
            case 5:
                _a.sent();
                return [4 /*yield*/, (0, exports.installPackageDependenciesAndPlugins)(c)];
            case 6:
                _a.sent();
                _a.label = 7;
            case 7: return [2 /*return*/];
        }
    });
}); };
exports.checkForPluginDependencies = checkForPluginDependencies;
var overrideTemplatePlugins = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var rnvPluginsDirs, appPluginDirs, appBasePluginDir;
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('overrideTemplatePlugins');
        rnvPluginsDirs = c.paths.rnv.pluginTemplates.dirs;
        appPluginDirs = c.paths.appConfig.pluginDirs;
        appBasePluginDir = c.paths.project.appConfigBase.pluginsDir;
        (0, exports.parsePlugins)(c, c.platform, function (plugin, pluginPlat, key) {
            var _a;
            if ((_a = plugin === null || plugin === void 0 ? void 0 : plugin._scopes) === null || _a === void 0 ? void 0 : _a.length) {
                plugin._scopes.forEach(function (pluginScope) {
                    var pluginOverridePath = rnvPluginsDirs[pluginScope];
                    if (pluginOverridePath) {
                        _overridePlugin(c, pluginOverridePath, key);
                    }
                });
            }
            if (appBasePluginDir) {
                _overridePlugin(c, appBasePluginDir, key);
            }
            if (appPluginDirs) {
                for (var k = 0; k < appPluginDirs.length; k++) {
                    _overridePlugin(c, appPluginDirs[k], key);
                }
            }
        }, true);
        return [2 /*return*/, true];
    });
}); };
exports.overrideTemplatePlugins = overrideTemplatePlugins;
var copyTemplatePluginsSync = function (c) {
    var platform = c.platform;
    var destPath = path_1.default.join((0, common_1.getAppFolder)(c));
    (0, logger_1.logTask)('copyTemplatePluginsSync', "(".concat(destPath, ")"));
    (0, exports.parsePlugins)(c, platform, function (plugin, pluginPlat, key) {
        var objectInject = (0, tslib_1.__spreadArray)([], c.configPropsInjects, true);
        if (plugin.props) {
            Object.keys(plugin.props).forEach(function (v) {
                objectInject.push({
                    pattern: "{{props.".concat(v, "}}"),
                    override: plugin.props[v]
                });
            });
        }
        // FOLDER MERGES FROM PROJECT CONFIG PLUGIN
        var sourcePathRnvPlugin = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(c.paths.rnv.pluginTemplates.dir, key));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePathRnvPlugin, destPath, true, false, false, objectInject);
        // FOLDER MERGES FROM PROJECT CONFIG PLUGIN
        var sourcePath3 = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(c.paths.project.appConfigBase.dir, "plugins/".concat(key)));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath3, destPath, true, false, false, objectInject);
        // FOLDER MERGERS FROM PROJECT CONFIG (PRIVATE)
        var sourcePath3secLegacy = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(c.paths.workspace.project.appConfigBase.dir_LEGACY, "plugins/".concat(key)));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath3secLegacy, destPath, true, false, false, objectInject);
        // FOLDER MERGES FROM PROJECT CONFIG PLUGIN (PRIVATE)
        var sourcePath3sec = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(c.paths.workspace.project.appConfigBase.dir, "plugins/".concat(key)));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath3sec, destPath, true, false, false, objectInject);
        if ((0, fileutils_1.fsExistsSync)(sourcePath3secLegacy)) {
            (0, logger_1.logWarning)("Path: ".concat((0, logger_1.chalk)().red(sourcePath3secLegacy), " is DEPRECATED.\n    Move your files to: ").concat((0, logger_1.chalk)().white(sourcePath3sec), " instead"));
        }
        // FOLDER MERGES FROM APP CONFIG PLUGIN
        var sourcePath2 = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(c.paths.appConfig.dir, "plugins/".concat(key)));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath2, destPath, true, false, false, objectInject);
        // FOLDER MERGES FROM APP CONFIG PLUGIN (PRIVATE)
        var sourcePath2sec = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(c.paths.workspace.appConfig.dir, "plugins/".concat(key)));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath2sec, destPath, true, false, false, objectInject);
        // FOLDER MERGES FROM SCOPED PLUGIN TEMPLATES
        Object.keys(c.paths.rnv.pluginTemplates.dirs).forEach(function (pathKey) {
            if (pathKey !== 'rnv') {
                var pluginTemplatePath = c.paths.rnv.pluginTemplates.dirs[pathKey];
                var sourcePath4sec = (0, common_1.getBuildsFolder)(c, platform, path_1.default.join(pluginTemplatePath, key));
                (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath4sec, destPath, true, false, false, objectInject);
            }
        });
    });
};
exports.copyTemplatePluginsSync = copyTemplatePluginsSync;
var sanitizePluginPath = function (str, name, mandatory, options) {
    var newStr = str;
    try {
        if (str === null || str === void 0 ? void 0 : str.replace) {
            newStr = str.replace('{{PLUGIN_ROOT}}', (0, resolve_1.doResolve)(name, mandatory, options));
        }
    }
    catch (e) {
        // Ignore
    }
    return newStr;
};
exports.sanitizePluginPath = sanitizePluginPath;
var includesPluginPath = function (str) {
    if (str === null || str === void 0 ? void 0 : str.includes) {
        return str.includes('{{PLUGIN_ROOT}}');
    }
    return false;
};
exports.includesPluginPath = includesPluginPath;
var getLocalRenativePlugin = function () { return ({
    version: 'file:../packages/renative',
    webpack: {
        modulePaths: [],
        moduleAliases: {
            renative: {
                projectPath: 'packages/renative'
            }
        }
    }
}); };
exports.getLocalRenativePlugin = getLocalRenativePlugin;
var getModuleConfigs = function (c, primaryKey) {
    var modulePaths = [];
    var moduleAliases = {};
    var doNotResolveModulePaths = [];
    // PLUGINS
    (0, exports.parsePlugins)(c, c.platform, function (plugin, pluginPlat, key) {
        var webpackConfig = plugin[primaryKey] || plugin.webpack || plugin.webpackConfig;
        if (webpackConfig) {
            if (webpackConfig.modulePaths) {
                if (webpackConfig.modulePaths === false) {
                    // ignore
                }
                else if (webpackConfig.modulePaths === true) {
                    modulePaths.push("node_modules/".concat(key));
                }
                else {
                    webpackConfig.modulePaths.forEach(function (v) {
                        if (typeof v === 'string') {
                            modulePaths.push(v);
                        }
                        else if ((0, exports.includesPluginPath)(v.projectPath)) {
                            doNotResolveModulePaths.push((0, exports.sanitizePluginPath)(v.projectPath, key));
                        }
                        else if (v === null || v === void 0 ? void 0 : v.projectPath) {
                            doNotResolveModulePaths.push(path_1.default.join(c.paths.project.dir, v.projectPath));
                        }
                    });
                }
            }
            if (webpackConfig.moduleAliases) {
                if (webpackConfig.moduleAliases === true) {
                    moduleAliases[key] = (0, resolve_1.doResolvePath)(key, true, {}, c.paths.project.nodeModulesDir);
                }
                else {
                    Object.keys(webpackConfig.moduleAliases).forEach(function (aKey) {
                        var mAlias = webpackConfig.moduleAliases[aKey];
                        if (typeof mAlias === 'string') {
                            moduleAliases[key] = (0, resolve_1.doResolvePath)(mAlias, true, {}, c.paths.project.nodeModulesDir);
                        }
                        else if (mAlias.path) {
                            moduleAliases[key] = path_1.default.join(c.paths.project.dir, mAlias.path);
                        }
                        else if ((0, exports.includesPluginPath)(mAlias.projectPath)) {
                            moduleAliases[key] = (0, exports.sanitizePluginPath)(mAlias.projectPath, key);
                        }
                        else if (mAlias.projectPath) {
                            moduleAliases[key] = path_1.default.join(c.paths.project.dir, mAlias.projectPath);
                        }
                    });
                }
            }
        }
    }, true);
    var moduleAliasesArray = [];
    Object.keys(moduleAliases).forEach(function (key) {
        moduleAliasesArray.push("".concat(key, ":").concat(moduleAliases[key]));
    });
    modulePaths = modulePaths
        .map(function (v) { return (0, resolve_1.doResolvePath)(v, true, {}, c.paths.project.dir); })
        .concat(doNotResolveModulePaths)
        .concat([c.paths.project.assets.dir])
        .filter(Boolean);
    return { modulePaths: modulePaths, moduleAliases: moduleAliases, moduleAliasesArray: moduleAliasesArray };
};
exports.getModuleConfigs = getModuleConfigs;
//# sourceMappingURL=index.js.map