"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildHooks = exports.executePipe = void 0;
var tslib_1 = require("tslib");
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var esbuild_1 = require("esbuild");
var logger_1 = require("../systemManager/logger");
var fileutils_1 = require("../systemManager/fileutils");
var common_1 = require("../common");
var resolve_1 = require("../systemManager/resolve");
var executePipe = function (c, key) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var pipe;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                (0, logger_1.logHook)('executePipe', ((_a = c === null || c === void 0 ? void 0 : c.program) === null || _a === void 0 ? void 0 : _a.json) ? key : "('".concat(key, "')"));
                return [4 /*yield*/, (0, exports.buildHooks)(c)];
            case 1:
                _b.sent();
                pipe = c.buildPipes ? c.buildPipes[key] : null;
                if (!Array.isArray(pipe)) return [3 /*break*/, 3];
                return [4 /*yield*/, pipe.reduce(function (accumulatorPromise, next) {
                        (0, logger_1.logHook)("buildHook.".concat(next === null || next === void 0 ? void 0 : next.name), '(EXECUTING)');
                        return accumulatorPromise.then(function () { return next(c); });
                    }, Promise.resolve())];
            case 2:
                _b.sent();
                return [3 /*break*/, 5];
            case 3:
                if (!pipe) return [3 /*break*/, 5];
                (0, logger_1.logHook)("buildHook.".concat(pipe === null || pipe === void 0 ? void 0 : pipe.name), '(EXECUTING)');
                return [4 /*yield*/, pipe(c)];
            case 4:
                _b.sent();
                _b.label = 5;
            case 5: return [2 /*return*/];
        }
    });
}); };
exports.executePipe = executePipe;
/* eslint-disable import/no-dynamic-require, global-require */
var buildHooks = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var enableHookRebuild, shouldBuildHook, confirmed, confirm_1, templatePath, buildHooksSource, e_1, h;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                (0, logger_1.logDebug)('buildHooks');
                enableHookRebuild = (0, common_1.getConfigProp)(c, c.platform, 'enableHookRebuild');
                shouldBuildHook = c.program.reset || c.program.resetHard || c.program.resetAssets
                    || c.program.hooks || !(0, fileutils_1.fsExistsSync)(c.paths.buildHooks.dist.dir) || enableHookRebuild === true
                    || c.runtime.forceBuildHookRebuild;
                if ((!(0, fileutils_1.fsExistsSync)(c.paths.buildHooks.index) && c.program.ci) || c.runtime.skipBuildHooks) {
                    (0, logger_1.logInfo)('No build hooks found and in --ci mode. SKIPPING');
                    return [2 /*return*/, true];
                }
                if (!!(0, fileutils_1.fsExistsSync)(c.paths.buildHooks.index)) return [3 /*break*/, 4];
                if (c.program.ci) {
                    c.runtime.skipBuildHooks = true;
                    return [2 /*return*/];
                }
                confirmed = void 0;
                if (!c.program.yes) return [3 /*break*/, 1];
                confirmed = true;
                return [3 /*break*/, 3];
            case 1: return [4 /*yield*/, inquirer_1.default.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: 'Build hooks not configured in this project. Configure?'
                })];
            case 2:
                confirm_1 = (_b.sent()).confirm;
                confirmed = confirm_1;
                _b.label = 3;
            case 3:
                if (confirmed) {
                    templatePath = c.buildConfig.currentTemplate ? (0, resolve_1.doResolve)(c.buildConfig.currentTemplate) : null;
                    buildHooksSource = void 0;
                    // if there is a template and has buildhooks folder, use that instead of the default
                    if (templatePath && (0, fileutils_1.fsExistsSync)("".concat(templatePath, "/buildHooks/src/index.js"))) {
                        buildHooksSource = path_1.default.join(templatePath, 'buildHooks/src');
                        shouldBuildHook = true;
                    }
                    else {
                        buildHooksSource = path_1.default.join(c.paths.rnv.dir, 'coreTemplateFiles/buildHooks/src');
                    }
                    (0, fileutils_1.copyFolderContentsRecursiveSync)(buildHooksSource, c.paths.buildHooks.dir);
                }
                else {
                    c.runtime.skipBuildHooks = true;
                    return [2 /*return*/];
                }
                _b.label = 4;
            case 4:
                if (!(!c.runtime.isFirstRunAfterNew && !((_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.isNew))) return [3 /*break*/, 10];
                if (!(shouldBuildHook && !c.isBuildHooksReady)) return [3 /*break*/, 9];
                _b.label = 5;
            case 5:
                _b.trys.push([5, 7, , 8]);
                (0, logger_1.logHook)('buildHooks', 'Build hooks not complied. BUILDING...');
                return [4 /*yield*/, (0, esbuild_1.build)({
                        entryPoints: ["".concat(c.paths.buildHooks.dir, "/index.js")],
                        bundle: true,
                        platform: 'node',
                        external: (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], Object.keys(c.files.project.package.dependencies || {}), true), Object.keys(c.files.project.package.devDependencies || {}), true),
                        outfile: "".concat(c.paths.buildHooks.dist.dir, "/index.js"),
                    })];
            case 6:
                _b.sent();
                return [3 /*break*/, 8];
            case 7:
                e_1 = _b.sent();
                // Fail Builds instead of warn when hook fails
                return [2 /*return*/, Promise.reject("BUILD_HOOK Failed with error: ".concat(e_1))];
            case 8:
                c.isBuildHooksReady = true;
                _b.label = 9;
            case 9:
                h = require(c.paths.buildHooks.dist.index);
                c.buildHooks = h.hooks;
                c.buildPipes = h.pipes;
                h = require(c.paths.buildHooks.dist.index);
                _b.label = 10;
            case 10: return [2 /*return*/, true];
        }
    });
}); };
exports.buildHooks = buildHooks;
//# sourceMappingURL=buildHooks.js.map