"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanPlaformAssets = exports.upgradeDependencies = exports.upgradeProjectDependencies = exports.versionCheck = exports.copyBuildsFolder = exports.copyAssetsFolder = exports.parseFonts = exports.copyRuntimeAssets = exports.configureFonts = exports.checkAndCreateBabelConfig = exports.checkAndCreateGitignore = exports.checkAndBootstrapIfRequired = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var prompt_1 = require("../../cli/prompt");
var common_1 = require("../common");
var constants_1 = require("../constants");
var engineManager_1 = require("../engineManager");
var platformManager_1 = require("../platformManager");
var pluginManager_1 = require("../pluginManager");
var fileutils_1 = require("../systemManager/fileutils");
var npmUtils_1 = require("../systemManager/npmUtils");
var exec_1 = require("../systemManager/exec");
var logger_1 = require("../systemManager/logger");
var templateManager_1 = require("../templateManager");
var configManager_1 = require("../configManager");
var checkAndBootstrapIfRequired = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var template, templateArr, templateDir, templatePath, templateObj, appConfigPath, appConfigObj, supportedPlatforms_1, engineTemplates_1, rnvPlatforms_1, activeEngineKeys_1, config_1, pkgJson_1, installPromises_1, appConfigsPath;
    var _a, _b, _c, _d, _e;
    return (0, tslib_1.__generator)(this, function (_f) {
        switch (_f.label) {
            case 0:
                (0, logger_1.logTask)('checkAndBootstrapIfRequired');
                template = c.program.template;
                if (!(!c.paths.project.configExists && template)) return [3 /*break*/, 9];
                return [4 /*yield*/, (0, exec_1.executeAsync)("".concat((0, npmUtils_1.isYarnInstalled)() ? 'yarn' : 'npm', " add ").concat(template), {
                        cwd: c.paths.project.dir
                    })];
            case 1:
                _f.sent();
                templateArr = template.split('@').filter(function (v) { return v !== ''; });
                templateDir = template.startsWith('@') ? "@".concat(templateArr[0]) : templateArr[0];
                templatePath = path_1.default.join(c.paths.project.dir, 'node_modules', templateDir);
                c.paths.template.dir = templatePath;
                c.paths.template.configTemplate = path_1.default.join(templatePath, constants_1.RENATIVE_CONFIG_TEMPLATE_NAME);
                templateObj = (0, fileutils_1.readObjectSync)(c.paths.template.configTemplate);
                appConfigPath = path_1.default.join(c.paths.project.appConfigsDir, c.program.appConfigID, 'renative.json');
                appConfigObj = (0, fileutils_1.readObjectSync)(appConfigPath);
                supportedPlatforms_1 = (_a = appConfigObj === null || appConfigObj === void 0 ? void 0 : appConfigObj.defaults) === null || _a === void 0 ? void 0 : _a.supportedPlatforms;
                engineTemplates_1 = (_c = (_b = c.files.rnv.projectTemplates) === null || _b === void 0 ? void 0 : _b.config) === null || _c === void 0 ? void 0 : _c.engineTemplates;
                rnvPlatforms_1 = (_e = (_d = c.files.rnv.projectTemplates) === null || _d === void 0 ? void 0 : _d.config) === null || _e === void 0 ? void 0 : _e.platforms;
                activeEngineKeys_1 = [];
                supportedPlatforms_1.forEach(function (supPlat) {
                    Object.keys(engineTemplates_1).forEach(function (eKey) {
                        var _a;
                        if (engineTemplates_1[eKey].id === ((_a = rnvPlatforms_1[supPlat]) === null || _a === void 0 ? void 0 : _a.engine)) {
                            activeEngineKeys_1.push(eKey);
                        }
                    });
                });
                config_1 = (0, tslib_1.__assign)({}, templateObj);
                // Clean unused engines
                Object.keys(config_1.engines).forEach(function (k) {
                    if (!activeEngineKeys_1.includes(k)) {
                        delete config_1.engines[k];
                    }
                });
                if (!config_1.templateConfig.packageTemplate) return [3 /*break*/, 4];
                pkgJson_1 = config_1.templateConfig.packageTemplate;
                if (!pkgJson_1.devDependencies)
                    pkgJson_1.devDependencies = {};
                if (!pkgJson_1.dependencies)
                    pkgJson_1.dependencies = {};
                c.files.project.package = pkgJson_1;
                installPromises_1 = [];
                Object.keys(pkgJson_1.devDependencies).forEach(function (devDepKey) {
                    if (activeEngineKeys_1.includes(devDepKey)) {
                        installPromises_1.push((0, exec_1.executeAsync)("npx yarn add ".concat(devDepKey, "@").concat(pkgJson_1.devDependencies[devDepKey]), {
                            cwd: c.paths.project.dir
                        }));
                    }
                });
                if (!installPromises_1.length) return [3 /*break*/, 3];
                return [4 /*yield*/, Promise.all(installPromises_1)];
            case 2:
                _f.sent();
                (0, logger_1.logInfo)('Installed engines DONE');
                activeEngineKeys_1.forEach(function (aek) {
                    var engineConfigPath = path_1.default.join(c.paths.project.dir, 'node_modules', aek, 'renative.engine.json');
                    var eConfig = (0, fileutils_1.readObjectSync)(engineConfigPath);
                    if (eConfig.platforms) {
                        supportedPlatforms_1.forEach(function (supPlat) {
                            var _a;
                            var engPlatNpm = (_a = eConfig.platforms[supPlat]) === null || _a === void 0 ? void 0 : _a.npm;
                            if (engPlatNpm) {
                                var engPlatDeps_1 = engPlatNpm.dependencies;
                                pkgJson_1.dependencies = pkgJson_1.dependencies || {};
                                if (engPlatDeps_1) {
                                    Object.keys(engPlatDeps_1).forEach(function (engPlatDepKey) {
                                        if (!pkgJson_1.dependencies[engPlatDepKey]) {
                                            (0, logger_1.logInfo)("Installing active engine dependency ".concat(engPlatDepKey));
                                            pkgJson_1.dependencies[engPlatDepKey] = engPlatDeps_1[engPlatDepKey];
                                        }
                                    });
                                }
                                var engPlatDevDeps_1 = engPlatNpm.devDependencies;
                                if (engPlatDevDeps_1) {
                                    Object.keys(engPlatDevDeps_1).forEach(function (engPlatDevDepKey) {
                                        pkgJson_1.devDependencies = pkgJson_1.devDependencies || {};
                                        if (!pkgJson_1.devDependencies[engPlatDevDepKey]) {
                                            (0, logger_1.logInfo)("Installing active engine dependency ".concat(engPlatDevDepKey));
                                            pkgJson_1.devDependencies[engPlatDevDepKey] = engPlatDevDeps_1[engPlatDevDepKey];
                                        }
                                    });
                                }
                                var engPlatOptDeps_1 = engPlatNpm.optionalDependencies;
                                if (engPlatOptDeps_1) {
                                    Object.keys(engPlatOptDeps_1).forEach(function (engPlatOptDepKey) {
                                        pkgJson_1.optionalDependencies = pkgJson_1.optionalDependencies || {};
                                        if (!pkgJson_1.optionalDependencies[engPlatOptDepKey]) {
                                            (0, logger_1.logInfo)("Installing active engine dependency ".concat(engPlatOptDepKey));
                                            pkgJson_1.optionalDependencies[engPlatOptDepKey] = engPlatOptDeps_1[engPlatOptDepKey];
                                        }
                                    });
                                }
                            }
                        });
                    }
                });
                _f.label = 3;
            case 3:
                (0, fileutils_1.writeFileSync)(c.paths.project.package, pkgJson_1);
                _f.label = 4;
            case 4:
                delete config_1.templateConfig;
                (0, fileutils_1.writeFileSync)(c.paths.project.config, config_1);
                appConfigsPath = path_1.default.join(templatePath, 'appConfigs');
                if ((0, fileutils_1.fsExistsSync)(appConfigsPath)) {
                    (0, fileutils_1.copyFolderContentsRecursiveSync)(appConfigsPath, path_1.default.join(c.paths.project.appConfigsDir));
                }
                return [4 /*yield*/, (0, npmUtils_1.installPackageDependencies)(c)];
            case 5:
                _f.sent();
                if (c.program.npxMode) {
                    return [2 /*return*/];
                }
                return [4 /*yield*/, (0, configManager_1.parseRenativeConfigs)(c)];
            case 6:
                _f.sent();
                return [4 /*yield*/, (0, templateManager_1.configureTemplateFiles)(c)];
            case 7:
                _f.sent();
                return [4 /*yield*/, (0, templateManager_1.configureEntryPoint)(c)];
            case 8:
                _f.sent();
                _f.label = 9;
            case 9: return [2 /*return*/, true];
        }
    });
}); };
exports.checkAndBootstrapIfRequired = checkAndBootstrapIfRequired;
var checkAndCreateGitignore = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var ignrPath;
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('checkAndCreateGitignore');
        ignrPath = path_1.default.join(c.paths.project.dir, '.gitignore');
        if (!(0, fileutils_1.fsExistsSync)(ignrPath)) {
            (0, logger_1.logInfo)('Your .gitignore is missing. CREATING...DONE');
            (0, fileutils_1.copyFileSync)(path_1.default.join(c.paths.rnv.dir, 'coreTemplateFiles/gitignore-template'), ignrPath);
        }
        return [2 /*return*/, true];
    });
}); };
exports.checkAndCreateGitignore = checkAndCreateGitignore;
var checkAndCreateBabelConfig = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('checkAndCreateBabelConfig');
        if (!c.paths.project.configExists)
            return [2 /*return*/, false];
        // Check babel-config
        (0, logger_1.logDebug)('configureProject:check babel config');
        if (!(0, fileutils_1.fsExistsSync)(c.paths.project.babelConfig)) {
            (0, logger_1.logInfo)("Your babel config file ".concat((0, logger_1.chalk)().white(c.paths.project.babelConfig), " is missing! CREATING...DONE"));
            (0, fileutils_1.copyFileSync)(path_1.default.join(c.paths.rnv.projectTemplate.dir, constants_1.RN_BABEL_CONFIG_NAME), c.paths.project.babelConfig);
        }
        return [2 /*return*/, true];
    });
}); };
exports.checkAndCreateBabelConfig = checkAndCreateBabelConfig;
var configureFonts = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var fontsObj, duplicateFontCheck, fontJsPath, existingFileContents, coreTemplateFiles;
    return (0, tslib_1.__generator)(this, function (_a) {
        fontsObj = 'export default [';
        duplicateFontCheck = [];
        (0, exports.parseFonts)(c, function (font, dir) {
            if (font.includes('.ttf') || font.includes('.otf') || font.includes('.woff')) {
                var keOriginal = font.split('.')[0];
                var keyNormalised = keOriginal.replace(/__/g, ' ');
                var includedFonts = (0, common_1.getConfigProp)(c, c.platform, 'includedFonts');
                if (includedFonts) {
                    if (includedFonts.includes('*')
                        || includedFonts.includes(keOriginal)
                        || includedFonts.includes(keyNormalised)) {
                        if (font && !duplicateFontCheck.includes(font)) {
                            duplicateFontCheck.push(font);
                            var fontSource = path_1.default.join(dir, font).replace(/\\/g, '\\\\');
                            if ((0, fileutils_1.fsExistsSync)(fontSource)) {
                                // const fontFolder = path.join(appFolder, 'app/src/main/assets/fonts');
                                // mkdirSync(fontFolder);
                                // const fontDest = path.join(fontFolder, font);
                                // copyFileSync(fontSource, fontDest);
                                fontsObj += "{\n                              fontFamily: '".concat(keyNormalised, "',\n                              file: require('").concat(fontSource, "'),\n                          },");
                            }
                            else {
                                (0, logger_1.logWarning)("Font ".concat((0, logger_1.chalk)().white(fontSource), " doesn't exist! Skipping."));
                            }
                        }
                    }
                }
            }
        });
        fontsObj += '];';
        if (!(0, fileutils_1.fsExistsSync)(c.paths.project.assets.runtimeDir)) {
            (0, fileutils_1.mkdirSync)(c.paths.project.assets.runtimeDir);
        }
        fontJsPath = path_1.default.join(c.paths.project.assets.dir, 'runtime', 'fonts.web.js');
        if ((0, fileutils_1.fsExistsSync)(fontJsPath)) {
            existingFileContents = (0, fileutils_1.fsReadFileSync)(fontJsPath).toString();
            if (existingFileContents !== fontsObj) {
                (0, logger_1.logDebug)('newFontsJsFile');
                (0, fileutils_1.fsWriteFileSync)(fontJsPath, fontsObj);
            }
        }
        else {
            (0, logger_1.logDebug)('newFontsJsFile');
            (0, fileutils_1.fsWriteFileSync)(fontJsPath, fontsObj);
        }
        coreTemplateFiles = path_1.default.resolve(c.paths.rnv.dir, 'coreTemplateFiles');
        (0, fileutils_1.copyFileSync)(path_1.default.resolve(coreTemplateFiles, 'fontManager.js'), path_1.default.resolve(c.paths.project.assets.dir, 'runtime', 'fontManager.js'));
        (0, fileutils_1.copyFileSync)(path_1.default.resolve(coreTemplateFiles, 'fontManager.js'), path_1.default.resolve(c.paths.project.assets.dir, 'runtime', 'fontManager.server.web.js'));
        (0, fileutils_1.copyFileSync)(path_1.default.resolve(coreTemplateFiles, 'fontManager.web.js'), path_1.default.resolve(c.paths.project.assets.dir, 'runtime', 'fontManager.web.js'));
        return [2 /*return*/, true];
    });
}); };
exports.configureFonts = configureFonts;
var copyRuntimeAssets = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var destPath, sourcePath;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        (0, logger_1.logTask)('copyRuntimeAssets');
        destPath = path_1.default.join(c.paths.project.assets.dir, 'runtime');
        // FOLDER MERGERS FROM APP CONFIG + EXTEND
        if (c.paths.appConfig.dirs) {
            c.paths.appConfig.dirs.forEach(function (v) {
                var sourcePath = path_1.default.join(v, 'assets/runtime');
                (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath, destPath);
            });
        }
        else if (c.paths.appConfig.dir) {
            sourcePath = path_1.default.join(c.paths.appConfig.dir, 'assets/runtime');
            (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath, destPath);
        }
        if (!((_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.common)) {
            (0, logger_1.logDebug)('BUILD_CONFIG', c.buildConfig);
            (0, logger_1.logWarning)("Your ".concat((0, logger_1.chalk)().white(c.paths.appConfig.config), " is misconfigured. (Maybe you have older version?). Missing ").concat((0, logger_1.chalk)().white('{ common: {} }'), " object at root"));
            return [2 /*return*/, true];
        }
        return [2 /*return*/, true];
    });
}); };
exports.copyRuntimeAssets = copyRuntimeAssets;
var parseFonts = function (c, callback) {
    (0, logger_1.logTask)('parseFonts');
    if (c.buildConfig) {
        // FONTS - PROJECT CONFIG
        if ((0, fileutils_1.fsExistsSync)(c.paths.project.appConfigBase.fontsDir)) {
            (0, fileutils_1.fsReaddirSync)(c.paths.project.appConfigBase.fontsDir).forEach(function (font) {
                if (callback) {
                    callback(font, c.paths.project.appConfigBase.fontsDir);
                }
            });
        }
        // FONTS - APP CONFIG
        if (c.paths.appConfig.fontsDirs) {
            c.paths.appConfig.fontsDirs.forEach(function (v) {
                if ((0, fileutils_1.fsExistsSync)(v)) {
                    (0, fileutils_1.fsReaddirSync)(v).forEach(function (font) {
                        if (callback)
                            callback(font, v);
                    });
                }
            });
        }
        else if ((0, fileutils_1.fsExistsSync)(c.paths.appConfig.fontsDir)) {
            (0, fileutils_1.fsReaddirSync)(c.paths.appConfig.fontsDir).forEach(function (font) {
                if (callback)
                    callback(font, c.paths.appConfig.fontsDir);
            });
        }
        _parseFontSources(c, (0, common_1.getConfigProp)(c, c.platform, 'fontSources', []), callback);
        // PLUGIN FONTS
        (0, pluginManager_1.parsePlugins)(c, c.platform, function (plugin) {
            var _a, _b;
            if ((_a = plugin.config) === null || _a === void 0 ? void 0 : _a.fontSources) {
                _parseFontSources(c, (_b = plugin.config) === null || _b === void 0 ? void 0 : _b.fontSources, callback);
            }
        }, true);
    }
};
exports.parseFonts = parseFonts;
var _parseFontSources = function (c, fontSourcesArr, callback) {
    var fontSources = fontSourcesArr.map(function (v) { return _resolvePackage(c, v); });
    fontSources.forEach(function (fontSourceDir) {
        if ((0, fileutils_1.fsExistsSync)(fontSourceDir)) {
            (0, fileutils_1.fsReaddirSync)(fontSourceDir).forEach(function (font) {
                if (callback)
                    callback(font, fontSourceDir);
            });
        }
    });
};
var _resolvePackage = function (c, v) {
    var _a;
    if ((_a = v === null || v === void 0 ? void 0 : v.startsWith) === null || _a === void 0 ? void 0 : _a.call(v, './')) {
        return path_1.default.join(c.paths.project.dir, v);
    }
    return (0, fileutils_1.resolvePackage)(v);
};
var _requiresAssetOverride = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var requiredAssets, assetsToCopy, assetsDir, actionOverride, actionMerge, actionSkip, chosenAction;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                requiredAssets = ((_c = (_b = (_a = c.runtime.engine) === null || _a === void 0 ? void 0 : _a.platforms) === null || _b === void 0 ? void 0 : _b[c.platform]) === null || _c === void 0 ? void 0 : _c.requiredAssets) || [];
                assetsToCopy = [];
                assetsDir = path_1.default.join(c.paths.project.appConfigBase.dir, 'assets', c.platform);
                requiredAssets.forEach(function (v) {
                    var sourcePath = path_1.default.join(c.runtime.engine.originalTemplateAssetsDir, c.platform, v);
                    var destPath = path_1.default.join(assetsDir, v);
                    if ((0, fileutils_1.fsExistsSync)(sourcePath)) {
                        if (!(0, fileutils_1.fsExistsSync)(destPath)) {
                            assetsToCopy.push({
                                sourcePath: sourcePath,
                                destPath: destPath,
                                value: v
                            });
                        }
                    }
                });
                actionOverride = 'Override exisitng folder';
                actionMerge = 'Merge with existing folder';
                actionSkip = 'Skip. Warning: this might fail your build';
                if (!(assetsToCopy.length > 0)) return [3 /*break*/, 4];
                if (!(0, fileutils_1.fsExistsSync)(assetsDir)) {
                    (0, logger_1.logInfo)("Required assets: ".concat((0, logger_1.chalk)().white(JSON.stringify(assetsToCopy.map(function (v) { return v.value; }))), " will be copied to ").concat((0, logger_1.chalk)().white('appConfigs/assets'), " folder"));
                    return [2 /*return*/, true];
                }
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        message: 'What to do next?',
                        type: 'list',
                        name: 'chosenAction',
                        choices: [actionOverride, actionMerge, actionSkip],
                        warningMessage: "Your appConfig/base/assets/".concat(c.platform, " exists but engine ").concat(c.runtime.engine.config.id, " requires some additional assets:\n").concat((0, logger_1.chalk)().red(requiredAssets.join(',')))
                    })];
            case 1:
                chosenAction = (_d.sent()).chosenAction;
                if (!(chosenAction === actionOverride)) return [3 /*break*/, 3];
                return [4 /*yield*/, (0, fileutils_1.removeDirs)([assetsDir])];
            case 2:
                _d.sent();
                _d.label = 3;
            case 3:
                if (chosenAction === actionOverride || chosenAction === actionMerge) {
                    return [2 /*return*/, true];
                }
                _d.label = 4;
            case 4: return [2 /*return*/, false];
        }
    });
}); };
var copyAssetsFolder = function (c, platform, subPath, customFn) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var tsPathsConfig, assetSources, validAssetSources, destPath, hasAssetFolder, requireOverride, sourcePath, sourcePath;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('copyAssetsFolder');
                if (!(0, platformManager_1.isPlatformActive)(c, platform))
                    return [2 /*return*/];
                tsPathsConfig = (0, common_1.getTimestampPathsConfig)(c, platform);
                assetSources = (0, common_1.getConfigProp)(c, c.platform, 'assetSources', []);
                validAssetSources = [];
                assetSources.forEach(function (v) {
                    var assetsPath = path_1.default.join(_resolvePackage(c, v), platform);
                    if ((0, fileutils_1.fsExistsSync)(assetsPath)) {
                        validAssetSources.push(assetsPath);
                    }
                });
                destPath = path_1.default.join((0, common_1.getPlatformProjectDir)(c, platform), subPath || '');
                // FOLDER MERGERS FROM EXTERNAL SOURCES
                if (validAssetSources.length > 0) {
                    (0, logger_1.logInfo)("Found custom assetSources at ".concat((0, logger_1.chalk)().white(validAssetSources.join('/n')), ". Will be used to generate assets."));
                    validAssetSources.forEach(function (sourcePath) {
                        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath, destPath, true, false, false, {}, tsPathsConfig, c);
                    });
                    return [2 /*return*/];
                }
                if (!c.paths.appConfig.dirs) return [3 /*break*/, 4];
                hasAssetFolder = c.paths.appConfig.dirs
                    .filter(function (v) { return (0, fileutils_1.fsExistsSync)(path_1.default.join(v, "assets/".concat(platform))); }).length;
                return [4 /*yield*/, _requiresAssetOverride(c)];
            case 1:
                requireOverride = _a.sent();
                if (!(!hasAssetFolder || requireOverride)) return [3 /*break*/, 3];
                return [4 /*yield*/, generateDefaultAssets(c, platform, path_1.default.join(c.paths.appConfig.dirs[0], "assets/".concat(platform)), requireOverride)];
            case 2:
                _a.sent();
                _a.label = 3;
            case 3: return [3 /*break*/, 6];
            case 4:
                sourcePath = path_1.default.join(c.paths.appConfig.dir, "assets/".concat(platform));
                if (!!(0, fileutils_1.fsExistsSync)(sourcePath)) return [3 /*break*/, 6];
                return [4 /*yield*/, generateDefaultAssets(c, platform, sourcePath)];
            case 5:
                _a.sent();
                _a.label = 6;
            case 6:
                if (customFn) {
                    return [2 /*return*/, customFn(c, platform)];
                }
                // FOLDER MERGERS FROM APP CONFIG + EXTEND
                if (c.paths.appConfig.dirs) {
                    c.paths.appConfig.dirs.forEach(function (v) {
                        var sourcePath = path_1.default.join(v, "assets/".concat(platform));
                        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath, destPath, true, false, false, {}, tsPathsConfig, c);
                    });
                }
                else {
                    sourcePath = path_1.default.join(c.paths.appConfig.dir, "assets/".concat(platform));
                    (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath, destPath, true, false, false, {}, tsPathsConfig, c);
                }
                return [2 /*return*/];
        }
    });
}); };
exports.copyAssetsFolder = copyAssetsFolder;
var generateDefaultAssets = function (c, platform, sourcePath, forceTrue) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var confirmAssets, confirm_1, engine;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('generateDefaultAssets');
                confirmAssets = true;
                if (!(c.program.ci !== true && c.program.yes !== true && !forceTrue)) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'confirm',
                        message: "It seems you don't have assets configured in ".concat((0, logger_1.chalk)().white(sourcePath), " do you want generate default ones?")
                    })];
            case 1:
                confirm_1 = (_a.sent()).confirm;
                confirmAssets = confirm_1;
                _a.label = 2;
            case 2:
                if (confirmAssets) {
                    engine = (0, engineManager_1.getEngineRunnerByPlatform)(c, c.platform);
                    if ((0, fileutils_1.fsExistsSync)(path_1.default.join(engine.originalTemplateAssetsDir, platform))) {
                        (0, fileutils_1.copyFolderContentsRecursiveSync)(path_1.default.join(engine.originalTemplateAssetsDir, platform), sourcePath);
                    }
                    else {
                        (0, logger_1.logWarning)('Currently used engine does not have default assets, creating an empty folder');
                        (0, fileutils_1.mkdirSync)(sourcePath);
                    }
                }
                return [2 /*return*/];
        }
    });
}); };
var copyBuildsFolder = function (c, platform) { return new Promise(function (resolve) {
    var _a;
    (0, logger_1.logTask)('copyBuildsFolder');
    if (!(0, platformManager_1.isPlatformActive)(c, platform, resolve))
        return;
    var destPath = path_1.default.join((0, common_1.getAppFolder)(c));
    var tsPathsConfig = (0, common_1.getTimestampPathsConfig)(c, platform);
    var configPropsInjects = [];
    constants_1.INJECTABLE_CONFIG_PROPS.forEach(function (v) {
        configPropsInjects.push({
            pattern: "{{configProps.".concat(v, "}}"),
            override: (0, common_1.getConfigProp)(c, c.platform, v)
        });
    });
    c.configPropsInjects = configPropsInjects;
    var allInjects = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], c.configPropsInjects, true), c.systemPropsInjects, true), c.runtimePropsInjects, true);
    // FOLDER MERGERS PROJECT CONFIG
    var sourcePath1 = (0, common_1.getBuildsFolder)(c, platform, c.paths.project.appConfigBase.dir);
    (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath1, destPath, true, false, false, allInjects, tsPathsConfig);
    // FOLDER MERGERS PROJECT CONFIG (PRIVATE)
    var sourcePath1secLegacy = (0, common_1.getBuildsFolder)(c, platform, c.paths.workspace.project.appConfigBase.dir_LEGACY);
    (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath1secLegacy, destPath, true, false, false, allInjects, tsPathsConfig);
    // FOLDER MERGERS PROJECT CONFIG (PRIVATE)
    var sourcePath1sec = (0, common_1.getBuildsFolder)(c, platform, c.paths.workspace.project.appConfigBase.dir);
    (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath1sec, destPath, true, false, false, allInjects, tsPathsConfig);
    if ((0, fileutils_1.fsExistsSync)(sourcePath1secLegacy)) {
        (0, logger_1.logWarning)("Path: ".concat((0, logger_1.chalk)().red(sourcePath1secLegacy), " is DEPRECATED.\nMove your files to: ").concat((0, logger_1.chalk)().white(sourcePath1sec), " instead"));
    }
    // DEPRECATED SHARED
    if ((_a = c.runtime.currentPlatform) === null || _a === void 0 ? void 0 : _a.isWebHosted) {
        var sourcePathShared = path_1.default.join(c.paths.project.appConfigBase.dir, 'builds/_shared');
        if ((0, fileutils_1.fsExistsSync)(sourcePathShared)) {
            (0, logger_1.logWarning)('Folder builds/_shared is DEPRECATED. use builds/<PLATFORM> instead ');
        }
        (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePathShared, (0, common_1.getPlatformBuildDir)(c), true, false, false, allInjects);
    }
    // FOLDER MERGERS FROM APP CONFIG + EXTEND
    if (c.paths.appConfig.dirs) {
        c.paths.appConfig.dirs.forEach(function (v) {
            var sourceV = (0, common_1.getBuildsFolder)(c, platform, v);
            (0, fileutils_1.copyFolderContentsRecursiveSync)(sourceV, destPath, true, false, false, allInjects, tsPathsConfig);
        });
    }
    else {
        (0, fileutils_1.copyFolderContentsRecursiveSync)((0, common_1.getBuildsFolder)(c, platform, c.paths.appConfig.dir), destPath, true, false, false, allInjects, tsPathsConfig);
    }
    // FOLDER MERGERS FROM APP CONFIG (PRIVATE)
    var sourcePath0sec = (0, common_1.getBuildsFolder)(c, platform, c.paths.workspace.appConfig.dir);
    (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath0sec, destPath, true, false, false, allInjects, tsPathsConfig);
    (0, pluginManager_1.copyTemplatePluginsSync)(c);
    resolve();
}); };
exports.copyBuildsFolder = copyBuildsFolder;
var SYNCED_DEPS = [
    'rnv',
    '@rnv/engine-rn',
    '@rnv/engine-rn-next',
    '@rnv/engine-lightning',
    '@rnv/engine-rn-web',
    '@rnv/engine-rn-electron',
    '@rnv/engine-rn-macos',
    '@rnv/engine-rn-windows',
    '@rnv/engine-rn-tvos',
    '@rnv/renative',
    '@rnv/template-starter',
    '@rnv/template-blank'
];
var SYNCED_TEMPLATES = [
    '@rnv/template-starter',
    '@rnv/template-blank'
];
var versionCheck = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var recCmd, actionNoUpdate, actionWithUpdate, actionUpgrade, chosenAction;
    var _a, _b, _c, _d, _e, _f, _g;
    return (0, tslib_1.__generator)(this, function (_h) {
        switch (_h.label) {
            case 0:
                (0, logger_1.logTask)('versionCheck');
                if (c.runtime.versionCheckCompleted || ((_b = (_a = c.files.project) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.skipAutoUpdate) || c.program.skipDependencyCheck) {
                    return [2 /*return*/, true];
                }
                c.runtime.rnvVersionRunner = (_d = (_c = c.files.rnv) === null || _c === void 0 ? void 0 : _c.package) === null || _d === void 0 ? void 0 : _d.version;
                c.runtime.rnvVersionProject = (_g = (_f = (_e = c.files.project) === null || _e === void 0 ? void 0 : _e.package) === null || _f === void 0 ? void 0 : _f.devDependencies) === null || _g === void 0 ? void 0 : _g.rnv;
                (0, logger_1.logTask)("versionCheck:rnvRunner:".concat(c.runtime.rnvVersionRunner, ",rnvProject:").concat(c.runtime.rnvVersionProject), (0, logger_1.chalk)().grey);
                if (!(c.runtime.rnvVersionRunner && c.runtime.rnvVersionProject)) return [3 /*break*/, 2];
                if (!((c.runtime.rnvVersionRunner !== c.runtime.rnvVersionProject) && !c.program.skipRnvCheck)) return [3 /*break*/, 2];
                recCmd = (0, logger_1.chalk)().white("$ npx ".concat((0, logger_1.getCurrentCommand)(true)));
                actionNoUpdate = 'Continue and skip updating package.json';
                actionWithUpdate = 'Continue and update package.json';
                actionUpgrade = "Upgrade project to ".concat(c.runtime.rnvVersionRunner);
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        message: 'What to do next?',
                        type: 'list',
                        name: 'chosenAction',
                        choices: [actionNoUpdate, actionWithUpdate, actionUpgrade],
                        warningMessage: "You are running $rnv v".concat((0, logger_1.chalk)().red(c.runtime.rnvVersionRunner), " against project built with rnv v").concat((0, logger_1.chalk)().red(c.runtime.rnvVersionProject), ". This might result in unexpected behaviour!\nIt is recommended that you run your rnv command with npx prefix: ").concat(recCmd, " . or manually update your devDependencies.rnv version in your package.json.")
                    })];
            case 1:
                chosenAction = (_h.sent()).chosenAction;
                c.runtime.versionCheckCompleted = true;
                c.runtime.skipPackageUpdate = chosenAction === actionNoUpdate;
                if (chosenAction === actionUpgrade) {
                    (0, exports.upgradeProjectDependencies)(c, c.runtime.rnvVersionRunner);
                }
                _h.label = 2;
            case 2: return [2 /*return*/, true];
        }
    });
}); };
exports.versionCheck = versionCheck;
var upgradeProjectDependencies = function (c, version) {
    (0, logger_1.logTask)('upgradeProjectDependencies');
    // const templates = c.files.project.config?.templates;
    // TODO: Make this dynamically injected
    // SYNC DEPS
    var result = (0, exports.upgradeDependencies)(c.files.project.package, c.paths.project.package, c.files.project.config, c.paths.project.config, version);
    c._requiresNpmInstall = true;
    return result;
};
exports.upgradeProjectDependencies = upgradeProjectDependencies;
var upgradeDependencies = function (packageFile, packagesPath, configFile, configPath, version) {
    // logTask('upgradeDependencies');
    var result = [];
    _fixDeps(packageFile === null || packageFile === void 0 ? void 0 : packageFile.devDependencies, version);
    _fixDeps(packageFile === null || packageFile === void 0 ? void 0 : packageFile.dependencies, version);
    _fixDeps(packageFile === null || packageFile === void 0 ? void 0 : packageFile.peerDependencies, version);
    SYNCED_TEMPLATES.forEach(function (templ) {
        var _a, _b;
        if ((_b = (_a = configFile === null || configFile === void 0 ? void 0 : configFile.templates) === null || _a === void 0 ? void 0 : _a[templ]) === null || _b === void 0 ? void 0 : _b.version) {
            configFile.templates[templ].version = version;
        }
    });
    if (packageFile) {
        (0, fileutils_1.writeFileSync)(packagesPath, packageFile);
        result.push(packagesPath);
    }
    if (configFile) {
        (0, fileutils_1.writeFileSync)(configPath, configFile);
        result.push(configPath);
    }
    return result;
};
exports.upgradeDependencies = upgradeDependencies;
var _fixDeps = function (deps, version) {
    SYNCED_DEPS.forEach(function (dep) {
        var d = deps === null || deps === void 0 ? void 0 : deps[dep];
        if (d) {
            var prefix = d.match(/~|>|>=|\^|<|<=/) || '';
            deps[dep] = "".concat(prefix).concat(version);
        }
    });
};
var cleanPlaformAssets = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('cleanPlaformAssets');
                return [4 /*yield*/, (0, fileutils_1.cleanFolder)(c.paths.project.assets.dir)];
            case 1:
                _a.sent();
                (0, fileutils_1.mkdirSync)(c.paths.project.assets.runtimeDir);
                return [2 /*return*/, true];
        }
    });
}); };
exports.cleanPlaformAssets = cleanPlaformAssets;
//# sourceMappingURL=index.js.map