"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/* eslint-disable global-require, import/no-dynamic-require */
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var semver_1 = (0, tslib_1.__importDefault)(require("semver"));
var config_1 = (0, tslib_1.__importDefault)(require("../configManager/config"));
var exec_1 = require("../systemManager/exec");
var fileutils_1 = require("../systemManager/fileutils");
var logger_1 = require("../systemManager/logger");
var bumpVersions = function (version) {
    var _a = config_1.default.getConfig().paths, dir = _a.project.dir, pluginTemplates = _a.rnv.pluginTemplates;
    // check for packages to bump
    var packagesDir = path_1.default.join(dir, 'packages');
    if ((0, fileutils_1.fsExistsSync)(packagesDir)) {
        var packages = (0, fileutils_1.fsReaddirSync)(packagesDir);
        packages.forEach(function (name) {
            var pkgPath = path_1.default.join(packagesDir, name);
            var pkgJsonPath = path_1.default.join(pkgPath, 'package.json');
            if ((0, fileutils_1.fsLstatSync)(pkgPath).isDirectory()
                && (0, fileutils_1.fsExistsSync)(pkgJsonPath)) {
                // we found a packaaaage, fist-bumpin' it
                var existingPkgJson = require(pkgJsonPath);
                existingPkgJson.version = version;
                (0, fileutils_1.writeFileSync)(pkgJsonPath, existingPkgJson);
            }
        });
        // check if it's our turf and do some extra magic
        var renativePkgPath = path_1.default.join(packagesDir, 'renative');
        if ((0, fileutils_1.fsExistsSync)(renativePkgPath)) {
            (0, fileutils_1.copyFileSync)(path_1.default.join(dir, 'README.md'), path_1.default.join(renativePkgPath, 'README.md'));
            (0, fileutils_1.updateObjectSync)(pluginTemplates.config, {
                pluginTemplates: {
                    renative: {
                        version: version
                    }
                }
            });
        }
    }
};
var publishAll = function () {
    var dir = config_1.default.getConfig().paths.project.dir;
    var packagesDir = path_1.default.join(dir, 'packages');
    if ((0, fileutils_1.fsExistsSync)(packagesDir)) {
        var packages = (0, fileutils_1.fsReaddirSync)(packagesDir);
        return Promise.all(packages.map(function (name) {
            var pkgPath = path_1.default.join(packagesDir, name);
            return (0, exec_1.executeAsync)('npm i', { cwd: pkgPath });
        }));
    }
    return true;
};
var taskRnvPkg = function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var args, firstArg, secondArg;
    return (0, tslib_1.__generator)(this, function (_a) {
        args = (0, tslib_1.__spreadArray)([], config_1.default.getConfig().program.rawArgs, true);
        args = args.slice(3);
        firstArg = args[0];
        secondArg = args[1];
        switch (firstArg) {
            case 'version':
                // sets the given version to all of the packages, if there are any
                if (!secondArg) {
                    return [2 /*return*/, (0, logger_1.logError)('No version specified', false, true)];
                }
                if (!semver_1.default.valid(secondArg)) {
                    return [2 /*return*/, (0, logger_1.logError)("Invalid version specified ".concat(secondArg), false, true)];
                }
                return [2 /*return*/, bumpVersions(secondArg)];
            case 'publish':
                return [2 /*return*/, publishAll()];
            default:
                (0, logger_1.logError)("Unknown argument ".concat(firstArg), false, true);
                break;
        }
        return [2 /*return*/];
    });
}); };
exports.default = taskRnvPkg;
//# sourceMappingURL=package.js.map