"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWorkspaceOptions = exports.getWorkspaceConnectionString = exports.getWorkspaceDirPath = exports.createWorkspace = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var prompt_1 = require("../../cli/prompt");
var logger_1 = require("../systemManager/logger");
var fileutils_1 = require("../systemManager/fileutils");
var createWorkspace = function (c, workspaceID, workspacePath) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var workspaceConfig;
    var _a, _b, _c, _d;
    return (0, tslib_1.__generator)(this, function (_e) {
        c.files.rnv.configWorkspaces.workspaces[workspaceID] = {
            path: workspacePath
        };
        workspaceConfig = {
            sdks: (_b = (_a = c.files.defaultWorkspace) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.sdks,
            defaultTargets: (_d = (_c = c.files.defaultWorkspace) === null || _c === void 0 ? void 0 : _c.config) === null || _d === void 0 ? void 0 : _d.defaultTargets
        };
        (0, fileutils_1.mkdirSync)(workspacePath);
        (0, fileutils_1.writeFileSync)(path_1.default.join(workspacePath, 'renative.json'), workspaceConfig);
        (0, fileutils_1.writeFileSync)(c.paths.rnv.configWorkspaces, c.files.rnv.configWorkspaces);
        return [2 /*return*/, true];
    });
}); };
exports.createWorkspace = createWorkspace;
var getWorkspaceDirPath = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var wss, ws, dirPath, wsDir, confirm_1, conf;
    var _a, _b, _c, _d, _e, _f;
    return (0, tslib_1.__generator)(this, function (_g) {
        switch (_g.label) {
            case 0:
                (0, logger_1.logTask)('getWorkspaceDirPath');
                wss = c.files.rnv.configWorkspaces;
                ws = c.runtime.selectedWorkspace || ((_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.workspaceID);
                if (!((wss === null || wss === void 0 ? void 0 : wss.workspaces) && ws)) return [3 /*break*/, 5];
                dirPath = (_b = wss.workspaces[ws]) === null || _b === void 0 ? void 0 : _b.path;
                if (!!dirPath) return [3 /*break*/, 5];
                wsDir = path_1.default.join(c.paths.home.dir, ".".concat(ws));
                if (!(0, fileutils_1.fsExistsSync)(wsDir)) return [3 /*break*/, 1];
                wss.workspaces[ws] = {
                    path: wsDir
                };
                (0, fileutils_1.writeFileSync)(c.paths.rnv.configWorkspaces, wss);
                (0, logger_1.logInfo)("Found workspace id ".concat(ws, " and compatible directory ").concat(wsDir, ". Your ").concat(c.paths.rnv.configWorkspaces, " has been updated."));
                return [3 /*break*/, 5];
            case 1:
                if (!(!c.runtime.isWSConfirmed || c.program.ci === true)) return [3 /*break*/, 5];
                confirm_1 = true;
                if (!(c.program.ci !== true)) return [3 /*break*/, 3];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'conf',
                        type: 'confirm',
                        message: "Your project belongs to workspace ".concat((0, logger_1.chalk)().white(ws), ". do you want to add new workspace ").concat((0, logger_1.chalk)().white(ws), " to your local system at ").concat((0, logger_1.chalk)().white(wsDir), "?"),
                        warningMessage: 'No app configs found for this project'
                    })];
            case 2:
                conf = (_g.sent()).conf;
                confirm_1 = conf;
                c.runtime.isWSConfirmed = true;
                _g.label = 3;
            case 3:
                if (!confirm_1) return [3 /*break*/, 5];
                return [4 /*yield*/, (0, exports.createWorkspace)(c, ws, wsDir)];
            case 4:
                _g.sent();
                _g.label = 5;
            case 5:
                if ((_d = (_c = c.buildConfig) === null || _c === void 0 ? void 0 : _c.paths) === null || _d === void 0 ? void 0 : _d.globalConfigDir) {
                    (0, logger_1.logWarning)("paths.globalConfigDir in ".concat(c.paths.project.config, " is DEPRECATED. use workspaceID instead. more info at https://renative.org/docs/workspaces"));
                }
                if (!dirPath) {
                    return [2 /*return*/, ((_f = (_e = c.buildConfig) === null || _e === void 0 ? void 0 : _e.paths) === null || _f === void 0 ? void 0 : _f.globalConfigDir) || c.paths.GLOBAL_RNV_DIR];
                }
                return [2 /*return*/, dirPath];
        }
    });
}); };
exports.getWorkspaceDirPath = getWorkspaceDirPath;
var getWorkspaceConnectionString = function (obj) {
    var _a;
    var remoteUrl = (_a = obj.remote) === null || _a === void 0 ? void 0 : _a.url;
    var connectMsg = remoteUrl
        ? (0, logger_1.chalk)().green("(".concat(obj.remote.type, ":").concat(remoteUrl, ")"))
        : '';
    return connectMsg;
};
exports.getWorkspaceConnectionString = getWorkspaceConnectionString;
var getWorkspaceOptions = function (c) {
    var _a;
    return (0, prompt_1.generateOptions)((_a = c.files.rnv.configWorkspaces) === null || _a === void 0 ? void 0 : _a.workspaces, false, null, function (i, obj, mapping, defaultVal) {
        (0, logger_1.logDebug)('getWorkspaceOptions');
        return " [".concat((0, logger_1.chalk)().grey(i + 1), "]> ").concat((0, logger_1.chalk)().bold(defaultVal), " ").concat((0, exports.getWorkspaceConnectionString)(obj), "\n");
    });
};
exports.getWorkspaceOptions = getWorkspaceOptions;
//# sourceMappingURL=workspace.js.map