"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkForActiveEmulator = exports.askForNewEmulator = exports.connectToWifiDevice = exports.getAndroidTargets = exports.resetAdb = exports.listAndroidTargets = exports.launchAndroidSimulator = exports.composeDevicesString = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var child_process_1 = (0, tslib_1.__importDefault)(require("child_process"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var os_1 = (0, tslib_1.__importDefault)(require("os"));
var fileutils_1 = require("../../systemManager/fileutils");
var exec_1 = require("../../systemManager/exec");
var common_1 = require("./common");
var utils_1 = require("../../systemManager/utils");
var logger_1 = require("../../systemManager/logger");
var constants_1 = require("../../constants");
var CHECK_INTEVAL = 5000;
var currentDeviceProps = {};
var composeDevicesString = function (devices, returnArray) {
    (0, logger_1.logTask)('composeDevicesString', "numDevices:".concat(devices ? devices.length : null));
    var devicesArray = [];
    devices.forEach(function (v, i) { return devicesArray.push(_getDeviceString(v, !returnArray ? i : null)); });
    if (returnArray)
        return devicesArray;
    return "\n".concat(devicesArray.join(''));
};
exports.composeDevicesString = composeDevicesString;
var launchAndroidSimulator = function (c, target, isIndependentThread) {
    if (isIndependentThread === void 0) { isIndependentThread = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var newTarget, device, list, devicesString, choices, response, actualTarget;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0:
                    (0, logger_1.logTask)('launchAndroidSimulator', "target:".concat(target, " independentThread:").concat(!!isIndependentThread));
                    newTarget = target;
                    if (!(target === true)) return [3 /*break*/, 3];
                    device = c.program.device;
                    return [4 /*yield*/, (0, exports.getAndroidTargets)(c, false, device, device)];
                case 1:
                    list = _a.sent();
                    devicesString = (0, exports.composeDevicesString)(list, true);
                    choices = devicesString;
                    return [4 /*yield*/, inquirer_1.default.prompt([
                            {
                                name: 'chosenEmulator',
                                type: 'list',
                                message: 'What emulator would you like to start?',
                                choices: choices
                            }
                        ])];
                case 2:
                    response = _a.sent();
                    newTarget = response.chosenEmulator;
                    _a.label = 3;
                case 3:
                    if (newTarget) {
                        actualTarget = newTarget.name || newTarget;
                        if (isIndependentThread) {
                            (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_EMULATOR, "-avd \"".concat(actualTarget, "\""), {
                                detached: isIndependentThread
                            }).catch(function (err) {
                                if (err.includes && err.includes('WHPX')) {
                                    (0, logger_1.logWarning)(err);
                                    return (0, logger_1.logError)('It seems you do not have the Windows Hypervisor Platform virtualization enabled. Enter windows features in the Windows search box and select Turn Windows features on or off in the search results. In the Windows Features dialog, enable both Hyper-V and Windows Hypervisor Platform.', true);
                                }
                                (0, logger_1.logError)(err);
                            });
                            return [2 /*return*/, Promise.resolve()];
                        }
                        return [2 /*return*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_EMULATOR, "-avd \"".concat(actualTarget, "\""), {
                                detached: isIndependentThread
                            })];
                    }
                    return [2 /*return*/, Promise.reject('No simulator -t target name specified!')];
            }
        });
    });
};
exports.launchAndroidSimulator = launchAndroidSimulator;
var listAndroidTargets = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var device, list, devices;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('listAndroidTargets');
                device = c.program.device;
                return [4 /*yield*/, (0, exports.resetAdb)(c)];
            case 1:
                _a.sent();
                return [4 /*yield*/, (0, exports.getAndroidTargets)(c, false, device, device)];
            case 2:
                list = _a.sent();
                return [4 /*yield*/, (0, exports.composeDevicesString)(list)];
            case 3:
                devices = _a.sent();
                (0, logger_1.logToSummary)("Android Targets:\n".concat(devices));
                if (devices.trim() === '') {
                    (0, logger_1.logToSummary)('Android Targets: No devices found');
                }
                return [2 /*return*/, devices];
        }
    });
}); };
exports.listAndroidTargets = listAndroidTargets;
var _getDeviceString = function (device, i) {
    var isTV = device.isTV, isTablet = device.isTablet, name = device.name, udid = device.udid, isDevice = device.isDevice, isActive = device.isActive, avdConfig = device.avdConfig, isWear = device.isWear, arch = device.arch;
    var deviceIcon = '';
    if (isTablet)
        deviceIcon = 'Tablet 💊 ';
    if (isTV)
        deviceIcon = 'TV 📺 ';
    if (isWear)
        deviceIcon = 'Wear ⌚ ';
    if (!deviceIcon && (udid !== 'unknown' || avdConfig)) {
        deviceIcon = 'Phone 📱 ';
    }
    var deviceString = "".concat((0, logger_1.chalk)().white(name), " | ").concat(deviceIcon, " | arch: ").concat(arch, " | udid: ").concat((0, logger_1.chalk)().grey(udid)).concat(isDevice ? (0, logger_1.chalk)().red(' (device)') : '', " ").concat(isActive ? (0, logger_1.chalk)().magenta(' (active)') : '');
    if (i === null) {
        return { key: name, name: deviceString, value: name, icon: deviceIcon };
    }
    return " [".concat(i + 1, "]> ").concat(deviceString, "\n");
};
var resetAdb = function (c, ranBefore) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var e_1, e_2;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 3, , 4]);
                if (!!ranBefore) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, 'kill-server')];
            case 1:
                _a.sent();
                _a.label = 2;
            case 2: return [3 /*break*/, 4];
            case 3:
                e_1 = _a.sent();
                (0, logger_1.logWarning)(e_1);
                return [3 /*break*/, 4];
            case 4:
                _a.trys.push([4, 6, , 7]);
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, 'start-server')];
            case 5:
                _a.sent();
                return [3 /*break*/, 7];
            case 6:
                e_2 = _a.sent();
                if (ranBefore) {
                    return [2 /*return*/, Promise.reject(e_2)];
                }
                (0, logger_1.logWarning)("Got error:\n".concat(e_2, "\nWill attemnt again in 5 seconds"));
                setTimeout(exports.resetAdb, 5000, c, true);
                return [3 /*break*/, 7];
            case 7: return [2 /*return*/];
        }
    });
}); };
exports.resetAdb = resetAdb;
var getAndroidTargets = function (c, skipDevices, skipAvds, deviceOnly) {
    if (deviceOnly === void 0) { deviceOnly = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var devicesResult, avdResult, e_3;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0:
                    (0, logger_1.logTask)('getAndroidTargets', "skipDevices:".concat(!!skipDevices, " skipAvds:").concat(!!skipAvds, " deviceOnly:").concat(!!deviceOnly));
                    // Temp workaround for race conditions receiving devices with offline status
                    return [4 /*yield*/, new Promise(function (r) { return setTimeout(r, 1000); })];
                case 1:
                    // Temp workaround for race conditions receiving devices with offline status
                    _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 7, , 8]);
                    devicesResult = void 0;
                    avdResult = void 0;
                    if (!!skipDevices) return [3 /*break*/, 4];
                    return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, 'devices -l')];
                case 3:
                    devicesResult = _a.sent();
                    _a.label = 4;
                case 4:
                    if (!!skipAvds) return [3 /*break*/, 6];
                    return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_EMULATOR, '-list-avds')];
                case 5:
                    avdResult = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, _parseDevicesResult(c, devicesResult, avdResult, deviceOnly)];
                case 7:
                    e_3 = _a.sent();
                    return [2 /*return*/, Promise.reject(e_3)];
                case 8: return [2 /*return*/];
            }
        });
    });
};
exports.getAndroidTargets = getAndroidTargets;
var calculateDeviceDiagonal = function (width, height, density) {
    // Calculate the diagonal in inches
    var widthInches = width / density;
    var heightInches = height / density;
    return Math.sqrt(widthInches * widthInches + heightInches * heightInches);
};
var getRunningDeviceProp = function (c, udid, prop) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var rawProps, reg, lines;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                // avoid multiple calls to the same device
                if (currentDeviceProps[udid]) {
                    if (!prop)
                        return [2 /*return*/, currentDeviceProps[udid]];
                    return [2 /*return*/, currentDeviceProps[udid][prop]];
                }
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, "-s ".concat(udid, " shell getprop"))];
            case 1:
                rawProps = _a.sent();
                reg = /\[.+\]: \[.*\n?[^[]*\]/gm;
                lines = rawProps.match(reg);
                lines.forEach(function (line) {
                    var words = line.split(']: [');
                    var key = words[0].slice(1);
                    var value = words[1].slice(0, words[1].length - 1);
                    if (!currentDeviceProps[udid])
                        currentDeviceProps[udid] = {};
                    currentDeviceProps[udid][key] = value;
                });
                return [2 /*return*/, getRunningDeviceProp(c, udid, prop)];
        }
    });
}); };
var decideIfTVRunning = function (c, device) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var udid, model, product, mod, name, flavor, clientIdBase, description, hdmi, modelGroup, configuration, cecEnabled, isTV;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                udid = device.udid, model = device.model, product = device.product;
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.product.model')];
            case 1:
                mod = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.product.name')];
            case 2:
                name = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.build.flavor')];
            case 3:
                flavor = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.com.google.clientidbase')];
            case 4:
                clientIdBase = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.build.description')];
            case 5:
                description = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'init.svc.hdmi')];
            case 6:
                hdmi = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.nrdp.modelgroup')];
            case 7:
                modelGroup = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.build.configuration')];
            case 8:
                configuration = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'persist.sys.cec.enabled')];
            case 9:
                cecEnabled = _a.sent();
                isTV = false;
                [mod, name, flavor, clientIdBase, description, model, product].forEach(function (string) {
                    if (string && string.toLowerCase().includes('tv'))
                        isTV = true;
                });
                if (model.includes('SHIELD'))
                    isTV = true;
                if (hdmi)
                    isTV = true;
                if (modelGroup && modelGroup.toLowerCase().includes('firetv'))
                    isTV = true;
                if (configuration === 'tv')
                    isTV = true;
                if (cecEnabled)
                    isTV = true;
                return [2 /*return*/, isTV];
        }
    });
}); };
var decideIfWearRunning = function (c, device) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var udid, model, product, fingerprint, name, mod, flavor, description, isWear;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                udid = device.udid, model = device.model, product = device.product;
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.vendor.build.fingerprint')];
            case 1:
                fingerprint = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.product.vendor.name')];
            case 2:
                name = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.product.vendor.model')];
            case 3:
                mod = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.build.flavor')];
            case 4:
                flavor = _a.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, udid, 'ro.build.description')];
            case 5:
                description = _a.sent();
                isWear = false;
                [fingerprint, name, mod, flavor, description, model, product].forEach(function (string) {
                    var cmp = string ? string.toLowerCase() : '';
                    if ((cmp.includes('wear') || cmp.includes('rubyfish') || cmp.includes('watch')))
                        isWear = true;
                });
                return [2 /*return*/, isWear];
        }
    });
}); };
var getDeviceType = function (device, c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var screenSizeResult, screenDensityResult, arch, screenProps, _a, width, height, density, _b, width, height, density, diagonalInches, _c, density, width, height, arch, sysdir, tagId, tagDisplay, deviceName, avdId, name_1, skin, image, diagonalInches;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logDebug)('getDeviceType - in', { device: device });
                if (!(device.udid !== 'unknown')) return [3 /*break*/, 7];
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, "-s ".concat(device.udid, " shell wm size"))];
            case 1:
                screenSizeResult = _d.sent();
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, "-s ".concat(device.udid, " shell wm density"))];
            case 2:
                screenDensityResult = _d.sent();
                return [4 /*yield*/, getRunningDeviceProp(c, device.udid, 'ro.product.cpu.abi')];
            case 3:
                arch = _d.sent();
                screenProps = void 0;
                if (screenSizeResult) {
                    _a = screenSizeResult
                        .split('Physical size: ')[1]
                        .split('x'), width = _a[0], height = _a[1];
                    screenProps = {
                        width: parseInt(width, 10),
                        height: parseInt(height, 10)
                    };
                }
                if (screenDensityResult) {
                    density = screenDensityResult.split('Physical density: ')[1];
                    screenProps = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, screenProps), { density: parseInt(density, 10) });
                }
                _b = device;
                return [4 /*yield*/, decideIfTVRunning(c, device)];
            case 4:
                _b.isTV = _d.sent();
                if (!(screenSizeResult && screenDensityResult)) return [3 /*break*/, 6];
                width = screenProps.width, height = screenProps.height, density = screenProps.density;
                diagonalInches = calculateDeviceDiagonal(width, height, density);
                screenProps = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, screenProps), { diagonalInches: diagonalInches });
                device.isTablet = !device.isTV
                    && diagonalInches > constants_1.IS_TABLET_ABOVE_INCH
                    && diagonalInches <= 15;
                _c = device;
                return [4 /*yield*/, decideIfWearRunning(c, device)];
            case 5:
                _c.isWear = _d.sent();
                _d.label = 6;
            case 6:
                device.isPhone = !device.isTablet && !device.isWear && !device.isTV;
                device.isMobile = !device.isWear && !device.isTV;
                device.screenProps = screenProps;
                device.arch = arch;
                (0, logger_1.logDebug)('getDeviceType - out', { device: device });
                return [2 /*return*/, device];
            case 7:
                if (device.avdConfig) {
                    density = parseInt(device.avdConfig['hw.lcd.density'], 10);
                    width = parseInt(device.avdConfig['hw.lcd.width'], 10);
                    height = parseInt(device.avdConfig['hw.lcd.height'], 10);
                    arch = device.avdConfig['abi.type'];
                    sysdir = device.avdConfig['image.sysdir.1'];
                    tagId = device.avdConfig['tag.id'];
                    tagDisplay = device.avdConfig['tag.display'];
                    deviceName = device.avdConfig['hw.device.name'];
                    device.isWear = false;
                    [sysdir, tagId, tagDisplay, deviceName].forEach(function (string) {
                        if (string && string.includes('wear'))
                            device.isWear = true;
                    });
                    avdId = device.avdConfig.AvdId;
                    name_1 = device.avdConfig['hw.device.name'];
                    skin = device.avdConfig['skin.name'];
                    image = device.avdConfig['image.sysdir.1'];
                    device.isTV = false;
                    [avdId, name_1, skin, image].forEach(function (string) {
                        if (string && string.toLowerCase().includes('tv')) {
                            device.isTV = true;
                        }
                    });
                    diagonalInches = calculateDeviceDiagonal(width, height, density);
                    device.isTablet = !device.isTV && diagonalInches > constants_1.IS_TABLET_ABOVE_INCH;
                    device.isPhone = !device.isTablet && !device.isWear && !device.isTV;
                    device.isMobile = !device.isWear && !device.isTV;
                    device.arch = arch;
                    (0, logger_1.logDebug)('getDeviceType - out', { device: device });
                    return [2 /*return*/, device];
                }
                return [2 /*return*/, device];
        }
    });
}); };
var getAvdDetails = function (c, deviceName) {
    var _a = process.env, ANDROID_SDK_HOME = _a.ANDROID_SDK_HOME, ANDROID_AVD_HOME = _a.ANDROID_AVD_HOME;
    // .avd dir might be in other place than homedir. (https://developer.android.com/studio/command-line/variables)
    var avdConfigPaths = [
        "".concat(ANDROID_AVD_HOME),
        "".concat(ANDROID_SDK_HOME, "/.android/avd"),
        "".concat(constants_1.USER_HOME_DIR, "/.android/avd")
    ];
    var results = {};
    avdConfigPaths.forEach(function (cPath) {
        if ((0, fileutils_1.fsExistsSync)(cPath)) {
            var filesPath = (0, fileutils_1.fsReaddirSync)(cPath);
            filesPath.forEach(function (fName) {
                var fPath = path_1.default.join(cPath, fName);
                var dirent = (0, fileutils_1.fsLstatSync)(fPath);
                if (!dirent.isDirectory() && fName === "".concat(deviceName, ".ini")) {
                    var avdData = (0, fileutils_1.fsReadFileSync)(fPath).toString();
                    var lines = avdData.trim().split(/\r?\n/);
                    lines.forEach(function (line) {
                        var _a = line.split('='), key = _a[0], value = _a[1];
                        if (key === 'path') {
                            var initData = (0, fileutils_1.fsReadFileSync)("".concat(value, "/config.ini"))
                                .toString();
                            var initLines = initData.trim().split(/\r?\n/);
                            var avdConfig_1 = {};
                            initLines.forEach(function (initLine) {
                                var _a = initLine.split('='), iniKey = _a[0], iniValue = _a[1];
                                // also remove the white space
                                avdConfig_1[iniKey.trim()] = iniValue.trim();
                            });
                            results.avdConfig = avdConfig_1;
                        }
                    });
                }
            });
        }
    });
    return results;
};
var getEmulatorName = function (c, words) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var emulator, port, emulatorReply, emulatorReplyArray, emulatorNameStr, emulatorName;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                emulator = words[0];
                port = emulator.split('-')[1];
                return [4 /*yield*/, (0, exec_1.executeTelnet)(c, port, 'avd name')];
            case 1:
                emulatorReply = _b.sent();
                emulatorReplyArray = emulatorReply.split('OK');
                emulatorNameStr = emulatorReplyArray[emulatorReplyArray.length - 2];
                emulatorName = ((_a = emulatorNameStr === null || emulatorNameStr === void 0 ? void 0 : emulatorNameStr.trim) === null || _a === void 0 ? void 0 : _a.call(emulatorNameStr)) || '(err: could not parse emulator name)';
                return [2 /*return*/, emulatorName];
        }
    });
}); };
var connectToWifiDevice = function (c, target) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var connect_str, deviceResponse;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                connect_str = "connect ".concat(target);
                if (!target.includes(':')) {
                    connect_str = "connect ".concat(target, ":5555");
                }
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_ADB, connect_str)];
            case 1:
                deviceResponse = _a.sent();
                if (deviceResponse.includes('connected'))
                    return [2 /*return*/, true];
                (0, logger_1.logError)("Failed to ".concat(connect_str), false, true);
                return [2 /*return*/, false];
        }
    });
}); };
exports.connectToWifiDevice = connectToWifiDevice;
var _parseDevicesResult = function (c, devicesString, avdsString, deviceOnly) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var devices, skipTargetCheck, lines, avdLines;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logDebug)("_parseDevicesResult:".concat(devicesString, ":").concat(avdsString, ":").concat(deviceOnly));
                devices = [];
                skipTargetCheck = c.program.skipTargetCheck;
                if (!devicesString) return [3 /*break*/, 2];
                lines = devicesString.trim().split(/\r?\n/);
                (0, logger_1.logDebug)('_parseDevicesResult 2', { lines: lines });
                if (!(lines.length !== 0)) return [3 /*break*/, 2];
                return [4 /*yield*/, Promise.all(lines.map(function (line) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                        var words, isDevice, name_2, model, product;
                        return (0, tslib_1.__generator)(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    words = line.split(/[ ,\t]+/).filter(function (w) { return w !== ''; });
                                    if (words.length === 0)
                                        return [2 /*return*/];
                                    (0, logger_1.logDebug)('_parseDevicesResult 3', { words: words });
                                    if (!(words[1] === 'device')) return [3 /*break*/, 4];
                                    isDevice = !words[0].includes('emulator');
                                    name_2 = _getDeviceProp(words, 'model:');
                                    model = name_2;
                                    product = _getDeviceProp(words, 'product:');
                                    (0, logger_1.logDebug)('_parseDevicesResult 4', { name: name_2 });
                                    if (!!isDevice) return [3 /*break*/, 3];
                                    return [4 /*yield*/, waitForEmulatorToBeReady(c, words[0])];
                                case 1:
                                    _a.sent();
                                    return [4 /*yield*/, getEmulatorName(c, words)];
                                case 2:
                                    name_2 = _a.sent();
                                    (0, logger_1.logDebug)('_parseDevicesResult 5', { name: name_2 });
                                    _a.label = 3;
                                case 3:
                                    (0, logger_1.logDebug)('_parseDevicesResult 6', {
                                        deviceOnly: deviceOnly,
                                        isDevice: isDevice
                                    });
                                    if ((deviceOnly && isDevice) || !deviceOnly) {
                                        devices.push({
                                            udid: words[0],
                                            isDevice: isDevice,
                                            isActive: true,
                                            name: name_2,
                                            model: model,
                                            product: product
                                        });
                                    }
                                    return [2 /*return*/, true];
                                case 4: return [2 /*return*/];
                            }
                        });
                    }); }))];
            case 1:
                _a.sent();
                _a.label = 2;
            case 2:
                if (!avdsString) return [3 /*break*/, 4];
                avdLines = avdsString.trim().split(/\r?\n/);
                (0, logger_1.logDebug)('_parseDevicesResult 7', { avdLines: avdLines });
                return [4 /*yield*/, Promise.all(avdLines.map(function (line) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                        var avdDetails, findProcess;
                        return (0, tslib_1.__generator)(this, function (_a) {
                            try {
                                avdDetails = getAvdDetails(c, line);
                            }
                            catch (e) {
                                (0, logger_1.logError)(e);
                            }
                            try {
                                (0, logger_1.logDebug)('_parseDevicesResult 8', { avdDetails: avdDetails });
                                findProcess = utils_1.isSystemWin
                                    ? "tasklist | find \"avd ".concat(line, "\"")
                                    : "ps x | grep \"avd ".concat(line, "\" | grep -v grep");
                                child_process_1.default.execSync(findProcess);
                                (0, logger_1.logDebug)('_parseDevicesResult 9 - excluding running emulator');
                            }
                            catch (e) {
                                if (avdDetails) {
                                    devices.push((0, tslib_1.__assign)({ udid: 'unknown', isDevice: false, isActive: false, name: line }, avdDetails));
                                }
                            }
                            return [2 /*return*/];
                        });
                    }); }))];
            case 3:
                _a.sent();
                _a.label = 4;
            case 4:
                (0, logger_1.logDebug)('_parseDevicesResult 10', { devices: devices });
                return [2 /*return*/, Promise.all(devices.map(function (device) { return getDeviceType(device, c); })).then(function (devicesArray) { return devicesArray.filter(function (device) {
                        // filter devices based on selected platform
                        var platform = c.platform;
                        if (skipTargetCheck)
                            return true; // return everything if skipTargetCheck is used
                        if (device.isNotEligibleAndroid)
                            return false;
                        var matches = (platform === constants_1.ANDROID && device.isTablet)
                            || (platform === constants_1.ANDROID_WEAR && device.isWear)
                            || (platform === constants_1.ANDROID_TV && device.isTV)
                            || (platform === constants_1.FIRE_TV && device.isTV)
                            || (platform === constants_1.ANDROID && device.isMobile);
                        (0, logger_1.logDebug)('getDeviceType - filter', {
                            device: device,
                            matches: matches,
                            platform: platform
                        });
                        return matches;
                    }); })];
        }
    });
}); };
var _getDeviceProp = function (arr, prop) {
    for (var i = 0; i < arr.length; i++) {
        var v = arr[i];
        if (v && v.includes(prop))
            return v.replace(prop, '');
    }
    return '';
};
var askForNewEmulator = function (c, platform) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var emuName, confirm, sdk, arch;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('askForNewEmulator');
                emuName = c.files.workspace.config.defaultTargets[platform];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'confirm',
                        type: 'confirm',
                        message: "Do you want ReNative to create new Emulator (".concat((0, logger_1.chalk)().white(emuName), ") for you?")
                    })];
            case 1:
                confirm = (_a.sent()).confirm;
                sdk = os_1.default.arch() === 'arm64' ? '30' : '28';
                arch = os_1.default.arch() === 'arm64' ? 'arm64-v8a' : 'x86';
                if (confirm) {
                    switch (platform) {
                        case 'android':
                            return [2 /*return*/, _createEmulator(c, sdk, 'google_apis', emuName, arch).then(function () { return (0, exports.launchAndroidSimulator)(c, emuName, true); })];
                        case 'androidtv':
                            return [2 /*return*/, _createEmulator(c, sdk, 'android-tv', emuName, arch).then(function () { return (0, exports.launchAndroidSimulator)(c, emuName, true); })];
                        case 'androidwear':
                            return [2 /*return*/, _createEmulator(c, sdk, 'android-wear', emuName, arch).then(function () { return (0, exports.launchAndroidSimulator)(c, emuName, true); })];
                        default:
                            return [2 /*return*/, Promise.reject('Cannot find any active or created emulators')];
                    }
                }
                return [2 /*return*/, Promise.reject('Action canceled!')];
        }
    });
}); };
exports.askForNewEmulator = askForNewEmulator;
var _createEmulator = function (c, apiVersion, emuPlatform, emuName, arch) {
    if (arch === void 0) { arch = 'x86'; }
    (0, logger_1.logTask)('_createEmulator');
    return (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_SDKMANAGER, "\"system-images;android-".concat(apiVersion, ";").concat(emuPlatform, ";").concat(arch, "\""))
        .then(function () { return (0, exec_1.execCLI)(c, constants_1.CLI_ANDROID_AVDMANAGER, "create avd -n ".concat(emuName, " -k \"system-images;android-").concat(apiVersion, ";").concat(emuPlatform, ";x86\""), {
        interactive: true
    }); })
        .catch(function (e) { return (0, logger_1.logError)(e, true); });
};
var waitForEmulatorToBeReady = function (c, emulator) { return (0, common_1.waitForEmulator)(c, constants_1.CLI_ANDROID_ADB, "-s ".concat(emulator, " shell getprop init.svc.bootanim"), function (res) { return res.includes('stopped'); }); };
var checkForActiveEmulator = function (c) { return new Promise(function (resolve, reject) {
    (0, logger_1.logTask)('checkForActiveEmulator');
    var platform = c.platform;
    var attempts = 1;
    var maxAttempts = utils_1.isSystemWin ? 20 : 10;
    var running = false;
    var poll = setInterval(function () {
        // Prevent the interval from running until enough promises return to make it stop or we get a result
        if (!running) {
            running = true;
            (0, exports.getAndroidTargets)(c, false, true, false)
                .then(function (v) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                return (0, tslib_1.__generator)(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            (0, logger_1.logDebug)('Available devices after filtering', v);
                            if (!(v.length > 0)) return [3 /*break*/, 1];
                            (0, logger_1.logSuccess)("Found active emulator! ".concat((0, logger_1.chalk)().white(v[0].udid), ". Will use it"));
                            clearInterval(poll);
                            resolve(v[0]);
                            return [3 /*break*/, 4];
                        case 1:
                            (0, logger_1.logRaw)("looking for active emulators: attempt ".concat(attempts, "/").concat(maxAttempts));
                            attempts++;
                            if (!([constants_1.ANDROID_TV, constants_1.FIRE_TV, constants_1.ANDROID_WEAR].includes(platform)
                                && attempts === 2)) return [3 /*break*/, 3];
                            return [4 /*yield*/, (0, exports.resetAdb)(c)];
                        case 2:
                            _a.sent(); // from time to time adb reports a recently started atv emu as being offline. Restarting adb fixes it
                            _a.label = 3;
                        case 3:
                            if (attempts > maxAttempts) {
                                clearInterval(poll);
                                reject('Could not find any active emulators');
                                // TODO: Asking for new emulator is worng as it diverts
                                // user from underlying failure of not being able to connect
                                // return _askForNewEmulator(c, platform);
                            }
                            running = false;
                            _a.label = 4;
                        case 4: return [2 /*return*/];
                    }
                });
            }); })
                .catch(function (e) {
                clearInterval(poll);
                (0, logger_1.logError)(e);
            });
        }
    }, CHECK_INTEVAL);
}); };
exports.checkForActiveEmulator = checkForActiveEmulator;
//# sourceMappingURL=android.js.map