"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.listAppleDevices = exports.launchAppleSimulator = exports.getAppleDevices = void 0;
var tslib_1 = require("tslib");
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var child_process_1 = (0, tslib_1.__importDefault)(require("child_process"));
var logger_1 = require("../../systemManager/logger");
var constants_1 = require("../../constants");
var exec_1 = require("../../systemManager/exec");
var getAppleDevices = function (c, ignoreDevices, ignoreSimulators) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var platform, skipTargetCheck, devicesAndSims, isXcode13, _a, simctl, _b, _c, availableSims, parseFunction, devicesArr, simulatorsArr, allDevices;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                platform = c.platform;
                (0, logger_1.logTask)('getAppleDevices', "ignoreDevices:".concat(ignoreDevices, " ignoreSimulators").concat(ignoreSimulators));
                skipTargetCheck = c.program.skipTargetCheck;
                isXcode13 = false;
                _d.label = 1;
            case 1:
                _d.trys.push([1, 3, , 5]);
                // xcode >= 13
                isXcode13 = true;
                return [4 /*yield*/, (0, exec_1.executeAsync)('xcrun xctrace list devices')];
            case 2:
                devicesAndSims = _d.sent();
                return [3 /*break*/, 5];
            case 3:
                _a = _d.sent();
                return [4 /*yield*/, (0, exec_1.executeAsync)('xcrun instruments -s')];
            case 4:
                // xcode < 13
                devicesAndSims = _d.sent();
                return [3 /*break*/, 5];
            case 5:
                _c = (_b = JSON).parse;
                return [4 /*yield*/, (0, exec_1.executeAsync)('xcrun simctl list --json')];
            case 6:
                simctl = _c.apply(_b, [_d.sent()]);
                availableSims = [];
                Object.keys(simctl.devices).forEach(function (runtime) {
                    (0, logger_1.logDebug)('runtime', runtime);
                    simctl.devices[runtime].forEach(function (device) {
                        if (device.isAvailable) {
                            availableSims.push((0, tslib_1.__assign)((0, tslib_1.__assign)({}, device), { version: runtime.split('.').pop() }));
                        }
                    });
                });
                parseFunction = _parseIOSDevicesList;
                if (isXcode13) {
                    parseFunction = _parseNewIOSDevicesList;
                }
                devicesArr = parseFunction(devicesAndSims, platform, ignoreDevices, ignoreSimulators);
                simulatorsArr = _parseIOSDevicesList(availableSims, platform, ignoreDevices, ignoreSimulators);
                allDevices = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], devicesArr, true), simulatorsArr, true);
                if (!skipTargetCheck) {
                    // filter watches
                    allDevices = allDevices.filter(function (d) { var _a; return !((_a = d.version) === null || _a === void 0 ? void 0 : _a.includes('watchOS')); });
                    // filter other platforms
                    allDevices = allDevices.filter(function (d) {
                        var _a, _b, _c;
                        if (platform === constants_1.IOS
                            && (((_a = d.icon) === null || _a === void 0 ? void 0 : _a.includes('Phone')) || ((_b = d.icon) === null || _b === void 0 ? void 0 : _b.includes('Tablet')))) {
                            return true;
                        }
                        if (platform === constants_1.TVOS && ((_c = d.icon) === null || _c === void 0 ? void 0 : _c.includes('TV')))
                            return true;
                        return false;
                    });
                }
                return [2 /*return*/, allDevices];
        }
    });
}); };
exports.getAppleDevices = getAppleDevices;
var _parseNewIOSDevicesList = function (rawDevices, platform, ignoreDevices) {
    if (ignoreDevices === void 0) { ignoreDevices = false; }
    var devices = [];
    if (ignoreDevices)
        return devices;
    var decideIcon = function (device) {
        var name = device.name, isDevice = device.isDevice;
        switch (platform) {
            case constants_1.IOS:
                if (name.includes('iPhone')
                    || name.includes('iPad')
                    || name.includes('iPod')) {
                    var icon = 'Phone 📱';
                    if (name.includes('iPad'))
                        icon = 'Tablet 💊';
                    return icon;
                }
                return null;
            case constants_1.TVOS:
                if (name.includes('TV')
                    && !name.includes('iPhone')
                    && !name.includes('iPad')) {
                    return 'TV 📺';
                }
                return null;
            default:
                if (isDevice) {
                    return 'Apple Device';
                }
                return null;
        }
    };
    var lines = rawDevices.split('\n');
    var devicesOnly = lines.slice(1, lines.indexOf(''));
    devicesOnly.forEach(function (device) {
        var udid = device.match(/\(([^()]*)\)$/)[1];
        var name = device.split(' (').slice(0, -1).join(' (');
        var icon = decideIcon({ name: name, isDevice: true });
        devices.push({
            udid: udid,
            name: name,
            icon: icon,
            isDevice: true
        });
    });
    return devices;
};
var _parseIOSDevicesList = function (rawDevices, platform, ignoreDevices, ignoreSimulators) {
    if (ignoreDevices === void 0) { ignoreDevices = false; }
    if (ignoreSimulators === void 0) { ignoreSimulators = false; }
    var devices = [];
    var decideIcon = function (device) {
        var name = device.name, isDevice = device.isDevice;
        switch (platform) {
            case constants_1.IOS:
                if (name.includes('iPhone')
                    || name.includes('iPad')
                    || name.includes('iPod')) {
                    var icon = 'Phone 📱';
                    if (name.includes('iPad'))
                        icon = 'Tablet 💊';
                    return icon;
                }
                return null;
            case constants_1.TVOS:
                if (name.includes('TV')
                    && !name.includes('iPhone')
                    && !name.includes('iPad')) {
                    return 'TV 📺';
                }
                return null;
            default:
                if (isDevice) {
                    return 'Apple Device';
                }
                return null;
        }
    };
    if (typeof rawDevices === 'string' && !ignoreDevices) {
        rawDevices.split('\n').forEach(function (line) {
            var s1 = line.match(/\[.*?\]/);
            var s2 = line.match(/\(.*?\)/g);
            // const s3 = line.substring(0, line.indexOf('(') - 1);
            var s4 = line.substring(0, line.indexOf('[') - 1);
            var isSim = false;
            if (s2 && s1) {
                if (s2[s2.length - 1] === '(Simulator)') {
                    isSim = true;
                    s2.pop();
                }
                var version = s2.pop();
                var name_1 = "".concat(s4.substring(0, s4.lastIndexOf('(') - 1));
                name_1 = name_1 || 'undefined';
                var udid = s1[0].replace(/\[|\]/g, '');
                var isDevice = !isSim;
                if (!isDevice)
                    return; // only take care of devices.
                if (!ignoreDevices) {
                    var device = { udid: udid, name: name_1, version: version, isDevice: isDevice };
                    devices.push((0, tslib_1.__assign)((0, tslib_1.__assign)({}, device), { icon: decideIcon(device) }));
                }
            }
        });
    }
    else if (typeof rawDevices === 'object' && !ignoreSimulators) {
        rawDevices.forEach(function (d) {
            var name = d.name, udid = d.udid, version = d.version;
            var device = {
                name: name,
                udid: udid,
                isDevice: false,
                version: version
            };
            devices.push((0, tslib_1.__assign)((0, tslib_1.__assign)({}, device), { icon: decideIcon(device) }));
        });
    }
    return devices;
};
var launchAppleSimulator = function (c, target) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var devicesArr, selectedDevice, i, devices, sim;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('launchAppleSimulator', "".concat(target));
                return [4 /*yield*/, (0, exports.getAppleDevices)(c, true)];
            case 1:
                devicesArr = _a.sent();
                for (i = 0; i < devicesArr.length; i++) {
                    if (devicesArr[i].name === target) {
                        selectedDevice = devicesArr[i];
                    }
                }
                if (!selectedDevice) return [3 /*break*/, 3];
                return [4 /*yield*/, _launchSimulator(selectedDevice)];
            case 2:
                _a.sent();
                return [2 /*return*/, selectedDevice.name];
            case 3:
                (0, logger_1.logWarning)("Your specified simulator target ".concat((0, logger_1.chalk)().white(target), " doesn't exists"));
                devices = devicesArr.map(function (v) { return ({
                    name: "".concat(v.name, " | ").concat(v.icon, " | v: ").concat((0, logger_1.chalk)().green(v.version), " | udid: ").concat((0, logger_1.chalk)().grey(v.udid)).concat(v.isDevice ? (0, logger_1.chalk)().red(' (device)') : ''),
                    value: v
                }); });
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'sim',
                        message: 'Select the simulator you want to launch',
                        type: 'list',
                        choices: devices
                    })];
            case 4:
                sim = (_a.sent()).sim;
                if (!sim) return [3 /*break*/, 6];
                return [4 /*yield*/, _launchSimulator(sim)];
            case 5:
                _a.sent();
                return [2 /*return*/, sim.name];
            case 6: return [2 /*return*/, Promise.reject('Action canceled!')];
        }
    });
}); };
exports.launchAppleSimulator = launchAppleSimulator;
var _launchSimulator = function (selectedDevice) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        try {
            child_process_1.default.spawnSync('xcrun', [
                'simctl',
                'boot',
                selectedDevice.udid
            ]);
        }
        catch (e) {
            // instruments always fail with 255 because it expects more arguments,
            // but we want it to only launch the simulator
        }
        return [2 /*return*/, true];
    });
}); };
var listAppleDevices = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var platform, devicesArr, devicesString;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('listAppleDevices');
                platform = c.platform;
                return [4 /*yield*/, (0, exports.getAppleDevices)(c)];
            case 1:
                devicesArr = _a.sent();
                devicesString = '';
                devicesArr.forEach(function (v, i) {
                    devicesString += " [".concat(i + 1, "]> ").concat((0, logger_1.chalk)().bold(v.name), " | ").concat(v.icon, " | v: ").concat((0, logger_1.chalk)().green(v.version), " | udid: ").concat((0, logger_1.chalk)().grey(v.udid)).concat(v.isDevice ? (0, logger_1.chalk)().red(' (device)') : '', "\n");
                });
                (0, logger_1.logToSummary)("".concat(platform, " Targets:\n\n").concat(devicesString));
                return [2 /*return*/];
        }
    });
}); };
exports.listAppleDevices = listAppleDevices;
//# sourceMappingURL=apple.js.map