"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runTizenSimOrDevice = exports.addDevelopTizenCertificate = exports.createDevelopTizenCertificate = exports.listTizenTargets = exports.launchTizenSimulator = exports.DEFAULT_SECURITY_PROFILE_NAME = exports.DEFAULT_CERTIFICATE_NAME = void 0;
var tslib_1 = require("tslib");
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var net_1 = (0, tslib_1.__importDefault)(require("net"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var common_1 = require("../../common");
var constants_1 = require("../../constants");
var exec_1 = require("../../systemManager/exec");
var fileutils_1 = require("../../systemManager/fileutils");
var logger_1 = require("../../systemManager/logger");
var common_2 = require("./common");
var xml2js = require('xml2js');
var parser = new xml2js.Parser();
exports.DEFAULT_CERTIFICATE_NAME = 'tizen_author';
var formatXMLObject = function (obj) {
    var _a, _b;
    var platArr = (_a = obj['model-config']) === null || _a === void 0 ? void 0 : _a.platform;
    var platKeyArr = ((_b = platArr === null || platArr === void 0 ? void 0 : platArr[0]) === null || _b === void 0 ? void 0 : _b.key) || (platArr === null || platArr === void 0 ? void 0 : platArr.key);
    if (platKeyArr) {
        return (0, tslib_1.__assign)({}, platKeyArr.reduce(function (acc, cur) {
            acc[cur.$.name] = cur._;
            return acc;
        }, {}));
    }
    (0, logger_1.logWarning)('Invalid object received from shell cat /etc/config/model-config.xml');
    return {};
};
exports.DEFAULT_SECURITY_PROFILE_NAME = 'RNVanillaCert';
var launchTizenSimulator = function (c, name) {
    (0, logger_1.logTask)("launchTizenSimulator:".concat(name));
    if (name) {
        return (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN_EMULATOR, "launch --name ".concat(name), {
            detached: true
        });
    }
    return Promise.reject('No simulator -t target name specified!');
};
exports.launchTizenSimulator = launchTizenSimulator;
var listTizenTargets = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var targets, targetArr, targetStr;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN_EMULATOR, 'list-vm', {
                    detached: true
                })];
            case 1:
                targets = _a.sent();
                targetArr = targets.split('\n');
                targetStr = '';
                Object.keys(targetArr).forEach(function (i) {
                    targetStr += "[".concat(i, "]> ").concat(targetArr[i], "\n");
                });
                (0, logger_1.logToSummary)("Tizen Targets:\n".concat(targetStr));
                return [2 /*return*/];
        }
    });
}); };
exports.listTizenTargets = listTizenTargets;
var createDevelopTizenCertificate = function (c) { return new Promise(function (resolve) {
    (0, logger_1.logTask)('createDevelopTizenCertificate');
    var certDirPath = c.paths.workspace.dir;
    var certFilename = exports.DEFAULT_CERTIFICATE_NAME;
    var certPassword = '1234';
    (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "certificate -- ".concat(certDirPath, " -a rnv -f ").concat(certFilename, " -p ").concat(certPassword))
        .then(function () { return (0, exports.addDevelopTizenCertificate)(c, {
        profileName: exports.DEFAULT_SECURITY_PROFILE_NAME,
        certPath: path_1.default.join(certDirPath, "".concat(certFilename, ".p12")),
        certPassword: certPassword,
    }); })
        .then(function () { return resolve(); })
        .catch(function (e) {
        (0, logger_1.logError)(e);
        resolve();
    });
}); };
exports.createDevelopTizenCertificate = createDevelopTizenCertificate;
var addDevelopTizenCertificate = function (c, secureProfileConfig) { return new Promise(function (resolve) {
    (0, logger_1.logTask)('addDevelopTizenCertificate');
    var _a = secureProfileConfig || {}, profileName = _a.profileName, certPath = _a.certPath, certPassword = _a.certPassword;
    (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "security-profiles add -n ".concat(profileName, " -a ").concat(certPath, " -p ").concat(certPassword), { privateParams: [certPassword] })
        .then(function () { return resolve(); })
        .catch(function (e) {
        (0, logger_1.logError)(e);
        resolve();
    });
}); };
exports.addDevelopTizenCertificate = addDevelopTizenCertificate;
var _getDeviceID = function (c, target) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var device, connectResponse, e_1, devicesList, lines, devices, deviceID;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                device = c.program.device;
                if (!device) return [3 /*break*/, 5];
                connectResponse = void 0;
                _a.label = 1;
            case 1:
                _a.trys.push([1, 3, , 4]);
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_SDB_TIZEN, "connect ".concat(target))];
            case 2:
                connectResponse = _a.sent();
                return [3 /*break*/, 4];
            case 3:
                e_1 = _a.sent();
                connectResponse = e_1;
                return [3 /*break*/, 4];
            case 4:
                if (connectResponse.includes('EPERM')) {
                    throw new Error("We can't connect to this device even though it is reachable. Please make sure you have enabled Developer Mode and you have added your IP in the Host PC IP section. For more information consult https://developer.samsung.com/tv/develop/getting-started/using-sdk/tv-device");
                }
                if (connectResponse.includes('failed to connect to remote target')) {
                    throw new Error("Failed to connect to ".concat(target, ". Make sure the IP is correct and you are connected on the same network."));
                }
                if (connectResponse.includes('error'))
                    throw new Error(connectResponse);
                _a.label = 5;
            case 5: return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_SDB_TIZEN, 'devices')];
            case 6:
                devicesList = _a.sent();
                if (devicesList.includes(target)) {
                    lines = devicesList.trim().split(/\r?\n/);
                    devices = lines.filter(function (line) { return line.includes(target); });
                    if (devices.length > 1) {
                        // @todo handle more than one
                    }
                    deviceID = devices[0].split('device')[1].trim();
                    return [2 /*return*/, deviceID];
                }
                return [2 /*return*/, Promise.reject("No device matching ".concat(target, " could be found."))];
        }
    });
}); };
var _getRunningDevices = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var platform, devicesList, lines, devices;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                platform = c.program.platform;
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_SDB_TIZEN, 'devices')];
            case 1:
                devicesList = _a.sent();
                lines = devicesList
                    .trim()
                    .split(/\r?\n/)
                    .filter(function (line) { return !line.includes('List of devices'); });
                devices = [];
                return [4 /*yield*/, Promise.all(lines.map(function (line) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                        var words, name_1, deviceInfoXML, deviceInfo, deviceType, parseObj;
                        return (0, tslib_1.__generator)(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    words = line.replace(/\t/g, '').split('    ');
                                    if (!(words.length >= 3)) return [3 /*break*/, 4];
                                    name_1 = words[0].trim();
                                    return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_SDB_TIZEN, "-s ".concat(name_1, " shell cat /etc/config/model-config.xml"), { ignoreErrors: true })];
                                case 1:
                                    deviceInfoXML = _a.sent();
                                    deviceInfo = void 0;
                                    deviceType = void 0;
                                    if (!(deviceInfoXML !== true && deviceInfoXML !== '')) return [3 /*break*/, 3];
                                    return [4 /*yield*/, parser.parseStringPromise(deviceInfoXML)];
                                case 2:
                                    parseObj = _a.sent();
                                    deviceInfo = formatXMLObject(parseObj);
                                    deviceType = deviceInfo['tizen.org/feature/profile'];
                                    _a.label = 3;
                                case 3:
                                    if ((platform === 'tizenmobile' && deviceType === 'mobile')
                                        || (platform === 'tizenwatch' && deviceType === 'wearable')
                                        || (platform === 'tizen' && !deviceType)) {
                                        devices.push({
                                            name: name_1,
                                            type: words[1].trim(),
                                            id: words[2].trim(),
                                            deviceType: deviceType
                                        });
                                    }
                                    _a.label = 4;
                                case 4: return [2 /*return*/];
                            }
                        });
                    }); }))];
            case 2:
                _a.sent();
                return [2 /*return*/, devices];
        }
    });
}); };
var _waitForEmulatorToBeReady = function (c, target) { return (0, common_2.waitForEmulator)(c, constants_1.CLI_SDB_TIZEN, 'devices', function (res) {
    var lines = res.trim().split(/\r?\n/);
    var devices = lines.filter(function (line) { return line.includes(target) && line.includes('device'); });
    return devices.length > 0;
}); };
var _composeDevicesString = function (devices) { return devices.map(function (device) { return ({
    key: device.id,
    name: device.name,
    value: device.id
}); }); };
// const startHostedServerIfRequired = (c) => {
//     if (Config.isWebHostEnabled) {
//         return rnvStart(c);
//     }
// };
var runTizenSimOrDevice = function (c, buildCoreWebpackProject) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var hosted, _a, target, engine, platform, platformConfig, bundleAssets, isHosted, isLightningEngine, tDir, tBuild, tOut, tId, wgt, wgtClean, certProfile, deviceID, askForEmulator, continueLaunching, isTargetSpecified, devices, targetDevice, e_2, choices, chosenEmulator;
    var _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        switch (_c.label) {
            case 0:
                hosted = c.program.hosted;
                _a = c.runtime, target = _a.target, engine = _a.engine;
                platform = c.platform;
                platformConfig = c.buildConfig.platforms[platform];
                bundleAssets = (0, common_1.getConfigProp)(c, platform, 'bundleAssets');
                isHosted = hosted !== null && hosted !== void 0 ? hosted : !bundleAssets;
                isLightningEngine = engine.config.id === 'engine-lightning';
                if (!bundleAssets && !hosted) {
                    // console.log('RUN WEINRE');
                }
                if (!platformConfig) {
                    throw new Error("runTizen: ".concat((0, logger_1.chalk)().grey(platform), " not defined in your ").concat((0, logger_1.chalk)().white(c.paths.appConfig.config)));
                }
                if (!platformConfig.appName) {
                    throw new Error("runTizen: ".concat((0, logger_1.chalk)().grey(platform), ".appName not defined in your ").concat((0, logger_1.chalk)().white(c.paths.appConfig.config)));
                }
                tDir = (0, common_1.getPlatformProjectDir)(c);
                tBuild = path_1.default.join(tDir, 'build');
                tOut = path_1.default.join(tDir, 'output');
                tId = platformConfig.id;
                wgt = "".concat(platformConfig.appName, ".wgt");
                wgtClean = "".concat(platformConfig.appName.replace(/[^a-z0-9]/gi, '_'), ".wgt");
                certProfile = (_b = platformConfig.certificateProfile) !== null && _b !== void 0 ? _b : exports.DEFAULT_SECURITY_PROFILE_NAME;
                askForEmulator = function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                    var startEmulator, defaultTarget, e_3, err_1;
                    return (0, tslib_1.__generator)(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, inquirer_1.default.prompt([
                                    {
                                        name: 'startEmulator',
                                        type: 'confirm',
                                        message: "Could not find or connect to the specified target (".concat(target, "). Would you like to start an emulator?")
                                    }
                                ])];
                            case 1:
                                startEmulator = (_a.sent()).startEmulator;
                                if (!startEmulator) return [3 /*break*/, 12];
                                defaultTarget = c.files.workspace.config.defaultTargets[platform];
                                _a.label = 2;
                            case 2:
                                _a.trys.push([2, 5, , 12]);
                                return [4 /*yield*/, (0, exports.launchTizenSimulator)(c, defaultTarget)];
                            case 3:
                                _a.sent();
                                deviceID = defaultTarget;
                                return [4 /*yield*/, _waitForEmulatorToBeReady(c, defaultTarget)];
                            case 4:
                                _a.sent();
                                return [2 /*return*/, continueLaunching()];
                            case 5:
                                e_3 = _a.sent();
                                (0, logger_1.logDebug)("askForEmulator:ERRROR: ".concat(e_3));
                                _a.label = 6;
                            case 6:
                                _a.trys.push([6, 10, , 11]);
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN_EMULATOR, "create -n ".concat(defaultTarget, " -p tv-samsung-5.0-x86"))];
                            case 7:
                                _a.sent();
                                return [4 /*yield*/, (0, exports.launchTizenSimulator)(c, defaultTarget)];
                            case 8:
                                _a.sent();
                                deviceID = defaultTarget;
                                return [4 /*yield*/, _waitForEmulatorToBeReady(c, defaultTarget)];
                            case 9:
                                _a.sent();
                                return [2 /*return*/, continueLaunching()];
                            case 10:
                                err_1 = _a.sent();
                                (0, logger_1.logDebug)(err_1);
                                (0, logger_1.logError)("Could not find the specified target and could not create the emulator automatically.\nPlease create one and then edit the default target from ".concat(c.paths.workspace.dir, "/").concat(constants_1.RENATIVE_CONFIG_NAME));
                                return [3 /*break*/, 11];
                            case 11: return [3 /*break*/, 12];
                            case 12: return [2 /*return*/];
                        }
                    });
                }); };
                continueLaunching = function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                    var hasDevice, _a, packageID, e_4, err_2, toReturn, packageID;
                    return (0, tslib_1.__generator)(this, function (_b) {
                        switch (_b.label) {
                            case 0:
                                hasDevice = false;
                                if (!(!isLightningEngine && buildCoreWebpackProject)) return [3 /*break*/, 5];
                                _a = !isHosted;
                                if (!_a) return [3 /*break*/, 2];
                                return [4 /*yield*/, buildCoreWebpackProject(c)];
                            case 1:
                                _a = (_b.sent());
                                _b.label = 2;
                            case 2:
                                _a;
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "build-web -- ".concat(tDir, " -out ").concat(tBuild))];
                            case 3:
                                _b.sent();
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "package -- ".concat(tBuild, " -s ").concat(certProfile, " -t wgt -o ").concat(tOut))];
                            case 4:
                                _b.sent();
                                _b.label = 5;
                            case 5:
                                _b.trys.push([5, 7, , 11]);
                                packageID = platform === 'tizenwatch' || platform === 'tizenmobile'
                                    ? tId.split('.')[0]
                                    : tId;
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "uninstall -p ".concat(packageID, " -t ").concat(deviceID), { ignoreErrors: true })];
                            case 6:
                                _b.sent();
                                hasDevice = true;
                                return [3 /*break*/, 11];
                            case 7:
                                e_4 = _b.sent();
                                if (!(e_4 && e_4.includes && e_4.includes('No device matching'))) return [3 /*break*/, 10];
                                return [4 /*yield*/, (0, exports.launchTizenSimulator)(c, target)];
                            case 8:
                                _b.sent();
                                return [4 /*yield*/, _waitForEmulatorToBeReady(c, target)];
                            case 9:
                                hasDevice = _b.sent();
                                _b.label = 10;
                            case 10: return [3 /*break*/, 11];
                            case 11:
                                try {
                                    if (wgtClean !== wgt) {
                                        (0, logger_1.logInfo)("Your app name contains characters like spaces, changing output from \"".concat(wgt, "\" to \"").concat(wgtClean, "\""));
                                        (0, fileutils_1.fsRenameSync)(path_1.default.join(tOut, wgt), path_1.default.join(tOut, wgtClean));
                                    }
                                }
                                catch (err) {
                                    (0, logger_1.logError)(err);
                                }
                                _b.label = 12;
                            case 12:
                                _b.trys.push([12, 14, , 17]);
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "install -- ".concat(tOut, " -n ").concat(wgtClean, " -t ").concat(deviceID))];
                            case 13:
                                _b.sent();
                                hasDevice = true;
                                return [3 /*break*/, 17];
                            case 14:
                                err_2 = _b.sent();
                                (0, logger_1.logError)(err_2);
                                (0, logger_1.logWarning)("There is no emulator or device connected! Let's try to launch it. \"".concat((0, logger_1.chalk)().white.bold("rnv target launch -p ".concat(platform, " -t ").concat(target)), "\""));
                                return [4 /*yield*/, (0, exports.launchTizenSimulator)(c, target)];
                            case 15:
                                _b.sent();
                                return [4 /*yield*/, _waitForEmulatorToBeReady(c, target)];
                            case 16:
                                hasDevice = _b.sent();
                                return [3 /*break*/, 17];
                            case 17:
                                toReturn = true;
                                if (!(platform !== 'tizenwatch'
                                    && platform !== 'tizenmobile'
                                    && hasDevice)) return [3 /*break*/, 19];
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "run -p ".concat(tId, " -t ").concat(deviceID))];
                            case 18:
                                _b.sent();
                                return [3 /*break*/, 21];
                            case 19:
                                if (!((platform === 'tizenwatch' || platform === 'tizenmobile')
                                    && hasDevice)) return [3 /*break*/, 21];
                                packageID = tId.split('.');
                                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_TIZEN, "run -p ".concat(packageID[0], " -t ").concat(deviceID))];
                            case 20:
                                _b.sent();
                                _b.label = 21;
                            case 21: return [2 /*return*/, toReturn];
                        }
                    });
                }); };
                isTargetSpecified = c.program.target;
                return [4 /*yield*/, _getRunningDevices(c)];
            case 1:
                devices = _c.sent();
                if (!isTargetSpecified) return [3 /*break*/, 9];
                if (!net_1.default.isIP(target)) return [3 /*break*/, 3];
                return [4 /*yield*/, _getDeviceID(c, target)];
            case 2:
                deviceID = _c.sent();
                return [2 /*return*/, continueLaunching()];
            case 3:
                if (devices.length > 0) {
                    targetDevice = devices.find(function (device) { return device.id === target || device.name === target; });
                    if (targetDevice) {
                        deviceID = targetDevice.id;
                        return [2 /*return*/, continueLaunching()];
                    }
                }
                _c.label = 4;
            case 4:
                _c.trys.push([4, 7, , 8]);
                // try to launch it, see if it's a simulator that's not started yet
                return [4 /*yield*/, (0, exports.launchTizenSimulator)(c, target)];
            case 5:
                // try to launch it, see if it's a simulator that's not started yet
                _c.sent();
                return [4 /*yield*/, _waitForEmulatorToBeReady(c, target)];
            case 6:
                _c.sent();
                deviceID = target;
                return [2 /*return*/, continueLaunching()];
            case 7:
                e_2 = _c.sent();
                return [2 /*return*/, askForEmulator()];
            case 8: return [3 /*break*/, 12];
            case 9:
                if (devices.length === 1) {
                    deviceID = devices[0].id;
                    return [2 /*return*/, continueLaunching()];
                }
                if (!(devices.length > 1)) return [3 /*break*/, 11];
                choices = _composeDevicesString(devices);
                return [4 /*yield*/, inquirer_1.default.prompt([
                        {
                            name: 'chosenEmulator',
                            type: 'list',
                            message: 'On what emulator would you like to run the app?',
                            choices: choices
                        }
                    ])];
            case 10:
                chosenEmulator = (_c.sent()).chosenEmulator;
                deviceID = chosenEmulator;
                return [2 /*return*/, continueLaunching()];
            case 11: return [2 /*return*/, askForEmulator()];
            case 12: return [2 /*return*/];
        }
    });
}); };
exports.runTizenSimOrDevice = runTizenSimOrDevice;
//# sourceMappingURL=tizen.js.map