"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runWebosSimOrDevice = exports.listWebOSTargets = exports.launchWebOSimulator = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var fileutils_1 = require("../../systemManager/fileutils");
var exec_1 = require("../../systemManager/exec");
var common_1 = require("../../common");
var common_2 = require("./common");
var logger_1 = require("../../systemManager/logger");
var constants_1 = require("../../constants");
var utils_1 = require("../../systemManager/utils");
var launchWebOSimulator = function (c) {
    var _a, _b, _c, _d;
    (0, logger_1.logTask)('launchWebOSimulator');
    var availableEmulatorVersions = (0, fileutils_1.getDirectories)(path_1.default.join((0, fileutils_1.getRealPath)(c, (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.sdks) === null || _b === void 0 ? void 0 : _b.WEBOS_SDK), 'Emulator'));
    var ePath = path_1.default.join((0, fileutils_1.getRealPath)(c, (_d = (_c = c.buildConfig) === null || _c === void 0 ? void 0 : _c.sdks) === null || _d === void 0 ? void 0 : _d.WEBOS_SDK), "Emulator/".concat((availableEmulatorVersions === null || availableEmulatorVersions === void 0 ? void 0 : availableEmulatorVersions[0]) || 'v4.0.0', "/LG_webOS_TV_Emulator").concat(utils_1.isSystemWin ? '.exe' : '_RCU.app'));
    if (!(0, fileutils_1.fsExistsSync)(ePath)) {
        return Promise.reject("Can't find emulator at path: ".concat(ePath));
    }
    if (utils_1.isSystemWin) {
        return (0, exec_1.executeAsync)(c, ePath, { detached: true, stdio: 'ignore' });
    }
    return (0, exec_1.executeAsync)(c, "".concat(exec_1.openCommand, " ").concat(ePath), { detached: true });
};
exports.launchWebOSimulator = launchWebOSimulator;
// const startHostedServerIfRequired = (c) => {
//     if (Config.isWebHostEnabled) {
//         return rnvStart(c);
//     }
// };
var parseDevices = function (c, devicesResponse) {
    var linesArray = devicesResponse
        .split('\n')
        .slice(2)
        .map(function (line) { return line.trim(); })
        .filter(function (line) { return line !== ''; });
    return Promise.all(linesArray.map(function (line) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var _a, name, device, connection, profile, deviceInfo, e_1;
        return (0, tslib_1.__generator)(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = line
                        .split(' ')
                        .map(function (word) { return word.trim(); })
                        .filter(function (word) { return word !== ''; }), name = _a[0], device = _a[1], connection = _a[2], profile = _a[3];
                    deviceInfo = '';
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_DEVICE_INFO, "-d ".concat(name), { silent: true, timeout: 10000 })];
                case 2:
                    deviceInfo = _b.sent();
                    return [3 /*break*/, 4];
                case 3:
                    e_1 = _b.sent();
                    deviceInfo = e_1;
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/, {
                        name: name,
                        device: device,
                        connection: connection,
                        profile: profile,
                        isDevice: !(0, utils_1.isUrlLocalhost)(device),
                        active: !deviceInfo.includes('ERR!')
                    }];
            }
        });
    }); }));
};
var installAndLaunchApp = function (c, target, appPath, tId) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var e_2, toReturn;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 4]);
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_INSTALL, "--device ".concat(target, " ").concat(appPath))];
            case 1:
                _a.sent();
                return [3 /*break*/, 4];
            case 2:
                e_2 = _a.sent();
                // installing it again if it fails. For some reason webosCLI says that it can't connect to
                // the device from time to time. Running it again works.
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_INSTALL, "--device ".concat(target, " ").concat(appPath))];
            case 3:
                // installing it again if it fails. For some reason webosCLI says that it can't connect to
                // the device from time to time. Running it again works.
                _a.sent();
                return [3 /*break*/, 4];
            case 4:
                toReturn = true;
                // if (isHosted) {
                //     toReturn = startHostedServerIfRequired(c);
                //     await waitForHost(c);
                // }
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_LAUNCH, "--device ".concat(target, " ").concat(tId))];
            case 5:
                // if (isHosted) {
                //     toReturn = startHostedServerIfRequired(c);
                //     await waitForHost(c);
                // }
                _a.sent();
                return [2 /*return*/, toReturn];
        }
    });
}); };
var buildDeviceChoices = function (devices) { return devices.map(function (device) { return ({
    key: device.name,
    name: "".concat(device.name, " - ").concat(device.device),
    value: device.name
}); }); };
var listWebOSTargets = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var devicesResponse, devices, deviceArray;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_DEVICE_INFO, '-D')];
            case 1:
                devicesResponse = _a.sent();
                return [4 /*yield*/, parseDevices(c, devicesResponse)];
            case 2:
                devices = _a.sent();
                deviceArray = devices.map(function (device, i) { return " [".concat(i + 1, "]> ").concat((0, logger_1.chalk)().bold(device.name), " | ").concat(device.device); });
                (0, logger_1.logToSummary)("WebOS Targets:\n".concat(deviceArray.join('\n')));
                return [2 /*return*/, true];
        }
    });
}); };
exports.listWebOSTargets = listWebOSTargets;
var waitForEmulatorToBeReady = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var devicesResponse, devices, emulator;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_DEVICE_INFO, '-D')];
            case 1:
                devicesResponse = _a.sent();
                return [4 /*yield*/, parseDevices(c, devicesResponse)];
            case 2:
                devices = _a.sent();
                emulator = devices.filter(function (d) { return !d.isDevice; })[0];
                if (!emulator)
                    throw new Error('No WebOS emulator configured');
                return [2 /*return*/, (0, common_2.waitForEmulator)(c, constants_1.CLI_WEBOS_ARES_DEVICE_INFO, "-d ".concat(emulator.name), function (res) { return res.includes('modelName'); })];
        }
    });
}); };
var runWebosSimOrDevice = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var device, tDir, tOut, tSim, configFilePath, cnfg, tId, appPath, devicesResponse, devices, activeDevices, actualDevices, response, newDeviceResponse, dev, actualDev, newDevice, tv, choices, response;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                device = c.program.device;
                tDir = (0, common_1.getPlatformProjectDir)(c);
                tOut = path_1.default.join((0, common_1.getPlatformBuildDir)(c), 'output');
                tSim = c.program.target || 'emulator';
                configFilePath = path_1.default.join(tDir, 'appinfo.json');
                cnfg = JSON.parse((0, fileutils_1.fsReadFileSync)(configFilePath, 'utf-8'));
                tId = cnfg.id;
                appPath = path_1.default.join(tOut, "".concat(tId, "_").concat(cnfg.version, "_all.ipk"));
                // Start the fun
                // await buildWeb(c);
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_PACKAGE, "-o ".concat(tOut, " ").concat(tDir, " -n"))];
            case 1:
                // Start the fun
                // await buildWeb(c);
                _a.sent();
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_DEVICE_INFO, '-D')];
            case 2:
                devicesResponse = _a.sent();
                return [4 /*yield*/, parseDevices(c, devicesResponse)];
            case 3:
                devices = _a.sent();
                activeDevices = devices.filter(function (d) { return d.active; });
                if (!device) return [3 /*break*/, 12];
                actualDevices = devices.filter(function (d) { return d.isDevice; });
                if (!!actualDevices.length) return [3 /*break*/, 10];
                return [4 /*yield*/, inquirer_1.default.prompt([
                        {
                            type: 'confirm',
                            name: 'setupDevice',
                            message: 'You want to deploy on a device but have none configured. Do you want to configure one?',
                            default: false
                        }
                    ])];
            case 4:
                response = _a.sent();
                if (!response.setupDevice) return [3 /*break*/, 9];
                // Yes, I would like that
                (0, logger_1.logInfo)('Please follow the instructions from http://webostv.developer.lge.com/develop/app-test/#installDevModeApp on how to setup the TV and the connection with the PC. Then follow the onscreen prompts\n');
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_SETUP_DEVICE, '', {
                        interactive: true
                    })];
            case 5:
                _a.sent();
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_DEVICE_INFO, '-D')];
            case 6:
                newDeviceResponse = _a.sent();
                return [4 /*yield*/, parseDevices(c, newDeviceResponse)];
            case 7:
                dev = _a.sent();
                actualDev = dev.filter(function (d) { return d.isDevice; });
                if (!(actualDev.length > 0)) return [3 /*break*/, 9];
                newDevice = actualDev[0];
                // Oh boy, oh boy, I did it! I have a TV connected!
                (0, logger_1.logInfo)("Please enter the `Passphrase` from the TV's Developer Mode app");
                return [4 /*yield*/, (0, exec_1.execCLI)(c, constants_1.CLI_WEBOS_ARES_NOVACOM, "--device ".concat(newDevice.name, " --getkey"), { stdio: 'inherit' })];
            case 8:
                _a.sent();
                return [2 /*return*/, installAndLaunchApp(c, newDevice.name, appPath, tId)];
            case 9: return [3 /*break*/, 11];
            case 10:
                if (actualDevices.length === 1) {
                    tv = actualDevices[0];
                    return [2 /*return*/, installAndLaunchApp(c, tv.name, appPath, tId)];
                }
                _a.label = 11;
            case 11: return [3 /*break*/, 19];
            case 12:
                if (!!c.program.target) return [3 /*break*/, 18];
                // No target specified
                if (activeDevices.length === 1) {
                    // One device present
                    return [2 /*return*/, installAndLaunchApp(c, devices[0].name, appPath, tId)];
                }
                if (!(activeDevices.length > 1)) return [3 /*break*/, 14];
                choices = buildDeviceChoices(devices);
                return [4 /*yield*/, inquirer_1.default.prompt([
                        {
                            name: 'chosenDevice',
                            type: 'list',
                            message: 'What device would you like to start the app?',
                            choices: choices
                        }
                    ])];
            case 13:
                response = _a.sent();
                if (response.chosenDevice) {
                    return [2 /*return*/, installAndLaunchApp(c, response.chosenDevice, appPath, tId)];
                }
                return [3 /*break*/, 17];
            case 14: return [4 /*yield*/, (0, exports.launchWebOSimulator)(c)];
            case 15:
                _a.sent();
                return [4 /*yield*/, waitForEmulatorToBeReady(c)];
            case 16:
                _a.sent();
                return [2 /*return*/, installAndLaunchApp(c, tSim, appPath, tId)];
            case 17: return [3 /*break*/, 19];
            case 18: 
            // Target specified, using that
            return [2 /*return*/, installAndLaunchApp(c, c.program.target, appPath, tId)];
            case 19: return [2 /*return*/];
        }
    });
}); };
exports.runWebosSimOrDevice = runWebosSimOrDevice;
//# sourceMappingURL=webos.js.map