"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkSdk = exports.checkAndConfigureSdks = exports.checkAndConfigureWebosSdks = exports.checkAndConfigureTizenSdks = exports.checkAndConfigureAndroidSdks = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var constants_1 = require("../constants");
var utils_1 = require("../systemManager/utils");
var fileutils_1 = require("../systemManager/fileutils");
var logger_1 = require("../systemManager/logger");
var setupManager_1 = (0, tslib_1.__importDefault)(require("../setupManager"));
var configManager_1 = require("../configManager");
var SDK_LOCATIONS = {
    android: [
        path_1.default.join('/usr/local/android-sdk'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'Library/Android/sdk'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'AppData/Local/Android/android-sdk'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'AppData/Local/Android/sdk'),
        path_1.default.join('Program Files (x86)/Android/android-sdk')
    ],
    'android-ndk': [
        path_1.default.join('/usr/local/android-sdk/ndk'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'Library/Android/sdk/ndk'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'AppData/Local/Android/android-sdk/ndk'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'AppData/Local/Android/sdk/ndk'),
        path_1.default.join('Program Files (x86)/Android/android-sdk/ndk'),
        path_1.default.join('/usr/local/android-sdk/ndk-bundle'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'Library/Android/sdk/ndk-bundle'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'AppData/Local/Android/android-sdk/ndk-bundle'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'AppData/Local/Android/sdk/ndk-bundle'),
        path_1.default.join('Program Files (x86)/Android/android-sdk/ndk-bundle')
    ],
    tizen: [
        path_1.default.join('usr/local/tizen-studio'),
        path_1.default.join(constants_1.USER_HOME_DIR, 'tizen-studio'),
        path_1.default.join('C:\\tizen-studio')
    ],
    webos: [
        path_1.default.join('/opt/webOS_TV_SDK'),
        path_1.default.join('C:\\webOS_TV_SDK')
    ]
};
var _logSdkWarning = function (c) {
    (0, logger_1.logWarning)("Your ".concat(c.paths.workspace.config, " is missing SDK configuration object"));
};
var checkAndConfigureAndroidSdks = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var sdk, sdkManagerPath, avdManagerPath;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        sdk = (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.sdks) === null || _b === void 0 ? void 0 : _b.ANDROID_SDK;
        (0, logger_1.logTask)('checkAndConfigureAndroidSdks', "(".concat(sdk, ")"));
        if (!sdk)
            return [2 /*return*/, _logSdkWarning(c)];
        sdkManagerPath = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "cmdline-tools/latest/bin/sdkmanager".concat(utils_1.isSystemWin ? '.bat' : '')));
        if (!(0, fileutils_1.fsExistsSync)(sdkManagerPath)) {
            sdkManagerPath = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "tools/bin/sdkmanager".concat(utils_1.isSystemWin ? '.bat' : '')));
        }
        avdManagerPath = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "cmdline-tools/latest/bin/avdmanager".concat(utils_1.isSystemWin ? '.bat' : '')));
        if (!(0, fileutils_1.fsExistsSync)(avdManagerPath)) {
            avdManagerPath = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "tools/bin/avdmanager".concat(utils_1.isSystemWin ? '.bat' : '')));
        }
        c.cli[constants_1.CLI_ANDROID_EMULATOR] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "emulator/emulator".concat(utils_1.isSystemWin ? '.exe' : '')));
        c.cli[constants_1.CLI_ANDROID_ADB] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "platform-tools/adb".concat(utils_1.isSystemWin ? '.exe' : '')));
        c.cli[constants_1.CLI_ANDROID_AVDMANAGER] = avdManagerPath;
        c.cli[constants_1.CLI_ANDROID_SDKMANAGER] = sdkManagerPath;
        return [2 /*return*/];
    });
}); };
exports.checkAndConfigureAndroidSdks = checkAndConfigureAndroidSdks;
var checkAndConfigureTizenSdks = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var sdk;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        (0, logger_1.logTask)("checkAndConfigureTizenSdks:".concat(c.platform));
        sdk = (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.sdks) === null || _b === void 0 ? void 0 : _b.TIZEN_SDK;
        if (sdk) {
            c.cli[constants_1.CLI_TIZEN_EMULATOR] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "tools/emulator/bin/em-cli".concat(utils_1.isSystemWin ? '.bat' : '')));
            c.cli[constants_1.CLI_TIZEN] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "tools/ide/bin/tizen".concat(utils_1.isSystemWin ? '.bat' : '')));
            c.cli[constants_1.CLI_SDB_TIZEN] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "tools/sdb".concat(utils_1.isSystemWin ? '.exe' : '')));
        }
        else {
            _logSdkWarning(c);
        }
        return [2 /*return*/];
    });
}); };
exports.checkAndConfigureTizenSdks = checkAndConfigureTizenSdks;
var checkAndConfigureWebosSdks = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var sdk;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        (0, logger_1.logTask)("checkAndConfigureWebosSdks:".concat(c.platform));
        sdk = (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.sdks) === null || _b === void 0 ? void 0 : _b.WEBOS_SDK;
        if (sdk) {
            c.cli[constants_1.CLI_WEBOS_ARES] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares".concat(utils_1.isSystemWin ? '.cmd' : '')));
            c.cli[constants_1.CLI_WEBOS_ARES_PACKAGE] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares-package".concat(utils_1.isSystemWin ? '.cmd' : '')));
            c.cli[constants_1.CLI_WEBOS_ARES_INSTALL] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares-install".concat(utils_1.isSystemWin ? '.cmd' : '')));
            c.cli[constants_1.CLI_WEBOS_ARES_LAUNCH] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares-launch".concat(utils_1.isSystemWin ? '.cmd' : '')));
            c.cli[constants_1.CLI_WEBOS_ARES_SETUP_DEVICE] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares-setup-device".concat(utils_1.isSystemWin ? '.cmd' : '')));
            c.cli[constants_1.CLI_WEBOS_ARES_DEVICE_INFO] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares-device-info".concat(utils_1.isSystemWin ? '.cmd' : '')));
            c.cli[constants_1.CLI_WEBOS_ARES_NOVACOM] = (0, fileutils_1.getRealPath)(c, path_1.default.join(sdk, "CLI/bin/ares-novacom".concat(utils_1.isSystemWin ? '.cmd' : '')));
        }
        else {
            _logSdkWarning(c);
        }
        return [2 /*return*/];
    });
}); };
exports.checkAndConfigureWebosSdks = checkAndConfigureWebosSdks;
var checkAndConfigureSdks = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('checkAndConfigureSdks');
        switch (c.platform) {
            case constants_1.ANDROID:
            case constants_1.ANDROID_TV:
            case constants_1.FIRE_TV:
            case constants_1.ANDROID_WEAR:
                return [2 /*return*/, (0, exports.checkAndConfigureAndroidSdks)(c)];
            case constants_1.TIZEN:
            case constants_1.TIZEN_MOBILE:
            case constants_1.TIZEN_WATCH:
                return [2 /*return*/, (0, exports.checkAndConfigureTizenSdks)(c)];
            case constants_1.WEBOS:
                return [2 /*return*/, (0, exports.checkAndConfigureWebosSdks)(c)];
            default:
                return [2 /*return*/, true];
        }
        return [2 /*return*/];
    });
}); };
exports.checkAndConfigureSdks = checkAndConfigureSdks;
var _getCurrentSdkPath = function (c) { var _a, _b; return (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.sdks) === null || _b === void 0 ? void 0 : _b[constants_1.SDK_PLATFORMS[c.platform]]; };
var _isSdkInstalled = function (c) {
    (0, logger_1.logTask)('_isSdkInstalled');
    if (!constants_1.SDK_PLATFORMS[c.platform])
        return true;
    var sdkPath = _getCurrentSdkPath(c, c.platform);
    return (0, fileutils_1.fsExistsSync)((0, fileutils_1.getRealPath)(c, sdkPath));
};
var _findFolderWithFile = function (dir, fileToFind) {
    var opt = path_1.default.join(dir, fileToFind);
    if ((0, fileutils_1.fsExistsSync)(opt)) {
        return dir;
    }
    var foundDir;
    (0, fileutils_1.fsReaddirSync)(dir).forEach(function (subDirName) {
        // not a directory check
        if (!(0, fileutils_1.fsLstatSync)(subDirName).isDirectory())
            return;
        var subDir = path_1.default.join(dir, subDirName);
        var foundSubDir = _findFolderWithFile(subDir, fileToFind);
        if (foundSubDir) {
            foundDir = foundSubDir;
        }
    });
    return foundDir;
};
var _attemptAutoFix = function (c, sdkPlatform, sdkKey, traverseUntilFoundFile) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var locations, _a, ANDROID_SDK_HOME, ANDROID_SDK_ROOT, ANDROID_HOME, ANDROID_SDK_ENV, ANDROID_NDK_HOME, result, subResult, confirmSdk, confirm_1, e_1, setupInstance;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                (0, logger_1.logTask)('_attemptAutoFix');
                if (c.program.hosted) {
                    (0, logger_1.logInfo)('HOSTED Mode. Skipping SDK checks');
                    return [2 /*return*/, true];
                }
                locations = SDK_LOCATIONS[sdkPlatform];
                // try common Android SDK env variables
                if (sdkKey === constants_1.ANDROID_SDK) {
                    _a = process.env, ANDROID_SDK_HOME = _a.ANDROID_SDK_HOME, ANDROID_SDK_ROOT = _a.ANDROID_SDK_ROOT, ANDROID_HOME = _a.ANDROID_HOME, ANDROID_SDK_ENV = _a.ANDROID_SDK;
                    locations = locations.concat([ANDROID_SDK_HOME, ANDROID_SDK_ROOT, ANDROID_HOME, ANDROID_SDK_ENV]);
                }
                if (sdkKey === constants_1.ANDROID_NDK) {
                    ANDROID_NDK_HOME = process.env.ANDROID_NDK_HOME;
                    locations.push(ANDROID_NDK_HOME);
                }
                result = locations.find(function (v) { return (0, fileutils_1.fsExistsSync)(v); });
                if (result && traverseUntilFoundFile) {
                    subResult = _findFolderWithFile(result, traverseUntilFoundFile);
                    if (subResult) {
                        result = subResult;
                    }
                    else {
                        result = null;
                    }
                }
                if (!result) return [3 /*break*/, 7];
                (0, logger_1.logSuccess)("Found existing ".concat(c.platform, " SDK location at ").concat((0, logger_1.chalk)().white(result)));
                confirmSdk = true;
                if (!!c.program.ci) return [3 /*break*/, 2];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        type: 'confirm',
                        name: 'confirm',
                        message: 'Do you want to use it?'
                    })];
            case 1:
                confirm_1 = (_b.sent()).confirm;
                confirmSdk = confirm_1;
                _b.label = 2;
            case 2:
                if (!confirmSdk) return [3 /*break*/, 7];
                _b.label = 3;
            case 3:
                _b.trys.push([3, 5, , 6]);
                if (!c.files.workspace.config.sdks)
                    c.files.workspace.config.sdks = {};
                c.files.workspace.config.sdks[sdkKey] = result;
                (0, fileutils_1.writeFileSync)(c.paths.workspace.config, c.files.workspace.config);
                (0, configManager_1.generateBuildConfig)(c);
                return [4 /*yield*/, (0, exports.checkAndConfigureSdks)(c)];
            case 4:
                _b.sent();
                return [3 /*break*/, 6];
            case 5:
                e_1 = _b.sent();
                (0, logger_1.logError)(e_1);
                return [3 /*break*/, 6];
            case 6: return [2 /*return*/, true];
            case 7:
                (0, logger_1.logTask)("_attemptAutoFix: no sdks found. searched at: ".concat(SDK_LOCATIONS[sdkPlatform].join(', ')));
                setupInstance = (0, setupManager_1.default)(c);
                return [4 /*yield*/, setupInstance.askToInstallSDK(sdkPlatform)];
            case 8:
                _b.sent();
                (0, configManager_1.generateBuildConfig)(c);
                return [2 /*return*/, true];
        }
    });
}); };
var checkSdk = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                (0, logger_1.logTask)('checkSdk');
                if (!!_isSdkInstalled(c)) return [3 /*break*/, 7];
                (0, logger_1.logWarning)("".concat(c.platform, " requires SDK to be installed. Your SDK path in ").concat((0, logger_1.chalk)().white(c.paths.workspace.config), " does not exist: ").concat((0, logger_1.chalk)().white(_getCurrentSdkPath(c))));
                _a = c.platform;
                switch (_a) {
                    case constants_1.ANDROID: return [3 /*break*/, 1];
                    case constants_1.ANDROID_TV: return [3 /*break*/, 1];
                    case constants_1.FIRE_TV: return [3 /*break*/, 1];
                    case constants_1.ANDROID_WEAR: return [3 /*break*/, 1];
                    case constants_1.TIZEN: return [3 /*break*/, 3];
                    case constants_1.TIZEN_MOBILE: return [3 /*break*/, 3];
                    case constants_1.TIZEN_WATCH: return [3 /*break*/, 3];
                    case constants_1.WEBOS: return [3 /*break*/, 4];
                }
                return [3 /*break*/, 5];
            case 1: return [4 /*yield*/, _attemptAutoFix(c, 'android', constants_1.ANDROID_SDK)];
            case 2:
                _b.sent();
                return [2 /*return*/, _attemptAutoFix(c, 'android-ndk', constants_1.ANDROID_NDK, 'source.properties')];
            case 3: return [2 /*return*/, _attemptAutoFix(c, 'tizen', constants_1.TIZEN_SDK)];
            case 4: return [2 /*return*/, _attemptAutoFix(c, 'webos', constants_1.WEBOS_SDK)];
            case 5: return [2 /*return*/, true];
            case 6: return [3 /*break*/, 9];
            case 7: return [4 /*yield*/, (0, exports.checkAndConfigureAndroidSdks)(c)];
            case 8:
                _b.sent();
                _b.label = 9;
            case 9: return [2 /*return*/, true];
        }
    });
}); };
exports.checkSdk = checkSdk;
//# sourceMappingURL=installer.js.map