"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var shelljs_1 = (0, tslib_1.__importDefault)(require("shelljs"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var os_1 = (0, tslib_1.__importDefault)(require("os"));
var exec_1 = require("../systemManager/exec");
var constants_1 = require("../constants");
var taskManager_1 = require("../taskManager");
var fileutils_1 = require("../systemManager/fileutils");
var config_1 = (0, tslib_1.__importDefault)(require("./config"));
var config_2 = (0, tslib_1.__importDefault)(require("../configManager/config"));
var logger_1 = require("../systemManager/logger");
var BasePlatformSetup = /** @class */ (function () {
    function BasePlatformSetup(os, c) {
        // eslint-disable-next-line no-param-reassign
        if (!c)
            c = config_2.default.getConfig();
        var paths = c.paths;
        this.os = os;
        this.c = c;
        this.globalConfigPath = paths.workspace.config;
        this.availableDownloader = null;
        this.androidSdkLocation = (0, fileutils_1.replaceHomeFolder)('~/Android');
        var sdksToInstallX86 = '"build-tools;28.0.3" "emulator" "extras;android;m2repository" "extras;google;m2repository" "patcher;v4" "platform-tools" "platforms;android-28" "sources;android-28" "system-images;android-28;google_apis_playstore;x86" "tools"';
        var sdksToInstallAppleSilicon = '"build-tools;30.0.3" "emulator" "extras;android;m2repository" "extras;google;m2repository" "patcher;v4" "platform-tools" "platforms;android-30" "sources;android-30" "system-images;android-30;google_apis_playstore;arm64-v8a" "tools"';
        this.sdksToInstall = os_1.default.arch() === 'arm64' ? sdksToInstallAppleSilicon : sdksToInstallX86;
    }
    BasePlatformSetup.prototype.checkPrereqs = function () {
        (0, logger_1.logInfo)("Platform ".concat(this.os));
        (0, logger_1.logInfo)('Checking if wget or curl is installed');
        if ((0, exec_1.commandExistsSync)('wget')) {
            this.availableDownloader = 'wget';
        }
        else if ((0, exec_1.commandExistsSync)('curl')) {
            this.availableDownloader = 'curl';
        }
    };
    BasePlatformSetup.prototype.installPrereqs = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                // to be overwritten
                return [2 /*return*/, true];
            });
        });
    };
    BasePlatformSetup.prototype.postInstall = function (sdk) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var location_1;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(sdk === 'android')) return [3 /*break*/, 3];
                        location_1 = config_1.default.android.location;
                        return [4 /*yield*/, (0, fileutils_1.updateConfigFile)({ androidSdk: location_1 }, this.globalConfigPath)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, (0, taskManager_1.executeTask)(this.c, constants_1.TASK_WORKSPACE_CONFIGURE)];
                    case 2:
                        _a.sent(); // trigger the configure to update the paths for clis
                        _a.label = 3;
                    case 3:
                        if (!(sdk === 'tizen')) return [3 /*break*/, 6];
                        return [4 /*yield*/, (0, fileutils_1.updateConfigFile)({ tizenSdk: this.tizenSdkPath }, this.globalConfigPath)];
                    case 4:
                        _a.sent();
                        return [4 /*yield*/, (0, taskManager_1.executeTask)(this.c, constants_1.TASK_WORKSPACE_CONFIGURE)];
                    case 5:
                        _a.sent(); // trigger the configure to update the paths for clis
                        _a.label = 6;
                    case 6:
                        if (!(sdk === 'webos')) return [3 /*break*/, 9];
                        return [4 /*yield*/, (0, fileutils_1.updateConfigFile)({ webosSdk: this.webosSdkPath }, this.globalConfigPath)];
                    case 7:
                        _a.sent();
                        return [4 /*yield*/, (0, taskManager_1.executeTask)(this.c, constants_1.TASK_WORKSPACE_CONFIGURE)];
                    case 8:
                        _a.sent(); // trigger the configure to update the paths for clis
                        _a.label = 9;
                    case 9: return [2 /*return*/];
                }
            });
        });
    };
    BasePlatformSetup.prototype.downloadSdk = function (sdk) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var downloader, aditionalArguments, locationArgument, command;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        downloader = this.availableDownloader;
                        if (!downloader)
                            throw new Error('Wget or cURL not installed!');
                        (0, logger_1.logDebug)("Downloading ".concat(sdk, " SDK to ").concat(config_1.default[sdk].downloadLocation, " using ").concat(downloader));
                        // remove the file if existing first
                        return [4 /*yield*/, shelljs_1.default.rm(config_1.default[sdk].downloadLocation)];
                    case 1:
                        // remove the file if existing first
                        _a.sent();
                        if (downloader === 'wget') {
                            aditionalArguments = '-q';
                            locationArgument = (0, fileutils_1.replaceHomeFolder)('-P ~/');
                        }
                        if (downloader === 'curl') {
                            aditionalArguments = '-#';
                            locationArgument = "--output ".concat(config_1.default[sdk].downloadLocation);
                        }
                        command = "".concat(downloader, " ").concat(aditionalArguments, " ").concat(config_1.default[sdk].sdkUrl, " ").concat(locationArgument);
                        (0, logger_1.logDebug)('Running', command);
                        (0, logger_1.logInfo)("Downloading ".concat(sdk, " SDK..."));
                        return [4 /*yield*/, shelljs_1.default.exec(command)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    BasePlatformSetup.prototype.unzipSdk = function (sdk) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        (0, logger_1.logDebug)("Unzipping from ".concat(config_1.default[sdk].downloadLocation, " to ").concat(config_1.default[sdk].location));
                        if (!(0, exec_1.commandExistsSync)('unzip')) {
                            throw new Error('unzip is not installed');
                        }
                        return [4 /*yield*/, shelljs_1.default.exec("unzip -qq -o ".concat(config_1.default[sdk].downloadLocation, " -d ").concat(config_1.default[sdk].location))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    BasePlatformSetup.prototype.installSdksAndEmulator = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        (0, logger_1.logTask)('installSdksAndEmulator');
                        (0, logger_1.logDebug)('Accepting licenses');
                        return [4 /*yield*/, shelljs_1.default.exec("yes | ".concat(config_1.default.android.location, "/tools/bin/sdkmanager --licenses > /dev/null"))];
                    case 1:
                        _a.sent();
                        (0, logger_1.logDebug)('Installing SDKs', this.sdksToInstall);
                        return [4 /*yield*/, shelljs_1.default.exec("".concat(config_1.default.android.location, "/tools/bin/sdkmanager ").concat(this.sdksToInstall, " > /dev/null"))];
                    case 2:
                        _a.sent();
                        (0, logger_1.logSuccess)("SDK succefully installed at: ".concat(config_1.default.android.location));
                        return [2 /*return*/];
                }
            });
        });
    };
    BasePlatformSetup.prototype.installSdk = function (sdk, skipPrereq) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var _a, _b;
            return (0, tslib_1.__generator)(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        (0, logger_1.logTask)("installSdk:".concat(sdk));
                        !skipPrereq && this.checkPrereqs();
                        _a = !skipPrereq;
                        if (!_a) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.installPrereqs()];
                    case 1:
                        _a = (_c.sent());
                        _c.label = 2;
                    case 2:
                        _a;
                        _b = sdk;
                        switch (_b) {
                            case 'android': return [3 /*break*/, 3];
                            case 'tizen': return [3 /*break*/, 7];
                            case 'webos': return [3 /*break*/, 9];
                            case 'fastlane': return [3 /*break*/, 11];
                            case 'docker': return [3 /*break*/, 13];
                            case 'aws': return [3 /*break*/, 15];
                        }
                        return [3 /*break*/, 17];
                    case 3: return [4 /*yield*/, this.downloadSdk(sdk)];
                    case 4:
                        _c.sent();
                        return [4 /*yield*/, this.unzipSdk(sdk)];
                    case 5:
                        _c.sent();
                        return [4 /*yield*/, this.installSdksAndEmulator()];
                    case 6:
                        _c.sent();
                        return [3 /*break*/, 18];
                    case 7: return [4 /*yield*/, this.installTizenSdk()];
                    case 8:
                        _c.sent();
                        return [3 /*break*/, 18];
                    case 9: return [4 /*yield*/, this.installWebosSdk()];
                    case 10:
                        _c.sent();
                        return [3 /*break*/, 18];
                    case 11: return [4 /*yield*/, this.installFastlane()];
                    case 12:
                        _c.sent();
                        return [3 /*break*/, 18];
                    case 13: return [4 /*yield*/, this.installDocker()];
                    case 14:
                        _c.sent();
                        return [3 /*break*/, 18];
                    case 15: return [4 /*yield*/, this.installAws()];
                    case 16:
                        _c.sent();
                        return [3 /*break*/, 18];
                    case 17: return [3 /*break*/, 18];
                    case 18:
                        this.postInstall(sdk);
                        return [2 /*return*/];
                }
            });
        });
    };
    BasePlatformSetup.prototype.installTizenSdk = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                // to be overwritten
                (0, logger_1.logError)('Install webos sdk not supported yet. Follow https://developer.tizen.org/development/tizen-studio/download to install it manually');
                return [2 /*return*/, true];
            });
        });
    };
    BasePlatformSetup.prototype.installWebosSdk = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                // to be overwritten
                (0, logger_1.logError)('Install webos sdk not supported yet. Follow http://webostv.developer.lge.com/sdk/installation/ to install it manually');
                return [2 /*return*/, true];
            });
        });
    };
    BasePlatformSetup.prototype.installFastlane = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                // to be overwritten
                (0, logger_1.logError)('Install fastlane not supported yet. Follow https://docs.fastlane.tools/getting-started/ios/setup/ to install it manually');
                return [2 /*return*/, true];
            });
        });
    };
    BasePlatformSetup.prototype.installDocker = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                // to be overwritten
                (0, logger_1.logError)('Install docker not supported yet');
                return [2 /*return*/, true];
            });
        });
    };
    BasePlatformSetup.prototype.installAws = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                // to be overwritten
                (0, logger_1.logError)('Install aws not supported yet. Follow https://docs.aws.amazon.com/cli/latest/userguide/install-macos.html to install it manually (version 1 required)');
                return [2 /*return*/, true];
            });
        });
    };
    BasePlatformSetup.prototype.askToInstallSDK = function (sdk) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var sdkInstall, response;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.c.program.ci) return [3 /*break*/, 2];
                        return [4 /*yield*/, inquirer_1.default.prompt([
                                {
                                    name: 'sdkInstall',
                                    type: 'confirm',
                                    message: "Do you want to install ".concat(sdk, " SDK?")
                                }
                            ])];
                    case 1:
                        response = _a.sent();
                        // eslint-disable-next-line prefer-destructuring
                        sdkInstall = response.sdkInstall;
                        _a.label = 2;
                    case 2:
                        if (!(this.c.program.ci || sdkInstall)) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.installSdk(sdk, ['fastlane', 'docker'].includes(sdk))];
                    case 3:
                        _a.sent(); // no prereqs needed for fastlane
                        return [3 /*break*/, 5];
                    case 4: throw new Error("You can't run the project on this platform without ".concat(sdk, " sdk installed"));
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    return BasePlatformSetup;
}());
exports.default = BasePlatformSetup;
//# sourceMappingURL=base.js.map