"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var shelljs_1 = (0, tslib_1.__importDefault)(require("shelljs"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var exec_1 = require("../systemManager/exec");
var logger_1 = require("../systemManager/logger");
var base_1 = (0, tslib_1.__importDefault)(require("./base"));
var fileutils_1 = require("../systemManager/fileutils");
var config_1 = (0, tslib_1.__importDefault)(require("./config"));
var constants_1 = require("../constants");
var LinuxPlatformSetup = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(LinuxPlatformSetup, _super);
    function LinuxPlatformSetup(c) {
        return _super.call(this, 'linux', c) || this;
    }
    LinuxPlatformSetup.prototype.installSoftware = function (software) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(0, exec_1.commandExistsSync)('apt-get')) return [3 /*break*/, 2];
                        return [4 /*yield*/, shelljs_1.default.exec("apt-get -qq update && apt-get install ".concat(software, " -y > /dev/null"))];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: 
                    // @todo also treat other linux flavours
                    return [2 /*return*/, true];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.installPrereqs = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.availableDownloader) return [3 /*break*/, 2];
                        (0, logger_1.logInfo)("You don't have wget or curl installed. We'll install wget for you");
                        return [4 /*yield*/, this.installSoftware('wget')];
                    case 1:
                        _a.sent();
                        this.availableDownloader = 'wget';
                        _a.label = 2;
                    case 2:
                        if (!!(0, exec_1.commandExistsSync)('unzip')) return [3 /*break*/, 4];
                        (0, logger_1.logInfo)("You don't have unzip installed. We'll install it for you");
                        return [4 /*yield*/, this.installSoftware('unzip')];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        if (!!(0, exec_1.commandExistsSync)('javac')) return [3 /*break*/, 6];
                        (0, logger_1.logInfo)("You don't have java installed. We'll install it for you");
                        return [4 /*yield*/, this.installSoftware('openjdk-8-jdk')];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6: return [2 /*return*/, true];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.postInstall = function (sdk) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var location_1, ANDROID_SDK;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(sdk === 'android')) return [3 /*break*/, 2];
                        location_1 = config_1.default.android.location;
                        (0, logger_1.logDebug)("Updating ".concat(this.globalConfigPath, " with ").concat(JSON.stringify({
                            androidSdk: location_1
                        })));
                        return [4 /*yield*/, (0, fileutils_1.updateConfigFile)({ androidSdk: location_1 }, this.globalConfigPath)];
                    case 1:
                        _a.sent();
                        // @todo find a more elegant way to update this
                        this.c.files.workspace.config.sdks.ANDROID_SDK = location_1;
                        ANDROID_SDK = this.c.files.workspace.config.sdks.ANDROID_SDK;
                        this.c.cli[constants_1.CLI_ANDROID_EMULATOR] = (0, fileutils_1.getRealPath)(this.c, path_1.default.join(ANDROID_SDK, 'emulator/emulator'));
                        this.c.cli[constants_1.CLI_ANDROID_ADB] = (0, fileutils_1.getRealPath)(this.c, path_1.default.join(ANDROID_SDK, 'platform-tools/adb'));
                        this.c.cli[constants_1.CLI_ANDROID_AVDMANAGER] = (0, fileutils_1.getRealPath)(this.c, path_1.default.join(ANDROID_SDK, 'tools/bin/avdmanager'));
                        this.c.cli[constants_1.CLI_ANDROID_SDKMANAGER] = (0, fileutils_1.getRealPath)(this.c, path_1.default.join(ANDROID_SDK, 'tools/bin/sdkmanager'));
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    return LinuxPlatformSetup;
}(base_1.default));
exports.default = LinuxPlatformSetup;
//# sourceMappingURL=linux.js.map