"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var shelljs_1 = (0, tslib_1.__importDefault)(require("shelljs"));
var get_installed_path_1 = require("get-installed-path");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var child_process_1 = require("child_process");
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var exec_1 = require("../systemManager/exec");
var logger_1 = require("../systemManager/logger");
var fileutils_1 = require("../systemManager/fileutils");
var base_1 = (0, tslib_1.__importDefault)(require("./base"));
var config_1 = (0, tslib_1.__importDefault)(require("./config"));
var LinuxPlatformSetup = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(LinuxPlatformSetup, _super);
    function LinuxPlatformSetup(c) {
        var _this = _super.call(this, 'win32', c) || this;
        _this.scoopInstalled = false;
        return _this;
    }
    LinuxPlatformSetup.prototype.checkPrereqs = function () {
        (0, logger_1.logInfo)("Platform ".concat(this.os));
        (0, logger_1.logInfo)('Checking wget is installed');
        if ((0, exec_1.commandExistsSync)('wget')) {
            this.availableDownloader = 'wget';
        }
        // check if scoop is installed
        if ((0, exec_1.commandExistsSync)('scoop')) {
            this.scoopInstalled = true;
        }
    };
    LinuxPlatformSetup.prototype.installSoftware = function (software) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, shelljs_1.default.exec((0, fileutils_1.replaceHomeFolder)("~/scoop/shims/scoop install ".concat(software)))];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.reloadPathEnv()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, true];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.addScoopBucket = function (bucket) {
        return shelljs_1.default.exec((0, fileutils_1.replaceHomeFolder)("~/scoop/shims/scoop bucket add ".concat(bucket)));
    };
    LinuxPlatformSetup.prototype.reloadPathEnv = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, shelljs_1.default.exec("".concat((0, get_installed_path_1.getInstalledPathSync)('rnv'), "/scripts/resetPath.vbs"))];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, shelljs_1.default.exec('%TEMP%/resetvars.bat')];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, true];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.installPrereqs = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.scoopInstalled) return [3 /*break*/, 3];
                        (0, logger_1.logInfo)('Installing Scoop...');
                        return [4 /*yield*/, shelljs_1.default.exec("powershell -executionpolicy remotesigned \"& \"\"".concat((0, get_installed_path_1.getInstalledPathSync)('rnv'), "/scripts/installPackageManager.ps1\"\"\""))];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.reloadPathEnv()];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        if (!!this.availableDownloader) return [3 /*break*/, 5];
                        (0, logger_1.logInfo)("You don't have wget or curl installed. We'll install wget for you");
                        return [4 /*yield*/, this.installSoftware('wget')];
                    case 4:
                        _a.sent();
                        this.availableDownloader = 'wget';
                        _a.label = 5;
                    case 5:
                        if (!!(0, exec_1.commandExistsSync)('unzip')) return [3 /*break*/, 7];
                        (0, logger_1.logInfo)("You don't have unzip installed. We'll install it for you");
                        return [4 /*yield*/, this.installSoftware('unzip')];
                    case 6:
                        _a.sent();
                        _a.label = 7;
                    case 7:
                        if (!!(0, exec_1.commandExistsSync)('javac')) return [3 /*break*/, 11];
                        (0, logger_1.logInfo)("You don't have java installed. We'll install it for you");
                        return [4 /*yield*/, this.installSoftware('shellcheck')];
                    case 8:
                        _a.sent();
                        return [4 /*yield*/, this.addScoopBucket('java')];
                    case 9:
                        _a.sent();
                        return [4 /*yield*/, this.installSoftware('ojdkbuild8')];
                    case 10:
                        _a.sent();
                        _a.label = 11;
                    case 11: return [2 /*return*/, true];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.installSdksAndEmulator = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        (0, logger_1.logDebug)('Accepting licenses');
                        return [4 /*yield*/, (0, exec_1.executeAsync)({}, "".concat(this.androidSdkLocation, "/tools/bin/sdkmanager.bat --licenses"))];
                    case 1:
                        _a.sent();
                        (0, logger_1.logDebug)('Installing SDKs', this.sdksToInstall);
                        return [4 /*yield*/, (0, exec_1.executeAsync)({}, "".concat(this.androidSdkLocation, "/tools/bin/sdkmanager.bat ").concat(this.sdksToInstall))];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.installTizenSdk = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var downloadDir, res;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        downloadDir = config_1.default.tizen.downloadLocation.split('/');
                        downloadDir.pop();
                        downloadDir = downloadDir.join('/');
                        (0, logger_1.logInfo)("Opening ".concat(downloadDir, ". Please install the SDK then continue after it finished installing."));
                        (0, child_process_1.exec)("start \"\" \"".concat(downloadDir, "\""));
                        return [4 /*yield*/, inquirer_1.default.prompt({
                                type: 'input',
                                name: 'sdkPath',
                                message: "Where did you install the SDK? (if you haven't changed the default just press enter)",
                                default: 'C:\\tizen-studio',
                                validate: function (value) {
                                    if ((0, fileutils_1.fsExistsSync)(value))
                                        return true;
                                    return 'Path does not exist';
                                }
                            })];
                    case 1:
                        res = _a.sent();
                        return [4 /*yield*/, inquirer_1.default.prompt({
                                type: 'confirm',
                                name: 'toolsInstalled',
                                message: 'Please open Package Manager and install: Tizen SDK Tools (Main SDK), TV Extensions-* (Extension SDK). Continue after you finished installing them.',
                                validate: function () {
                                    return ((0, fileutils_1.fsExistsSync)(path_1.default.join(res.sdkPath, 'tools/ide/bin/tizen.bat')) || 'This does not look like a Tizen SDK path');
                                }
                            })];
                    case 2:
                        _a.sent();
                        this.tizenSdkPath = res.sdkPath;
                        return [2 /*return*/];
                }
            });
        });
    };
    LinuxPlatformSetup.prototype.installWebosSdk = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var downloadLink, res;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        downloadLink = config_1.default.webos.downloadLink;
                        (0, logger_1.logInfo)("Opening ".concat(downloadLink, ". Please download and install the SDK then continue after it finished installing.\nMake sure you also install the CLI and Emulator components"));
                        (0, child_process_1.exec)("".concat(exec_1.openCommand, " ").concat(downloadLink));
                        return [4 /*yield*/, inquirer_1.default.prompt({
                                type: 'input',
                                name: 'sdkPath',
                                message: "Where did you install the SDK? (if you haven't changed the default just press enter)",
                                default: 'C:\\webOS_TV_SDK',
                                validate: function (value) {
                                    if ((0, fileutils_1.fsExistsSync)(value))
                                        return true;
                                    return 'Path does not exist';
                                }
                            })];
                    case 1:
                        res = _a.sent();
                        return [4 /*yield*/, inquirer_1.default.prompt({
                                type: 'confirm',
                                name: 'toolsInstalled',
                                message: 'Are the CLI and Emulator components installed?',
                                validate: function () {
                                    return ((0, fileutils_1.fsExistsSync)(path_1.default.join(res.sdkPath, 'tools/ide/bin/tizen.bat')) || 'This does not look like a Tizen SDK path');
                                }
                            })];
                    case 2:
                        _a.sent();
                        this.webosSdkPath = res.sdkPath;
                        return [2 /*return*/];
                }
            });
        });
    };
    return LinuxPlatformSetup;
}(base_1.default));
exports.default = LinuxPlatformSetup;
//# sourceMappingURL=windows.js.map