"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var integrations_1 = require("@sentry/integrations");
var node_machine_id_1 = require("node-machine-id");
var axios_1 = (0, tslib_1.__importDefault)(require("axios"));
var os_1 = (0, tslib_1.__importDefault)(require("os"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var config_1 = (0, tslib_1.__importDefault)(require("../configManager/config"));
var package_json_1 = (0, tslib_1.__importDefault)(require("../../../package.json"));
var constants_1 = require("../constants");
// deal with useless duplicate errors on sentry because of different error texts
var sanitizeError = function (err) {
    if (err) {
        if (err.includes('file if your SDK path is correct.')) {
            return err.toLowerCase().split('. check your ')[0];
        }
        if (err.includes('AppConfig error - ')) {
            return err.split(' - ')[0];
        }
    }
    return err;
};
var Redash = /** @class */ (function () {
    function Redash() {
    }
    Redash.prototype.captureEvent = function (e) {
        var defaultProps = {
            fingerprint: (0, node_machine_id_1.machineIdSync)(),
            os: os_1.default.platform(),
            rnvVersion: package_json_1.default.version
        };
        return axios_1.default
            .post(constants_1.REDASH_URL, (0, tslib_1.__assign)((0, tslib_1.__assign)({}, e), defaultProps), {
            headers: {
                'x-api-key': constants_1.REDASH_KEY
            }
        })
            .catch(function () { return true; });
    };
    return Redash;
}());
var Analytics = /** @class */ (function () {
    function Analytics() {
        this.errorFixer = null;
        this.knowItAll = null;
    }
    Analytics.prototype.initialize = function () {
        if (config_1.default.isAnalyticsEnabled) {
            // ERROR HANDLING
            // eslint-disable-next-line global-require
            this.errorFixer = require('@sentry/node');
            this.errorFixer.init({
                dsn: constants_1.SENTRY_ENDPOINT,
                release: "rnv@".concat(package_json_1.default.version),
                integrations: [
                    new integrations_1.RewriteFrames({
                        root: '/',
                        iteratee: function (frame) {
                            if (frame.filename.includes("rnv".concat(path_1.default.sep, "dist").concat(path_1.default.sep))
                                || frame.filename.includes("rnv".concat(path_1.default.sep, "src").concat(path_1.default.sep))) {
                                if (frame.filename.includes("rnv".concat(path_1.default.sep, "dist").concat(path_1.default.sep))) {
                                    frame.filename = frame.filename.split("rnv".concat(path_1.default.sep, "dist").concat(path_1.default.sep))[1];
                                }
                                else {
                                    frame.filename = frame.filename.split("rnv".concat(path_1.default.sep, "src").concat(path_1.default.sep))[1];
                                }
                            }
                            else if (frame.filename.includes("".concat(path_1.default.sep, "node_modules").concat(path_1.default.sep))) {
                                frame.filename = "node_modules/".concat(frame.filename.split("".concat(path_1.default.sep, "node_modules").concat(path_1.default.sep))[1]);
                            }
                            return frame;
                        }
                    })
                ]
            });
            // EVENT HANDLING
            this.knowItAll = new Redash();
        }
    };
    Analytics.prototype.captureException = function (e, context) {
        var _this = this;
        if (context === void 0) { context = {}; }
        if (config_1.default.isAnalyticsEnabled && this.errorFixer) {
            this.errorFixer.withScope(function (scope) {
                var _a = context.extra, extra = _a === void 0 ? {} : _a, _b = context.tags, tags = _b === void 0 ? {} : _b;
                scope.setTags((0, tslib_1.__assign)((0, tslib_1.__assign)({}, tags), { os: os_1.default.platform() }));
                scope.setExtras((0, tslib_1.__assign)((0, tslib_1.__assign)({}, extra), { fingerprint: (0, node_machine_id_1.machineIdSync)() }));
                if (e instanceof Error) {
                    _this.errorFixer.captureException(e);
                }
                else {
                    _this.errorFixer.captureException(new Error(sanitizeError(e)));
                }
            });
        }
    };
    Analytics.prototype.captureEvent = function (e) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                if (config_1.default.isAnalyticsEnabled && this.knowItAll) {
                    return [2 /*return*/, this.knowItAll.captureEvent(e)];
                }
                return [2 /*return*/, true];
            });
        });
    };
    Analytics.prototype.teardown = function () {
        var _this = this;
        return new Promise(function (resolve) {
            if (config_1.default.isAnalyticsEnabled && _this.errorFixer) {
                var client = _this.errorFixer.getCurrentHub().getClient();
                if (client) {
                    return client.close(2000).then(resolve);
                }
                return resolve();
            }
            return resolve();
        });
    };
    return Analytics;
}());
exports.default = new Analytics();
//# sourceMappingURL=analytics.js.map