"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkCrypto = exports.getEnvVar = exports.getEnvExportCmd = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var logger_1 = require("./logger");
var utils_1 = require("./utils");
var fileutils_1 = require("./fileutils");
var constants_1 = require("../constants");
var taskManager_1 = require("../taskManager");
var getEnvExportCmd = function (envVar, key) {
    if (utils_1.isSystemWin) {
        return "".concat((0, logger_1.chalk)().white("setx ".concat(envVar, " \"").concat(key, "\"")));
    }
    return "".concat((0, logger_1.chalk)().white("export ".concat(envVar, "=\"").concat(key, "\"")));
};
exports.getEnvExportCmd = getEnvExportCmd;
var getEnvVar = function (c) {
    var p1 = c.paths.workspace.dir
        .split('/')
        .pop()
        .replace('.', '');
    var p2 = c.files.project.package.name
        .replace('@', '')
        .replace('/', '_')
        .replace(/-/g, '_');
    var envVar = "CRYPTO_".concat(p1, "_").concat(p2).toUpperCase();
    (0, logger_1.logDebug)('encrypt looking for env var:', envVar);
    return envVar;
};
exports.getEnvVar = getEnvVar;
var checkCrypto = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var sourceRaw, destRaw, source, tsProjectPath, wsPath, tsWorkspacePath, tsWorkspace, tsProject;
    var _a, _b, _c, _d, _e, _f, _g, _h;
    return (0, tslib_1.__generator)(this, function (_j) {
        switch (_j.label) {
            case 0:
                (0, logger_1.logTask)('checkCrypto');
                if (c.program.ci || ((_b = (_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.crypto) === null || _b === void 0 ? void 0 : _b.optional))
                    return [2 /*return*/];
                sourceRaw = (_e = (_d = (_c = c.files.project.config) === null || _c === void 0 ? void 0 : _c.crypto) === null || _d === void 0 ? void 0 : _d.decrypt) === null || _e === void 0 ? void 0 : _e.source;
                destRaw = (_h = (_g = (_f = c.files.project.config) === null || _f === void 0 ? void 0 : _f.crypto) === null || _g === void 0 ? void 0 : _g.encrypt) === null || _h === void 0 ? void 0 : _h.dest;
                if (!destRaw) return [3 /*break*/, 4];
                if (!(sourceRaw && destRaw)) return [3 /*break*/, 4];
                source = "".concat((0, fileutils_1.getRealPath)(c, sourceRaw, 'decrypt.source'));
                tsProjectPath = "".concat(source, ".timestamp");
                wsPath = path_1.default.join(c.paths.workspace.dir, c.files.project.package.name);
                tsWorkspacePath = path_1.default.join(wsPath, 'timestamp');
                if (!!(0, fileutils_1.fsExistsSync)(source)) return [3 /*break*/, 1];
                (0, logger_1.logWarning)("This project uses encrypted files but you don't have them installed");
                return [3 /*break*/, 4];
            case 1:
                tsWorkspace = 0;
                tsProject = 0;
                if ((0, fileutils_1.fsExistsSync)(tsWorkspacePath)) {
                    tsWorkspace = parseInt((0, fileutils_1.fsReadFileSync)(tsWorkspacePath).toString(), 10);
                }
                if ((0, fileutils_1.fsExistsSync)(tsProjectPath)) {
                    tsProject = parseInt((0, fileutils_1.fsReadFileSync)(tsProjectPath).toString(), 10);
                }
                if (!(tsProject > tsWorkspace)) return [3 /*break*/, 3];
                (0, logger_1.logWarning)("Your ".concat(tsWorkspacePath, " is out of date.\nproject timestamp: ").concat((0, logger_1.chalk)().grey("".concat(tsProject, " - ").concat(new Date(tsProject))), "\nworkspace timestamp: ").concat((0, logger_1.chalk)().grey("".concat(tsWorkspace, " - ").concat(new Date(tsWorkspace))), "\nyou should run decrypt"));
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_CRYPTO_DECRYPT, parentTask, originTask)];
            case 2:
                _j.sent();
                return [2 /*return*/];
            case 3:
                if (tsProject < tsWorkspace) {
                    (0, logger_1.logWarning)("Your ".concat(tsWorkspacePath, " is newer than your project one."));
                }
                _j.label = 4;
            case 4: return [2 /*return*/];
        }
    });
}); };
exports.checkCrypto = checkCrypto;
//# sourceMappingURL=crypto.js.map