"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixPackageObject = exports.fixPackageJson = void 0;
var fileutils_1 = require("./fileutils");
var constants_1 = require("../constants");
var logger_1 = require("./logger");
var getSortedObject = function (obj) {
    if (obj !== null && typeof obj === 'object' && !Array.isArray(obj)) {
        var keys = Object.keys(obj).sort();
        var newObj_1 = {};
        var addedKeys_1 = {};
        keys.forEach(function (v) {
            if (!addedKeys_1[v]) {
                newObj_1[v] = obj[v];
                addedKeys_1[v] = true;
            }
        });
        return newObj_1;
    }
    if (Array.isArray(obj)) {
        return obj.sort();
    }
    return obj;
};
var checkForDuplicates = function (arr) {
    var dupCheck = {};
    arr.forEach(function (v) {
        if (v) {
            Object.keys(v).forEach(function (k) {
                if (dupCheck[k]) {
                    (0, logger_1.logWarning)("Key ".concat((0, logger_1.chalk)().white(k), " is duplicated in your package.json"));
                }
                dupCheck[k] = true;
            });
        }
    });
};
var fixPackageJson = function (c, pkgPath) { return new Promise(function (resolve) {
    var pth = pkgPath || c.paths.project.package;
    var pp = (0, fileutils_1.readObjectSync)(pth);
    var output = fixPackageObject(pp);
    (0, fileutils_1.writeFileSync)(pth, output, 4);
    resolve();
}); };
exports.fixPackageJson = fixPackageJson;
var fixPackageObject = function (pp) {
    var output = {};
    var usedKeys = {};
    constants_1.PACKAGE_JSON_FILEDS.forEach(function (v) {
        if (pp[v] !== null) {
            output[v] = getSortedObject(pp[v]);
            usedKeys[v] = true;
        }
    });
    Object.keys(pp).forEach(function (k) {
        if (!usedKeys[k]) {
            output[k] = pp[k];
        }
    });
    checkForDuplicates([pp.dependencies, pp.devDependencies]);
    return output;
};
exports.fixPackageObject = fixPackageObject;
exports.default = {
    fixPackageJson: fixPackageJson,
    fixPackageObject: fixPackageObject
};
//# sourceMappingURL=doctor.js.map