"use strict";
/* eslint-disable no-control-regex */
/* eslint-disable no-bitwise */
/* eslint-disable no-await-in-loop */
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeTelnet = exports.openCommand = exports.commandExistsSync = exports.commandExists = exports.execCLI = exports.executeAsync = exports.parseErrorMessage = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var fs_1 = require("fs");
var execa_1 = (0, tslib_1.__importDefault)(require("execa"));
var client_1 = (0, tslib_1.__importDefault)(require("netcat/client"));
var ora_1 = (0, tslib_1.__importDefault)(require("../../cli/ora"));
var config_1 = (0, tslib_1.__importDefault)(require("../configManager/config"));
var logger_1 = require("./logger");
var fileutils_1 = require("./fileutils");
var utils_1 = require("./utils");
var _a = require('child_process'), exec = _a.exec, execSync = _a.execSync;
/**
 *
 * Also accepts the Node's child_process exec/spawn options
 *
 * @typedef {Object} Opts
 * @property {Object} privateParams - private params that will be masked in the logs
 * @property {Boolean} silent - don't print anything
 * @property {Boolean} ignoreErrors - will print the loader but it will finish with a
 * checkmark regardless of the outcome. Also, it never throws a catch.
 * @property {Boolean} interactive - when you want to execute a command that requires user input
 *
 * Execute commands
 *
 * @param {String} command - command to be executed
 * @param {Opts} [opts={}] - the options for the command
 * @returns {Promise}
 *
 */
var _execute = function (c, command, opts) {
    var _a, _b, _d, _e, _f, _g;
    if (opts === void 0) { opts = {}; }
    var defaultOpts = {
        stdio: 'pipe',
        localDir: path_1.default.resolve('./node_modules/.bin'),
        preferLocal: true,
        all: true,
        maxErrorLength: (_a = c.program) === null || _a === void 0 ? void 0 : _a.maxErrorLength,
        mono: (_b = c.program) === null || _b === void 0 ? void 0 : _b.mono,
    };
    if (opts.interactive) {
        defaultOpts.silent = true;
        defaultOpts.stdio = 'inherit';
        defaultOpts.shell = true;
    }
    var mergedOpts = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, defaultOpts), opts);
    var env = opts.env && c.program.info ? Object.keys(opts.env).map(function (k) { return "".concat(k, "=").concat(opts.env[k]); }).join(' ') : null;
    var cleanCommand = command;
    var interval;
    var intervalTimer = 30000; // 30s
    var timer = intervalTimer;
    var privateMask = '*******';
    var cleanRawCmd = ((_d = opts.rawCommand) === null || _d === void 0 ? void 0 : _d.args) || [];
    if (Array.isArray(command))
        cleanCommand = command.join(' ');
    cleanCommand += cleanRawCmd.join(' ');
    var logMessage = cleanCommand;
    var privateParams = mergedOpts.privateParams;
    if (privateParams && Array.isArray(privateParams)) {
        logMessage = (0, utils_1.replaceOverridesInString)(command, privateParams, privateMask);
    }
    logMessage = "".concat(env ? "".concat(env, " ") : '').concat(logMessage);
    (0, logger_1.logDebug)("_execute: ".concat(logMessage));
    var silent = mergedOpts.silent, mono = mergedOpts.mono, maxErrorLength = mergedOpts.maxErrorLength, ignoreErrors = mergedOpts.ignoreErrors;
    var spinner = !silent && !mono && (0, ora_1.default)({ text: "Executing: ".concat(logMessage) }).start();
    if (opts.interactive) {
        (0, logger_1.logRaw)("".concat((0, logger_1.chalk)().green('✔'), " Executing: ").concat(logMessage, "\n"));
    }
    if (mono) {
        interval = setInterval(function () {
            (0, logger_1.logRaw)("Executing: ".concat(logMessage, " - ").concat(timer / 1000, "s"));
            timer += intervalTimer;
        }, intervalTimer);
    }
    var child;
    if (opts.rawCommand) {
        var args = opts.rawCommand.args;
        child = (0, execa_1.default)(command, args, mergedOpts);
    }
    else {
        child = execa_1.default.command(cleanCommand, mergedOpts);
    }
    var MAX_OUTPUT_LENGTH = 200;
    var printLastLine = function (buffer) {
        var text = Buffer.from(buffer)
            .toString()
            .trim();
        var lastLine = text.split('\n').pop();
        spinner.text = (0, utils_1.replaceOverridesInString)(lastLine.substring(0, MAX_OUTPUT_LENGTH), privateParams, privateMask);
        if (lastLine.length === MAX_OUTPUT_LENGTH)
            spinner.text += '...\n';
    };
    if (((_e = c.program) === null || _e === void 0 ? void 0 : _e.info) && ((_f = child === null || child === void 0 ? void 0 : child.stdout) === null || _f === void 0 ? void 0 : _f.pipe)) {
        child.stdout.pipe(process.stdout);
    }
    else if (spinner && ((_g = child === null || child === void 0 ? void 0 : child.stdout) === null || _g === void 0 ? void 0 : _g.on)) {
        child.stdout.on('data', printLastLine);
    }
    return child
        .then(function (res) {
        var _a;
        if ((_a = child === null || child === void 0 ? void 0 : child.stdout) === null || _a === void 0 ? void 0 : _a.off) {
            spinner && child.stdout.off('data', printLastLine);
        }
        !silent && !mono && spinner.succeed("Executing: ".concat(logMessage));
        (0, logger_1.logDebug)((0, utils_1.replaceOverridesInString)(res.all, privateParams, privateMask));
        interval && clearInterval(interval);
        // logDebug(res);
        return res.stdout;
    })
        .catch(function (err) {
        var _a;
        if ((_a = child === null || child === void 0 ? void 0 : child.stdout) === null || _a === void 0 ? void 0 : _a.off) {
            spinner && child.stdout.off('data', printLastLine);
        }
        if (!silent && !mono && !ignoreErrors) {
            spinner.fail("FAILED: ".concat(logMessage));
        } // parseErrorMessage will return false if nothing is found, default to previous implementation
        (0, logger_1.logDebug)((0, utils_1.replaceOverridesInString)(err.all, privateParams, privateMask));
        interval && clearInterval(interval);
        // logDebug(err);
        if (ignoreErrors && !silent && !mono) {
            spinner.succeed("Executing: ".concat(logMessage));
            return true;
        }
        var errMessage = (0, exports.parseErrorMessage)(err.all, maxErrorLength);
        if (!errMessage) {
            errMessage = '';
        }
        else {
            errMessage += '\n\n';
        }
        if (err.message) {
            errMessage += "".concat(err.message, "\n\n");
        }
        if (err.stderr) {
            errMessage += "".concat(err.stderr, "\n\n");
        }
        if (err.stack) {
            errMessage += "".concat(err.stack, "\n\n");
        }
        errMessage = (0, utils_1.replaceOverridesInString)(errMessage, privateParams, privateMask);
        return Promise.reject("COMMAND: \n\n".concat(logMessage, " \n\nFAILED with ERROR: \n\n").concat(errMessage)); // parseErrorMessage will return false if nothing is found, default to previous implementation
    });
};
/**
 *
 * Execute CLI command
 *
 * @param {Object} c - the trusty old c object
 * @param {String} cli - the cli to be executed
 * @param {String} command - the command to be executed
 * @param {Opts} [opts={}] - the options for the command
 * @returns {Promise}
 *
 */
var execCLI = function (c, cli, command, opts) {
    var _a;
    if (opts === void 0) { opts = {}; }
    if (!c.program) {
        return Promise.reject('You need to pass c object as first parameter to execCLI()');
    }
    var p = c.cli[cli];
    if (!(0, fileutils_1.fsExistsSync)(p)) {
        (0, logger_1.logDebug)("execCLI error: ".concat(cli, " | ").concat(command), '\nCLI Config:\n', c.cli, '\nSDK Config:\n', (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.sdks);
        return Promise.reject("Location of your cli ".concat((0, logger_1.chalk)().white(p), " does not exists. check your ").concat((0, logger_1.chalk)().white(c.paths.workspace.config), " file if your ").concat((0, logger_1.chalk)().white('sdks'), " paths are correct"));
    }
    return _execute(c, "".concat(p, " ").concat(command), (0, tslib_1.__assign)((0, tslib_1.__assign)({}, opts), { shell: true }));
};
exports.execCLI = execCLI;
/**
 *
 * Execute a plain command
 *
 * @param {String} command - the command to be executed
 * @param {Opts} [opts={}] - the options for the command
 * @returns {Promise}
 *
 */
var executeAsync = function (_c, _cmd, _opts) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var c, cmd, opts, result;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                c = _c;
                cmd = _cmd;
                opts = _opts;
                if (typeof c === 'string') {
                    opts = cmd;
                    cmd = c;
                    c = config_1.default.getConfig();
                }
                opts = opts || {};
                if (cmd.includes('npm') && process.platform === 'win32') {
                    cmd.replace('npm', 'npm.cmd');
                }
                return [4 /*yield*/, _execute(c, cmd, opts)];
            case 1:
                result = _a.sent();
                return [2 /*return*/, result];
        }
    });
}); };
exports.executeAsync = executeAsync;
/**
 *
 * Connect to a local telnet server and execute a command
 *
 * @param {Number|String} port - where do you want me to connect to?
 * @param {String} command - the command to be executed once I'm connected
 * @returns {Promise}
 *
 */
var executeTelnet = function (c, port, command) { return new Promise(function (resolve) {
    (0, logger_1.logDebug)("execTelnet: ".concat(port, " ").concat(command));
    try {
        var output_1 = '';
        var nc2_1 = new client_1.default();
        nc2_1.addr(c.runtime.localhost)
            .port(parseInt(port, 10))
            .connect()
            .send("".concat(command, "\n"));
        nc2_1.on('data', function (data) {
            var resp = Buffer.from(data).toString();
            output_1 += resp;
            if (output_1.includes('OK'))
                nc2_1.close();
        });
        nc2_1.on('close', function () { return resolve(output_1); });
    }
    catch (e) {
        (0, logger_1.logError)(e);
        resolve();
    }
}); };
exports.executeTelnet = executeTelnet;
// Legacy error parser
// export const parseErrorMessage = (text, maxErrorLength = 800) => {
//     const errors = [];
//     const toSearch = /(exception|error|fatal|\[!])/i;
//
//     const extractError = (t) => {
//         const errorFound = t ? t.search(toSearch) : -1;
//         if (errorFound === -1) return errors.length ? errors.join(' ') : false; // return the errors or false if we found nothing at all
//         const usefulString = t.substring(errorFound); // dump first part of the string that doesn't contain what we look for
//         let extractedError = usefulString.substring(0, maxErrorLength);
//         if (extractedError.length === maxErrorLength) extractedError += '...'; // add elipsis if string is bigger than maxErrorLength
//         errors.push(extractedError); // save the error
//         const newString = usefulString.substring(100); // dump everything we processed and continue
//         return extractError(newString);
//     };
//
//     return extractError(text);
// };
var parseErrorMessage = function (text, maxErrorLength) {
    if (maxErrorLength === void 0) { maxErrorLength = 800; }
    if (!text)
        return '';
    // Gradle specific
    var gradleFailIndex = text.indexOf('FAILURE: Build failed with an exception.');
    if (gradleFailIndex !== -1) {
        return text.substring(gradleFailIndex);
    }
    // NextJS Specific
    var nextFailIndex = text.indexOf('> Build error occurred');
    if (nextFailIndex !== -1) {
        return text.substring(nextFailIndex);
    }
    var toSearch = /(exception|error|fatal|\[!])/i;
    var arr = text.split('\n');
    var errFound = 0;
    arr = arr.filter(function (v) {
        if (v === '')
            return false;
        // Cleaner iOS reporting
        if (v.includes('-Werror')
            || v.includes('following modules are linked manually')
            || v.includes('warn ')
            || v.includes('note: ')
            || v.includes('warning: ')
            || v.includes('Could not find the following native modules')
            || v.includes('⚠️')
            || v.includes('/Applications/Xcode.app/Contents/Developer/Toolchains/XcodeDefault.xctoolchain')) {
            return false;
        }
        // Cleaner Android reporting
        if (v.includes('[DEBUG]')
            || v.includes('[INFO]')
            || v.includes('[LIFECYCLE]')
            || v.includes('[WARN]')
            || v.includes(':+HeapDumpOnOutOfMemoryError')
            || v.includes('.errors.')
            || v.includes('-exception-')
            || v.includes('error_prone_annotations')) {
            return false;
        }
        // Special Helper for iOS
        // if (v.endsWith('^')) {
        //     arr[i - 1] = chalk().red(arr[i - 1]);
        // }
        if (v.search(toSearch) !== -1) {
            errFound = 5;
            return true;
        }
        if (errFound > 0) {
            errFound -= 1;
            return true;
        }
        return false;
    });
    arr = arr.map(function (str) {
        var v = str.replace(/\s{2,}/g, ' ');
        var extractedError = v.substring(0, maxErrorLength);
        if (extractedError.length === maxErrorLength)
            extractedError += '...';
        return extractedError;
    });
    return arr.join('\n');
};
exports.parseErrorMessage = parseErrorMessage;
var isUsingWindows = process.platform === 'win32';
var fileNotExists = function (commandName, callback) {
    (0, fs_1.access)(commandName, fs_1.constants.F_OK, function (err) {
        callback(!err);
    });
};
var fileNotExistsSync = function (commandName) {
    try {
        (0, fs_1.accessSync)(commandName, fs_1.constants.F_OK);
        return false;
    }
    catch (e) {
        return true;
    }
};
var localExecutable = function (commandName, callback) {
    (0, fs_1.access)(commandName, fs_1.constants.F_OK | fs_1.constants.X_OK, function (err) {
        callback(null, !err);
    });
};
var localExecutableSync = function (commandName) {
    try {
        (0, fs_1.accessSync)(commandName, fs_1.constants.F_OK | fs_1.constants.X_OK);
        return true;
    }
    catch (e) {
        return false;
    }
};
var commandExistsUnix = function (commandName, cleanedCommandName, callback) {
    fileNotExists(commandName, function (isFile) {
        if (!isFile) {
            exec("command -v ".concat(cleanedCommandName, " 2>/dev/null")
                + " && { echo >&1 ".concat(cleanedCommandName, "; exit 0; }"), function (error, stdout) {
                callback(null, !!stdout);
            });
            return;
        }
        localExecutable(commandName, callback);
    });
};
var commandExistsWindows = function (commandName, cleanedCommandName, callback) {
    if (/[\x00-\x1f<>:"|?*]/.test(commandName)) {
        callback(null, false);
        return;
    }
    exec("where ".concat(cleanedCommandName), function (error) {
        if (error !== null) {
            callback(null, false);
        }
        else {
            callback(null, true);
        }
    });
};
var commandExistsUnixSync = function (commandName, cleanedCommandName) {
    if (fileNotExistsSync(commandName)) {
        try {
            var stdout = execSync("command -v ".concat(cleanedCommandName, " 2>/dev/null")
                + " && { echo >&1 ".concat(cleanedCommandName, "; exit 0; }"));
            return !!stdout;
        }
        catch (error) {
            return false;
        }
    }
    return localExecutableSync(commandName);
};
var commandExistsWindowsSync = function (commandName, cleanedCommandName) {
    if (/[\x00-\x1f<>:"|?*]/.test(commandName)) {
        return false;
    }
    try {
        var stdout = execSync("where ".concat(cleanedCommandName), { stdio: [] });
        return !!stdout;
    }
    catch (error) {
        return false;
    }
};
var cleanInput = function (_s) {
    var s = _s;
    if (/[^A-Za-z0-9_/:=-]/.test(s)) {
        s = "'".concat(s.replace(/'/g, "'\\''"), "'");
        s = s
            .replace(/^(?:'')+/g, '') // unduplicate single-quote at the beginning
            .replace(/\\'''/g, "\\'"); // remove non-escaped single-quote if there are enclosed between 2 escaped
    }
    return s;
};
if (isUsingWindows) {
    cleanInput = function (s) {
        var isPathName = /[\\]/.test(s);
        if (isPathName) {
            var dirname = "\"".concat(path_1.default.dirname(s), "\"");
            var basename = "\"".concat(path_1.default.basename(s), "\"");
            return "".concat(dirname, ":").concat(basename);
        }
        return "\"".concat(s, "\"");
    };
}
var commandExists = function (commandName, callback) {
    var cleanedCommandName = cleanInput(commandName);
    if (!callback && typeof Promise !== 'undefined') {
        return new Promise(function (resolve, reject) {
            commandExists(commandName, function (error, output) {
                if (output) {
                    resolve(commandName);
                }
                else {
                    reject(error);
                }
            });
        });
    }
    if (isUsingWindows) {
        commandExistsWindows(commandName, cleanedCommandName, callback);
    }
    else {
        commandExistsUnix(commandName, cleanedCommandName, callback);
    }
};
exports.commandExists = commandExists;
var commandExistsSync = function (commandName) {
    var cleanedCommandName = cleanInput(commandName);
    if (isUsingWindows) {
        return commandExistsWindowsSync(commandName, cleanedCommandName);
    }
    return commandExistsUnixSync(commandName, cleanedCommandName);
};
exports.commandExistsSync = commandExistsSync;
// eslint-disable-next-line no-nested-ternary
var openCommand = process.platform === 'darwin'
    ? 'open'
    : process.platform === 'win32'
        ? 'start'
        : 'xdg-open';
exports.openCommand = openCommand;
exports.default = {
    executeAsync: executeAsync,
    execCLI: execCLI,
    openCommand: openCommand,
    executeTelnet: executeTelnet,
    commandExistsSync: commandExistsSync
};
//# sourceMappingURL=exec.js.map