"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanEmptyFoldersRecursively = exports.getDirectories = exports.formatBytes = exports.loadFile = exports.getFileListSync = exports.replaceHomeFolder = exports.updateConfigFile = exports.mergeObjects = exports.sanitizeDynamicProps = exports.resolvePackage = exports.sanitizeDynamicRefs = exports.arrayMerge = exports.getRealPath = exports.updateObjectSync = exports.readObjectSync = exports.writeObjectSync = exports.writeFileSync = exports.removeDirSync = exports.removeDirs = exports.removeDirsSync = exports.removeFilesSync = exports.cleanFolder = exports.mkdirSync = exports.removeDir = exports.saveAsJs = exports.copyFolderContentsRecursive = exports.copyFolderContentsRecursiveSync = exports.copyFolderRecursiveSync = exports.invalidatePodsChecksum = exports.copyFileWithInjectSync = exports.readCleanFile = exports.writeCleanFile = exports.copyFileSync = exports.fsReaddir = exports.fsReadFile = exports.fsSymlinkSync = exports.fsUnlinkSync = exports.fsMkdirSync = exports.fsStatSync = exports.fsRenameSync = exports.fsChmodSync = exports.fsReadFileSync = exports.fsLstatSync = exports.fsReaddirSync = exports.fsExistsSync = exports.fsCopyFileSync = exports.fsWriteFileSync = exports.configureFilesystem = void 0;
var tslib_1 = require("tslib");
var fs_1 = (0, tslib_1.__importDefault)(require("fs"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var rimraf_1 = (0, tslib_1.__importDefault)(require("rimraf"));
var svg2js_1 = (0, tslib_1.__importDefault)(require("svg2js"));
var shelljs_1 = (0, tslib_1.__importDefault)(require("shelljs"));
var deepmerge_1 = (0, tslib_1.__importDefault)(require("deepmerge"));
var ncp_1 = (0, tslib_1.__importDefault)(require("ncp"));
var logger_1 = require("./logger");
var configureFilesystem = function (_getConfigProp, _doResolve, _isSystemWin) {
    global.getConfigProp = _getConfigProp;
    global.doResolve = _doResolve;
    global.isSystemWin = _isSystemWin;
};
exports.configureFilesystem = configureFilesystem;
var fsWriteFileSync = function (dest, data, encoding) {
    // if (dest && dest.includes('renative.json')) {
    //     console.log('FS_WRITE', dest, data.length);
    // }
    fs_1.default.writeFileSync(dest, data, encoding);
};
exports.fsWriteFileSync = fsWriteFileSync;
var fsCopyFileSync = function (source, dest) {
    // console.log('FS_COPY', source);
    fs_1.default.copyFileSync(source, dest);
};
exports.fsCopyFileSync = fsCopyFileSync;
var fsExistsSync = function (dest) { return fs_1.default.existsSync(dest); };
exports.fsExistsSync = fsExistsSync;
var fsReaddirSync = function (dest) { return fs_1.default.readdirSync(dest); };
exports.fsReaddirSync = fsReaddirSync;
var fsLstatSync = function (dest) { return fs_1.default.lstatSync(dest); };
exports.fsLstatSync = fsLstatSync;
var fsReadFileSync = function (dest) { return fs_1.default.readFileSync(dest); };
exports.fsReadFileSync = fsReadFileSync;
var fsChmodSync = function (dest, flag) { return fs_1.default.chmodSync(dest, flag); };
exports.fsChmodSync = fsChmodSync;
var fsRenameSync = function (arg1, arg2) { return fs_1.default.renameSync(arg1, arg2); };
exports.fsRenameSync = fsRenameSync;
var fsStatSync = function (arg1) { return fs_1.default.statSync(arg1); };
exports.fsStatSync = fsStatSync;
var fsMkdirSync = function (arg1) { return fs_1.default.mkdirSync(arg1); };
exports.fsMkdirSync = fsMkdirSync;
var fsUnlinkSync = function (arg1) { return fs_1.default.unlinkSync(arg1); };
exports.fsUnlinkSync = fsUnlinkSync;
var fsSymlinkSync = function (arg1, arg2) {
    fs_1.default.symlinkSync(arg1, arg2);
};
exports.fsSymlinkSync = fsSymlinkSync;
var fsReadFile = function (arg1, arg2) {
    fs_1.default.readFile(arg1, arg2);
};
exports.fsReadFile = fsReadFile;
var fsReaddir = function (arg1, arg2) { return fs_1.default.readdir(arg1, arg2); };
exports.fsReaddir = fsReaddir;
var _getSanitizedPath = function (origPath, timestampPathsConfig) {
    var _a;
    if (((_a = timestampPathsConfig === null || timestampPathsConfig === void 0 ? void 0 : timestampPathsConfig.paths) === null || _a === void 0 ? void 0 : _a.length) && (timestampPathsConfig === null || timestampPathsConfig === void 0 ? void 0 : timestampPathsConfig.timestamp)) {
        var pths = timestampPathsConfig.paths;
        if (pths.includes(origPath)) {
            var ext = path_1.default.extname(origPath);
            var fileName = path_1.default.basename(origPath, ext);
            var dirPath = path_1.default.dirname(origPath);
            var newPath = path_1.default.join(dirPath, "".concat(fileName, "-").concat(timestampPathsConfig.timestamp).concat(ext));
            return newPath;
        }
    }
    return origPath;
};
var copyFileSync = function (source, target, skipOverride, timestampPathsConfig) {
    (0, logger_1.logDebug)('copyFileSync', source);
    var targetFile = target;
    // if target is a directory a new file with the same name will be created
    if (source.indexOf('.DS_Store') !== -1)
        return;
    if (fs_1.default.existsSync(target)) {
        if (fs_1.default.lstatSync(target).isDirectory()) {
            targetFile = path_1.default.join(target, path_1.default.basename(source));
        }
    }
    if (fs_1.default.existsSync(targetFile)) {
        if (skipOverride)
            return;
        var src = fs_1.default.readFileSync(source);
        var dst = fs_1.default.readFileSync(targetFile);
        if (Buffer.compare(src, dst) === 0)
            return;
    }
    (0, logger_1.logDebug)('copyFileSync', source, targetFile, 'executed');
    try {
        (0, exports.fsCopyFileSync)(source, _getSanitizedPath(targetFile, timestampPathsConfig));
    }
    catch (e) {
        (0, logger_1.logDebug)('copyFileSync', e);
    }
};
exports.copyFileSync = copyFileSync;
var SKIP_INJECT_EXTENSIONS = ['.jpg', '.jpeg', '.png', '.svg', '.jar', '.zip', '.ico'];
var writeCleanFile = function (source, destination, overrides, timestampPathsConfig, c) {
    // logTask(`writeCleanFile`)
    // console.log('writeCleanFile', destination);
    if (!fs_1.default.existsSync(source)) {
        (0, logger_1.logError)("Cannot write file. source path doesn't exists: ".concat(source));
        return;
    }
    if (!fs_1.default.existsSync(destination)) {
        (0, logger_1.logDebug)("destination path doesn't exists: ".concat(destination, ". will create new one"));
        // return;
    }
    var ext = path_1.default.extname(source);
    if (SKIP_INJECT_EXTENSIONS.includes(ext)) {
        (0, exports.fsCopyFileSync)(source, _getSanitizedPath(destination, timestampPathsConfig));
    }
    else {
        var pFile = fs_1.default.readFileSync(source, 'utf8');
        if (/\ufffd/.test(pFile) === true) {
            // Handle uncaught binary files
            (0, exports.fsCopyFileSync)(source, _getSanitizedPath(destination, timestampPathsConfig));
        }
        else {
            var pFileClean_1 = pFile;
            if (overrides === null || overrides === void 0 ? void 0 : overrides.forEach) {
                overrides.forEach(function (v) {
                    var regEx = new RegExp(v.pattern, 'g');
                    pFileClean_1 = pFileClean_1.replace(regEx, v.override);
                });
            }
            if (c) {
                var regEx = /{{configProps.([\s\S]*?)}}/g;
                var occurences = pFileClean_1.match(regEx);
                if (occurences) {
                    occurences.forEach(function (occ) {
                        var val = occ.replace('{{configProps.', '').replace('}}', '');
                        var configVal = global.getConfigProp(c, c.platform, val, '');
                        pFileClean_1 = pFileClean_1.replace(occ, configVal);
                    });
                }
            }
            (0, exports.fsWriteFileSync)(_getSanitizedPath(destination, timestampPathsConfig), pFileClean_1, 'utf8');
        }
    }
};
exports.writeCleanFile = writeCleanFile;
var readCleanFile = function (source, overrides) {
    // logTask(`writeCleanFile`)
    // console.log('readCleanFile', source);
    if (!fs_1.default.existsSync(source)) {
        (0, logger_1.logError)("Cannot read file. source path doesn't exists: ".concat(source));
        return;
    }
    var pFile = fs_1.default.readFileSync(source, 'utf8');
    var pFileClean = pFile;
    if (overrides === null || overrides === void 0 ? void 0 : overrides.forEach) {
        overrides.forEach(function (v) {
            var regEx = new RegExp(v.pattern, 'g');
            pFileClean = pFileClean.replace(regEx, v.override);
        });
    }
    return Buffer.from(pFileClean, 'utf8');
};
exports.readCleanFile = readCleanFile;
var copyFileWithInjectSync = function (source, target, skipOverride, injectObject, timestampPathsConfig, c) {
    (0, logger_1.logDebug)('copyFileWithInjectSync', source);
    var targetFile = target;
    // if target is a directory a new file with the same name will be created
    if (source.indexOf('.DS_Store') !== -1)
        return;
    if (fs_1.default.existsSync(target)) {
        if (fs_1.default.lstatSync(target).isDirectory()) {
            targetFile = path_1.default.join(target, path_1.default.basename(source));
        }
    }
    if (fs_1.default.existsSync(targetFile)) {
        if (skipOverride)
            return;
        var src = (0, exports.readCleanFile)(source, injectObject);
        var dst = fs_1.default.readFileSync(targetFile);
        if (Buffer.compare(src, dst) === 0)
            return;
    }
    (0, logger_1.logDebug)('copyFileSync', source, targetFile, 'executed');
    try {
        (0, exports.writeCleanFile)(source, targetFile, injectObject, timestampPathsConfig, c);
    }
    catch (e) {
        (0, logger_1.logDebug)('copyFileSync', e);
    }
};
exports.copyFileWithInjectSync = copyFileWithInjectSync;
var invalidatePodsChecksum = function (c) {
    var appFolder = path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(c.platform));
    var podChecksumPath = path_1.default.join(appFolder, 'Podfile.checksum');
    if (fs_1.default.existsSync(podChecksumPath)) {
        fs_1.default.unlinkSync(podChecksumPath);
    }
};
exports.invalidatePodsChecksum = invalidatePodsChecksum;
var copyFolderRecursiveSync = function (source, target, convertSvg, skipOverride, injectObject, timestampPathsConfig, c, extFilter) {
    if (convertSvg === void 0) { convertSvg = true; }
    if (injectObject === void 0) { injectObject = null; }
    if (timestampPathsConfig === void 0) { timestampPathsConfig = null; }
    if (extFilter === void 0) { extFilter = null; }
    (0, logger_1.logDebug)('copyFolderRecursiveSync', source, target);
    if (!fs_1.default.existsSync(source))
        return;
    var files = [];
    // check if folder needs to be created or integrated
    var targetFolder = path_1.default.join(target, path_1.default.basename(source));
    if (!fs_1.default.existsSync(targetFolder)) {
        (0, exports.mkdirSync)(targetFolder);
    }
    // copy
    if (fs_1.default.lstatSync(source).isDirectory()) {
        files = fs_1.default.readdirSync(source);
        files.forEach(function (file) {
            var curSource = path_1.default.join(source, file);
            if (fs_1.default.lstatSync(curSource).isDirectory()) {
                (0, exports.copyFolderRecursiveSync)(curSource, targetFolder, convertSvg, skipOverride, injectObject, timestampPathsConfig, c, extFilter);
            }
            else if (path_1.default.extname(curSource) === '.svg'
                && convertSvg === true) {
                var jsDest = path_1.default.join(targetFolder, "".concat(path_1.default.basename(curSource), ".js"));
                (0, logger_1.logDebug)("file ".concat(curSource, " is svg and convertSvg is set to true. converting to ").concat(jsDest));
                (0, exports.saveAsJs)(curSource, jsDest);
            }
            else if (injectObject !== null) {
                (0, exports.copyFileWithInjectSync)(curSource, targetFolder, skipOverride, injectObject, timestampPathsConfig, c);
            }
            else if ((extFilter === null || extFilter === void 0 ? void 0 : extFilter.length) > 0) {
                if (extFilter.includes(path_1.default.extname(curSource)) || extFilter.includes(path_1.default.basename(curSource))) {
                    (0, exports.copyFileSync)(curSource, targetFolder, skipOverride, timestampPathsConfig);
                }
            }
            else {
                (0, exports.copyFileSync)(curSource, targetFolder, skipOverride, timestampPathsConfig);
            }
        });
    }
};
exports.copyFolderRecursiveSync = copyFolderRecursiveSync;
var copyFolderContentsRecursiveSync = function (source, target, convertSvg, skipPaths, skipOverride, injectObject, timestampPathsConfig, c, extFilter) {
    if (convertSvg === void 0) { convertSvg = true; }
    if (injectObject === void 0) { injectObject = null; }
    if (timestampPathsConfig === void 0) { timestampPathsConfig = null; }
    if (extFilter === void 0) { extFilter = null; }
    (0, logger_1.logDebug)('copyFolderContentsRecursiveSync', source, target, skipPaths);
    if (!fs_1.default.existsSync(source))
        return;
    var files = [];
    var targetFolder = path_1.default.join(target);
    if (!fs_1.default.existsSync(targetFolder)) {
        (0, exports.mkdirSync)(targetFolder);
    }
    if (fs_1.default.lstatSync(source).isDirectory()) {
        files = fs_1.default.readdirSync(source);
        files.forEach(function (file) {
            var curSource = path_1.default.join(source, file);
            if (!skipPaths || (skipPaths && !skipPaths.includes(curSource))) {
                if (fs_1.default.lstatSync(curSource).isDirectory()) {
                    (0, exports.copyFolderRecursiveSync)(curSource, targetFolder, convertSvg, skipOverride, injectObject, timestampPathsConfig, c, extFilter);
                }
                else if (injectObject !== null) {
                    (0, exports.copyFileWithInjectSync)(curSource, targetFolder, skipOverride, injectObject, timestampPathsConfig, c);
                }
                else if (path_1.default.extname(curSource) === '.svg' && convertSvg === true) {
                    var jsDest = path_1.default.join(targetFolder, "".concat(path_1.default.basename(curSource), ".js"));
                    (0, logger_1.logDebug)("file ".concat(curSource, " is svg and convertSvg is set to true. converting to ").concat(jsDest));
                    (0, exports.saveAsJs)(curSource, jsDest);
                }
                else if ((extFilter === null || extFilter === void 0 ? void 0 : extFilter.length) > 0) {
                    if (extFilter.includes(path_1.default.extname(curSource)) || extFilter.includes(path_1.default.basename(curSource))) {
                        (0, exports.copyFileSync)(curSource, targetFolder, skipOverride, timestampPathsConfig);
                    }
                }
                else {
                    (0, exports.copyFileSync)(curSource, targetFolder, skipOverride, timestampPathsConfig);
                }
            }
        });
    }
};
exports.copyFolderContentsRecursiveSync = copyFolderContentsRecursiveSync;
var copyFolderContentsRecursive = function (source, target, convertSvg, skipPaths) {
    if (convertSvg === void 0) { convertSvg = true; }
    return new Promise(function (resolve, reject) {
        (0, logger_1.logDebug)('copyFolderContentsRecursive', source, target, skipPaths, convertSvg);
        if (!fs_1.default.existsSync(source))
            return;
        var targetFolder = path_1.default.resolve(target);
        if (!fs_1.default.existsSync(targetFolder)) {
            (0, exports.mkdirSync)(targetFolder);
        }
        (0, ncp_1.default)(source, targetFolder, function (err) {
            if (err) {
                return reject(err);
            }
            return resolve();
        });
    });
};
exports.copyFolderContentsRecursive = copyFolderContentsRecursive;
var saveAsJs = function (source, dest) {
    svg2js_1.default.createSync({
        source: source,
        destination: dest
    });
};
exports.saveAsJs = saveAsJs;
var removeDir = function (pth, callback) {
    (0, rimraf_1.default)(pth, callback);
};
exports.removeDir = removeDir;
var mkdirSync = function (dir) {
    if (!dir)
        return;
    if (fs_1.default.existsSync(dir))
        return;
    try {
        shelljs_1.default.mkdir('-p', dir);
    }
    catch (e) {
        (0, logger_1.logWarning)("shelljs.mkdir failed for dir: ".concat(dir, " with error: ").concat(e));
    }
};
exports.mkdirSync = mkdirSync;
var cleanFolder = function (d) { return new Promise(function (resolve) {
    (0, logger_1.logDebug)('cleanFolder', d);
    (0, exports.removeDir)(d, function () {
        (0, exports.mkdirSync)(d);
        resolve();
    });
}); };
exports.cleanFolder = cleanFolder;
var removeFilesSync = function (filePaths) {
    (0, logger_1.logDebug)('removeFilesSync', filePaths);
    filePaths.forEach(function (filePath) {
        try {
            if (fs_1.default.existsSync(filePath)) {
                fs_1.default.unlinkSync(filePath);
            }
            else {
                (0, logger_1.logDebug)("Path ".concat(filePath, " does not exist"));
            }
        }
        catch (e) {
            (0, logger_1.logError)(e);
        }
    });
};
exports.removeFilesSync = removeFilesSync;
var removeDirsSync = function (dirPaths) {
    (0, logger_1.logDebug)('removeDirsSync', dirPaths);
    for (var i = 0; i < dirPaths.length; i++) {
        try {
            (0, exports.removeDirSync)(dirPaths[i]);
        }
        catch (e) {
            (0, logger_1.logError)(e);
        }
    }
};
exports.removeDirsSync = removeDirsSync;
/* eslint-disable no-loop-func */
var removeDirs = function (dirPaths) { return new Promise(function (resolve) {
    (0, logger_1.logDebug)('removeDirs', dirPaths);
    var allFolders = dirPaths.length;
    var deletedFolders = 0;
    for (var i = 0; i < allFolders; i++) {
        (0, rimraf_1.default)(dirPaths[i], function (e) {
            if (e) {
                (0, logger_1.logError)(e);
            }
            deletedFolders++;
            if (deletedFolders >= allFolders)
                resolve();
        });
    }
    if (allFolders === 0)
        resolve();
}); };
exports.removeDirs = removeDirs;
var removeDirSync = function (_dir, _rmSelf) {
    var dir = _dir;
    var rmSelf = _rmSelf;
    var files;
    rmSelf = rmSelf === undefined ? true : rmSelf;
    dir += '/';
    try {
        files = fs_1.default.readdirSync(dir);
    }
    catch (e) {
        (0, logger_1.logDebug)('!Oops, directory not exist.');
        return;
    }
    if (files.length > 0) {
        files.forEach(function (x) {
            try {
                if (fs_1.default.statSync(dir + x).isDirectory()) {
                    (0, exports.removeDirSync)(dir + x);
                }
                else {
                    fs_1.default.unlinkSync(dir + x);
                }
            }
            catch (e) {
                (0, logger_1.logDebug)("removeDirSync error:".concat(e, ". will try to unlink"));
                try {
                    fs_1.default.unlinkSync(dir + x);
                }
                catch (e2) {
                    (0, logger_1.logDebug)("removeDirSync error:".concat(e));
                }
            }
        });
    }
    if (rmSelf) {
        // check if user want to delete the directory ir just the files in this directory
        fs_1.default.rmdirSync(dir);
    }
};
exports.removeDirSync = removeDirSync;
var writeFileSync = function (filePath, obj, spaces, addNewLine) {
    if (spaces === void 0) { spaces = 4; }
    if (addNewLine === void 0) { addNewLine = true; }
    (0, logger_1.logDebug)('writeFileSync', filePath);
    if (filePath.includes('?') || filePath.includes('undefined'))
        return;
    var output;
    if (typeof obj === 'string') {
        output = obj;
    }
    else {
        output = "".concat(JSON.stringify(obj, null, spaces)).concat(addNewLine ? '\n' : '');
    }
    if (fs_1.default.existsSync(filePath)) {
        if (fs_1.default.readFileSync(filePath).toString() === output)
            return;
    }
    (0, logger_1.logDebug)('writeFileSync', filePath, 'executed', "size:".concat(output === null || output === void 0 ? void 0 : output.length));
    (0, exports.fsWriteFileSync)(filePath, output);
    return output;
};
exports.writeFileSync = writeFileSync;
var writeObjectSync = function (filePath, obj, spaces, addNewLine) {
    if (addNewLine === void 0) { addNewLine = true; }
    (0, logger_1.logDebug)('writeObjectSync', filePath);
    (0, logger_1.logWarning)('writeObjectSync is DEPRECATED. use writeFileSync instead');
    return (0, exports.writeFileSync)(filePath, obj, spaces, addNewLine);
};
exports.writeObjectSync = writeObjectSync;
var readObjectSync = function (filePath, sanitize, c) {
    if (sanitize === void 0) { sanitize = false; }
    (0, logger_1.logDebug)("readObjectSync:".concat(sanitize, ":").concat(filePath));
    if (!filePath) {
        (0, logger_1.logDebug)('readObjectSync: filePath is undefined');
        return null;
    }
    if (!fs_1.default.existsSync(filePath)) {
        (0, logger_1.logDebug)("readObjectSync: File at ".concat(filePath, " does not exist"));
        return null;
    }
    var obj;
    try {
        obj = JSON.parse(fs_1.default.readFileSync(filePath));
        if (sanitize) {
            (0, logger_1.logDebug)("readObjectSync: will sanitize file at: ".concat(filePath));
            if (c) {
                obj = (0, exports.sanitizeDynamicRefs)(c, obj);
            }
            if (obj._refs) {
                obj = (0, exports.sanitizeDynamicProps)(obj, {
                    files: c.files,
                    runtimeProps: c.runtime,
                    props: obj._refs,
                    configProps: c.configPropsInjects
                });
            }
        }
    }
    catch (e) {
        (0, logger_1.logError)("readObjectSync: Parsing of ".concat((0, logger_1.chalk)().white(filePath), " failed with ").concat(e));
        return null;
    }
    return obj;
};
exports.readObjectSync = readObjectSync;
var updateObjectSync = function (filePath, updateObj) {
    var output;
    var obj = (0, exports.readObjectSync)(filePath);
    if (obj) {
        output = (0, deepmerge_1.default)(obj, updateObj);
    }
    else {
        output = updateObj;
    }
    (0, exports.writeFileSync)(filePath, output);
    return output;
};
exports.updateObjectSync = updateObjectSync;
var getRealPath = function (c, p, key, original) {
    if (key === void 0) { key = 'undefined'; }
    if (!p) {
        if (original) {
            (0, logger_1.logDebug)("Path ".concat((0, logger_1.chalk)().white(key), " is not defined. using default: ").concat((0, logger_1.chalk)().white(original)));
        }
        return original;
    }
    if (p.startsWith('./')) {
        return path_1.default.join(c.paths.project.dir, p);
    }
    var output = p
        .replace(/\$RNV_HOME/g, c.paths.rnv.dir)
        .replace(/~/g, c.paths.home.dir)
        .replace(/\$USER_HOME/g, c.paths.home.dir)
        .replace(/\$PROJECT_HOME/g, c.paths.project.dir)
        .replace(/\$WORKSPACE_HOME/g, c.paths.workspace.dir)
        .replace(/RNV_HOME/g, c.paths.rnv.dir)
        .replace(/USER_HOME/g, c.paths.home.dir)
        .replace(/PROJECT_HOME/g, c.paths.project.dir);
    return output;
};
exports.getRealPath = getRealPath;
var _refToValue = function (c, ref, key) {
    var val = ref.replace('$REF$:', '').split('$...');
    var realPath = (0, exports.getRealPath)(c, val[0], key);
    if (realPath && realPath.includes('.json') && val.length === 2) {
        if (fs_1.default.existsSync(realPath)) {
            var obj = (0, exports.readObjectSync)(realPath);
            try {
                var output = val[1].split('.').reduce(function (o, i) { return o[i]; }, obj);
                return output;
            }
            catch (e) {
                (0, logger_1.logWarning)("_refToValue: ".concat(e));
            }
        }
        else {
            (0, logger_1.logWarning)("_refToValue: ".concat((0, logger_1.chalk)().white(realPath), " does not exist!"));
        }
    }
    return ref;
};
var arrayMerge = function (destinationArray, sourceArray) {
    var jointArray = destinationArray.concat(sourceArray);
    var uniqueArray = jointArray.filter(function (item, index) { return jointArray.indexOf(item) === index; });
    return uniqueArray;
};
exports.arrayMerge = arrayMerge;
var _arrayMergeOverride = function (destinationArray, sourceArray) { return sourceArray; };
var sanitizeDynamicRefs = function (c, obj) {
    if (!obj)
        return obj;
    if (Array.isArray(obj)) {
        obj.forEach(function (v) {
            (0, exports.sanitizeDynamicRefs)(c, v);
        });
    }
    Object.keys(obj).forEach(function (key) {
        var val = obj[key];
        if (val) {
            if (typeof val === 'string') {
                if (val.startsWith('$REF$:')) {
                    obj[key] = _refToValue(c, val, key);
                }
            }
            else {
                (0, exports.sanitizeDynamicRefs)(c, val);
            }
        }
    });
    return obj;
};
exports.sanitizeDynamicRefs = sanitizeDynamicRefs;
var resolvePackage = function (text) {
    if (typeof text !== 'string')
        return text;
    var regEx = /{{resolvePackage\(([\s\S]*?)\)}}/g;
    var matches = text.match(regEx);
    var newText = text;
    if (matches === null || matches === void 0 ? void 0 : matches.length) {
        matches.forEach(function (match) {
            var val = match.replace('{{resolvePackage(', '').replace(')}}', '');
            // TODO: Figure out WIN vs LINUX treatment here
            // forceForwardPaths is required for WIN Android to work correctly
            newText = newText.replace(match, global.doResolve(val, false, { forceForwardPaths: true }));
        });
    }
    return newText;
};
exports.resolvePackage = resolvePackage;
var sanitizeDynamicProps = function (obj, propConfig) {
    if (!obj) {
        return obj;
    }
    if (Array.isArray(obj)) {
        obj.forEach(function (v, i) {
            var val = v;
            if (typeof val === 'string') {
                _bindStringVals(obj, val, i, propConfig);
            }
            else {
                (0, exports.sanitizeDynamicProps)(v, propConfig);
            }
        });
    }
    else if (typeof obj === 'object') {
        Object.keys(obj).forEach(function (key) {
            var val = obj[key];
            // Some values are passed as keys so have to validate keys as well
            var newKey = (0, exports.resolvePackage)(key);
            delete obj[key];
            obj[newKey] = val;
            if (val) {
                if (typeof val === 'string') {
                    _bindStringVals(obj, val, newKey, propConfig);
                }
                else {
                    (0, exports.sanitizeDynamicProps)(val, propConfig);
                }
            }
        });
    }
    else if (typeof obj === 'string') {
        return (0, exports.resolvePackage)(obj);
    }
    return obj;
};
exports.sanitizeDynamicProps = sanitizeDynamicProps;
var BIND_FILES = '{{files.';
var BIND_PROPS = '{{props.';
var BIND_CONFIG_PROPS = '{{configProps.';
var BIND_RUNTIME_PROPS = '{{runtimeProps.';
var BIND_ENV = '{{env.';
var _bindStringVals = function (obj, _val, newKey, propConfig) {
    var _a = propConfig.props, props = _a === void 0 ? {} : _a, _b = propConfig.configProps, configProps = _b === void 0 ? {} : _b, _c = propConfig.runtimeProps, runtimeProps = _c === void 0 ? {} : _c;
    var val = _val;
    if (val.includes(BIND_FILES)) {
        var key = val.replace(BIND_FILES, '').replace('}}', '');
        var nVal = key.split('.').reduce(function (o, i) { return o === null || o === void 0 ? void 0 : o[i]; }, propConfig.files);
        obj[newKey] = (0, exports.resolvePackage)(nVal);
    }
    else if (val.includes(BIND_PROPS)) {
        Object.keys(props).forEach(function (pk) {
            val = val.replace("".concat(BIND_PROPS).concat(pk, "}}"), props === null || props === void 0 ? void 0 : props[pk]);
            obj[newKey] = (0, exports.resolvePackage)(val);
        });
    }
    else if (val.includes(BIND_CONFIG_PROPS)) {
        Object.keys(configProps).forEach(function (pk2) {
            val = val.replace("".concat(BIND_CONFIG_PROPS).concat(pk2, "}}"), configProps[pk2]);
            obj[newKey] = (0, exports.resolvePackage)(val);
        });
    }
    else if (val.includes(BIND_RUNTIME_PROPS)) {
        Object.keys(runtimeProps).forEach(function (pk3) {
            val = val.replace("".concat(BIND_RUNTIME_PROPS).concat(pk3, "}}"), runtimeProps[pk3]);
            obj[newKey] = (0, exports.resolvePackage)(val);
        });
    }
    else if (val.includes(BIND_ENV)) {
        var key = val.replace(BIND_ENV, '').replace('}}', '');
        obj[newKey] = process.env[key];
    }
};
var mergeObjects = function (c, obj1, obj2, dynamicRefs, replaceArrays) {
    if (dynamicRefs === void 0) { dynamicRefs = true; }
    if (replaceArrays === void 0) { replaceArrays = false; }
    if (!obj2)
        return obj1;
    if (!obj1)
        return obj2;
    var obj = (0, deepmerge_1.default)(obj1, obj2, {
        arrayMerge: replaceArrays ? _arrayMergeOverride : exports.arrayMerge
    });
    return dynamicRefs ? (0, exports.sanitizeDynamicRefs)(c, obj) : obj;
};
exports.mergeObjects = mergeObjects;
var updateConfigFile = function (update, globalConfigPath) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var configContents;
    return (0, tslib_1.__generator)(this, function (_a) {
        configContents = JSON.parse(fs_1.default.readFileSync(globalConfigPath));
        if (update.androidSdk) {
            configContents.sdks.ANDROID_SDK = update.androidSdk;
        }
        if (update.tizenSdk) {
            configContents.sdks.TIZEN_SDK = update.tizenSdk;
        }
        if (update.webosSdk) {
            configContents.sdks.WEBOS_SDK = update.webosSdk;
        }
        (0, logger_1.logDebug)("Updating ".concat(globalConfigPath, ". New file ").concat(JSON.stringify(configContents, null, 3)));
        (0, exports.fsWriteFileSync)(globalConfigPath, JSON.stringify(configContents, null, 3));
        return [2 /*return*/];
    });
}); };
exports.updateConfigFile = updateConfigFile;
var replaceHomeFolder = function (p) {
    if (global.isSystemWin)
        return p.replace('~', process.env.USERPROFILE);
    return p.replace('~', process.env.HOME);
};
exports.replaceHomeFolder = replaceHomeFolder;
var getFileListSync = function (dir) {
    var results = [];
    var list = fs_1.default.readdirSync(dir);
    list.forEach(function (file) {
        var fileFixed = "".concat(dir, "/").concat(file);
        var stat = fs_1.default.statSync(fileFixed);
        if (stat && stat.isDirectory()) {
            /* Recurse into a subdirectory */
            results = results.concat((0, exports.getFileListSync)(fileFixed));
        }
        else {
            /* Is a file */
            results.push(fileFixed);
        }
    });
    return results;
};
exports.getFileListSync = getFileListSync;
var loadFile = function (fileObj, pathObj, key) {
    var pKey = "".concat(key, "Exists");
    if (!(0, exports.fsExistsSync)(pathObj[key])) {
        pathObj[pKey] = false;
        (0, logger_1.logDebug)("WARNING: loadFile: Path ".concat(pathObj[key], " does not exists!"));
        (0, logger_1.logDebug)("FILE_EXISTS: ".concat(key, ":false path:").concat(pathObj[key]));
        return false;
    }
    pathObj[pKey] = true;
    try {
        var fileString = (0, exports.fsReadFileSync)(pathObj[key]).toString();
        fileObj[key] = JSON.parse(fileString);
        pathObj[pKey] = true;
        (0, logger_1.logDebug)("FILE_EXISTS: ".concat(key, ":true size:").concat((0, exports.formatBytes)(Buffer.byteLength(fileString, 'utf8'))));
        // if (validateRuntimeObjectSchema && fileObj[key]) {
        //     const valid = ajv.validate(schemaRoot, fileObj[key]);
        //     if (!valid) {
        //         logWarning(`Invalid schema in ${pathObj[key]}. ISSUES: ${JSON.stringify(ajv.errors, null, 2)}`);
        //     }
        // }
        // if (pathObj[key].includes?.('renative.json')) {
        //     console.log(`FILE_EXISTS: ${key}:true size:${formatBytes(Buffer.byteLength(fileString, 'utf8'))}`);
        // }
        return fileObj[key];
    }
    catch (e) {
        (0, logger_1.logError)("loadFile: ".concat(pathObj[key], " :: ").concat(e), true); // crash if there's an error in the config file
        return false;
    }
};
exports.loadFile = loadFile;
var formatBytes = function (bytes, decimals) {
    if (decimals === void 0) { decimals = 2; }
    if (bytes === 0)
        return '0 Bytes';
    var k = 1024;
    var dm = decimals < 0 ? 0 : decimals;
    var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    var i = Math.floor(Math.log(bytes) / Math.log(k));
    return "".concat(parseFloat((bytes / (Math.pow(k, i))).toFixed(dm)), " ").concat(sizes[i]);
};
exports.formatBytes = formatBytes;
// Return all directories within a directory
var getDirectories = function (source) { return fs_1.default.readdirSync(source, { withFileTypes: true })
    .filter(function (dirent) { return dirent.isDirectory(); })
    .map(function (dirent) { return dirent.name; }); };
exports.getDirectories = getDirectories;
var cleanEmptyFoldersRecursively = function (folder) {
    var isDir = (0, exports.fsStatSync)(folder).isDirectory();
    if (!isDir) {
        return;
    }
    var files = (0, exports.fsReaddirSync)(folder);
    if (files.length > 0) {
        files.forEach(function (file) {
            var fullPath = path_1.default.join(folder, file);
            (0, exports.cleanEmptyFoldersRecursively)(fullPath);
        });
        // re-evaluate files; after deleting subfolder
        // we may have parent folder empty now
        files = (0, exports.fsReaddirSync)(folder);
    }
    if (files.length === 0) {
        fs_1.default.rmdirSync(folder);
    }
};
exports.cleanEmptyFoldersRecursively = cleanEmptyFoldersRecursively;
exports.default = {
    sanitizeDynamicRefs: exports.sanitizeDynamicRefs,
    getFileListSync: exports.getFileListSync,
    removeDirs: exports.removeDirs,
    copyFileSync: exports.copyFileSync,
    copyFolderRecursiveSync: exports.copyFolderRecursiveSync,
    removeDir: exports.removeDir,
    removeDirsSync: exports.removeDirsSync,
    removeFilesSync: exports.removeFilesSync,
    saveAsJs: exports.saveAsJs,
    mkdirSync: exports.mkdirSync,
    copyFolderContentsRecursive: exports.copyFolderContentsRecursive,
    copyFolderContentsRecursiveSync: exports.copyFolderContentsRecursiveSync,
    cleanFolder: exports.cleanFolder,
    writeFileSync: exports.writeFileSync,
    readObjectSync: exports.readObjectSync,
    updateObjectSync: exports.updateObjectSync,
    arrayMerge: exports.arrayMerge,
    mergeObjects: exports.mergeObjects,
    updateConfigFile: exports.updateConfigFile,
    replaceHomeFolder: exports.replaceHomeFolder,
    getDirectories: exports.getDirectories,
    resolvePackage: exports.resolvePackage,
    cleanEmptyFoldersRecursively: exports.cleanEmptyFoldersRecursively
};
//# sourceMappingURL=fileutils.js.map