"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanNodeModules = exports.jetifyIfRequired = exports.installPackageDependencies = exports.isYarnInstalled = exports.checkIfProjectAndNodeModulesExists = exports.listAndSelectNpmVersion = exports.areNodeModulesInstalled = exports.checkAndCreateProjectPackage = exports.checkNpxIsInstalled = void 0;
var tslib_1 = require("tslib");
// import semver from 'semver';
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var exec_1 = require("./exec");
var fileutils_1 = require("./fileutils");
var logger_1 = require("./logger");
var constants_1 = require("../constants");
var resolve_1 = require("./resolve");
var prompt_1 = require("../../cli/prompt");
var common_1 = require("../common");
var packageJsonIsValid = function (c) {
    if (!(0, fileutils_1.fsExistsSync)(c.paths.project.package))
        return false;
    var pkg = (0, fileutils_1.readObjectSync)(c.paths.project.package);
    if (!pkg)
        return false;
    // yarn add creates a package.json with only dependencies, that is not valid
    if (Object.keys(pkg).length === 1 && Object.keys(pkg)[0] === 'dependencies')
        return false;
    return true;
};
var checkNpxIsInstalled = function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var confirm_1;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('checkNpxIsInstalled');
                if (!!(0, exec_1.commandExistsSync)('npx')) return [3 /*break*/, 3];
                (0, logger_1.logWarning)('npx is not installed, please install it before running this command');
                confirm_1 = (0, prompt_1.inquirerPrompt)({
                    type: 'confirm',
                    message: 'Do you want to install npx it now?',
                }).confirm;
                if (!confirm_1) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, exec_1.executeAsync)('npm install -g npx')];
            case 1:
                _a.sent();
                return [2 /*return*/, true];
            case 2: throw new Error('npx is not installed');
            case 3: return [2 /*return*/];
        }
    });
}); };
exports.checkNpxIsInstalled = checkNpxIsInstalled;
var checkAndCreateProjectPackage = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var packageName, version, templateName, rnvVersion, templateObj, pkgJson, pkgJsonStringClean;
    var _a, _b, _c, _d, _e, _f;
    return (0, tslib_1.__generator)(this, function (_g) {
        (0, logger_1.logTask)('checkAndCreateProjectPackage');
        if (!packageJsonIsValid(c)) {
            (0, logger_1.logInfo)("Your ".concat(c.paths.project.package, " is missing. CREATING...DONE"));
            packageName = c.files.project.config.projectName
                || c.paths.project.dir.split('/').pop();
            version = ((_b = (_a = c.files.project.config.defaults) === null || _a === void 0 ? void 0 : _a.package) === null || _b === void 0 ? void 0 : _b.version) || '0.1.0';
            templateName = (_c = c.files.project.config) === null || _c === void 0 ? void 0 : _c.currentTemplate;
            if (!templateName) {
                (0, logger_1.logWarning)('You are missing currentTemplate in your renative.json');
            }
            rnvVersion = c.files.rnv.package.version;
            c.paths.template.configTemplate = path_1.default.join(c.paths.project.dir, 'node_modules', templateName, constants_1.RENATIVE_CONFIG_TEMPLATE_NAME);
            templateObj = (0, fileutils_1.readObjectSync)(c.paths.template.configTemplate);
            pkgJson = ((_d = templateObj === null || templateObj === void 0 ? void 0 : templateObj.templateConfig) === null || _d === void 0 ? void 0 : _d.packageTemplate) || {};
            pkgJson.name = packageName;
            pkgJson.version = version;
            pkgJson.dependencies = pkgJson.dependencies || {};
            // No longer good option to assume same version
            // pkgJson.dependencies.renative = rnvVersion;
            pkgJson.devDependencies = pkgJson.devDependencies || {};
            pkgJson.devDependencies.rnv = rnvVersion;
            pkgJson.devDependencies[templateName] = (_f = (_e = c.files.project.config) === null || _e === void 0 ? void 0 : _e.templates[templateName]) === null || _f === void 0 ? void 0 : _f.version;
            pkgJsonStringClean = JSON.stringify(pkgJson, null, 2);
            (0, fileutils_1.fsWriteFileSync)(c.paths.project.package, pkgJsonStringClean);
        }
        (0, fileutils_1.loadFile)(c.files.project, c.paths.project, 'package');
        return [2 /*return*/, true];
    });
}); };
exports.checkAndCreateProjectPackage = checkAndCreateProjectPackage;
var areNodeModulesInstalled = function () { return !!(0, resolve_1.doResolve)('resolve', false); };
exports.areNodeModulesInstalled = areNodeModulesInstalled;
var listAndSelectNpmVersion = function (c, npmPackage) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var templateVersionsStr, versionArr, templateTagsStr, tagArr, validVersions, recommendedVersion, inputTemplateVersion;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, exec_1.executeAsync)(c, "npm view ".concat(npmPackage, " versions"))];
            case 1:
                templateVersionsStr = _a.sent();
                versionArr = templateVersionsStr
                    .replace(/\r?\n|\r|\s|'|\[|\]/g, '')
                    .split(',');
                return [4 /*yield*/, (0, exec_1.executeAsync)(c, "npm dist-tag ls ".concat(npmPackage))];
            case 2:
                templateTagsStr = _a.sent();
                tagArr = [];
                templateTagsStr.split('\n').forEach(function (tString) {
                    var tArr = tString.split(': ');
                    tagArr.push({
                        name: tArr[0],
                        version: tArr[1]
                    });
                });
                versionArr.reverse();
                validVersions = versionArr.map(function (v) { return ({ name: v, value: v }); });
                validVersions.forEach(function (item) {
                    var matchStr = '';
                    var matchArr = [];
                    tagArr.forEach(function (tag) {
                        if (tag.version === item.value) {
                            matchArr.push(tag.name);
                        }
                    });
                    if (matchArr.length) {
                        matchStr = " (HEAD: ".concat(matchArr.join(', '), ")");
                        item.name = "".concat(item.value).concat(matchStr);
                        if (matchArr[0] === 'latest') {
                            recommendedVersion = item;
                        }
                    }
                });
                if (!recommendedVersion) {
                    recommendedVersion = validVersions[0];
                }
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'inputTemplateVersion',
                        type: 'list',
                        message: "What ".concat(npmPackage, " version to use?"),
                        default: recommendedVersion.value,
                        loop: false,
                        choices: validVersions
                    })];
            case 3:
                inputTemplateVersion = (_a.sent()).inputTemplateVersion;
                return [2 /*return*/, inputTemplateVersion];
        }
    });
}); };
exports.listAndSelectNpmVersion = listAndSelectNpmVersion;
var checkIfProjectAndNodeModulesExists = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('checkIfProjectAndNodeModulesExists');
                if (!(c.paths.project.configExists && !(0, fileutils_1.fsExistsSync)(c.paths.project.nodeModulesDir))) return [3 /*break*/, 2];
                c._requiresNpmInstall = false;
                (0, logger_1.logInfo)('node_modules folder is missing. INSTALLING...');
                return [4 /*yield*/, (0, exports.installPackageDependencies)(c)];
            case 1:
                _a.sent();
                _a.label = 2;
            case 2: return [2 /*return*/];
        }
    });
}); };
exports.checkIfProjectAndNodeModulesExists = checkIfProjectAndNodeModulesExists;
var _getInstallScript = function (c) {
    var _a, _b;
    var tasks = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.tasks;
    if (!tasks)
        return null;
    if (Array.isArray(tasks)) {
        for (var i = 0; i < tasks.length; i++) {
            if (tasks[i].name === 'install') {
                return tasks[i].script;
            }
        }
    }
    else {
        return (_b = tasks.install) === null || _b === void 0 ? void 0 : _b.script;
    }
};
var isYarnInstalled = function () { return (0, exec_1.commandExistsSync)('yarn') || (0, resolve_1.doResolve)('yarn', false); };
exports.isYarnInstalled = isYarnInstalled;
var installPackageDependencies = function (c, failOnError) {
    if (failOnError === void 0) { failOnError = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var customScript, isMonorepo, yarnLockPath, npmLockPath, command, yarnLockExists, packageLockExists, packageManager, e_1, npmErr_1, plats;
        var _a, _b, _c;
        return (0, tslib_1.__generator)(this, function (_d) {
            switch (_d.label) {
                case 0:
                    c.runtime.forceBuildHookRebuild = true;
                    customScript = _getInstallScript(c);
                    if (!customScript) return [3 /*break*/, 2];
                    (0, logger_1.logTask)('installPackageDependencies');
                    (0, logger_1.logInfo)("Found custom task for install: ".concat(customScript, "."));
                    return [4 /*yield*/, (0, exec_1.executeAsync)(customScript)];
                case 1:
                    _d.sent();
                    c._requiresNpmInstall = false;
                    return [2 /*return*/, true];
                case 2:
                    isMonorepo = (0, common_1.getConfigProp)(c, c.platform, 'isMonorepo');
                    if (isMonorepo) {
                        (0, logger_1.logSuccess)('This project is marked as part of monorepo and it has no custom install tasks. Run your usual monorepo bootstrap procedure and re-run command again.');
                        return [2 /*return*/, Promise.reject('Cancelled')];
                    }
                    yarnLockPath = path_1.default.join(c.paths.project.dir, 'yarn.lock');
                    npmLockPath = path_1.default.join(c.paths.project.dir, 'package-lock.json');
                    command = 'npm install';
                    yarnLockExists = (0, fileutils_1.fsExistsSync)(yarnLockPath);
                    packageLockExists = (0, fileutils_1.fsExistsSync)(npmLockPath);
                    if (!(yarnLockExists || packageLockExists)) return [3 /*break*/, 3];
                    // a lock file exists, defaulting to whichever is present
                    if (yarnLockExists && !(0, exports.isYarnInstalled)())
                        throw new Error('You have a yarn.lock file but you don\'t have yarn installed. Install it or delete yarn.lock');
                    command = yarnLockExists ? 'yarn' : 'npm install';
                    return [3 /*break*/, 6];
                case 3:
                    if (!c.program.packageManager) return [3 /*break*/, 4];
                    // no lock file check cli option
                    if (['yarn', 'npm'].includes(c.program.packageManager)) {
                        command = c.program.packageManager === 'yarn' ? 'yarn' : 'npm install';
                        if (command === 'yarn' && !(0, exports.isYarnInstalled)())
                            throw new Error('You specified yarn as packageManager but it\'s not installed');
                    }
                    else {
                        throw new Error("Unsupported package manager ".concat(c.program.packageManager, ". Only yarn and npm are supported at the moment."));
                    }
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'list',
                        name: 'packageManager',
                        message: 'What package manager would you like to use?',
                        choices: ['yarn', 'npm'],
                        default: 'npm'
                    })];
                case 5:
                    packageManager = (_d.sent()).packageManager;
                    if (packageManager === 'yarn')
                        command = 'yarn';
                    _d.label = 6;
                case 6:
                    (0, logger_1.logTask)('installPackageDependencies', "packageManager:(".concat(command, ")"));
                    _d.label = 7;
                case 7:
                    _d.trys.push([7, 10, , 16]);
                    return [4 /*yield*/, (0, exec_1.executeAsync)(command)];
                case 8:
                    _d.sent();
                    return [4 /*yield*/, (0, fileutils_1.invalidatePodsChecksum)(c)];
                case 9:
                    _d.sent();
                    return [3 /*break*/, 16];
                case 10:
                    e_1 = _d.sent();
                    if (failOnError) {
                        (0, logger_1.logError)(e_1);
                        throw e_1;
                    }
                    (0, logger_1.logWarning)("".concat(e_1, "\n Seems like your node_modules is corrupted by other libs. ReNative will try to fix it for you"));
                    _d.label = 11;
                case 11:
                    _d.trys.push([11, 14, , 15]);
                    return [4 /*yield*/, (0, exports.cleanNodeModules)(c)];
                case 12:
                    _d.sent();
                    return [4 /*yield*/, (0, exports.installPackageDependencies)(c, true)];
                case 13:
                    _d.sent();
                    return [3 /*break*/, 15];
                case 14:
                    npmErr_1 = _d.sent();
                    (0, logger_1.logError)(npmErr_1);
                    throw npmErr_1;
                case 15: return [3 /*break*/, 16];
                case 16:
                    try {
                        plats = (_b = (_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.defaults) === null || _b === void 0 ? void 0 : _b.supportedPlatforms;
                        if (Array.isArray(plats) && (plats.includes(constants_1.ANDROID)
                            || plats.includes(constants_1.ANDROID_TV)
                            || plats.includes(constants_1.FIRE_TV)
                            || plats.includes(constants_1.ANDROID_WEAR))) {
                            if (!c.files.project.configLocal) {
                                c.files.project.configLocal = {};
                            }
                            if (!((_c = c.files.project.configLocal) === null || _c === void 0 ? void 0 : _c._meta)) {
                                c.files.project.configLocal._meta = {};
                            }
                            c.files.project.configLocal._meta.requiresJetify = true;
                            (0, fileutils_1.writeFileSync)(c.paths.project.configLocal, c.files.project.configLocal);
                        }
                        c._requiresNpmInstall = false;
                        return [2 /*return*/, true];
                    }
                    catch (jetErr) {
                        (0, logger_1.logError)(jetErr);
                        return [2 /*return*/, false];
                    }
                    return [2 /*return*/];
            }
        });
    });
};
exports.installPackageDependencies = installPackageDependencies;
var jetifyIfRequired = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        switch (_c.label) {
            case 0:
                (0, logger_1.logTask)('jetifyIfRequired');
                if (!((_b = (_a = c.files.project.configLocal) === null || _a === void 0 ? void 0 : _a._meta) === null || _b === void 0 ? void 0 : _b.requiresJetify)) return [3 /*break*/, 2];
                if (!(0, resolve_1.doResolve)('jetifier')) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, exec_1.executeAsync)('npx jetify')];
            case 1:
                _c.sent();
                c.files.project.configLocal._meta.requiresJetify = false;
                (0, fileutils_1.writeFileSync)(c.paths.project.configLocal, c.files.project.configLocal);
                _c.label = 2;
            case 2: return [2 /*return*/, true];
        }
    });
}); };
exports.jetifyIfRequired = jetifyIfRequired;
var cleanNodeModules = function () { return new Promise(function (resolve, reject) {
    (0, logger_1.logTask)('cleanNodeModules');
    var dirs = [
        'react-native-safe-area-view/.git',
        '@react-navigation/native/node_modules/react-native-safe-area-view/.git',
        'react-navigation/node_modules/react-native-safe-area-view/.git',
        'react-native-safe-area-view/.git',
        '@react-navigation/native/node_modules/react-native-safe-area-view/.git',
        'react-navigation/node_modules/react-native-safe-area-view/.git'
    ].reduce(function (acc, dir) {
        var _a = dir.match(/([^/]+)\/(.*)/), _all = _a[0], aPackage = _a[1], aPath = _a[2];
        (0, logger_1.logDebug)("Cleaning: ".concat(_all));
        var resolved = (0, resolve_1.doResolve)(aPackage, false);
        if (resolved) {
            acc.push("".concat(resolved, "/").concat(aPath));
        }
        return acc;
    }, []);
    (0, fileutils_1.removeDirs)(dirs)
        .then(function () { return resolve(); })
        .catch(function (e) { return reject(e); });
    // removeDirs([
    //     path.join(c.paths.project.nodeModulesDir, 'react-native-safe-area-view/.git'),
    //     path.join(c.paths.project.nodeModulesDir, '@react-navigation/native/node_modules/react-native-safe-area-view/.git'),
    //     path.join(c.paths.project.nodeModulesDir, 'react-navigation/node_modules/react-native-safe-area-view/.git'),
    //     path.join(c.paths.rnv.nodeModulesDir, 'react-native-safe-area-view/.git'),
    //     path.join(c.paths.rnv.nodeModulesDir, '@react-navigation/native/node_modules/react-native-safe-area-view/.git'),
    //     path.join(c.paths.rnv.nodeModulesDir, 'react-navigation/node_modules/react-native-safe-area-view/.git')
    // ]).then(() => resolve()).catch(e => reject(e));
}); };
exports.cleanNodeModules = cleanNodeModules;
//# sourceMappingURL=npmUtils.js.map