"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeEngineTask = exports.shouldSkipTask = exports.executeOrSkipTask = exports.executeTask = exports.findSuitableTask = exports.initializeTask = exports.registerCustomTask = void 0;
var tslib_1 = require("tslib");
var logger_1 = require("../systemManager/logger");
var analytics_1 = (0, tslib_1.__importDefault)(require("../systemManager/analytics"));
var buildHooks_1 = require("../projectManager/buildHooks");
var prompt_1 = require("../../cli/prompt");
var npmUtils_1 = require("../systemManager/npmUtils");
var engineManager_1 = require("../engineManager");
var constants_1 = require("../constants");
var executedTasks = {};
var CUSTOM_TASKS = {};
var registerCustomTask = function (c, task) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        if (task.task) {
            CUSTOM_TASKS[task.task] = task;
        }
        return [2 /*return*/];
    });
}); };
exports.registerCustomTask = registerCustomTask;
var initializeTask = function (c, task) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('initializeTask', task);
                c.runtime.task = task;
                executedTasks = {};
                analytics_1.default.captureEvent({
                    type: "".concat(task, "Project"),
                    platform: c.platform
                });
                return [4 /*yield*/, (0, exports.executeTask)(c, task, null, task, true)];
            case 1:
                _a.sent();
                return [2 /*return*/, true];
        }
    });
}); };
exports.initializeTask = initializeTask;
var _getTaskOption = function (_a) {
    var taskInstance = _a.taskInstance, hasMultipleSubTasks = _a.hasMultipleSubTasks;
    if (hasMultipleSubTasks) {
        return "".concat(taskInstance.task.split(' ')[0], "...");
    }
    if (taskInstance.description && taskInstance.description !== '') {
        return "".concat(taskInstance.task.split(' ')[0], " ").concat((0, logger_1.chalk)().grey("(".concat(taskInstance.description, ")")));
    }
    return "".concat(taskInstance.task.split(' ')[0]);
};
var _getTaskObj = function (taskInstance) {
    var key = taskInstance.task.split(' ')[0];
    var hasMultipleSubTasks = false;
    if (taskInstance.task.includes(' '))
        hasMultipleSubTasks = true;
    return {
        key: key,
        taskInstance: taskInstance,
        hasMultipleSubTasks: hasMultipleSubTasks
    };
};
var findSuitableTask = function (c, specificTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var REGISTERED_ENGINES, task, suitableTaskInstances_1, taskInstances, tasks, defaultCmd, tasksCommands, filteredTasks, addendum, command, suitableEngines, autocompleteEngines, isAutoComplete_1, message, supportedSubtasksArr_1, supportedSubtasks_1, supportedSubtasksFilter_1, subTasks, subCommand, customTask;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('findSuitableTask');
                REGISTERED_ENGINES = (0, engineManager_1.getRegisteredEngines)(c);
                if (!!specificTask) return [3 /*break*/, 13];
                if (!!c.command) return [3 /*break*/, 2];
                suitableTaskInstances_1 = {};
                REGISTERED_ENGINES.forEach(function (engine) {
                    Object.values(engine.tasks).forEach(function (taskInstance) {
                        var taskObj = _getTaskObj(taskInstance);
                        suitableTaskInstances_1[taskObj.key] = taskObj;
                    });
                });
                Object.values(CUSTOM_TASKS).forEach(function (taskInstance) {
                    var taskObj = _getTaskObj(taskInstance);
                    suitableTaskInstances_1[taskObj.key] = taskObj;
                });
                taskInstances = Object.values(suitableTaskInstances_1);
                tasks = void 0;
                defaultCmd = 'new';
                tasksCommands = void 0;
                filteredTasks = void 0;
                addendum = '';
                if (!c.paths.project.configExists) {
                    filteredTasks = taskInstances.filter(function (v) { return v.taskInstance.isGlobalScope; });
                    tasks = filteredTasks.map(function (v) { return _getTaskOption(v); }).sort();
                    tasksCommands = filteredTasks.map(function (v) { return v.taskInstance.task.split(' ')[0]; }).sort();
                    addendum = ' (Not a ReNative project. options will be limited)';
                }
                else {
                    tasks = taskInstances.map(function (v) { return _getTaskOption(v); }).sort();
                    tasksCommands = taskInstances.map(function (v) { return v.taskInstance.task.split(' ')[0]; }).sort();
                    defaultCmd = tasks.find(function (v) { return v.startsWith('run'); });
                }
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'list',
                        default: defaultCmd,
                        name: 'command',
                        message: "Pick a command".concat(addendum),
                        choices: tasks,
                        pageSize: 15,
                        logMessage: 'Welcome to the brave new world...'
                    })];
            case 1:
                command = (_a.sent()).command;
                c.command = tasksCommands[tasks.indexOf(command)];
                _a.label = 2;
            case 2:
                task = c.command;
                if (c.subCommand)
                    task += " ".concat(c.subCommand);
                suitableEngines = REGISTERED_ENGINES
                    .filter(function (engine) { return (0, engineManager_1.hasEngineTask)(task, engine.tasks, c.paths.project.configExists); });
                autocompleteEngines = REGISTERED_ENGINES
                    .filter(function (engine) { return (0, engineManager_1.getEngineSubTasks)(task, engine.tasks, true).length; });
                isAutoComplete_1 = !suitableEngines.length && !!c.command && !autocompleteEngines.length;
                message = isAutoComplete_1 ? "Autocomplete action for \"".concat(c.command, "\"") : "Pick a subCommand for ".concat(c.command);
                if (!!suitableEngines.length) return [3 /*break*/, 4];
                supportedSubtasksArr_1 = [];
                REGISTERED_ENGINES.forEach(function (engine) {
                    (0, engineManager_1.getEngineSubTasks)(task, engine.tasks).forEach(function (taskInstance) {
                        var _a, _b;
                        var isNotViable = !c.paths.project.configExists && !taskInstance.isGlobalScope;
                        if (!isNotViable) {
                            var taskKey = isAutoComplete_1 ? taskInstance.task : taskInstance.task.split(' ')[1];
                            supportedSubtasksArr_1.push({
                                desc: (_b = (_a = taskInstance.description) === null || _a === void 0 ? void 0 : _a.toLowerCase) === null || _b === void 0 ? void 0 : _b.call(_a),
                                taskKey: taskKey
                            });
                        }
                    });
                });
                Object.values(CUSTOM_TASKS).forEach(function (taskInstance) {
                    var _a, _b;
                    var tskArr = taskInstance.task.split(' ');
                    if (task === tskArr[0]) {
                        var taskKey = isAutoComplete_1 ? taskInstance.task : taskInstance.task.split(' ')[1];
                        supportedSubtasksArr_1.push({
                            desc: (_b = (_a = taskInstance.description) === null || _a === void 0 ? void 0 : _a.toLowerCase) === null || _b === void 0 ? void 0 : _b.call(_a),
                            taskKey: taskKey
                        });
                    }
                });
                supportedSubtasks_1 = {};
                supportedSubtasksFilter_1 = {};
                supportedSubtasksArr_1.forEach(function (tsk) {
                    var mergedTask = supportedSubtasksFilter_1[tsk.taskKey];
                    if (!mergedTask) {
                        supportedSubtasksFilter_1[tsk.taskKey] = tsk;
                    }
                    else if (!mergedTask.desc.includes(tsk.desc)) {
                        mergedTask.desc += ", ".concat(tsk.desc);
                    }
                });
                // Generate final list object
                Object.values(supportedSubtasksFilter_1).forEach(function (v) {
                    var desc = v.desc ? "(".concat(v.desc, ")") : '';
                    var key = "".concat(v.taskKey, " ").concat((0, logger_1.chalk)().grey(desc));
                    supportedSubtasks_1[key] = {
                        taskKey: v.taskKey
                    };
                });
                subTasks = Object.keys(supportedSubtasks_1);
                if (!subTasks.length) return [3 /*break*/, 4];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'list',
                        name: 'subCommand',
                        message: message,
                        choices: subTasks,
                    })];
            case 3:
                subCommand = (_a.sent()).subCommand;
                if (isAutoComplete_1) {
                    task = supportedSubtasks_1[subCommand].taskKey;
                    c.command = task.split(' ')[0];
                    c.subCommand = task.split(' ')[1];
                    if (c.subCommand) {
                        task = "".concat(c.command, " ").concat(c.subCommand);
                    }
                    else {
                        task = "".concat(c.command);
                    }
                }
                else {
                    c.subCommand = supportedSubtasks_1[subCommand].taskKey;
                    task = "".concat(c.command, " ").concat(c.subCommand);
                }
                suitableEngines = REGISTERED_ENGINES
                    .filter(function (engine) { return (0, engineManager_1.hasEngineTask)(task, engine.tasks, c.paths.project.configExists); });
                _a.label = 4;
            case 4:
                if (!CUSTOM_TASKS[task]) return [3 /*break*/, 5];
                // Custom tasks are executed by core engine
                (0, logger_1.logInfo)("Running custom task ".concat(task));
                return [3 /*break*/, 8];
            case 5:
                if (!!suitableEngines.length) return [3 /*break*/, 8];
                if (!!c.runtime.hasAllEnginesRegistered) return [3 /*break*/, 7];
                // No platform was specified. we have no option other than load all engines and offer platform list next round
                return [4 /*yield*/, (0, engineManager_1.registerAllPlatformEngines)(c)];
            case 6:
                // No platform was specified. we have no option other than load all engines and offer platform list next round
                _a.sent();
                return [2 /*return*/, (0, exports.findSuitableTask)(c)];
            case 7:
                (0, logger_1.logInfo)("could not find suitable task for ".concat((0, logger_1.chalk)().white(c.command), ". GETTING OPTIONS..."));
                c.command = null;
                c.subCommand = null;
                return [2 /*return*/, (0, exports.findSuitableTask)(c)];
            case 8:
                if (!(!c.platform || c.platform === true)) return [3 /*break*/, 10];
                return [4 /*yield*/, _selectPlatform(c, suitableEngines, task)];
            case 9:
                _a.sent();
                _a.label = 10;
            case 10:
                c.runtime.engine = (0, engineManager_1.getEngineRunner)(c, task, CUSTOM_TASKS, false);
                if (!!c.runtime.engine) return [3 /*break*/, 12];
                return [4 /*yield*/, _selectPlatform(c, suitableEngines, task)];
            case 11:
                _a.sent();
                c.runtime.engine = (0, engineManager_1.getEngineRunner)(c, task, CUSTOM_TASKS);
                _a.label = 12;
            case 12:
                if (c.runtime.engine.runtimeExtraProps) {
                    c.runtime.runtimeExtraProps = c.runtime.engine.runtimeExtraProps;
                }
                (0, logger_1.logInfo)("Current Engine: ".concat((0, logger_1.chalk)().bold.white(c.runtime.engine.config.id), " path: ").concat((0, logger_1.chalk)().grey(c.runtime.engine.rootPath)));
                customTask = CUSTOM_TASKS[task];
                if (customTask) {
                    c.runtime.availablePlatforms = customTask.platforms;
                    _populateExtraParameters(c, customTask);
                    return [2 /*return*/, customTask];
                }
                return [3 /*break*/, 14];
            case 13:
                task = specificTask;
                c.runtime.engine = (0, engineManager_1.getEngineRunner)(c, task);
                _a.label = 14;
            case 14:
                c.runtime.availablePlatforms = Object.keys(c.runtime.engine.platforms || []);
                return [2 /*return*/, (0, engineManager_1.getEngineTask)(task, c.runtime.engine.tasks)];
        }
    });
}); };
exports.findSuitableTask = findSuitableTask;
var _populateExtraParameters = function (c, task) {
    if (task.params) {
        task.params.forEach(function (param) {
            var cmd = '';
            if (param.shortcut) {
                cmd += "-".concat(param.shortcut, ", ");
            }
            cmd += "--".concat(param.key);
            if (param.value) {
                if (param.isRequired) {
                    cmd += " <".concat(param.value, ">");
                }
                else {
                    cmd += " [".concat(param.value, "]");
                }
            }
            c.program.option(cmd, param.description);
        });
        c.program.parse(process.argv);
    }
};
var _selectPlatform = function (c, suitableEngines, task) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var supportedPlatforms, platforms, platform;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                supportedPlatforms = {};
                suitableEngines.forEach(function (engine) {
                    (0, engineManager_1.getEngineTask)(task, engine.tasks).platforms.forEach(function (plat) {
                        supportedPlatforms[plat] = true;
                    });
                });
                platforms = Object.keys(supportedPlatforms);
                if (!platforms.length) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'list',
                        name: 'platform',
                        message: "Pick a platform for ".concat(task),
                        choices: platforms,
                    })];
            case 1:
                platform = (_a.sent()).platform;
                c.platform = platform;
                _a.label = 2;
            case 2: return [2 /*return*/];
        }
    });
}); };
var _executePipe = function (c, task, phase) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () { return (0, tslib_1.__generator)(this, function (_a) {
    return [2 /*return*/, (0, buildHooks_1.executePipe)(c, "".concat(task.split(' ').join(':'), ":").concat(phase))];
}); }); };
var TASK_LIMIT = 20;
var executeTask = function (c, task, parentTask, originTask, isFirstTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var pt, prt;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                pt = parentTask ? "=> [".concat(parentTask, "] ") : '';
                c._currentTask = task;
                (0, logger_1.logInitTask)("".concat(pt, "=> [").concat((0, logger_1.chalk)().bold.rgb(170, 106, 170)(task), "]"));
                if (!executedTasks[task])
                    executedTasks[task] = 0;
                if (executedTasks[task] > TASK_LIMIT) {
                    return [2 /*return*/, Promise.reject("You reached maximum amount of executions per one task (".concat(TASK_LIMIT, ") task: ").concat(task, ".\nThis is to warn you ended up in task loop.\n(").concat(task, " calls same or another task which calls ").concat(task, " again)\nbut issue migh not be necessarily with this task\n\nTo avoid that test your task code against parentTask and avoid executing same task X from within task X"))];
                }
                return [4 /*yield*/, (0, exports.executeEngineTask)(c, task, parentTask, originTask, (0, engineManager_1.getEngineRunner)(c, task, CUSTOM_TASKS).tasks, isFirstTask)];
            case 1:
                _a.sent();
                // await getEngineRunner(c, task, CUSTOM_TASKS).executeTask(c, task, parentTask, originTask, isFirstTask);
                executedTasks[task]++;
                c._currentTask = parentTask;
                prt = parentTask ? "<= [".concat((0, logger_1.chalk)().rgb(170, 106, 170)(parentTask), "] ") : '';
                (0, logger_1.logExitTask)("".concat(prt, "<= ").concat(task));
                return [2 /*return*/];
        }
    });
}); };
exports.executeTask = executeTask;
var executeOrSkipTask = function (c, task, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        if (!c.program.only) {
            return [2 /*return*/, (0, exports.executeTask)(c, task, parentTask, originTask)];
        }
        return [2 /*return*/, (0, exports.executeTask)(c, constants_1.TASK_CONFIGURE_SOFT, parentTask, originTask)];
    });
}); };
exports.executeOrSkipTask = executeOrSkipTask;
var ACCEPTED_CONDITIONS = [
    'platform',
    'target',
    'appId',
    'scheme'
];
var _logSkip = function (task) {
    (0, logger_1.logInfo)("Original RNV task ".concat((0, logger_1.chalk)().white(task), " marked to ignore. SKIPPING..."));
};
var shouldSkipTask = function (c, task, originTask) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    var tasks = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.tasks;
    c.runtime.platform = c.platform;
    if (!tasks)
        return;
    if ((_b = c.program.skipTasks) === null || _b === void 0 ? void 0 : _b.split) {
        var skipTaskArr = c.program.skipTasks.split(',');
        if (skipTaskArr.includes(task))
            return true;
    }
    if (Array.isArray(tasks)) {
        var _loop_1 = function (k) {
            var t = tasks[k];
            if (t.name === task) {
                if (t.filter) {
                    var conditions = t.filter.split('&');
                    var conditionsToMatch_1 = conditions.length;
                    conditions.forEach(function (con) {
                        var conArr = con.split('=');
                        if (ACCEPTED_CONDITIONS.includes(conArr[0])) {
                            if (c.runtime[conArr[0]] === conArr[1]) {
                                conditionsToMatch_1--;
                            }
                        }
                        else {
                            (0, logger_1.logWarning)("Condition ".concat(con, " not valid. only following keys are valid: ").concat(ACCEPTED_CONDITIONS.join(','), " SKIPPING..."));
                        }
                    });
                    if (conditionsToMatch_1 === 0) {
                        if (t.ignore) {
                            _logSkip(task);
                            return { value: true };
                        }
                    }
                }
                else if (t.ignore) {
                    _logSkip(task);
                    return { value: true };
                }
            }
        };
        for (var k = 0; k < tasks.length; k++) {
            var state_1 = _loop_1(k);
            if (typeof state_1 === "object")
                return state_1.value;
        }
    }
    else {
        var ignoreTask = (_e = (_d = (_c = tasks[task]) === null || _c === void 0 ? void 0 : _c.platform) === null || _d === void 0 ? void 0 : _d[c.platform]) === null || _e === void 0 ? void 0 : _e.ignore;
        if (ignoreTask) {
            _logSkip(task);
            return true;
        }
        var ignoreTasks = ((_h = (_g = (_f = tasks[originTask]) === null || _f === void 0 ? void 0 : _f.platform) === null || _g === void 0 ? void 0 : _g[c.platform]) === null || _h === void 0 ? void 0 : _h.ignoreTasks) || [];
        if (ignoreTasks.includes(task)) {
            (0, logger_1.logInfo)("Task ".concat(task, " marked to skip during rnv ").concat(originTask, ". SKIPPING..."));
            return true;
        }
    }
    return false;
};
exports.shouldSkipTask = shouldSkipTask;
var executeEngineTask = function (c, task, parentTask, originTask, tasks, isFirstTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var needsHelp, t, inOnlyMode, doPipe;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                needsHelp = Object.prototype.hasOwnProperty.call(c.program, 'help');
                t = (0, engineManager_1.getEngineTask)(task, tasks, CUSTOM_TASKS);
                if (!(needsHelp && !parentTask)) return [3 /*break*/, 4];
                (0, logger_1.logRaw)("\n=======================================================\nINTERACTIVE HELP FOR TASK: ".concat((0, logger_1.chalk)().green(t.task), "\n\nDescription: ").concat(t.description, "\n\nOptions:\n\n").concat(t.params.map(function (v) {
                    var option = v.shortcut ? "`-".concat(v.shortcut, "`, ") : '';
                    return "".concat(option, "`--").concat(v.key, "` - ").concat(v.description);
                }).join('\n'), "\n\n  "));
                if (!t.fnHelp) return [3 /*break*/, 2];
                return [4 /*yield*/, t.fnHelp(c, parentTask, originTask)];
            case 1:
                _a.sent();
                _a.label = 2;
            case 2: return [4 /*yield*/, (0, prompt_1.pressAnyKeyToContinue)()];
            case 3:
                _a.sent();
                (0, logger_1.logRaw)("\n=======================================================");
                _a.label = 4;
            case 4:
                if (!(!t.isGlobalScope && isFirstTask)) return [3 /*break*/, 6];
                if (!c.files.project.package) return [3 /*break*/, 6];
                // This has to happen in order for hooks to be able to run
                return [4 /*yield*/, (0, npmUtils_1.checkIfProjectAndNodeModulesExists)(c)];
            case 5:
                // This has to happen in order for hooks to be able to run
                _a.sent();
                _a.label = 6;
            case 6:
                if (isFirstTask) {
                    c.runtime.forceBuildHookRebuild = t.forceBuildHookRebuild;
                }
                inOnlyMode = c.program.only;
                doPipe = !t.isGlobalScope && (!inOnlyMode || (inOnlyMode && isFirstTask));
                if (!doPipe) return [3 /*break*/, 8];
                return [4 /*yield*/, _executePipe(c, task, 'before')];
            case 7:
                _a.sent();
                _a.label = 8;
            case 8: return [4 /*yield*/, t.fn(c, parentTask, originTask)];
            case 9:
                _a.sent();
                if (!doPipe) return [3 /*break*/, 11];
                return [4 /*yield*/, _executePipe(c, task, 'after')];
            case 10:
                _a.sent();
                _a.label = 11;
            case 11: return [2 /*return*/];
        }
    });
}); };
exports.executeEngineTask = executeEngineTask;
//# sourceMappingURL=index.js.map