"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyTemplate = exports.isTemplateInstalled = exports.getInstalledTemplateOptions = exports.getTemplateOptions = exports.configureEntryPoint = exports.configureTemplateFiles = exports.checkIfTemplateConfigured = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var constants_1 = require("../constants");
var fileutils_1 = require("../systemManager/fileutils");
var logger_1 = require("../systemManager/logger");
var prompt_1 = require("../../cli/prompt");
var common_1 = require("../common");
var configManager_1 = require("../configManager");
var resolve_1 = require("../systemManager/resolve");
var npmUtils_1 = require("../systemManager/npmUtils");
var checkIfTemplateConfigured = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('checkIfTemplateConfigured');
        if (c.program.skipDependencyCheck || c.files.project.config.isTemplate)
            return [2 /*return*/, true];
        if (!c.buildConfig.templates) {
            (0, logger_1.logWarning)("Your ".concat((0, logger_1.chalk)().white(c.paths.project.config), " does not contain ").concat((0, logger_1.chalk)().white('templates'), " object. ReNative will skip template generation"));
            return [2 /*return*/, false];
        }
        Object.keys(c.buildConfig.templates).forEach(function (k) {
            var obj = c.buildConfig.templates[k];
            if (!(0, resolve_1.doResolve)(k.version, false, { basedir: '../' })
                && !(0, resolve_1.doResolve)(k, false)) {
                (0, logger_1.logInfo)("Your ".concat((0, logger_1.chalk)().white("".concat(k, "@").concat(obj.version)), " template is missing in renative.json. CONFIGURING...DONE"));
                c._requiresNpmInstall = true;
                c.runtime.requiresBootstrap = true;
            }
            if (c.files.project.package.devDependencies) {
                if (c.files.project.package.devDependencies[k] !== obj.version) {
                    (0, logger_1.logInfo)("Updating template ".concat((0, logger_1.chalk)().white("".concat(k)), " => ").concat((0, logger_1.chalk)().green(obj.version), ". ...DONE"));
                }
                c.files.project.package.devDependencies[k] = obj.version;
            }
        });
        _writeObjectSync(c, c.paths.project.package, c.files.project.package);
        return [2 /*return*/, true];
    });
}); };
exports.checkIfTemplateConfigured = checkIfTemplateConfigured;
var _cleanProjectTemplateSync = function (c) {
    (0, logger_1.logTask)('_cleanProjectTemplateSync');
    var dirsToRemove = [
        path_1.default.join(c.paths.project.appConfigBase.dir),
        path_1.default.join(c.paths.project.srcDir),
        path_1.default.join(c.paths.project.appConfigsDir)
    ];
    var filesToRemove = c.buildConfig.defaults.supportedPlatforms.map(function (p) { return path_1.default.join(c.paths.project.dir, "index.".concat(p, ".js")); });
    (0, fileutils_1.removeDirsSync)(dirsToRemove);
    // TODO: NOT SERVED FROM TEMPLATE YET
    (0, fileutils_1.removeFilesSync)(filesToRemove);
};
var _applyTemplate = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        (0, logger_1.logTask)('_applyTemplate', "current:".concat(c.buildConfig.currentTemplate, " selected:").concat(c.runtime.selectedTemplate));
        if (c.runtime.selectedTemplate) {
            _cleanProjectTemplateSync(c);
            // if (isMonorepo()) {
            //     // @todo - have the templates report their absolute locations
            //     c.paths.template.dir = path.join(
            //         getMonorepoRoot(),
            //         'packages',
            //         c.runtime.selectedTemplate
            //     );
            // } else {
            c.paths.template.dir = (0, resolve_1.doResolve)(c.runtime.selectedTemplate);
            // c.paths.template.dir = path.join(c.paths.project.nodeModulesDir, c.runtime.selectedTemplate);
            // }
        }
        else {
            c.paths.template.dir = (0, resolve_1.doResolve)(c.buildConfig.currentTemplate);
            // c.paths.template.dir = path.join(c.paths.project.nodeModulesDir, c.buildConfig.currentTemplate);
        }
        c.paths.template.configTemplate = path_1.default.join(c.paths.template.dir, constants_1.RENATIVE_CONFIG_TEMPLATE_NAME);
        c.paths.template.config = path_1.default.join(c.paths.template.dir, constants_1.RENATIVE_CONFIG_NAME);
        // if (fsExistsSync(c.paths.template.config)) {
        //     c.files.template.config = readObjectSync(c.paths.template.config);
        // }
        if (!(0, fileutils_1.fsExistsSync)(c.paths.template.configTemplate)) {
            (0, logger_1.logWarning)("Template file ".concat((0, logger_1.chalk)().white(c.paths.template.configTemplate), " does not exist. check your ").concat((0, logger_1.chalk)().white(c.paths.template.dir), ". skipping"));
            return [2 /*return*/, true];
        }
        (0, logger_1.logDebug)("_applyTemplate:".concat(c.runtime.selectedTemplate, ":").concat(c.paths.template.dir));
        c.paths.template.appConfigsDir = path_1.default.join(c.paths.template.dir, 'appConfigs');
        c.paths.template.appConfigBase.dir = path_1.default.join(c.paths.template.appConfigsDir, 'base');
        c.runtime.currentTemplate = c.files.project.config.currentTemplate;
        if (!c.runtime.currentTemplate) {
            c.runtime.currentTemplate = Object.keys(c.files.project.config.templates)[0];
            c.runtime.requiresForcedTemplateApply = true;
        }
        if (c.buildConfig.currentTemplate) {
            c.runtime.activeTemplate = "".concat(c.buildConfig.currentTemplate, "@").concat((_b = (_a = c.buildConfig.templates) === null || _a === void 0 ? void 0 : _a[c.buildConfig.currentTemplate]) === null || _b === void 0 ? void 0 : _b.version);
        }
        return [2 /*return*/, true];
    });
}); };
var _configureSrc = function (c) { return new Promise(function (resolve) {
    // Check src
    (0, logger_1.logDebug)('configureProject:check src');
    if (!(0, fileutils_1.fsExistsSync)(c.paths.project.srcDir)) {
        (0, logger_1.logInfo)("Your src folder ".concat((0, logger_1.chalk)().white(c.paths.project.srcDir), " is missing! CREATING...DONE"));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(path_1.default.join(c.paths.template.dir, 'src'), c.paths.project.srcDir);
    }
    resolve();
}); };
var _configureAppConfigs = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var appConfigIds, supPlats_1;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        // Check appConfigs
        (0, logger_1.logDebug)('configureProject:check appConfigs');
        //
        if (!(0, fileutils_1.fsExistsSync)(c.paths.project.appConfigsDir)) {
            (0, logger_1.logInfo)("Your appConfig folder ".concat((0, logger_1.chalk)().white(c.paths.project.appConfigsDir), " is missing! ReNative will create one from template."));
            // TODO: GET CORRECT PROJECT TEMPLATE
            (0, fileutils_1.copyFolderContentsRecursiveSync)(c.paths.template.appConfigsDir, c.paths.project.appConfigsDir);
            appConfigIds = (0, configManager_1.listAppConfigsFoldersSync)(c, false);
            // Update App Title to match package.json
            try {
                supPlats_1 = (_c = (_b = (_a = c.files.project) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.defaults) === null || _c === void 0 ? void 0 : _c.supportedPlatforms;
                appConfigIds.forEach(function (v) {
                    var _a, _b, _c, _d;
                    var appConfigPath = path_1.default.join(c.paths.project.appConfigsDir, v, constants_1.RENATIVE_CONFIG_NAME);
                    var appConfig = (0, fileutils_1.readObjectSync)(appConfigPath);
                    if (appConfig) {
                        if (appConfig.skipBootstrapCopy) {
                            (0, fileutils_1.fsUnlinkSync)(appConfigPath);
                            if (v !== 'base') {
                                (0, fileutils_1.removeDirSync)(path_1.default.join(c.paths.project.appConfigsDir, v));
                            }
                        }
                        else if (!appConfig.hidden) {
                            appConfig.common = appConfig.common || {};
                            appConfig.common.title = (_b = (_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.defaults) === null || _b === void 0 ? void 0 : _b.title;
                            appConfig.common.id = (_d = (_c = c.files.project.config) === null || _c === void 0 ? void 0 : _c.defaults) === null || _d === void 0 ? void 0 : _d.id;
                            if (supPlats_1) {
                                Object.keys(appConfig.platforms).forEach(function (pk) {
                                    if (!supPlats_1.includes(pk)) {
                                        delete appConfig.platforms[pk];
                                    }
                                });
                            }
                            _writeObjectSync(c, appConfigPath, appConfig);
                        }
                    }
                });
            }
            catch (e) {
                (0, logger_1.logError)(e);
            }
        }
        return [2 /*return*/];
    });
}); };
var _configureProjectConfig = function (c) { return new Promise(function (resolve) {
    // Check projectConfigs
    (0, logger_1.logDebug)('configureProject:check projectConfigs');
    if (!(0, fileutils_1.fsExistsSync)(c.paths.project.appConfigBase.dir)) {
        (0, logger_1.logInfo)("Your projectConfig folder ".concat((0, logger_1.chalk)().white(c.paths.project.appConfigBase.dir), " is missing! CREATING...DONE"));
        (0, fileutils_1.copyFolderContentsRecursiveSync)(c.paths.template.appConfigBase.dir, c.paths.project.appConfigBase.dir);
    }
    resolve();
}); };
var _configureRenativeConfig = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var templateConfig, mergedObj;
    return (0, tslib_1.__generator)(this, function (_a) {
        templateConfig = (0, fileutils_1.readObjectSync)(c.paths.template.configTemplate);
        (0, logger_1.logDebug)('configureProject:check renative.json');
        if (c.runtime.selectedTemplate
            || c.runtime.requiresForcedTemplateApply
            || c.files.project.config.isNew) {
            (0, logger_1.logInfo)("Your ".concat(c.paths.project.config, " needs to be updated with ").concat(c.paths.template.configTemplate, ". UPDATING...DONE"));
            mergedObj = (0, fileutils_1.mergeObjects)(c, templateConfig, c.files.project.config_original, false, true);
            // Do not override supportedPlatforms
            mergedObj.defaults.supportedPlatforms = c.files.project.config_original.defaults.supportedPlatforms;
            // Do not override engines
            mergedObj.engines = c.files.project.config_original.engines;
            // Set current template
            mergedObj.currentTemplate = c.runtime.currentTemplate;
            if (mergedObj.isNew) {
                c.runtime.isFirstRunAfterNew = true;
            }
            // mergedObj.isNew = null;
            delete mergedObj.isNew;
            delete mergedObj.templateConfig;
            // c.files.project.config = mergedObj;
            _writeObjectSync(c, c.paths.project.config, mergedObj);
            (0, configManager_1.loadFileExtended)(c, c.files.project, c.paths.project, 'config');
        }
        return [2 /*return*/, true];
    });
}); };
var configureTemplateFiles = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var templateConfig, includedPaths;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        (0, logger_1.logTask)('configureTemplateFiles');
        templateConfig = (0, fileutils_1.readObjectSync)(c.paths.template.configTemplate);
        includedPaths = (_a = templateConfig === null || templateConfig === void 0 ? void 0 : templateConfig.templateConfig) === null || _a === void 0 ? void 0 : _a.includedPaths;
        if (includedPaths) {
            includedPaths.forEach(function (name) {
                var sourcePath = path_1.default.join(c.paths.template.dir, name);
                var destPath = path_1.default.join(c.paths.project.dir, name);
                if (!(0, fileutils_1.fsExistsSync)(destPath) && (0, fileutils_1.fsExistsSync)(sourcePath)) {
                    try {
                        if ((0, fileutils_1.fsLstatSync)(sourcePath).isDirectory()) {
                            (0, logger_1.logInfo)("Missing directory ".concat((0, logger_1.chalk)().white("".concat(destPath, ".js")), ". COPYING from TEMPATE...DONE"));
                            (0, fileutils_1.copyFolderContentsRecursiveSync)(sourcePath, destPath);
                        }
                        else {
                            (0, logger_1.logInfo)("Missing file ".concat((0, logger_1.chalk)().white("".concat(destPath, ".js")), ". COPYING from TEMPATE...DONE"));
                            (0, fileutils_1.copyFileSync)(sourcePath, destPath);
                        }
                    }
                    catch (e) {
                        // Get some beer and order hookers
                    }
                }
            });
        }
        return [2 /*return*/];
    });
}); };
exports.configureTemplateFiles = configureTemplateFiles;
var configureEntryPoint = function (c, platform) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var entryFile, source, dest;
    return (0, tslib_1.__generator)(this, function (_a) {
        (0, logger_1.logTask)('configureEntryPoint');
        if (c.files.project.config.isTemplate)
            return [2 /*return*/, true];
        entryFile = (0, common_1.getConfigProp)(c, platform, 'entryFile');
        try {
            source = path_1.default.join(c.paths.template.dir, "".concat(entryFile, ".js"));
            dest = path_1.default.join(c.paths.project.dir, "".concat(entryFile, ".js"));
            if (!(0, fileutils_1.fsExistsSync)(dest)) {
                if (!entryFile) {
                    (0, logger_1.logWarning)("Missing entryFile key for ".concat((0, logger_1.chalk)().white(c.platform), " platform in your ").concat((0, logger_1.chalk)().white(c.paths.appConfig.config), "."));
                }
                else if (!(0, fileutils_1.fsExistsSync)(source)) {
                    (0, logger_1.logWarning)("Missing entry file ".concat((0, logger_1.chalk)().white("".concat(entryFile, ".js")), ". Not available in your current template. You need to create one manually"));
                }
                else {
                    (0, logger_1.logInfo)("Missing entry file ".concat((0, logger_1.chalk)().white("".concat(entryFile, ".js")), ". COPYING from TEMPATE...DONE"));
                    (0, fileutils_1.copyFileSync)(source, dest);
                }
            }
        }
        catch (e) {
            return [2 /*return*/, Promise.reject(e)];
        }
        return [2 /*return*/, true];
    });
}); };
exports.configureEntryPoint = configureEntryPoint;
var _writeObjectSync = function (c, p, s) {
    (0, fileutils_1.writeFileSync)(p, s);
    (0, configManager_1.generateBuildConfig)(c);
};
var getTemplateOptions = function (c, isGlobalScope) {
    var defaultProjectTemplates;
    if (isGlobalScope) {
        defaultProjectTemplates = c.files.rnv.projectTemplates.config.projectTemplates;
    }
    else {
        defaultProjectTemplates = c.buildConfig.projectTemplates || {};
    }
    return (0, prompt_1.generateOptions)(defaultProjectTemplates, false, null, function (i, obj, mapping, defaultVal) {
        var _a;
        var exists = (_a = c.buildConfig.templates) === null || _a === void 0 ? void 0 : _a[defaultVal];
        var installed = exists ? (0, logger_1.chalk)().yellow(' (installed)') : '';
        return " [".concat((0, logger_1.chalk)().grey(i + 1), "]> ").concat((0, logger_1.chalk)().bold(defaultVal)).concat(installed, " \n");
    });
};
exports.getTemplateOptions = getTemplateOptions;
var getInstalledTemplateOptions = function (c) {
    if (c.files.project.config.isTemplate)
        return [];
    if (c.buildConfig.templates) {
        return (0, prompt_1.generateOptions)(c.buildConfig.templates);
    }
    (0, logger_1.logError)("You don't have any local templates installed", false, true);
    return [];
};
exports.getInstalledTemplateOptions = getInstalledTemplateOptions;
var isTemplateInstalled = function (c) { return (c.buildConfig.currentTemplate
    ? (0, resolve_1.doResolve)(c.buildConfig.currentTemplate) : false); };
exports.isTemplateInstalled = isTemplateInstalled;
var applyTemplate = function (c, selectedTemplate) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var opts, template, templateIsInstalled;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('applyTemplate', "".concat(c.buildConfig.currentTemplate, "=>").concat(selectedTemplate));
                if (c.files.project.config.isTemplate)
                    return [2 /*return*/, true];
                c.runtime.selectedTemplate = selectedTemplate;
                if (!!c.buildConfig.currentTemplate) return [3 /*break*/, 2];
                (0, logger_1.logWarning)("You don't have any current template selected");
                opts = (0, exports.getInstalledTemplateOptions)(c);
                return [4 /*yield*/, inquirer_1.default.prompt({
                        type: 'list',
                        name: 'template',
                        message: 'Pick which template to apply',
                        choices: opts.keysAsArray
                    })];
            case 1:
                template = (_a.sent()).template;
                c.buildConfig.currentTemplate = template;
                c.files.project.config.currentTemplate = template;
                _writeObjectSync(c, c.paths.project.config, c.files.project.config);
                _a.label = 2;
            case 2:
                templateIsInstalled = (0, resolve_1.doResolve)(c.buildConfig.currentTemplate);
                if (!!templateIsInstalled) return [3 /*break*/, 4];
                // We Need template to be installed before other dependency resolutions (due to scoping)
                return [4 /*yield*/, (0, npmUtils_1.checkIfProjectAndNodeModulesExists)(c)];
            case 3:
                // We Need template to be installed before other dependency resolutions (due to scoping)
                _a.sent();
                _a.label = 4;
            case 4: return [4 /*yield*/, _applyTemplate(c)];
            case 5:
                _a.sent();
                return [4 /*yield*/, _configureSrc(c)];
            case 6:
                _a.sent();
                return [4 /*yield*/, _configureAppConfigs(c)];
            case 7:
                _a.sent();
                return [4 /*yield*/, _configureProjectConfig(c)];
            case 8:
                _a.sent();
                return [4 /*yield*/, _configureRenativeConfig(c)];
            case 9:
                _a.sent();
                return [2 /*return*/];
        }
    });
}); };
exports.applyTemplate = applyTemplate;
//# sourceMappingURL=index.js.map