"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvAppConfigure = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var configManager_1 = require("../../core/configManager");
var runtimeManager_1 = require("../../core/runtimeManager");
var constants_1 = require("../../core/constants");
var fileutils_1 = require("../../core/systemManager/fileutils");
var logger_1 = require("../../core/systemManager/logger");
var prompt_1 = require("../../cli/prompt");
var _loadAppConfigIDfromDir = function (dirName, appConfigsDir) {
    (0, logger_1.logDebug)("_loadAppConfigIDfromDir:".concat(dirName, ":").concat(appConfigsDir), (0, logger_1.chalk)().grey);
    var filePath = path_1.default.join(appConfigsDir, 'renative.json');
    if ((0, fileutils_1.fsExistsSync)(filePath)) {
        try {
            var renativeConf = JSON.parse((0, fileutils_1.fsReadFileSync)(filePath));
            return { dir: dirName, id: renativeConf.id };
        }
        catch (e) {
            (0, logger_1.logError)("File ".concat(filePath, " is MALFORMED:\n").concat(e));
        }
    }
    return { dir: dirName, id: null };
};
var _askUserAboutConfigs = function (c, dir, id, basePath) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var choice, conf, filePath, fileContents;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('_askUserAboutConfigs');
                (0, logger_1.logWarning)("AppConfig error - It seems you have a mismatch between appConfig folder name (".concat(dir, ") and the id defined in renative.json (").concat(id, "). They must match."));
                if (c.program.ci === true) {
                    throw new Error('You cannot continue if you set --ci flag. please fix above error first');
                }
                return [4 /*yield*/, inquirer_1.default.prompt({
                        type: 'list',
                        name: 'choice',
                        message: 'You must choose what you want to keep',
                        choices: [
                            // {
                            //     name: `Keep ID from renative.json (${id}) and rename the folder (${dir} -> ${id})`,
                            //     value: 'keepID'
                            // },
                            {
                                name: "Keep folder name (".concat(dir, ") and rename the ID from renative.json (").concat(id, " -> ").concat(dir, ")"),
                                value: 'keepFolder'
                            },
                            new inquirer_1.default.Separator(),
                            {
                                name: "I'll do it manually",
                                value: 'manually'
                            }
                        ]
                    })];
            case 1:
                choice = (_a.sent()).choice;
                if (choice === 'manually') {
                    throw new Error('Please do the changes and rerun the command');
                }
                conf = { id: id, dir: dir };
                if (choice === 'keepID') {
                    conf.dir = path_1.default.join(basePath, id);
                    (0, fileutils_1.fsRenameSync)(path_1.default.join(basePath, dir), conf.dir);
                }
                if (choice === 'keepFolder') {
                    filePath = path_1.default.join(basePath, dir, 'renative.json');
                    fileContents = JSON.parse((0, fileutils_1.fsReadFileSync)(filePath));
                    fileContents.id = dir;
                    conf.id = dir;
                    (0, fileutils_1.writeFileSync)(filePath, fileContents);
                }
                return [2 /*return*/, conf];
        }
    });
}); };
/* eslint-disable no-await-in-loop */
var matchAppConfigID = function (c, appConfigID) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _a, appConfigsDirs, appConfigsDirNames, acIndex, conf, dir, id;
    return (0, tslib_1.__generator)(this, function (_b) {
        switch (_b.label) {
            case 0:
                (0, logger_1.logTask)('matchAppConfigID', "appId:".concat(appConfigID));
                if (!appConfigID)
                    return [2 /*return*/, false];
                _a = c.paths.project, appConfigsDirs = _a.appConfigsDirs, appConfigsDirNames = _a.appConfigsDirNames;
                acIndex = appConfigsDirNames.indexOf(appConfigID);
                if (!(acIndex !== -1)) return [3 /*break*/, 3];
                conf = _loadAppConfigIDfromDir(appConfigID, appConfigsDirs[acIndex]);
                dir = conf.dir, id = conf.id;
                if (!(id !== dir)) return [3 /*break*/, 2];
                return [4 /*yield*/, _askUserAboutConfigs(c, conf.dir, conf.id, path_1.default.join(appConfigsDirs[acIndex], '..'))];
            case 1:
                conf = _b.sent();
                _b.label = 2;
            case 2: return [2 /*return*/, conf.id];
            case 3: return [2 /*return*/, false];
        }
    });
}); };
var _findAndSwitchAppConfigDir = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var appConfigsDirNames, conf;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('_findAndSwitchAppConfigDir');
                appConfigsDirNames = c.paths.project.appConfigsDirNames;
                if (!appConfigsDirNames.length) return [3 /*break*/, 2];
                if (appConfigsDirNames.length === 1) {
                    // we have only one, skip the question
                    (0, logger_1.logInfo)("Found only one app config available. Will use ".concat((0, logger_1.chalk)().white(appConfigsDirNames[0])));
                    _setAppId(c, appConfigsDirNames[0]);
                    return [2 /*return*/, true];
                }
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'conf',
                        type: 'list',
                        message: 'Which one would you like to pick?',
                        choices: appConfigsDirNames,
                        pageSize: 50,
                        logMessage: 'ReNative found multiple existing appConfigs'
                    })];
            case 1:
                conf = (_a.sent()).conf;
                if (conf) {
                    _setAppId(c, conf);
                    return [2 /*return*/, true];
                }
                _a.label = 2;
            case 2: return [2 /*return*/, false];
        }
    });
}); };
var _setAppId = function (c, appId) {
    var _a, _b, _c;
    var currentAppConfigId = (_c = (_b = (_a = c.files.project) === null || _a === void 0 ? void 0 : _a.configLocal) === null || _b === void 0 ? void 0 : _b._meta) === null || _c === void 0 ? void 0 : _c.currentAppConfigId;
    (0, logger_1.logTask)('_setAppId', "appId:".concat(appId, " runtime.appId:").concat(c.runtime.appId, " _meta.appId:").concat(currentAppConfigId));
    c.runtime.appId = appId || c.runtime.appId || currentAppConfigId;
    c.runtime.appDir = path_1.default.join(c.paths.project.builds.dir, "".concat(c.runtime.appId, "_").concat(c.platform));
};
var taskRnvAppConfigure = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var appConfigsDirsExt, hasAppConfig, aid, hasAppConfig;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        switch (_c.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvAppConfigure');
                c.paths.project.appConfigsDirNames = (0, configManager_1.listAppConfigsFoldersSync)(c, true);
                c.paths.project.appConfigsDirNames.forEach(function (dirName) {
                    c.paths.project.appConfigsDirs.push(path_1.default.join(c.paths.project.appConfigsDir, dirName));
                });
                appConfigsDirsExt = (_b = (_a = c.buildConfig) === null || _a === void 0 ? void 0 : _a.paths) === null || _b === void 0 ? void 0 : _b.appConfigsDirs;
                if (appConfigsDirsExt) {
                    appConfigsDirsExt.forEach(function (apePath) {
                        var appConfigsExt = (0, configManager_1.listAppConfigsFoldersSync)(c, true, apePath);
                        appConfigsExt.forEach(function (appExtName) {
                            c.paths.project.appConfigsDirNames.push(appExtName);
                            c.paths.project.appConfigsDirs.push(path_1.default.join(apePath, appExtName));
                        });
                    });
                }
                // Reset appId if appConfig no longer exists but renative.local.json still has reference to it
                if (!c.paths.project.appConfigsDirNames.includes(c.runtime.appId)) {
                    c.runtime.appId = null;
                }
                if (!(c.program.appConfigID === true || (!c.program.appConfigID && !c.runtime.appId))) return [3 /*break*/, 2];
                return [4 /*yield*/, _findAndSwitchAppConfigDir(c)];
            case 1:
                hasAppConfig = _c.sent();
                if (!hasAppConfig) {
                    // await executeTask(c, TASK_APP_CREATE, TASK_APP_CONFIGURE);
                    // return Promise.reject('No app configs found for this project');
                    (0, logger_1.logWarning)('No app configs found for this project');
                    return [2 /*return*/, true];
                }
                return [3 /*break*/, 6];
            case 2:
                if (!c.program.appConfigID) return [3 /*break*/, 6];
                return [4 /*yield*/, matchAppConfigID(c, c.program.appConfigID)];
            case 3:
                aid = _c.sent();
                if (!!aid) return [3 /*break*/, 5];
                (0, logger_1.logWarning)("Cannot find app config ".concat((0, logger_1.chalk)().white(c.program.appConfigID)));
                return [4 /*yield*/, _findAndSwitchAppConfigDir(c)];
            case 4:
                hasAppConfig = _c.sent();
                if (!hasAppConfig) {
                    // await executeTask(c, TASK_APP_CREATE, TASK_APP_CONFIGURE);
                    // return Promise.reject('No app configs found for this project');
                    (0, logger_1.logWarning)('No app configs found for this project');
                    return [2 /*return*/, true];
                }
                _c.label = 5;
            case 5:
                _setAppId(c, aid);
                _c.label = 6;
            case 6:
                // Generate true path to appConfig (ensure external appConfigsDirs are included)
                c.runtime.appConfigDir = c.paths.project.appConfigsDirs[c.paths.project.appConfigsDirNames.indexOf(c.runtime.appId)];
                return [4 /*yield*/, (0, runtimeManager_1.updateRenativeConfigs)(c)];
            case 7:
                _c.sent();
                (0, logger_1.logAppInfo)(c);
                return [2 /*return*/, true];
        }
    });
}); };
exports.taskRnvAppConfigure = taskRnvAppConfigure;
exports.default = {
    description: 'Configure project with specific appConfig',
    fn: exports.taskRnvAppConfigure,
    task: constants_1.TASK_APP_CONFIGURE,
    params: constants_1.PARAMS.withBase(constants_1.PARAMS.withConfigure()),
    platforms: [],
};
//# sourceMappingURL=task.rnv.app.configure.js.map