"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvAppCreate = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var constants_1 = require("../../core/constants");
var logger_1 = require("../../core/systemManager/logger");
var prompt_1 = require("../../cli/prompt");
var runtimeManager_1 = require("../../core/runtimeManager");
var fileutils_1 = require("../../core/systemManager/fileutils");
var resolve_1 = require("../../core/systemManager/resolve");
var taskRnvAppCreate = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var sourcePath, sourceAppConfigDirPath, tacPath, tacDirsPath_1, tacDirs, appConfigChoicesObj_1, appConfigChoices_1, acDirs, tacPath, tacDirsPath_2, tacDirs, sourceAppConfig, destPath, appConfigId, confName, confObjPath, confObj, appConfigTitle, confTitle, bundleId, confId;
    var _a, _b;
    return (0, tslib_1.__generator)(this, function (_c) {
        switch (_c.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvAppCreate');
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 1:
                _c.sent();
                if (!c.program.sourceAppConfigID) return [3 /*break*/, 2];
                sourceAppConfigDirPath = path_1.default.join(c.paths.project.appConfigsDir, c.program.sourceAppConfigID);
                if ((0, fileutils_1.fsExistsSync)(sourceAppConfigDirPath)) {
                    sourcePath = sourceAppConfigDirPath;
                }
                return [3 /*break*/, 5];
            case 2:
                if (!c.program.ci) return [3 /*break*/, 3];
                tacPath = (0, resolve_1.doResolve)(c.buildConfig.currentTemplate);
                if ((0, fileutils_1.fsExistsSync)(tacPath)) {
                    tacDirsPath_1 = path_1.default.join(tacPath, 'appConfigs');
                    tacDirs = (0, fileutils_1.fsReaddirSync)(tacDirsPath_1);
                    tacDirs.forEach(function (v) {
                        if (v !== 'base') {
                            var pth = path_1.default.join(tacDirsPath_1, v);
                            if ((0, fileutils_1.fsLstatSync)(pth).isDirectory()) {
                                sourcePath = pth;
                            }
                        }
                    });
                }
                return [3 /*break*/, 5];
            case 3:
                appConfigChoicesObj_1 = {};
                appConfigChoices_1 = [];
                acDirs = (0, fileutils_1.fsReaddirSync)(c.paths.project.appConfigsDir);
                acDirs.forEach(function (v) {
                    if (v !== 'base') {
                        var pth = path_1.default.join(c.paths.project.appConfigsDir, v);
                        if ((0, fileutils_1.fsLstatSync)(pth).isDirectory()) {
                            var key = "project>".concat(v);
                            appConfigChoices_1.push(key);
                            appConfigChoicesObj_1[key] = {
                                path: pth
                            };
                        }
                    }
                });
                tacPath = (0, resolve_1.doResolve)(c.buildConfig.currentTemplate);
                if ((0, fileutils_1.fsExistsSync)(tacPath)) {
                    tacDirsPath_2 = path_1.default.join(tacPath, 'appConfigs');
                    tacDirs = (0, fileutils_1.fsReaddirSync)(tacDirsPath_2);
                    tacDirs.forEach(function (v) {
                        if (v !== 'base') {
                            var pth = path_1.default.join(tacDirsPath_2, v);
                            if ((0, fileutils_1.fsLstatSync)(pth).isDirectory()) {
                                var key = "template>".concat(v);
                                appConfigChoices_1.push(key);
                                appConfigChoicesObj_1[key] = {
                                    path: pth
                                };
                            }
                        }
                    });
                }
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'sourceAppConfig',
                        type: 'list',
                        choices: appConfigChoices_1,
                        message: 'Which App config to use as copy source?',
                    })];
            case 4:
                sourceAppConfig = (_c.sent()).sourceAppConfig;
                sourcePath = appConfigChoicesObj_1[sourceAppConfig].path;
                _c.label = 5;
            case 5:
                if (!c.program.appConfigID) return [3 /*break*/, 6];
                appConfigId = c.program.appConfigID.toLowerCase();
                destPath = path_1.default.join(c.paths.project.appConfigsDir, appConfigId);
                return [3 /*break*/, 8];
            case 6: return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                    name: 'confName',
                    type: 'input',
                    message: 'Type name of app config (lowercase)',
                })];
            case 7:
                confName = (_c.sent()).confName;
                appConfigId = confName.toLowerCase();
                destPath = path_1.default.join(c.paths.project.appConfigsDir, appConfigId);
                _c.label = 8;
            case 8:
                (0, logger_1.logInfo)('Copying new app config...');
                return [4 /*yield*/, (0, fileutils_1.copyFolderContentsRecursive)(sourcePath, destPath)];
            case 9:
                _c.sent();
                (0, logger_1.logInfo)('Copying new app config...DONE');
                confObjPath = path_1.default.join(destPath, 'renative.json');
                confObj = (0, fileutils_1.readObjectSync)(confObjPath);
                confObj.id = appConfigId;
                confObj.common = confObj.common || {};
                if (!c.program.title) return [3 /*break*/, 10];
                appConfigTitle = c.program.title;
                return [3 /*break*/, 13];
            case 10:
                if (!c.program.ci) return [3 /*break*/, 11];
                return [3 /*break*/, 13];
            case 11: return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                    name: 'confTitle',
                    type: 'input',
                    default: ((_a = confObj.common) === null || _a === void 0 ? void 0 : _a.title) || '',
                    message: 'Type the title of the app',
                })];
            case 12:
                confTitle = (_c.sent()).confTitle;
                appConfigTitle = confTitle;
                _c.label = 13;
            case 13:
                confObj.common.title = appConfigTitle || confObj.common.title;
                if (!c.program.id) return [3 /*break*/, 14];
                bundleId = c.program.id;
                return [3 /*break*/, 17];
            case 14:
                if (!c.program.ci) return [3 /*break*/, 15];
                return [3 /*break*/, 17];
            case 15: return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                    name: 'confId',
                    type: 'input',
                    default: ((_b = confObj.common) === null || _b === void 0 ? void 0 : _b.id) || '',
                    message: 'Type the id of the app (bundle identifier)',
                })];
            case 16:
                confId = (_c.sent()).confId;
                bundleId = confId;
                _c.label = 17;
            case 17:
                confObj.common.id = bundleId || confObj.common.id;
                (0, fileutils_1.writeFileSync)(confObjPath, confObj);
                return [2 /*return*/, true];
        }
    });
}); };
exports.taskRnvAppCreate = taskRnvAppCreate;
exports.default = {
    description: 'Create new app config',
    fn: exports.taskRnvAppCreate,
    task: constants_1.TASK_APP_CREATE,
    params: constants_1.PARAMS.withBase(),
    skipPlatforms: true,
    platforms: [],
};
//# sourceMappingURL=task.rnv.app.create.js.map