"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvClean = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var fileutils_1 = require("../../core/systemManager/fileutils");
var logger_1 = require("../../core/systemManager/logger");
var exec_1 = require("../../core/systemManager/exec");
var constants_1 = require("../../core/constants");
var utils_1 = require("../../core/systemManager/utils");
function clearWindowsCacheFiles() {
    var opts = {
        detached: false,
        stdio: 'ignore'
    };
    // TODO using executeAsync for these scripts returns an error, so this is just a temporary workaround
    // This should resolve as it used internally by @react-native-community/cli
    // eslint-disable-next-line global-require
    var child_process_1 = require('child_process');
    // Temporary cache files located in C:/Users/<UserName>/AppData/Local/Temp
    child_process_1.spawn('cmd.exe', ['/C', 'del /q/f/s %TEMP%\\*'], opts);
    // NuGet cache
    child_process_1.spawn('cmd.exe', ['/C', 'dotnet nuget locals all --clear'], opts);
    // Yarn/NPM cache
    child_process_1.spawn('cmd.exe', ['/C', 'npm cache clean --force & yarn cache clean --all'], opts);
    // Watchman cache
    child_process_1.spawn('cmd.exe', ['/C', 'watchman watch-del-all'], opts);
    return true;
}
var taskRnvClean = function (c, skipQuestionParam) {
    if (skipQuestionParam === void 0) { skipQuestionParam = false; }
    return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var skipQuestion, pathsToRemove, localFiles, immediateNodeModuleDir, pkgLock, yarnLock, packagesFolder, buildDirs, local1, local2, local3, answers, confirm_1, confirmBuilds, confirmLocals, confirmCache, e_1;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0:
                    (0, logger_1.logTask)('taskRnvClean');
                    skipQuestion = c.program.ci ? true : skipQuestionParam;
                    pathsToRemove = [];
                    localFiles = [];
                    immediateNodeModuleDir = path_1.default.join(c.paths.project.dir, 'node_modules');
                    pkgLock = path_1.default.join(c.paths.project.dir, 'package-lock.json');
                    if ((0, fileutils_1.fsExistsSync)(immediateNodeModuleDir)) {
                        pathsToRemove.push(immediateNodeModuleDir);
                    }
                    if ((0, fileutils_1.fsExistsSync)(pkgLock))
                        pathsToRemove.push(pkgLock);
                    yarnLock = path_1.default.join(c.paths.project.dir, 'yarn.lock');
                    if ((0, fileutils_1.fsExistsSync)(yarnLock)) {
                        pathsToRemove.push(yarnLock);
                    }
                    packagesFolder = path_1.default.join(c.paths.project.dir, 'packages');
                    if ((0, fileutils_1.fsExistsSync)(packagesFolder)) {
                        (0, fileutils_1.fsReaddirSync)(packagesFolder).forEach(function (dir) {
                            if (dir === '.DS_Store') {
                                var pth = path_1.default.join(packagesFolder, dir);
                                if ((0, fileutils_1.fsExistsSync)(pth)) {
                                    pathsToRemove.push(pth);
                                }
                            }
                            else {
                                var pth2 = path_1.default.join(packagesFolder, dir, 'node_modules');
                                if ((0, fileutils_1.fsExistsSync)(pth2)) {
                                    pathsToRemove.push(pth2);
                                }
                                var pth3 = path_1.default.join(packagesFolder, dir, 'package-lock.json');
                                if ((0, fileutils_1.fsExistsSync)(pth3)) {
                                    pathsToRemove.push(pth3);
                                }
                                var pth4 = path_1.default.join(packagesFolder, dir, 'yarn.lock');
                                if ((0, fileutils_1.fsExistsSync)(pth4)) {
                                    pathsToRemove.push(pth4);
                                }
                                var pth5 = path_1.default.join(packagesFolder, dir, 'dist');
                                if ((0, fileutils_1.fsExistsSync)(pth5)) {
                                    pathsToRemove.push(pth5);
                                }
                            }
                        });
                    }
                    buildDirs = [];
                    if ((0, fileutils_1.fsExistsSync)(c.paths.project.builds.dir)) {
                        buildDirs.push(c.paths.project.builds.dir);
                    }
                    if ((0, fileutils_1.fsExistsSync)(c.paths.project.assets.dir)) {
                        buildDirs.push(c.paths.project.assets.dir);
                    }
                    local1 = path_1.default.join(c.paths.project.dir, '.DS_Store');
                    if ((0, fileutils_1.fsExistsSync)(local1))
                        localFiles.push(local1);
                    local2 = path_1.default.join(c.paths.project.dir, 'renative.local.json');
                    if ((0, fileutils_1.fsExistsSync)(local2))
                        localFiles.push(local2);
                    local3 = path_1.default.join(c.paths.project.dir, 'metro.config.local.js');
                    if ((0, fileutils_1.fsExistsSync)(local3))
                        localFiles.push(local3);
                    answers = {
                        modules: false,
                        builds: false,
                        cache: false,
                        nothingToClean: !skipQuestion
                    };
                    if (!pathsToRemove.length) return [3 /*break*/, 3];
                    if (!!skipQuestion) return [3 /*break*/, 2];
                    return [4 /*yield*/, inquirer_1.default.prompt({
                            name: 'confirm',
                            type: 'confirm',
                            message: "Do you want to remove node_module related files/folders? \n".concat((0, logger_1.chalk)().red(pathsToRemove.join('\n')))
                        })];
                case 1:
                    confirm_1 = (_a.sent()).confirm;
                    answers.modules = confirm_1;
                    if (confirm_1)
                        answers.nothingToClean = false;
                    return [3 /*break*/, 3];
                case 2:
                    answers.modules = true;
                    _a.label = 3;
                case 3:
                    if (!buildDirs.length) return [3 /*break*/, 6];
                    if (!!skipQuestion) return [3 /*break*/, 5];
                    return [4 /*yield*/, inquirer_1.default.prompt({
                            name: 'confirmBuilds',
                            type: 'confirm',
                            message: "Do you want to clean your platformBuilds and platformAssets? \n".concat((0, logger_1.chalk)().red(buildDirs.join('\n')))
                        })];
                case 4:
                    confirmBuilds = (_a.sent()).confirmBuilds;
                    answers.builds = confirmBuilds;
                    if (confirmBuilds)
                        answers.nothingToClean = false;
                    return [3 /*break*/, 6];
                case 5:
                    answers.builds = true;
                    _a.label = 6;
                case 6:
                    if (!localFiles.length) return [3 /*break*/, 9];
                    if (!!skipQuestion) return [3 /*break*/, 8];
                    return [4 /*yield*/, inquirer_1.default.prompt({
                            name: 'confirmLocals',
                            type: 'confirm',
                            message: "Do you want to clean local files? \n".concat((0, logger_1.chalk)().red(localFiles.join('\n')))
                        })];
                case 7:
                    confirmLocals = (_a.sent()).confirmLocals;
                    answers.locals = confirmLocals;
                    if (confirmLocals)
                        answers.nothingToClean = false;
                    return [3 /*break*/, 9];
                case 8:
                    answers.locals = true;
                    _a.label = 9;
                case 9:
                    if (!!skipQuestion) return [3 /*break*/, 11];
                    return [4 /*yield*/, inquirer_1.default.prompt({
                            name: 'confirmCache',
                            type: 'confirm',
                            message: 'Do you want to clean your npm/bundler cache?'
                        })];
                case 10:
                    confirmCache = (_a.sent()).confirmCache;
                    answers.cache = confirmCache;
                    if (confirmCache)
                        answers.nothingToClean = false;
                    return [3 /*break*/, 12];
                case 11:
                    answers.cache = true;
                    _a.label = 12;
                case 12:
                    if (answers.nothingToClean) {
                        (0, logger_1.logToSummary)('Nothing to clean');
                        return [2 /*return*/, Promise.resolve()];
                    }
                    if (!answers.modules) return [3 /*break*/, 14];
                    return [4 /*yield*/, (0, fileutils_1.removeDirs)(pathsToRemove)];
                case 13:
                    _a.sent();
                    _a.label = 14;
                case 14:
                    if (!answers.builds) return [3 /*break*/, 16];
                    return [4 /*yield*/, (0, fileutils_1.removeDirs)(buildDirs)];
                case 15:
                    _a.sent();
                    _a.label = 16;
                case 16:
                    if (!answers.locals) return [3 /*break*/, 18];
                    return [4 /*yield*/, (0, fileutils_1.removeDirs)(localFiles)];
                case 17:
                    _a.sent();
                    _a.label = 18;
                case 18:
                    if (!answers.cache) return [3 /*break*/, 24];
                    if (!utils_1.isSystemWin) return [3 /*break*/, 19];
                    clearWindowsCacheFiles();
                    return [3 /*break*/, 24];
                case 19:
                    _a.trys.push([19, 21, , 22]);
                    return [4 /*yield*/, (0, exec_1.executeAsync)(c, 'watchman watch-del-all')];
                case 20:
                    _a.sent();
                    return [3 /*break*/, 22];
                case 21:
                    e_1 = _a.sent();
                    (0, logger_1.logDebug)('watchman not installed. skipping');
                    return [3 /*break*/, 22];
                case 22: return [4 /*yield*/, (0, exec_1.executeAsync)(c, 'rm -rf $TMPDIR/metro-* && rm -rf $TMPDIR/react-* && rm -rf $TMPDIR/haste-*')];
                case 23:
                    _a.sent();
                    _a.label = 24;
                case 24: return [2 /*return*/, true];
            }
        });
    });
};
exports.taskRnvClean = taskRnvClean;
exports.default = {
    description: 'Automatically removes all node_modules and lock in your project and its dependencies',
    fn: exports.taskRnvClean,
    task: 'clean',
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    isGlobalScope: true
};
//# sourceMappingURL=task.rnv.clean.js.map