"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvCryptoDecrypt = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var tar_1 = (0, tslib_1.__importDefault)(require("tar"));
var logger_1 = require("../../core/systemManager/logger");
var fileutils_1 = require("../../core/systemManager/fileutils");
var prompt_1 = require("../../cli/prompt");
var crypto_1 = require("../../core/systemManager/crypto");
var taskManager_1 = require("../../core/taskManager");
var constants_1 = require("../../core/constants");
var iocane = require('iocane');
var _unzipAndCopy = function (c, shouldCleanFolder, destTemp, wsPath, ts, destFolder) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                if (!shouldCleanFolder) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, fileutils_1.cleanFolder)(wsPath)];
            case 1:
                _a.sent();
                _a.label = 2;
            case 2: return [4 /*yield*/, tar_1.default.x({
                    file: destTemp,
                    cwd: c.paths.workspace.dir
                })];
            case 3:
                _a.sent();
                (0, fileutils_1.removeFilesSync)([destTemp]);
                if ((0, fileutils_1.fsExistsSync)(ts)) {
                    (0, fileutils_1.copyFileSync)(ts, path_1.default.join(c.paths.workspace.dir, c.files.project.package.name, 'timestamp'));
                }
                (0, logger_1.logSuccess)("Files succesfully extracted into ".concat(destFolder));
                return [2 /*return*/];
        }
    });
}); };
var taskRnvCryptoDecrypt = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var crypto, sourceRaw, source, ts, destFolder, destTemp, envVar, shouldCleanFolder, wsPath, isCryptoReset, options, option, confirm_1, key, data, e_1, err;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvCryptoDecrypt');
                if (!!parentTask) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_PROJECT_CONFIGURE, constants_1.TASK_CRYPTO_DECRYPT, originTask)];
            case 1:
                _d.sent();
                _d.label = 2;
            case 2:
                crypto = (_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.crypto;
                sourceRaw = (_b = crypto === null || crypto === void 0 ? void 0 : crypto.decrypt) === null || _b === void 0 ? void 0 : _b.source;
                if (!(!crypto.isOptional && sourceRaw)) return [3 /*break*/, 14];
                source = "".concat((0, fileutils_1.getRealPath)(c, sourceRaw, 'decrypt.source'));
                ts = "".concat(source, ".timestamp");
                destFolder = path_1.default.join(c.paths.workspace.dir, c.files.project.package.name);
                destTemp = "".concat(path_1.default.join(c.paths.workspace.dir, c.files.project.package.name.replace('/', '-')), ".tgz");
                envVar = (0, crypto_1.getEnvVar)(c);
                shouldCleanFolder = false;
                wsPath = path_1.default.join(c.paths.workspace.dir, c.files.project.package.name);
                isCryptoReset = c.command === 'crypto' && c.program.reset === true;
                if (!(c.program.ci !== true && !isCryptoReset)) return [3 /*break*/, 4];
                options = [
                    'Yes - override (recommended)',
                    'Yes - merge',
                    'Skip'
                ];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        name: 'option',
                        type: 'list',
                        choices: options,
                        message: "How to decrypt to ".concat((0, logger_1.chalk)().white(destFolder), " ?")
                    })];
            case 3:
                option = (_d.sent()).option;
                if (option === options[0]) {
                    shouldCleanFolder = true;
                }
                else if (option === options[2]) {
                    return [2 /*return*/, true];
                }
                return [3 /*break*/, 5];
            case 4:
                shouldCleanFolder = true;
                _d.label = 5;
            case 5:
                if (!(0, fileutils_1.fsExistsSync)(destTemp)) return [3 /*break*/, 8];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'confirm',
                        message: "Found existing decrypted file at ".concat((0, logger_1.chalk)().white(destTemp), ". want to use it and skip decrypt ?")
                    })];
            case 6:
                confirm_1 = (_d.sent()).confirm;
                if (!confirm_1) return [3 /*break*/, 8];
                return [4 /*yield*/, _unzipAndCopy(c, shouldCleanFolder, destTemp, wsPath, ts, destFolder)];
            case 7:
                _d.sent();
                return [2 /*return*/, true];
            case 8:
                key = c.program.key || c.process.env[envVar];
                if (!key) {
                    return [2 /*return*/, Promise.reject("encrypt: You must pass ".concat((0, logger_1.chalk)().white('--key'), " or have env var defined:\n\n").concat((0, crypto_1.getEnvExportCmd)(envVar, 'REPLACE_WITH_ENV_VARIABLE'), "\n\n"))];
                }
                if (!(0, fileutils_1.fsExistsSync)(source)) {
                    return [2 /*return*/, Promise.reject("Can't decrypt. ".concat((0, logger_1.chalk)().white(source), " is missing!"))];
                }
                data = void 0;
                _d.label = 9;
            case 9:
                _d.trys.push([9, 11, , 12]);
                return [4 /*yield*/, iocane.createSession()
                        .use('cbc')
                        .decrypt((0, fileutils_1.fsReadFileSync)(source), key)];
            case 10:
                data = _d.sent();
                return [3 /*break*/, 12];
            case 11:
                e_1 = _d.sent();
                if ((_c = e_1 === null || e_1 === void 0 ? void 0 : e_1.message) === null || _c === void 0 ? void 0 : _c.includes) {
                    if (e_1.message.includes('Signature mismatch')) {
                        err = "You're trying to decode crypto file encoded with previous version of crypto.\nthis change was introduced in \"rnv@0.29.0\"\n\n".concat(e_1, "\n\n      ").concat((0, logger_1.chalk)().green('SUGGESTION:'), "\n\n      ").concat((0, logger_1.chalk)().yellow('STEP 1:'), "\n      run: ").concat((0, logger_1.chalk)().white('rnv crypto encrypt'), " locally at least once and commit the result back to your repository\n\n      ").concat((0, logger_1.chalk)().yellow('STEP 2:'), "\n      you should be able to use: ").concat((0, logger_1.chalk)().white('rnv crypto decrypt'), " properly now\n\n      ").concat((0, logger_1.chalk)().yellow('IF ALL HOPE IS LOST:'), "\n      Raise new issue and copy this SUMMARY box output at:\n      ").concat((0, logger_1.chalk)().white('https://github.com/renative-org/renative/issues'), "\n      and we will try to help!\n\n      ");
                        return [2 /*return*/, Promise.reject(err)];
                    }
                    if (e_1.message.includes('Authentication failed')) {
                        return [2 /*return*/, Promise.reject("It seems like you provided invalid decryption key.\n\n".concat(e_1.stack, "\n\n").concat((0, logger_1.chalk)().green('SUGGESTION:'), "\n\n").concat((0, logger_1.chalk)().yellow('STEP 1:'), "\ncheck if your ENV VAR is correct: ").concat((0, crypto_1.getEnvExportCmd)(envVar, '***********'), "\nor if someone did not encrypt ").concat((0, logger_1.chalk)().white(source), " with a different key\n\n").concat((0, logger_1.chalk)().yellow('STEP 2:'), "\nrun crypto decrypt again\n\n").concat((0, logger_1.chalk)().yellow('IF ALL HOPE IS LOST:'), "\nRaise new issue and copy this SUMMARY box output at:\n").concat((0, logger_1.chalk)().white('https://github.com/renative-org/renative/issues'), "\nand we will try to help!\n\n"))];
                    }
                }
                return [2 /*return*/, Promise.reject(e_1)];
            case 12:
                (0, fileutils_1.fsWriteFileSync)(destTemp, data);
                //         try {
                //             await executeAsync(
                //                 c,
                //                 `${_getOpenSllPath(
                //                     c
                //                 )} enc -aes-256-cbc -md md5 -d -in ${source} -out ${destTemp} -k ${key}`,
                //                 { privateParams: [key] }
                //             );
                //         } catch (e) {
                //             const cmd1 = chalk().white(
                //                 `openssl enc -aes-256-cbc -md md5 -d -in ${source} -out ${destTemp} -k $${envVar}`
                //             );
                //             return Promise.reject(`${e}
                // ${chalk().green('SUGGESTION:')}
                // ${chalk().yellow('STEP 1:')}
                // ${cmd1}
                // ${chalk().yellow('STEP 2:')}
                // ${chalk().white(
                //         'run your previous command again and choose to skip openssl once asked'
                //     )}`);
                //         }
                return [4 /*yield*/, _unzipAndCopy(c, shouldCleanFolder, destTemp, wsPath, ts, destFolder)];
            case 13:
                //         try {
                //             await executeAsync(
                //                 c,
                //                 `${_getOpenSllPath(
                //                     c
                //                 )} enc -aes-256-cbc -md md5 -d -in ${source} -out ${destTemp} -k ${key}`,
                //                 { privateParams: [key] }
                //             );
                //         } catch (e) {
                //             const cmd1 = chalk().white(
                //                 `openssl enc -aes-256-cbc -md md5 -d -in ${source} -out ${destTemp} -k $${envVar}`
                //             );
                //             return Promise.reject(`${e}
                // ${chalk().green('SUGGESTION:')}
                // ${chalk().yellow('STEP 1:')}
                // ${cmd1}
                // ${chalk().yellow('STEP 2:')}
                // ${chalk().white(
                //         'run your previous command again and choose to skip openssl once asked'
                //     )}`);
                //         }
                _d.sent();
                return [3 /*break*/, 15];
            case 14:
                (0, logger_1.logWarning)("You don't have {{ crypto.encrypt.dest }} specificed in ".concat((0, logger_1.chalk)().white(c.paths.appConfigBase)));
                return [2 /*return*/, true];
            case 15: return [2 /*return*/];
        }
    });
}); };
exports.taskRnvCryptoDecrypt = taskRnvCryptoDecrypt;
exports.default = {
    description: 'Decrypt encrypted project files into local ~/<wokspace>/<project>/..',
    fn: exports.taskRnvCryptoDecrypt,
    task: constants_1.TASK_CRYPTO_DECRYPT,
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    skipPlatforms: true,
};
//# sourceMappingURL=task.rnv.crypto.decrypt.js.map