"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvCryptoEncrypt = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var tar_1 = (0, tslib_1.__importDefault)(require("tar"));
var util_1 = require("util");
var logger_1 = require("../../core/systemManager/logger");
var fileutils_1 = require("../../core/systemManager/fileutils");
var prompt_1 = require("../../cli/prompt");
var crypto_1 = require("../../core/systemManager/crypto");
var taskManager_1 = require("../../core/taskManager");
var constants_1 = require("../../core/constants");
var iocane = require('iocane');
var readdirAsync = (0, util_1.promisify)(fileutils_1.fsReaddir);
var generateRandomKey = function (length) { return Array(length)
    .fill('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz#$%^&*')
    .map(function (x) { return x[Math.floor(Math.random() * x.length)]; })
    .join(''); };
var _checkAndConfigureCrypto = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var source, location_1, sourceFolder, appConfigsDirs, envVar, key, keyGenerated, confirm_1;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                source = "./".concat(c.files.project.package.name);
                if (!(c.files.project.config && !c.files.project.config.crypto)) return [3 /*break*/, 2];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'input',
                        name: 'location',
                        message: 'Where would you like your secrets to be residing? (path relative to root, without leading or trailing slash. Ex. `myPrivateConfig/encrypt`)',
                        default: 'secrets'
                    })];
            case 1:
                location_1 = (_a.sent()).location;
                c.files.project.config_original.crypto = {
                    encrypt: {
                        dest: "PROJECT_HOME/".concat(location_1, "/privateConfigs.enc")
                    },
                    decrypt: {
                        source: "PROJECT_HOME/".concat(location_1, "/privateConfigs.enc")
                    }
                };
                (0, fileutils_1.writeFileSync)(c.paths.project.config, c.files.project.config_original);
                _a.label = 2;
            case 2:
                sourceFolder = path_1.default.join(c.paths.workspace.dir, source);
                if (!!(0, fileutils_1.fsExistsSync)(sourceFolder)) return [3 /*break*/, 5];
                (0, logger_1.logInfo)("It seems you are running encrypt for the first time. Directory ".concat((0, logger_1.chalk)().white(sourceFolder), " does not exist yet.\nRNV will create it for you, make sure you add whatever you want encrypted in it and then run the command again"));
                (0, fileutils_1.mkdirSync)(sourceFolder);
                (0, fileutils_1.mkdirSync)(path_1.default.join(sourceFolder, 'certs'));
                (0, fileutils_1.writeFileSync)(path_1.default.join(sourceFolder, 'renative.private.json'), {});
                return [4 /*yield*/, readdirAsync(c.paths.project.appConfigsDir)];
            case 3:
                appConfigsDirs = _a.sent();
                appConfigsDirs.forEach(function (item) {
                    var appConfigDir = path_1.default.join(sourceFolder, item);
                    (0, fileutils_1.mkdirSync)(appConfigDir);
                    (0, fileutils_1.mkdirSync)(path_1.default.join(appConfigDir, 'certs'));
                    (0, fileutils_1.writeFileSync)(path_1.default.join(appConfigDir, 'renative.private.json'), {});
                });
                // writeFileSync(path.join(sourceFolder), c.files.project.config);
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'confirm',
                        message: 'Once ready, Continue?'
                    })];
            case 4:
                // writeFileSync(path.join(sourceFolder), c.files.project.config);
                _a.sent();
                _a.label = 5;
            case 5:
                envVar = (0, crypto_1.getEnvVar)(c);
                key = c.program.key || c.process.env[envVar];
                keyGenerated = false;
                if (!!key) return [3 /*break*/, 7];
                return [4 /*yield*/, (0, prompt_1.inquirerPrompt)({
                        type: 'confirm',
                        message: "You haven't passed a key with --key or set an env variable named ".concat((0, logger_1.chalk)().yellow(envVar), " for the encryption key. Would you like to generate one?")
                    })];
            case 6:
                confirm_1 = (_a.sent()).confirm;
                if (confirm_1) {
                    key = generateRandomKey(20);
                    keyGenerated = true;
                }
                else {
                    return [2 /*return*/, Promise.reject("encrypt: You must pass ".concat((0, logger_1.chalk)().white('--key'), " or have env var defined:\n\n").concat((0, crypto_1.getEnvExportCmd)(envVar, 'REPLACE_WITH_ENV_VARIABLE'), "\n\n"))];
                }
                if (keyGenerated) {
                    (0, logger_1.logSuccess)("The files were encrypted with key ".concat((0, logger_1.chalk)().red(key), ". Make sure you keep it safe! Pass it with --key on decryption or set it as following env variable:\n\n").concat((0, crypto_1.getEnvExportCmd)(envVar, key), "\n\n"));
                    c.process.env[envVar] = key;
                }
                _a.label = 7;
            case 7: return [2 /*return*/];
        }
    });
}); };
var taskRnvCryptoEncrypt = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var source, destRaw, tsWorkspacePath, envVar, key, dest, destTemp, timestamp, destFolder, data;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvCryptoEncrypt');
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_PROJECT_CONFIGURE, constants_1.TASK_CRYPTO_ENCRYPT, originTask)];
            case 1:
                _d.sent();
                source = "./".concat(c.files.project.package.name);
                return [4 /*yield*/, _checkAndConfigureCrypto(c)];
            case 2:
                _d.sent();
                destRaw = (_c = (_b = (_a = c.files.project.config) === null || _a === void 0 ? void 0 : _a.crypto) === null || _b === void 0 ? void 0 : _b.encrypt) === null || _c === void 0 ? void 0 : _c.dest;
                tsWorkspacePath = path_1.default.join(c.paths.workspace.dir, c.files.project.package.name, 'timestamp');
                envVar = (0, crypto_1.getEnvVar)(c);
                key = c.program.key || c.process.env[envVar];
                if (!destRaw) return [3 /*break*/, 5];
                dest = "".concat((0, fileutils_1.getRealPath)(c, destRaw, 'encrypt.dest'));
                destTemp = "".concat(path_1.default.join(c.paths.workspace.dir, c.files.project.package.name.replace('/', '-')), ".tgz");
                timestamp = new Date().getTime();
                destFolder = path_1.default.join(dest, '../../core/');
                !(0, fileutils_1.fsExistsSync)(destFolder) && (0, fileutils_1.mkdirSync)(destFolder);
                return [4 /*yield*/, tar_1.default.c({
                        gzip: true,
                        file: destTemp,
                        cwd: c.paths.workspace.dir
                    }, [source])];
            case 3:
                _d.sent();
                return [4 /*yield*/, iocane.createSession()
                        .use('cbc')
                        .encrypt((0, fileutils_1.fsReadFileSync)(destTemp), key)];
            case 4:
                data = _d.sent();
                (0, fileutils_1.fsWriteFileSync)(dest, data);
                // await executeAsync(
                //     c,
                //     `${_getOpenSllPath(
                //         c
                //     )} enc -aes-256-cbc -md md5 -salt -in ${destTemp} -out ${dest} -k ${key}`,
                //     { privateParams: [key] }
                // );
                (0, fileutils_1.removeFilesSync)([destTemp]);
                (0, fileutils_1.fsWriteFileSync)("".concat(dest, ".timestamp"), "".concat(timestamp));
                (0, fileutils_1.fsWriteFileSync)("".concat(tsWorkspacePath), "".concat(timestamp));
                (0, logger_1.logSuccess)("Files succesfully encrypted into ".concat(dest));
                return [3 /*break*/, 6];
            case 5:
                (0, logger_1.logWarning)("You don't have {{ crypto.encrypt.dest }} specificed in ".concat((0, logger_1.chalk)().white(c.paths.appConfigBase)));
                _d.label = 6;
            case 6: return [2 /*return*/];
        }
    });
}); };
exports.taskRnvCryptoEncrypt = taskRnvCryptoEncrypt;
exports.default = {
    description: 'Encrypts secure files from ~/<wokspace>/<project>/.. to project',
    fn: exports.taskRnvCryptoEncrypt,
    task: constants_1.TASK_CRYPTO_ENCRYPT,
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    skipPlatforms: true,
};
//# sourceMappingURL=task.rnv.crypto.encrypt.js.map