"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvDoctor = void 0;
var tslib_1 = require("tslib");
var lodash_get_1 = (0, tslib_1.__importDefault)(require("lodash.get"));
var logger_1 = require("../../core/systemManager/logger");
var constants_1 = require("../../core/constants");
var taskManager_1 = require("../../core/taskManager");
var runtimeManager_1 = require("../../core/runtimeManager");
var fileutils_1 = require("../../core/systemManager/fileutils");
var schemaManager_1 = require("../../core/schemaManager");
var configTargets = [
    'workspace.config',
    'workspace.project.config',
    'workspace.appConfig.configs',
    'project.config',
    'appConfig.configs'
];
var taskRnvDoctor = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var configPaths, errMsg, hasErrors;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvDoctor');
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 1:
                _a.sent();
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_APP_CONFIGURE, parentTask, originTask)];
            case 2:
                _a.sent();
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 3:
                _a.sent();
                configPaths = [];
                configTargets.forEach(function (target) {
                    var tPath = (0, lodash_get_1.default)(c.paths, target);
                    if (tPath) {
                        if (Array.isArray(tPath)) {
                            configPaths.push.apply(configPaths, tPath);
                        }
                        else {
                            configPaths.push(tPath);
                        }
                    }
                });
                errMsg = 'RENATIVE JSON SCHEMA VALIDITY CHECK:\n\n';
                hasErrors = false;
                configPaths.forEach(function (cPath) {
                    if ((0, fileutils_1.fsExistsSync)(cPath)) {
                        var cObj = (0, fileutils_1.readObjectSync)(cPath);
                        var _a = (0, schemaManager_1.validateRenativeJsonSchema)(cObj), valid = _a[0], ajv = _a[1];
                        if (!valid) {
                            hasErrors = true;
                            // console.log('ERROR', ajv.errors);
                            errMsg += (0, logger_1.chalk)().yellow("\nInvalid schema in ".concat(
                            // cPath}. ISSUES: ${JSON.stringify(ajv.errors, null, 2)}\n`);
                            cPath, ". ISSUES:\n\n"));
                            ajv.errors.forEach(function (err) {
                                errMsg += (0, logger_1.chalk)().yellow("".concat((0, logger_1.chalk)().grey(err.dataPath === '' ? '/' : err.dataPath), ": ").concat(err.message, " ").concat(Object.keys(err.params).map(function (k) { return "=> ".concat((0, logger_1.chalk)().red(err.params[k])); }).join('\n'), "\n"));
                            });
                        }
                    }
                });
                if (!hasErrors) {
                    errMsg += (0, logger_1.chalk)().green("PASSED ".concat(configPaths.length, " files"));
                }
                // const [valid, ajv] = validateRuntimeObjectSchema(c);
                // if (!valid) {
                //     console.log('ERROR', ajv.errors);
                // }
                (0, logger_1.logToSummary)(errMsg);
                return [2 /*return*/];
        }
    });
}); };
exports.taskRnvDoctor = taskRnvDoctor;
exports.default = {
    description: 'Checks validity and config health of your project',
    fn: exports.taskRnvDoctor,
    task: constants_1.TASK_DOCTOR,
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    isGlobalScope: true
};
//# sourceMappingURL=task.rnv.doctor.js.map