"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvNew = void 0;
var tslib_1 = require("tslib");
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var lodash_set_1 = (0, tslib_1.__importDefault)(require("lodash.set"));
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var semver_1 = (0, tslib_1.__importDefault)(require("semver"));
var constants_1 = require("../../core/constants");
var templateManager_1 = require("../../core/templateManager");
var fileutils_1 = require("../../core/systemManager/fileutils");
var projectManager_1 = require("../../core/projectManager");
var workspace_1 = require("../../core/projectManager/workspace");
var runtimeManager_1 = require("../../core/runtimeManager");
var analytics_1 = (0, tslib_1.__importDefault)(require("../../core/systemManager/analytics"));
var exec_1 = require("../../core/systemManager/exec");
var gitUtils_1 = require("../../core/systemManager/gitUtils");
var logger_1 = require("../../core/systemManager/logger");
var npmUtils_1 = require("../../core/systemManager/npmUtils");
var highlight = (0, logger_1.chalk)().green;
var _prepareProjectOverview = function (c, data) {
    data.appTitle = data.inputAppTitle || data.defaultAppTitle;
    data.teamID = '';
    data.appID = data.inputAppID
        ? data.inputAppID.replace(/\s+/g, '-').toLowerCase()
        : data.appID;
    data.version = data.inputVersion || data.defaultVersion;
    var tempString = "".concat(data.optionTemplates.selectedOption, "@").concat(data.optionTemplates.selectedVersion);
    var str = (0, logger_1.printBoxStart)('🚀  ReNative Project Generator');
    str += (0, logger_1.printIntoBox)('');
    str += (0, logger_1.printIntoBox)("Project Name (folder): ".concat(highlight(data.projectName)), 1);
    str += (0, logger_1.printIntoBox)("Workspace: ".concat(highlight(data.optionWorkspaces.selectedOption)), 1);
    str += (0, logger_1.printIntoBox)("Project Title: ".concat(highlight(data.appTitle)), 1);
    str += (0, logger_1.printIntoBox)("Project Version: ".concat(highlight(data.version)), 1);
    str += (0, logger_1.printIntoBox)("App ID: ".concat(highlight(data.appID)), 1);
    str += (0, logger_1.printIntoBox)("Project Template: ".concat(highlight(tempString)), 1);
    str += (0, logger_1.printIntoBox)("Git Enabled: ".concat(highlight(data.gitEnabled)), 1);
    str += (0, logger_1.printIntoBox)('');
    str += (0, logger_1.printIntoBox)('Project Platforms:');
    str += (0, logger_1.printArrIntoBox)(data.optionPlatforms.selectedOptions);
    str += (0, logger_1.printIntoBox)('');
    str += (0, logger_1.printIntoBox)('Project Structure:');
    str += (0, logger_1.printIntoBox)('');
    str += (0, logger_1.printIntoBox)(data.projectName);
    str += (0, logger_1.chalk)()
        .gray("\u2502   \u251C\u2500\u2500 appConfigs            # Application flavour configuration files/assets \u2502\n\u2502   \u2502   \u2514\u2500\u2500 [APP_ID]          # Example application flavour                    \u2502\n\u2502   \u2502       \u251C\u2500\u2500 assets        # Platform assets injected to ./platformAssets   \u2502\n\u2502   \u2502       \u251C\u2500\u2500 builds        # Platform files injected to ./platformBuilds    \u2502\n\u2502   \u2502       \u251C\u2500\u2500 fonts             # Folder for all custom fonts                \u2502\n\u2502   \u2502       \u251C\u2500\u2500 plugins           # Multi-platform plugins injections          \u2502\n\u2502   \u2502       \u2514\u2500\u2500 renative.json # Application flavour config                     \u2502\n\u2502   \u251C\u2500\u2500 platformAssets        # Generated cross-platform assets                \u2502\n\u2502   \u251C\u2500\u2500 platformBuilds        # Generated platform app projects                \u2502\n\u2502   \u251C\u2500\u2500 src                   # Source code files                              \u2502\n\u2502   \u251C\u2500\u2500 index.*.js            # Entry files                                    \u2502\n\u2502   \u2514\u2500\u2500 renative.json         # ReNative project configuration                 \u2502\n");
    str += (0, logger_1.printIntoBox)('');
    str += (0, logger_1.printBoxEnd)();
    str += '\n';
    data.confirmString = str;
};
var interactiveQuestion = function (results, bootstrapQuestions, providedAnswers) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _loop_1, i;
    var _a, _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                if (!(bootstrapQuestions === null || bootstrapQuestions === void 0 ? void 0 : bootstrapQuestions.length)) return [3 /*break*/, 4];
                _loop_1 = function (i) {
                    var q, qKey, qKeyClean, choicesObj, answer, value, inqQuestion, result, val;
                    return (0, tslib_1.__generator)(this, function (_e) {
                        switch (_e.label) {
                            case 0:
                                q = bootstrapQuestions[i];
                                qKey = q.configProp.key;
                                qKeyClean = qKey.replace('.', '__');
                                choicesObj = {};
                                if (q.options) {
                                    q.options.forEach(function (opt) {
                                        choicesObj[opt.title] = opt;
                                    });
                                }
                                answer = providedAnswers[qKey.replace('__', '.')];
                                if (!answer) return [3 /*break*/, 1];
                                value = void 0;
                                if (typeof answer === 'string' && q.type === 'list') {
                                    value = (_a = choicesObj[answer]) === null || _a === void 0 ? void 0 : _a.value;
                                }
                                else {
                                    value = answer;
                                }
                                results[qKey] = {
                                    answer: answer,
                                    configProp: q.configProp,
                                    value: value,
                                };
                                return [3 /*break*/, 3];
                            case 1:
                                inqQuestion = {
                                    name: qKeyClean,
                                    type: q.type,
                                    message: q.title,
                                    choices: Object.keys(choicesObj),
                                };
                                return [4 /*yield*/, inquirer_1.default.prompt(inqQuestion)];
                            case 2:
                                result = _e.sent();
                                val = q.type === 'list' ? (_b = choicesObj[result[qKeyClean]]) === null || _b === void 0 ? void 0 : _b.value : result[qKeyClean];
                                results[qKey] = {
                                    answer: result[qKeyClean],
                                    configProp: q.configProp,
                                    value: val,
                                };
                                _e.label = 3;
                            case 3:
                                if (!((_c = choicesObj[results[qKey].answer]) === null || _c === void 0 ? void 0 : _c.bootstrapQuestions)) return [3 /*break*/, 5];
                                // eslint-disable-next-line no-await-in-loop
                                return [4 /*yield*/, interactiveQuestion(results, choicesObj[results[qKey].answer].bootstrapQuestions, providedAnswers)];
                            case 4:
                                // eslint-disable-next-line no-await-in-loop
                                _e.sent();
                                _e.label = 5;
                            case 5: return [2 /*return*/];
                        }
                    });
                };
                i = 0;
                _d.label = 1;
            case 1:
                if (!(i < bootstrapQuestions.length)) return [3 /*break*/, 4];
                return [5 /*yield**/, _loop_1(i)];
            case 2:
                _d.sent();
                _d.label = 3;
            case 3:
                i++;
                return [3 /*break*/, 1];
            case 4: return [2 /*return*/];
        }
    });
}); };
var taskRnvNew = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var _a, ci, projectName, title, id, appVersion, workspace, projectTemplate, templateVersion, platform, gitEnabled, confirmInRnvProject, confirmDeleteNodeModules, data, inputProjectName, inputProjectNameObj, confirm_1, inputAppTitle, inputAppID, inputVersion, answers, inputWorkspace, answer, options, getTemplateKey, customTemplate, selectedInputTemplate, inputTemplate, inputTemplateCustom, inputTemplateVersion, confirmAddTemplate, renativeTemplateConfig, renativeConfig, supportedPlatforms, inputSupportedPlatforms, answer, renativeTemplateConfigExt, bootstrapQuestions, results, providedAnswers, isGitEnabled, response, confirm_2, e_1, templates, config, supPlats;
    var _b, _c, _d, _e;
    return (0, tslib_1.__generator)(this, function (_f) {
        switch (_f.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvNew');
                _a = c.program, ci = _a.ci, projectName = _a.projectName, title = _a.title, id = _a.id, appVersion = _a.appVersion, workspace = _a.workspace, projectTemplate = _a.projectTemplate, templateVersion = _a.templateVersion, platform = _a.platform, gitEnabled = _a.gitEnabled;
                if (!(0, fileutils_1.fsExistsSync)(c.paths.project.config)) return [3 /*break*/, 2];
                (0, logger_1.logWarning)("You are in ReNative project. Found: ".concat(c.paths.project.config));
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'confirmInRnvProject',
                        type: 'confirm',
                        message: 'Are you sure you want to continue?',
                    })];
            case 1:
                confirmInRnvProject = (_f.sent()).confirmInRnvProject;
                if (!confirmInRnvProject) {
                    return [2 /*return*/, Promise.reject('Cancelled')];
                }
                _f.label = 2;
            case 2:
                if (!(0, fileutils_1.fsExistsSync)(c.paths.project.nodeModulesDir)) return [3 /*break*/, 5];
                (0, logger_1.logWarning)("Found node_modules directory at your location. If you continue it will be deleted: ".concat(c.paths.project.nodeModulesDir));
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'confirmDeleteNodeModules',
                        type: 'confirm',
                        message: 'Are you sure you want to continue?',
                    })];
            case 3:
                confirmDeleteNodeModules = (_f.sent()).confirmDeleteNodeModules;
                if (!confirmDeleteNodeModules) return [3 /*break*/, 5];
                return [4 /*yield*/, (0, fileutils_1.removeDirs)([c.paths.project.nodeModulesDir])];
            case 4:
                _f.sent();
                _f.label = 5;
            case 5:
                data = {
                    defaultVersion: '0.1.0',
                    defaultTemplate: '@rnv/template-starter',
                    defaultProjectName: 'helloRenative',
                    defaultAppTitle: 'Hello Renative',
                    defaultWorkspace: 'rnv',
                };
                data.optionPlatforms = {};
                data.optionTemplates = {};
                data.optionWorkspaces = (0, workspace_1.getWorkspaceOptions)(c);
                if (!(projectName && projectName !== '')) return [3 /*break*/, 6];
                inputProjectName = projectName;
                return [3 /*break*/, 8];
            case 6: return [4 /*yield*/, inquirer_1.default.prompt({
                    name: 'inputProjectName',
                    type: 'input',
                    validate: function (value) { return !!value; },
                    message: "What's your project Name? (no spaces, folder based on ID will be created in this directory)",
                })];
            case 7:
                inputProjectNameObj = _f.sent();
                inputProjectName = inputProjectNameObj === null || inputProjectNameObj === void 0 ? void 0 : inputProjectNameObj.inputProjectName;
                _f.label = 8;
            case 8:
                data.projectName = inputProjectName;
                c.paths.project.dir = path_1.default.join(constants_1.CURRENT_DIR, data.projectName.replace(/(\s+)/g, '_'));
                if (!(0, fileutils_1.fsExistsSync)(c.paths.project.dir)) return [3 /*break*/, 11];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        type: 'confirm',
                        name: 'confirm',
                        message: "Folder ".concat(c.paths.project.dir, " already exists. RNV will override it. Continue?"),
                    })];
            case 9:
                confirm_1 = (_f.sent()).confirm;
                if (!confirm_1) {
                    return [2 /*return*/, Promise.reject('Cancelled by user')];
                }
                return [4 /*yield*/, (0, fileutils_1.cleanFolder)(c.paths.project.dir)];
            case 10:
                _f.sent();
                _f.label = 11;
            case 11:
                (0, fileutils_1.mkdirSync)(c.paths.project.dir);
                if (!(title
                    && title !== ''
                    && id
                    && id !== ''
                    && appVersion
                    && appVersion !== '')) return [3 /*break*/, 12];
                inputAppTitle = title;
                inputAppID = id;
                inputVersion = appVersion;
                return [3 /*break*/, 14];
            case 12: return [4 /*yield*/, inquirer_1.default.prompt([
                    {
                        name: 'inputAppTitle',
                        type: 'input',
                        default: data.defaultAppTitle,
                        validate: function (val) { return !!val || 'Please enter a title'; },
                        message: "What's your project Title?",
                    },
                    {
                        name: 'inputAppID',
                        type: 'input',
                        default: function () {
                            data.appID = "com.mycompany.".concat(inputProjectName
                                .replace(/\s+/g, '')
                                .toLowerCase());
                            return data.appID;
                        },
                        validate: function (appId) { return !!appId.match(/[a-z]+\.[a-z0-9]+\.[a-z0-9]+/)
                            || 'Please enter a valid appID (com.test.app)'; },
                        message: "What's your App ID?",
                    },
                    {
                        name: 'inputVersion',
                        type: 'input',
                        default: data.defaultVersion,
                        validate: function (v) { return !!semver_1.default.valid(semver_1.default.coerce(v))
                            || 'Please enter a valid semver version (1.0.0, 42.6.7.9.3-alpha, etc.)'; },
                        message: "What's your Version?",
                    },
                ])];
            case 13:
                answers = _f.sent();
                inputAppTitle = answers === null || answers === void 0 ? void 0 : answers.inputAppTitle;
                inputAppID = answers === null || answers === void 0 ? void 0 : answers.inputAppID;
                inputVersion = answers === null || answers === void 0 ? void 0 : answers.inputVersion;
                _f.label = 14;
            case 14:
                if (!(workspace && workspace !== '')) return [3 /*break*/, 15];
                inputWorkspace = workspace;
                return [3 /*break*/, 18];
            case 15:
                if (!ci) return [3 /*break*/, 16];
                inputWorkspace = data.defaultWorkspace;
                return [3 /*break*/, 18];
            case 16: return [4 /*yield*/, inquirer_1.default.prompt([
                    {
                        name: 'inputWorkspace',
                        type: 'list',
                        message: 'What workspace to use?',
                        default: data.defaultWorkspace,
                        choices: data.optionWorkspaces.keysAsArray,
                    },
                ])];
            case 17:
                answer = _f.sent();
                inputWorkspace = answer === null || answer === void 0 ? void 0 : answer.inputWorkspace;
                _f.label = 18;
            case 18:
                data.optionWorkspaces.selectedOption = inputWorkspace;
                c.runtime.selectedWorkspace = inputWorkspace;
                return [4 /*yield*/, (0, runtimeManager_1.updateRenativeConfigs)(c)];
            case 19:
                _f.sent();
                data.optionTemplates = (0, templateManager_1.getTemplateOptions)(c);
                options = [];
                Object.keys(data.optionTemplates.valuesAsObject).forEach(function (k) {
                    var val = data.optionTemplates.valuesAsObject[k];
                    if (val.description) {
                        val.title = "".concat(k, " ").concat((0, logger_1.chalk)().grey("- ".concat(val.description)));
                    }
                    else {
                        val.title = k;
                    }
                    val.key = k;
                    options.push(val.title);
                });
                getTemplateKey = function (val) { var _a; return (_a = data.optionTemplates.valuesAsArray.find(function (v) { return v.title === val; })) === null || _a === void 0 ? void 0 : _a.key; };
                customTemplate = 'Custom Template ...';
                // data.optionTemplates.keysAsArray.push(customTemplate);
                options.push(customTemplate);
                if (!(projectTemplate && projectTemplate !== '')) return [3 /*break*/, 20];
                selectedInputTemplate = projectTemplate;
                return [3 /*break*/, 24];
            case 20: return [4 /*yield*/, inquirer_1.default.prompt({
                    name: 'inputTemplate',
                    type: 'list',
                    message: 'What template to use?',
                    default: data.defaultTemplate,
                    choices: options,
                })];
            case 21:
                inputTemplate = (_f.sent()).inputTemplate;
                if (!(inputTemplate === customTemplate)) return [3 /*break*/, 23];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'inputTemplateCustom',
                        type: 'input',
                        message: 'Type exact name of your template NPM package.',
                    })];
            case 22:
                inputTemplateCustom = (_f.sent()).inputTemplateCustom;
                selectedInputTemplate = inputTemplateCustom;
                return [3 /*break*/, 24];
            case 23:
                selectedInputTemplate = getTemplateKey(inputTemplate);
                _f.label = 24;
            case 24:
                data.optionTemplates.selectedOption = selectedInputTemplate;
                if (!(templateVersion && templateVersion !== '')) return [3 /*break*/, 25];
                inputTemplateVersion = templateVersion;
                return [3 /*break*/, 27];
            case 25: return [4 /*yield*/, (0, npmUtils_1.listAndSelectNpmVersion)(c, data.optionTemplates.selectedOption, Object.keys(c.files.rnv.projectTemplates.config.projectTemplates))];
            case 26:
                inputTemplateVersion = _f.sent();
                _f.label = 27;
            case 27:
                data.optionTemplates.selectedVersion = inputTemplateVersion;
                return [4 /*yield*/, (0, exec_1.executeAsync)("".concat((0, npmUtils_1.isYarnInstalled)() ? 'yarn' : 'npm', " add ").concat(selectedInputTemplate, "@").concat(inputTemplateVersion), {
                        cwd: c.paths.project.dir,
                    })];
            case 28:
                _f.sent();
                // Check if node_modules folder exists
                if (!(0, fileutils_1.fsExistsSync)(path_1.default.join(c.paths.project.dir, 'node_modules'))) {
                    (0, logger_1.logError)("".concat((0, npmUtils_1.isYarnInstalled)() ? 'yarn' : 'npm', " add ").concat(selectedInputTemplate, "@").concat(inputTemplateVersion, " : FAILED. this could happen if you have package.json accidentally created somewhere in parent directory"));
                    return [2 /*return*/];
                }
                if (!!data.optionTemplates.keysAsArray.includes(selectedInputTemplate)) return [3 /*break*/, 31];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'confirmAddTemplate',
                        type: 'confirm',
                        message: "Would you like to add ".concat((0, logger_1.chalk)().white(selectedInputTemplate), " to your ").concat(c.runtime.selectedWorkspace, " workspace template list?"),
                    })];
            case 29:
                confirmAddTemplate = (_f.sent()).confirmAddTemplate;
                if (!confirmAddTemplate) return [3 /*break*/, 31];
                if (!((_b = c.files.workspace.config) === null || _b === void 0 ? void 0 : _b.projectTemplates)) {
                    c.files.workspace.config.projectTemplates = {};
                }
                c.files.workspace.config.projectTemplates[selectedInputTemplate] = {};
                (0, fileutils_1.writeFileSync)(c.paths.workspace.config, c.files.workspace.config);
                return [4 /*yield*/, (0, runtimeManager_1.updateRenativeConfigs)(c)];
            case 30:
                _f.sent();
                (0, logger_1.logInfo)("Updating ".concat(c.paths.workspace.config, "...DONE"));
                _f.label = 31;
            case 31:
                renativeTemplateConfig = (0, fileutils_1.readObjectSync)(path_1.default.join(c.paths.project.dir, 'node_modules', selectedInputTemplate, constants_1.RENATIVE_CONFIG_TEMPLATE_NAME));
                renativeConfig = (0, fileutils_1.readObjectSync)(path_1.default.join(c.paths.project.dir, 'node_modules', selectedInputTemplate, constants_1.RENATIVE_CONFIG_NAME));
                supportedPlatforms = ((_c = renativeTemplateConfig === null || renativeTemplateConfig === void 0 ? void 0 : renativeTemplateConfig.defaults) === null || _c === void 0 ? void 0 : _c.supportedPlatforms)
                    || ((_d = renativeConfig === null || renativeConfig === void 0 ? void 0 : renativeConfig.defaults) === null || _d === void 0 ? void 0 : _d.supportedPlatforms)
                    || [];
                if (supportedPlatforms.length === 0) {
                    (0, logger_1.logError)("Template ".concat(selectedInputTemplate, " does not seem to export any default platforms to support. contact the author."));
                }
                if (!(platform && platform !== '')) return [3 /*break*/, 32];
                inputSupportedPlatforms = platform.split(',');
                return [3 /*break*/, 34];
            case 32: return [4 /*yield*/, inquirer_1.default.prompt({
                    name: 'inputSupportedPlatforms',
                    type: 'checkbox',
                    pageSize: 20,
                    message: 'What platforms would you like to use?',
                    validate: function (val) { return !!val.length || 'Please select at least a platform'; },
                    default: supportedPlatforms,
                    choices: supportedPlatforms,
                })];
            case 33:
                answer = _f.sent();
                inputSupportedPlatforms = answer === null || answer === void 0 ? void 0 : answer.inputSupportedPlatforms;
                _f.label = 34;
            case 34:
                renativeTemplateConfigExt = {};
                bootstrapQuestions = (_e = renativeTemplateConfig === null || renativeTemplateConfig === void 0 ? void 0 : renativeTemplateConfig.templateConfig) === null || _e === void 0 ? void 0 : _e.bootstrapQuestions;
                results = {};
                providedAnswers = {};
                if (c.program.answer) {
                    c.program.answer.forEach(function (a) {
                        var key = a.split('=')[0];
                        var value;
                        try {
                            value = JSON.parse(a.split('=')[1]);
                        }
                        catch (e) {
                            value = a.split('=')[1];
                        }
                        providedAnswers[key] = value;
                    });
                }
                return [4 /*yield*/, interactiveQuestion(results, bootstrapQuestions, providedAnswers)];
            case 35:
                _f.sent();
                console.log('asnwer', JSON.stringify(results, null, 2));
                Object.keys(results).forEach(function (targetKey) {
                    var objValue = results[targetKey].value;
                    console.log('setting', targetKey, objValue);
                    if (targetKey) {
                        (0, lodash_set_1.default)(renativeTemplateConfigExt, targetKey, objValue);
                    }
                });
                isGitEnabled = gitEnabled === 'true' || gitEnabled === true;
                if (!(gitEnabled === undefined && !ci)) return [3 /*break*/, 37];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'gitEnabled',
                        type: 'confirm',
                        message: 'Do you want to set-up git in your new project?',
                    })];
            case 36:
                response = _f.sent();
                isGitEnabled = response.gitEnabled;
                _f.label = 37;
            case 37:
                // ==================================================
                // INPUT: Confirm Overview
                // ==================================================
                data = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, data), { inputProjectName: inputProjectName, inputAppTitle: inputAppTitle, inputAppID: inputAppID, inputVersion: inputVersion, inputTemplate: selectedInputTemplate, inputSupportedPlatforms: inputSupportedPlatforms, inputWorkspace: inputWorkspace, gitEnabled: isGitEnabled });
                data.optionPlatforms.selectedOptions = inputSupportedPlatforms;
                _prepareProjectOverview(c, data);
                if (!!ci) return [3 /*break*/, 39];
                return [4 /*yield*/, inquirer_1.default.prompt({
                        type: 'confirm',
                        name: 'confirm',
                        message: "\n".concat(data.confirmString, "\nIs all this correct?"),
                    })];
            case 38:
                confirm_2 = (_f.sent()).confirm;
                if (!confirm_2) {
                    return [2 /*return*/];
                }
                _f.label = 39;
            case 39:
                _f.trys.push([39, 41, , 42]);
                return [4 /*yield*/, analytics_1.default.captureEvent({
                        type: 'newProject',
                        template: selectedInputTemplate,
                        platforms: inputSupportedPlatforms,
                    })];
            case 40:
                _f.sent();
                return [3 /*break*/, 42];
            case 41:
                e_1 = _f.sent();
                (0, logger_1.logDebug)(e_1);
                return [3 /*break*/, 42];
            case 42:
                c.paths.project.package = path_1.default.join(c.paths.project.dir, 'package.json');
                c.paths.project.config = path_1.default.join(c.paths.project.dir, constants_1.RENATIVE_CONFIG_NAME);
                data.packageName = data.appTitle.replace(/\s+/g, '-').toLowerCase();
                templates = {};
                (0, logger_1.logTask)("_generateProject:".concat(data.optionTemplates.selectedOption, ":").concat(data.optionTemplates.selectedVersion), (0, logger_1.chalk)().grey);
                templates[data.optionTemplates.selectedOption] = {
                    version: data.optionTemplates.selectedVersion,
                };
                config = (0, tslib_1.__assign)((0, tslib_1.__assign)((0, tslib_1.__assign)({ platforms: {} }, renativeTemplateConfig), renativeTemplateConfigExt), { projectName: data.projectName, workspaceID: data.optionWorkspaces.selectedOption, 
                    // paths: {
                    //     appConfigsDir: './appConfigs',
                    //     entryDir: './',
                    //     platformAssetsDir: './platformAssets',
                    //     platformBuildsDir: './platformBuilds',
                    // },
                    defaults: {
                        title: data.appTitle,
                        id: data.appID,
                        supportedPlatforms: data.optionPlatforms.selectedOptions,
                    }, engines: {}, templates: templates, currentTemplate: data.optionTemplates.selectedOption, isNew: true, isMonorepo: false });
                supPlats = config.defaults.supportedPlatforms;
                // Remove unused platforms
                Object.keys(config.platforms).forEach(function (k) {
                    if (!supPlats.includes(k)) {
                        delete config.platforms[k];
                    }
                });
                if (renativeTemplateConfig.engines) {
                    // Remove unused engines based on selected platforms
                    supPlats.forEach(function (k) {
                        var _a, _b;
                        var selectedEngineId = ((_a = config.platforms[k]) === null || _a === void 0 ? void 0 : _a.engine)
                            || ((_b = c.files.rnv.projectTemplates.config.platforms[k]) === null || _b === void 0 ? void 0 : _b.engine);
                        if (selectedEngineId) {
                            var selectedEngine = findEngineKeyById(c, selectedEngineId);
                            config.engines[selectedEngine.key] = renativeTemplateConfig.engines[selectedEngine.key];
                        }
                    });
                }
                delete config.templateConfig;
                if (!config.platforms) {
                    config.platforms = {};
                }
                (0, fileutils_1.writeFileSync)(c.paths.project.config, config);
                if (!data.gitEnabled) return [3 /*break*/, 45];
                return [4 /*yield*/, (0, projectManager_1.checkAndCreateGitignore)(c)];
            case 43:
                _f.sent();
                return [4 /*yield*/, (0, gitUtils_1.configureGit)(c)];
            case 44:
                _f.sent();
                _f.label = 45;
            case 45:
                (0, logger_1.logSuccess)("Your project is ready! navigate to project ".concat((0, logger_1.chalk)().white("cd ".concat(data.projectName)), " and run ").concat((0, logger_1.chalk)().white('rnv run'), " to see magic happen!"));
                return [2 /*return*/];
        }
    });
}); };
exports.taskRnvNew = taskRnvNew;
var findEngineKeyById = function (c, id) {
    var engineTemplates = c.files.rnv.projectTemplates.config.engineTemplates;
    var etk = Object.keys(engineTemplates);
    for (var i = 0; i < etk.length; i++) {
        var engine = engineTemplates[etk[i]];
        if (engine.id === id) {
            engine.key = etk[i];
            return engine;
        }
    }
};
exports.default = {
    description: 'Create new ReNative project',
    fn: exports.taskRnvNew,
    task: 'new',
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    isGlobalScope: true,
};
//# sourceMappingURL=task.rnv.new.js.map