"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvPlatformConnect = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var inquirer_1 = (0, tslib_1.__importDefault)(require("inquirer"));
var logger_1 = require("../../core/systemManager/logger");
var fileutils_1 = require("../../core/systemManager/fileutils");
var platformManager_1 = require("../../core/platformManager");
var taskManager_1 = require("../../core/taskManager");
var constants_1 = require("../../core/constants");
var taskRnvPlatformConnect = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var selectedPlatforms, connectedPlatforms, deletePlatformFolder, pathsToRemove_1;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvPlatformConnect');
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_PROJECT_CONFIGURE, constants_1.TASK_PLATFORM_CONNECT, originTask)];
            case 1:
                _a.sent();
                if (!c.files.project.config.paths.platformTemplatesDirs) {
                    (0, logger_1.logToSummary)('All supported platforms are connected. nothing to do.');
                    return [2 /*return*/];
                }
                if (!c.platform) return [3 /*break*/, 2];
                selectedPlatforms = [c.platform];
                return [3 /*break*/, 4];
            case 2: return [4 /*yield*/, inquirer_1.default.prompt({
                    name: 'connectedPlatforms',
                    message: 'This will point platformTemplates folders from your local project to ReNative managed one. Select platforms you would like to connect',
                    type: 'checkbox',
                    choices: (0, platformManager_1.generatePlatformChoices)(c).map(function (choice) { return ((0, tslib_1.__assign)((0, tslib_1.__assign)({}, choice), { disabled: choice.isConnected })); })
                })];
            case 3:
                connectedPlatforms = (_a.sent()).connectedPlatforms;
                selectedPlatforms = connectedPlatforms;
                _a.label = 4;
            case 4:
                if (selectedPlatforms.length) {
                    selectedPlatforms.forEach(function (platform) {
                        var _a, _b, _c;
                        if ((_b = (_a = c.files.project.config_original.paths) === null || _a === void 0 ? void 0 : _a.platformTemplatesDirs) === null || _b === void 0 ? void 0 : _b[platform]) {
                            delete c.files.project.config_original.paths.platformTemplatesDirs[platform];
                        }
                        if (!Object.keys(((_c = c.files.project.config_original.paths) === null || _c === void 0 ? void 0 : _c.platformTemplatesDirs) || {}).length) {
                            delete c.files.project.config_original.paths.platformTemplatesDirs; // also cleanup the empty object
                        }
                        (0, fileutils_1.writeFileSync)(c.paths.project.config, c.files.project.config_original);
                    });
                }
                return [4 /*yield*/, inquirer_1.default.prompt({
                        name: 'deletePlatformFolder',
                        type: 'confirm',
                        message: 'Would you also like to delete the previously used platform folder?'
                    })];
            case 5:
                deletePlatformFolder = (_a.sent()).deletePlatformFolder;
                if (!deletePlatformFolder) return [3 /*break*/, 7];
                pathsToRemove_1 = [];
                selectedPlatforms.forEach(function (platform) {
                    pathsToRemove_1.push(path_1.default.join(c.paths.project.platformTemplatesDirs[platform], platform));
                });
                // TODO: Remove shared folders as well
                return [4 /*yield*/, (0, fileutils_1.removeDirs)(pathsToRemove_1)];
            case 6:
                // TODO: Remove shared folders as well
                _a.sent();
                _a.label = 7;
            case 7:
                (0, logger_1.logSuccess)("".concat((0, logger_1.chalk)().white(selectedPlatforms.join(',')), " now using ReNative platformTemplates located associated platform engines."));
                return [2 /*return*/];
        }
    });
}); };
exports.taskRnvPlatformConnect = taskRnvPlatformConnect;
exports.default = {
    description: 'Connect platform template back to rnv',
    fn: exports.taskRnvPlatformConnect,
    task: constants_1.TASK_PLATFORM_CONNECT,
    params: constants_1.PARAMS.withBase(),
    platforms: [],
};
//# sourceMappingURL=task.rnv.platform.connect.js.map