"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvProjectConfigure = void 0;
var tslib_1 = require("tslib");
var pluginManager_1 = require("../../core/pluginManager");
var logger_1 = require("../../core/systemManager/logger");
var configManager_1 = require("../../core/configManager");
var runtimeManager_1 = require("../../core/runtimeManager");
var templateManager_1 = require("../../core/templateManager");
var fileutils_1 = require("../../core/systemManager/fileutils");
var crypto_1 = require("../../core/systemManager/crypto");
var migrator_1 = require("../../core/projectManager/migrator");
var constants_1 = require("../../core/constants");
var projectManager_1 = require("../../core/projectManager");
var engineManager_1 = require("../../core/engineManager");
var taskManager_1 = require("../../core/taskManager");
var taskRnvProjectConfigure = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var taskInstance;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvProjectConfigure');
                if (c.paths.project.builds.dir && !(0, fileutils_1.fsExistsSync)(c.paths.project.builds.dir)) {
                    (0, logger_1.logInfo)("Creating folder ".concat(c.paths.project.builds.dir, " ...DONE"));
                    (0, fileutils_1.fsMkdirSync)(c.paths.project.builds.dir);
                }
                return [4 /*yield*/, (0, migrator_1.checkAndMigrateProject)(c)];
            case 1:
                _a.sent();
                return [4 /*yield*/, (0, runtimeManager_1.updateRenativeConfigs)(c)];
            case 2:
                _a.sent();
                return [4 /*yield*/, (0, configManager_1.checkIsRenativeProject)(c)];
            case 3:
                _a.sent();
                // await checkAndCreateProjectPackage(c);
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_WORKSPACE_CONFIGURE, constants_1.TASK_PROJECT_CONFIGURE, originTask)];
            case 4:
                // await checkAndCreateProjectPackage(c);
                _a.sent();
                if (!(c.program.only && !!parentTask)) return [3 /*break*/, 8];
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 5:
                _a.sent();
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_APP_CONFIGURE, constants_1.TASK_PROJECT_CONFIGURE, originTask)];
            case 6:
                _a.sent();
                return [4 /*yield*/, (0, configManager_1.generateRuntimeConfig)(c)];
            case 7:
                _a.sent();
                return [2 /*return*/, true];
            case 8: return [4 /*yield*/, (0, templateManager_1.checkIfTemplateConfigured)(c)];
            case 9:
                _a.sent();
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_INSTALL, constants_1.TASK_PROJECT_CONFIGURE, originTask)];
            case 10:
                _a.sent();
                return [4 /*yield*/, (0, crypto_1.checkCrypto)(c, parentTask, originTask)];
            case 11:
                _a.sent();
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 12:
                _a.sent();
                if (!(originTask !== constants_1.TASK_TEMPLATE_APPLY)) return [3 /*break*/, 39];
                if (!((c.runtime.requiresBootstrap || !(0, templateManager_1.isTemplateInstalled)(c)) && !c.files.project.config.isTemplate)) return [3 /*break*/, 15];
                return [4 /*yield*/, (0, templateManager_1.applyTemplate)(c)];
            case 13:
                _a.sent();
                // We'll have to install the template first and reset current engine
                (0, logger_1.logInfo)('Your template has been bootstraped. Command reset is required. RESTRATING...DONE');
                return [4 /*yield*/, (0, taskManager_1.findSuitableTask)(c)];
            case 14:
                taskInstance = _a.sent();
                c.runtime.requiresBootstrap = false;
                return [2 /*return*/, (0, taskManager_1.initializeTask)(c, taskInstance.task)];
            case 15: return [4 /*yield*/, (0, templateManager_1.applyTemplate)(c)];
            case 16:
                _a.sent();
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 17:
                _a.sent();
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_INSTALL, constants_1.TASK_PROJECT_CONFIGURE, originTask)];
            case 18:
                _a.sent();
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_APP_CONFIGURE, constants_1.TASK_PROJECT_CONFIGURE, originTask)];
            case 19:
                _a.sent();
                // IMPORTANT: configurePlugins must run after appConfig present to ensure merge of all configs/plugins
                return [4 /*yield*/, (0, projectManager_1.versionCheck)(c)];
            case 20:
                // IMPORTANT: configurePlugins must run after appConfig present to ensure merge of all configs/plugins
                _a.sent();
                return [4 /*yield*/, (0, engineManager_1.configureEngines)(c)];
            case 21:
                _a.sent();
                return [4 /*yield*/, (0, pluginManager_1.resolvePluginDependants)(c)];
            case 22:
                _a.sent();
                return [4 /*yield*/, (0, pluginManager_1.configurePlugins)(c)];
            case 23:
                _a.sent();
                return [4 /*yield*/, (0, runtimeManager_1.configureRuntimeDefaults)(c)];
            case 24:
                _a.sent();
                if (!!c.runtime.disableReset) return [3 /*break*/, 28];
                if (!c.program.resetHard) return [3 /*break*/, 26];
                (0, logger_1.logInfo)("You passed ".concat((0, logger_1.chalk)().white('-R, --resetHard'), " argument. \"").concat((0, logger_1.chalk)().white('./platformAssets'), "\" will be cleaned up first"));
                return [4 /*yield*/, (0, projectManager_1.cleanPlaformAssets)(c)];
            case 25:
                _a.sent();
                return [3 /*break*/, 28];
            case 26:
                if (!c.program.resetAssets) return [3 /*break*/, 28];
                (0, logger_1.logInfo)("You passed ".concat((0, logger_1.chalk)().white('-a, --resetAssets'), " argument. \"").concat((0, logger_1.chalk)().white('./platformAssets'), "\" will be cleaned up first"));
                return [4 /*yield*/, (0, projectManager_1.cleanPlaformAssets)(c)];
            case 27:
                _a.sent();
                _a.label = 28;
            case 28: return [4 /*yield*/, (0, projectManager_1.copyRuntimeAssets)(c)];
            case 29:
                _a.sent();
                return [4 /*yield*/, (0, templateManager_1.configureTemplateFiles)(c)];
            case 30:
                _a.sent();
                return [4 /*yield*/, (0, projectManager_1.checkAndCreateGitignore)(c)];
            case 31:
                _a.sent();
                return [4 /*yield*/, (0, projectManager_1.checkAndCreateBabelConfig)(c)];
            case 32:
                _a.sent();
                if (!!c.buildConfig.platforms) return [3 /*break*/, 34];
                return [4 /*yield*/, (0, runtimeManager_1.updateRenativeConfigs)(c)];
            case 33:
                _a.sent();
                _a.label = 34;
            case 34: 
            // NOTE: Migrated to engines
            // await configureEntryPoints(c);
            return [4 /*yield*/, (0, configManager_1.generateRuntimeConfig)(c)];
            case 35:
                // NOTE: Migrated to engines
                // await configureEntryPoints(c);
                _a.sent();
                return [4 /*yield*/, (0, pluginManager_1.overrideTemplatePlugins)(c)];
            case 36:
                _a.sent();
                // NOTE: this is needed to ensure missing rnv plugin sub-deps are caught
                return [4 /*yield*/, (0, pluginManager_1.checkForPluginDependencies)(c)];
            case 37:
                // NOTE: this is needed to ensure missing rnv plugin sub-deps are caught
                _a.sent();
                return [4 /*yield*/, (0, projectManager_1.configureFonts)(c)];
            case 38:
                _a.sent();
                _a.label = 39;
            case 39: return [2 /*return*/, true];
        }
    });
}); };
exports.taskRnvProjectConfigure = taskRnvProjectConfigure;
exports.default = {
    description: 'Configure current project',
    fn: exports.taskRnvProjectConfigure,
    task: constants_1.TASK_PROJECT_CONFIGURE,
    params: constants_1.PARAMS.withBase(),
    platforms: [],
};
//# sourceMappingURL=task.rnv.project.configure.js.map