"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvProjectUpgrade = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var fs_1 = (0, tslib_1.__importDefault)(require("fs"));
var constants_1 = require("../../core/constants");
var logger_1 = require("../../core/systemManager/logger");
var projectManager_1 = require("../../core/projectManager");
var taskManager_1 = require("../../core/taskManager");
var npmUtils_1 = require("../../core/systemManager/npmUtils");
var pluginManager_1 = require("../../core/pluginManager");
var fileutils_1 = require("../../core/systemManager/fileutils");
var taskRnvProjectUpgrade = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var upgradedPaths, selectedVersion, packagesPath_1, selectedVersion_1, dirs;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvProjectUpgrade');
                upgradedPaths = [];
                if (!(0, fileutils_1.fsExistsSync)(c.paths.project.config)) return [3 /*break*/, 4];
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_PROJECT_CONFIGURE, constants_1.TASK_PROJECT_UPGRADE, originTask)];
            case 1:
                _a.sent();
                return [4 /*yield*/, (0, npmUtils_1.listAndSelectNpmVersion)(c, 'rnv')];
            case 2:
                selectedVersion = _a.sent();
                upgradedPaths.push.apply(upgradedPaths, (0, projectManager_1.upgradeProjectDependencies)(c, selectedVersion));
                return [4 /*yield*/, (0, pluginManager_1.installPackageDependenciesAndPlugins)(c)];
            case 3:
                _a.sent();
                return [3 /*break*/, 6];
            case 4:
                (0, logger_1.logInfo)('Your are running rnv upgrade outside of renative project');
                packagesPath_1 = path_1.default.join(c.paths.project.dir, 'packages');
                if (!((0, fileutils_1.fsExistsSync)(c.paths.project.package) && (0, fileutils_1.fsExistsSync)(packagesPath_1))) return [3 /*break*/, 6];
                return [4 /*yield*/, (0, npmUtils_1.listAndSelectNpmVersion)(c, 'rnv')];
            case 5:
                selectedVersion_1 = _a.sent();
                upgradedPaths.push.apply(upgradedPaths, (0, projectManager_1.upgradeDependencies)(c.files.project.package, c.paths.project.package, null, null, selectedVersion_1));
                dirs = fs_1.default.readdirSync(packagesPath_1);
                dirs.forEach(function (dir) {
                    var dirPath = path_1.default.join(packagesPath_1, dir);
                    if (fs_1.default.statSync(dirPath).isDirectory()) {
                        var pkgPath = path_1.default.join(dirPath, 'package.json');
                        var rnvPath = path_1.default.join(dirPath, 'renative.json');
                        var pkgFile = void 0;
                        var rnvFile = void 0;
                        if ((0, fileutils_1.fsExistsSync)(pkgPath)) {
                            pkgFile = (0, fileutils_1.readObjectSync)(pkgPath);
                        }
                        if ((0, fileutils_1.fsExistsSync)(rnvPath)) {
                            rnvFile = (0, fileutils_1.readObjectSync)(rnvPath);
                        }
                        upgradedPaths.push.apply(upgradedPaths, (0, projectManager_1.upgradeDependencies)(pkgFile, pkgPath, rnvFile, rnvPath, selectedVersion_1));
                    }
                });
                _a.label = 6;
            case 6:
                (0, logger_1.logToSummary)("Upgraded following files:\n".concat(upgradedPaths.join('\n')), true);
                return [2 /*return*/, true];
        }
    });
}); };
exports.taskRnvProjectUpgrade = taskRnvProjectUpgrade;
exports.default = {
    description: 'Upgrade or downgrade RNV dependencies in your ReNative project',
    fn: exports.taskRnvProjectUpgrade,
    task: constants_1.TASK_PROJECT_UPGRADE,
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    skipAppConfig: true,
    skipPlatforms: true,
    isGlobalScope: true
};
//# sourceMappingURL=task.rnv.project.upgrade.js.map