"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvPublish = void 0;
var tslib_1 = require("tslib");
var semver_1 = (0, tslib_1.__importDefault)(require("semver"));
var config_1 = (0, tslib_1.__importDefault)(require("../../core/configManager/config"));
var exec_1 = require("../../core/systemManager/exec");
var logger_1 = require("../../core/systemManager/logger");
var fileutils_1 = require("../../core/systemManager/fileutils");
var taskManager_1 = require("../../core/taskManager");
var constants_1 = require("../../core/constants");
var includesPre = function (version) {
    if (version.includes('alpha'))
        return 'alpha';
    if (version.includes('beta'))
        return 'beta';
    if (version.includes('rc'))
        return 'rc';
    return false;
};
/*
 *
 * Usage
 * rnv publish
 * rnv publish patch|minor|major
 * rnv publish patch|minor|major alpha|beta|rc
 * rnv publish 1.0.0
 * rnv publish 1.0.0-alpha.1
 * rnv publish ... --dry-run
 *
 * Basically the same as release-it documentation. The only difference is that you don't need to specify --preRelease=beta
 * if you are publishing a beta/alpha/rc. That is done automatically by checking if the second arg is alpha, beta, rc.
 *
 */
var taskRnvPublish = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var pkgJson, existingPath, args, maybeVersion, secondArg, prereleaseMark, dir, execCommonOpts, ci, publishMode, _a, skipRootPublish, rootPublishCommand, rootPublishIfNecessary, releaseIt;
    var _b, _c;
    return (0, tslib_1.__generator)(this, function (_d) {
        switch (_d.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvPublish');
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_PROJECT_CONFIGURE, constants_1.TASK_PUBLISH, originTask)];
            case 1:
                _d.sent();
                // make sure release-it is installed
                return [4 /*yield*/, config_1.default.checkRequiredPackage(c, 'release-it', '12.4.3', 'devDependencies')];
            case 2:
                // make sure release-it is installed
                _d.sent();
                pkgJson = c.files.project.package;
                existingPath = c.paths.project.package;
                if (!pkgJson['release-it']) {
                    pkgJson['release-it'] = {
                        git: {
                            // eslint-disable-next-line no-template-curly-in-string
                            tagName: 'v${version}',
                            requireCleanWorkingDir: false
                        },
                        npm: {
                            publish: false
                        },
                        hooks: {
                            // eslint-disable-next-line no-template-curly-in-string
                            'before:git': 'npx rnv pkg version ${version}'
                        }
                    };
                    (0, fileutils_1.writeFileSync)(existingPath, pkgJson);
                }
                // backwards compatibility and user change friendly
                if (!((_c = (_b = pkgJson['release-it']) === null || _b === void 0 ? void 0 : _b.hooks) === null || _c === void 0 ? void 0 : _c['before:git'])) {
                    if (!pkgJson['release-it'].hooks) {
                        pkgJson['release-it'].hooks = {};
                    }
                    // eslint-disable-next-line no-template-curly-in-string
                    pkgJson['release-it'].hooks['before:git'] = 'npx rnv pkg version ${version}';
                    (0, fileutils_1.writeFileSync)(existingPath, pkgJson);
                }
                if (!pkgJson['release-it'].publish) {
                    pkgJson['release-it'].publish = 'local';
                    pkgJson['release-it'].skipRootPublish = true;
                    pkgJson['release-it'].rootPublishCommand = 'npx rnv deploy -p ios -s debug';
                    (0, fileutils_1.writeFileSync)(existingPath, pkgJson);
                }
                args = (0, tslib_1.__spreadArray)([], config_1.default.getConfig().program.rawArgs, true);
                args = args.slice(3);
                maybeVersion = args[0];
                secondArg = args[1];
                prereleaseMark = '';
                // for handling `rnv publish patch alpha`
                if (['alpha', 'beta', 'rc'].includes(secondArg)) {
                    args.splice(1, 1); // remove it so it won't interfere with release-it
                    prereleaseMark = "--preRelease=".concat(secondArg);
                }
                // for handling `rnv publish 1.0.0-alpha.1`
                if (semver_1.default.valid(maybeVersion) && includesPre(maybeVersion)) {
                    prereleaseMark = "--preRelease=".concat(includesPre(maybeVersion));
                }
                dir = config_1.default.getConfig().paths.project.dir;
                execCommonOpts = { interactive: true, env: process.env, cwd: dir };
                ci = config_1.default.getConfig().program.ci;
                publishMode = pkgJson['release-it'].publish || 'local';
                _a = pkgJson['release-it'], skipRootPublish = _a.skipRootPublish, rootPublishCommand = _a.rootPublishCommand;
                rootPublishIfNecessary = function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
                    return (0, tslib_1.__generator)(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, (0, exec_1.executeAsync)('npx rnv pkg publish', execCommonOpts)];
                            case 1:
                                _a.sent();
                                if (!skipRootPublish) {
                                    if (!rootPublishCommand) {
                                        throw new Error("You don't have a rootPublishCommand specified in package.json");
                                    }
                                    return [2 /*return*/, (0, exec_1.executeAsync)(rootPublishCommand, execCommonOpts)];
                                }
                                return [2 /*return*/];
                        }
                    });
                }); };
                releaseIt = function () { return (0, exec_1.executeAsync)("npx release-it ".concat(args.join(' '), " ").concat(prereleaseMark), execCommonOpts)
                    .catch(function (e) {
                    if (e.includes('SIGINT'))
                        return Promise.resolve();
                    if (e.includes('--no-git.requireUpstream')) {
                        return Promise.reject(new Error('Seems like you have no upstream configured for current branch. Run `git push -u <origin> <your_branch>` to fix it then try again.'));
                    }
                    return Promise.reject(e);
                })
                    .then(rootPublishIfNecessary); };
                // we have a ci flag, checking if the project is configured for ci releases to do a bumpless deploy
                if (ci) {
                    if (publishMode !== 'ci') {
                        return [2 /*return*/, (0, logger_1.logWarning)('You are running publish with --ci flag but this project is set for local deployments. Check package.json release-it.publish property')];
                    }
                    return [2 /*return*/, rootPublishIfNecessary()];
                }
                return [2 /*return*/, releaseIt()];
        }
    });
}); };
exports.taskRnvPublish = taskRnvPublish;
exports.default = {
    description: '',
    fn: exports.taskRnvPublish,
    task: 'publish',
    params: constants_1.PARAMS.withBase(),
    platforms: [],
};
//# sourceMappingURL=task.rnv.publish.js.map