"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvTargetLaunch = void 0;
var tslib_1 = require("tslib");
var platformManager_1 = require("../../core/platformManager");
var logger_1 = require("../../core/systemManager/logger");
var constants_1 = require("../../core/constants");
var installer_1 = require("../../core/sdkManager/installer");
var tizen_1 = require("../../core/sdkManager/deviceUtils/tizen");
var webos_1 = require("../../core/sdkManager/deviceUtils/webos");
var android_1 = require("../../core/sdkManager/deviceUtils/android");
var apple_1 = require("../../core/sdkManager/deviceUtils/apple");
var kaios_1 = require("../../core/sdkManager/deviceUtils/kaios");
var taskManager_1 = require("../../core/taskManager");
var taskRnvTargetLaunch = function (c, parentTask, originTask) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var platform, program, target;
    return (0, tslib_1.__generator)(this, function (_a) {
        switch (_a.label) {
            case 0:
                (0, logger_1.logTask)('taskRnvTargetLaunch');
                return [4 /*yield*/, (0, platformManager_1.isPlatformSupported)(c, true)];
            case 1:
                _a.sent();
                return [4 /*yield*/, (0, taskManager_1.executeTask)(c, constants_1.TASK_WORKSPACE_CONFIGURE, constants_1.TASK_TARGET_LAUNCH, originTask)];
            case 2:
                _a.sent();
                platform = c.platform, program = c.program;
                target = program.target || c.files.workspace.config.defaultTargets[platform];
                return [4 /*yield*/, (0, installer_1.checkSdk)(c)];
            case 3:
                _a.sent();
                switch (platform) {
                    case constants_1.ANDROID:
                    case constants_1.ANDROID_TV:
                    case constants_1.FIRE_TV:
                    case constants_1.ANDROID_WEAR:
                        return [2 /*return*/, (0, android_1.launchAndroidSimulator)(c, target)];
                    case constants_1.IOS:
                    case constants_1.TVOS:
                        return [2 /*return*/, (0, apple_1.launchAppleSimulator)(c, target)];
                    case constants_1.TIZEN:
                        return [2 /*return*/, (0, tizen_1.launchTizenSimulator)(c, target)];
                    case constants_1.WEBOS:
                        return [2 /*return*/, (0, webos_1.launchWebOSimulator)(c, target)];
                    case constants_1.KAIOS:
                        return [2 /*return*/, (0, kaios_1.launchKaiOSSimulator)(c, target)];
                    default:
                        return [2 /*return*/, Promise.reject("\"target launch\" command does not support ".concat((0, logger_1.chalk)().white.bold(platform), " platform yet. You will have to launch the emulator manually. Working on it!"))];
                }
                return [2 /*return*/];
        }
    });
}); };
exports.taskRnvTargetLaunch = taskRnvTargetLaunch;
exports.default = {
    description: 'Launch specific emulator',
    fn: exports.taskRnvTargetLaunch,
    task: 'target launch',
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    isGlobalScope: true
};
//# sourceMappingURL=task.rnv.target.launch.js.map