"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskRnvWorkspaceConfigure = void 0;
var tslib_1 = require("tslib");
var path_1 = (0, tslib_1.__importDefault)(require("path"));
var constants_1 = require("../../core/constants");
var fileutils_1 = require("../../core/systemManager/fileutils");
var logger_1 = require("../../core/systemManager/logger");
var taskRnvWorkspaceConfigure = function (c) { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
    var oldGlobalConfigPath, defaultConfig, newConfig;
    var _a;
    return (0, tslib_1.__generator)(this, function (_b) {
        (0, logger_1.logTask)('taskRnvWorkspaceConfigure');
        // Check globalConfig Dir
        if ((0, fileutils_1.fsExistsSync)(c.paths.workspace.dir)) {
            (0, logger_1.logDebug)("".concat(c.paths.workspace.dir, " folder exists!"));
        }
        else {
            (0, logger_1.logInfo)("".concat(c.paths.workspace.dir, " folder missing! Creating one for you..."));
            (0, fileutils_1.mkdirSync)(c.paths.workspace.dir);
        }
        // Check globalConfig
        if ((0, fileutils_1.fsExistsSync)(c.paths.workspace.config)) {
            (0, logger_1.logDebug)("".concat(c.paths.workspace.dir, "/").concat(constants_1.RENATIVE_CONFIG_NAME, " file exists!"));
        }
        else {
            oldGlobalConfigPath = path_1.default.join(c.paths.workspace.dir, 'config.json');
            if ((0, fileutils_1.fsExistsSync)(oldGlobalConfigPath)) {
                (0, logger_1.logWarning)('Found old version of your config. will copy it to new renative.json config');
                (0, fileutils_1.copyFileSync)(oldGlobalConfigPath, c.paths.workspace.config);
            }
            else {
                (0, logger_1.logInfo)("".concat(c.paths.workspace.dir, "/").concat(constants_1.RENATIVE_CONFIG_NAME, " file missing! Creating one for you..."));
                (0, fileutils_1.copyFileSync)(path_1.default.join(c.paths.rnv.dir, 'coreTemplateFiles', 'global-config-template.json'), c.paths.workspace.config);
            }
        }
        if ((0, fileutils_1.fsExistsSync)(c.paths.workspace.config)) {
            c.files.workspace.config = JSON.parse((0, fileutils_1.fsReadFileSync)(c.paths.workspace.config).toString());
            if ((_a = c.files.workspace.config) === null || _a === void 0 ? void 0 : _a.appConfigsPath) {
                if (!(0, fileutils_1.fsExistsSync)(c.files.workspace.config.appConfigsPath)) {
                    (0, logger_1.logWarning)("Your custom global appConfig is pointing to ".concat((0, logger_1.chalk)().white(c.files.workspace.config.appConfigsPath), " which doesn't exist! Make sure you create one in that location"));
                }
                else {
                    (0, logger_1.logInfo)("Found custom appConfing location pointing to ".concat((0, logger_1.chalk)().white(c.files.workspace.config.appConfigsPath), ". ReNativewill now swith to that location!"));
                    c.paths.project.appConfigsDir = c.files.workspace.config.appConfigsPath;
                }
            }
            // Check config sanity
            if (c.files.workspace.config.defaultTargets === undefined) {
                (0, logger_1.logWarning)("You're missing defaultTargets in your config ".concat((0, logger_1.chalk)().white(c.paths.workspace.config), ". Let's add them!"));
                defaultConfig = JSON.parse((0, fileutils_1.fsReadFileSync)(path_1.default.join(c.paths.rnv.dir, 'coreTemplateFiles', 'global-config-template.json')).toString());
                newConfig = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, c.files.workspace.config), { defaultTargets: defaultConfig.defaultTargets });
                (0, fileutils_1.fsWriteFileSync)(c.paths.workspace.config, JSON.stringify(newConfig, null, 2));
            }
        }
        return [2 /*return*/, true];
    });
}); };
exports.taskRnvWorkspaceConfigure = taskRnvWorkspaceConfigure;
exports.default = {
    description: '',
    fn: exports.taskRnvWorkspaceConfigure,
    task: 'workspace configure',
    params: constants_1.PARAMS.withBase(),
    platforms: [],
    skipPlatforms: true,
    isGlobalScope: true
};
//# sourceMappingURL=task.rnv.workspace.configure.js.map