export function getRenativeJsonSchema(): {
    $id: string;
    definitions: {
        android: {
            additionalProperties: boolean;
            type: string;
            properties: {
                enableAndroidX: {
                    type: string;
                    default: boolean;
                    examples: boolean[];
                };
                enableHermes: {
                    type: string;
                    default: boolean;
                    description: string;
                    examples: boolean[];
                };
                reactNativeEngine: {
                    type: string;
                    default: string;
                    description: string;
                    examples: boolean[];
                };
                signingConfig: {
                    type: string;
                    default: string;
                    description: string;
                    examples: string[];
                };
                minSdkVersion: {
                    type: string;
                    default: number;
                    examples: number[];
                };
                multipleAPKs: {
                    type: string;
                    default: boolean;
                    description: string;
                    examples: boolean[];
                };
                aab: {
                    type: string;
                    description: string;
                    default: boolean;
                    examples: boolean[];
                };
                extraGradleParams: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                minifyEnabled: {
                    type: string;
                    description: string;
                    default: boolean;
                    examples: boolean[];
                };
                targetSdkVersion: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                compileSdkVersion: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                gradleBuildToolsVersion: {
                    type: string;
                    description: string;
                    default: string;
                    examples: string[];
                };
                gradleWrapperVersion: {
                    type: string;
                    description: string;
                    default: string;
                    examples: string[];
                };
                mainActivity: {
                    type: string;
                    description: string;
                    default: string;
                    properties: {
                        onCreate: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                    };
                };
                storeFile: {
                    type: string;
                };
                storePassword: {
                    type: string;
                    description: string;
                };
                keyAlias: {
                    type: string;
                };
                keyPassword: {
                    type: string;
                    description: string;
                };
                excludedFeatures: {
                    type: string;
                };
                includedFeatures: {
                    type: string;
                };
                'gradle.properties': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        'gradle.properties': {
                            'android.debug.obsoleteApi': boolean;
                            'debug.keystore': string;
                            'org.gradle.daemon': boolean;
                            'org.gradle.parallel': boolean;
                            'org.gradle.configureondemand': boolean;
                        };
                    }[];
                };
                'build.gradle': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        allprojects: {
                            repositories: {
                                'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                            };
                        };
                    }[];
                };
                'app/build.gradle': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        apply: string[];
                    }[];
                };
                AndroidManifest: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: ({
                        children: {
                            tag: string;
                            'android:name': string;
                            children: {
                                tag: string;
                                'android:name': string;
                            }[];
                        }[];
                    } | {
                        children: {
                            tag: string;
                            'android:name': string;
                            'android:allowBackup': boolean;
                            'android:largeHeap': boolean;
                            'android:usesCleartextTraffic': boolean;
                            'tools:targetApi': number;
                        }[];
                    })[];
                };
                applyPlugin: {
                    type: string;
                };
                BuildGradle: {
                    type: string;
                    description: string;
                    properties: {
                        allprojects: {
                            type: string;
                            properties: {
                                repositories: {
                                    type: string;
                                    description: string;
                                    additionalProperties: boolean;
                                    examples: {
                                        "flatDir { dirs 'libs'}": boolean;
                                    }[];
                                };
                            };
                        };
                    };
                };
                implementation: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        ios: {
            additionalProperties: boolean;
            type: string;
            properties: {
                deploymentTarget: {
                    type: string;
                };
                teamID: {
                    type: string;
                };
                excludedArchs: {
                    type: string;
                    description: string;
                    default: null;
                    examples: string[][];
                };
                teamIdentifier: {
                    type: string;
                };
                scheme: {
                    type: string;
                };
                appleId: {
                    type: string;
                };
                orientationSupport: {
                    type: string;
                    properties: {
                        phone: {
                            type: string;
                        };
                        tab: {
                            type: string;
                        };
                    };
                    examples: {
                        phone: string[];
                        tab: string[];
                    }[];
                };
                provisioningStyle: {
                    type: string;
                };
                codeSignIdentity: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                commandLineArguments: {
                    type: string;
                    description: string;
                    examples: string[][];
                };
                provisionProfileSpecifier: {
                    type: string;
                };
                provisioningProfiles: {
                    additionalProperties: boolean;
                    type: string;
                };
                systemCapabilities: {
                    additionalProperties: boolean;
                    type: string;
                    examples: {
                        'com.apple.SafariKeychain': boolean;
                        'com.apple.Wallet': boolean;
                        'com.apple.HealthKit': boolean;
                        'com.apple.ApplicationGroups.iOS': boolean;
                        'com.apple.iCloud': boolean;
                        'com.apple.DataProtection': boolean;
                        'com.apple.HomeKit': boolean;
                        'com.apple.ClassKit': boolean;
                        'com.apple.VPNLite': boolean;
                        'com.apple.AutoFillCredentialProvider': boolean;
                        'com.apple.AccessWiFi': boolean;
                        'com.apple.InAppPurchase': boolean;
                        'com.apple.HotspotConfiguration': boolean;
                        'com.apple.Multipath': boolean;
                        'com.apple.GameCenter.iOS': boolean;
                        'com.apple.BackgroundModes': boolean;
                        'com.apple.InterAppAudio': boolean;
                        'com.apple.WAC': boolean;
                        'com.apple.Push': boolean;
                        'com.apple.NearFieldCommunicationTagReading': boolean;
                        'com.apple.ApplePay': boolean;
                        'com.apple.Keychain': boolean;
                        'com.apple.Maps.iOS': boolean;
                        'com.apple.Siri': boolean;
                        'com.apple.NetworkExtensions.iOS': boolean;
                    }[];
                };
                entitlements: {
                    additionalProperties: boolean;
                    type: string;
                };
                runScheme: {
                    type: string;
                };
                sdk: {
                    type: string;
                };
                testFlightId: {
                    type: string;
                };
                firebaseId: {
                    type: string;
                };
                exportOptions: {
                    type: string;
                    additionalProperties: boolean;
                    properties: {
                        method: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        uploadBitcode: {
                            type: string;
                        };
                        compileBitcode: {
                            type: string;
                        };
                        uploadSymbols: {
                            type: string;
                        };
                        signingStyle: {
                            type: string;
                        };
                        signingCertificate: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                    };
                };
                Podfile: {
                    additionalProperties: boolean;
                    type: string;
                };
                xcodeproj: {
                    additionalProperties: boolean;
                    type: string;
                };
                plist: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateApplicationMethods: {
                    type: string;
                    properties: {
                        didFinishLaunchingWithOptions: {
                            type: string;
                        };
                        open: {
                            type: string;
                        };
                        supportedInterfaceOrientationsFor: {
                            type: string;
                        };
                        didReceiveRemoteNotification: {
                            type: string;
                        };
                        didFailToRegisterForRemoteNotificationsWithError: {
                            type: string;
                        };
                        didReceive: {
                            type: string;
                        };
                        didRegister: {
                            type: string;
                        };
                        didRegisterForRemoteNotificationsWithDeviceToken: {
                            type: string;
                        };
                        didConnectCarInterfaceController: {
                            type: string;
                        };
                        didDisconnectCarInterfaceController: {
                            type: string;
                        };
                    };
                };
                appDelegateMethods: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateImports: {
                    type: string;
                };
                appDelegateExtensions: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        tvos: {
            additionalProperties: boolean;
            type: string;
            properties: {
                deploymentTarget: {
                    type: string;
                };
                teamID: {
                    type: string;
                };
                excludedArchs: {
                    type: string;
                    description: string;
                    default: null;
                    examples: string[][];
                };
                teamIdentifier: {
                    type: string;
                };
                scheme: {
                    type: string;
                };
                appleId: {
                    type: string;
                };
                orientationSupport: {
                    type: string;
                    properties: {
                        phone: {
                            type: string;
                        };
                        tab: {
                            type: string;
                        };
                    };
                    examples: {
                        phone: string[];
                        tab: string[];
                    }[];
                };
                provisioningStyle: {
                    type: string;
                };
                codeSignIdentity: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                commandLineArguments: {
                    type: string;
                    description: string;
                    examples: string[][];
                };
                provisionProfileSpecifier: {
                    type: string;
                };
                provisioningProfiles: {
                    additionalProperties: boolean;
                    type: string;
                };
                systemCapabilities: {
                    additionalProperties: boolean;
                    type: string;
                    examples: {
                        'com.apple.SafariKeychain': boolean;
                        'com.apple.Wallet': boolean;
                        'com.apple.HealthKit': boolean;
                        'com.apple.ApplicationGroups.iOS': boolean;
                        'com.apple.iCloud': boolean;
                        'com.apple.DataProtection': boolean;
                        'com.apple.HomeKit': boolean;
                        'com.apple.ClassKit': boolean;
                        'com.apple.VPNLite': boolean;
                        'com.apple.AutoFillCredentialProvider': boolean;
                        'com.apple.AccessWiFi': boolean;
                        'com.apple.InAppPurchase': boolean;
                        'com.apple.HotspotConfiguration': boolean;
                        'com.apple.Multipath': boolean;
                        'com.apple.GameCenter.iOS': boolean;
                        'com.apple.BackgroundModes': boolean;
                        'com.apple.InterAppAudio': boolean;
                        'com.apple.WAC': boolean;
                        'com.apple.Push': boolean;
                        'com.apple.NearFieldCommunicationTagReading': boolean;
                        'com.apple.ApplePay': boolean;
                        'com.apple.Keychain': boolean;
                        'com.apple.Maps.iOS': boolean;
                        'com.apple.Siri': boolean;
                        'com.apple.NetworkExtensions.iOS': boolean;
                    }[];
                };
                entitlements: {
                    additionalProperties: boolean;
                    type: string;
                };
                runScheme: {
                    type: string;
                };
                sdk: {
                    type: string;
                };
                testFlightId: {
                    type: string;
                };
                firebaseId: {
                    type: string;
                };
                exportOptions: {
                    type: string;
                    additionalProperties: boolean;
                    properties: {
                        method: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        uploadBitcode: {
                            type: string;
                        };
                        compileBitcode: {
                            type: string;
                        };
                        uploadSymbols: {
                            type: string;
                        };
                        signingStyle: {
                            type: string;
                        };
                        signingCertificate: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                    };
                };
                Podfile: {
                    additionalProperties: boolean;
                    type: string;
                };
                xcodeproj: {
                    additionalProperties: boolean;
                    type: string;
                };
                plist: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateApplicationMethods: {
                    type: string;
                    properties: {
                        didFinishLaunchingWithOptions: {
                            type: string;
                        };
                        open: {
                            type: string;
                        };
                        supportedInterfaceOrientationsFor: {
                            type: string;
                        };
                        didReceiveRemoteNotification: {
                            type: string;
                        };
                        didFailToRegisterForRemoteNotificationsWithError: {
                            type: string;
                        };
                        didReceive: {
                            type: string;
                        };
                        didRegister: {
                            type: string;
                        };
                        didRegisterForRemoteNotificationsWithDeviceToken: {
                            type: string;
                        };
                        didConnectCarInterfaceController: {
                            type: string;
                        };
                        didDisconnectCarInterfaceController: {
                            type: string;
                        };
                    };
                };
                appDelegateMethods: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateImports: {
                    type: string;
                };
                appDelegateExtensions: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        tizen: {
            additionalProperties: boolean;
            type: string;
            properties: {
                package: {
                    type: string;
                };
                certificateProfile: {
                    type: string;
                };
                appName: {
                    type: string;
                };
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        webos: {
            additionalProperties: boolean;
            type: string;
            properties: {
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        web: {
            additionalProperties: boolean;
            type: string;
            properties: {
                pagesDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                outputDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                exportDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                environment: {
                    type: string;
                };
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        webtv: {
            additionalProperties: boolean;
            type: string;
            properties: {
                pagesDir: {
                    type: string;
                };
                environment: {
                    type: string;
                };
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        chromecast: {
            additionalProperties: boolean;
            type: string;
            properties: {
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        firefox: {
            additionalProperties: boolean;
            type: string;
            properties: {
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        macos: {
            additionalProperties: boolean;
            type: string;
            properties: {
                appleId: {
                    type: string;
                };
                environment: {
                    type: string;
                };
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        linux: {
            additionalProperties: boolean;
            type: string;
            properties: {
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        windows: {
            additionalProperties: boolean;
            type: string;
            properties: {
                language: {
                    type: string;
                    description: string;
                };
                experimentalNuGetDependency: {
                    type: string;
                };
                useWinUI3: {
                    type: string;
                };
                nuGetTestVersion: {
                    type: string;
                };
                reactNativeEngine: {
                    type: string;
                };
                nuGetTestFeed: {
                    type: string;
                };
                overwrite: {
                    type: string;
                    description: string;
                };
                release: {
                    type: string;
                    description: string;
                };
                root: {
                    type: string;
                    description: string;
                };
                arch: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                singleproc: {
                    type: string;
                    description: string;
                };
                emulator: {
                    type: string;
                };
                device: {
                    type: string;
                };
                target: {
                    type: string;
                };
                remoteDebugging: {
                    type: string;
                };
                logging: {
                    type: string;
                    description: string;
                };
                packager: {
                    type: string;
                };
                bundle: {
                    type: string;
                };
                launch: {
                    type: string;
                    description: string;
                };
                autolink: {
                    type: string;
                    description: string;
                };
                build: {
                    type: string;
                    description: string;
                };
                deploy: {
                    type: string;
                };
                sln: {
                    type: string;
                    description: string;
                };
                proj: {
                    type: string;
                    description: string;
                };
                appPath: {
                    type: string;
                    description: string;
                };
                msbuildprops: {
                    type: string;
                    description: string;
                };
                buildLogDirectory: {
                    type: string;
                    description: string;
                };
                info: {
                    type: string;
                    description: string;
                };
                directDebugging: {
                    type: string;
                };
                telemetry: {
                    type: string;
                    description: string;
                };
                devPort: {
                    type: string;
                };
                additionalMetroOptions: {
                    type: string;
                };
                packageExtension: {
                    type: string;
                };
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        xbox: {
            additionalProperties: boolean;
            type: string;
            properties: {
                language: {
                    type: string;
                    description: string;
                };
                experimentalNuGetDependency: {
                    type: string;
                };
                useWinUI3: {
                    type: string;
                };
                nuGetTestVersion: {
                    type: string;
                };
                reactNativeEngine: {
                    type: string;
                };
                nuGetTestFeed: {
                    type: string;
                };
                overwrite: {
                    type: string;
                    description: string;
                };
                release: {
                    type: string;
                    description: string;
                };
                root: {
                    type: string;
                    description: string;
                };
                arch: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                singleproc: {
                    type: string;
                    description: string;
                };
                emulator: {
                    type: string;
                };
                device: {
                    type: string;
                };
                target: {
                    type: string;
                };
                remoteDebugging: {
                    type: string;
                };
                logging: {
                    type: string;
                    description: string;
                };
                packager: {
                    type: string;
                };
                bundle: {
                    type: string;
                };
                launch: {
                    type: string;
                    description: string;
                };
                autolink: {
                    type: string;
                    description: string;
                };
                build: {
                    type: string;
                    description: string;
                };
                deploy: {
                    type: string;
                };
                sln: {
                    type: string;
                    description: string;
                };
                proj: {
                    type: string;
                    description: string;
                };
                appPath: {
                    type: string;
                    description: string;
                };
                msbuildprops: {
                    type: string;
                    description: string;
                };
                buildLogDirectory: {
                    type: string;
                    description: string;
                };
                info: {
                    type: string;
                    description: string;
                };
                directDebugging: {
                    type: string;
                };
                telemetry: {
                    type: string;
                    description: string;
                };
                devPort: {
                    type: string;
                };
                additionalMetroOptions: {
                    type: string;
                };
                packageExtension: {
                    type: string;
                };
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
    };
} | {
    $id: string;
    type: string;
    additionalProperties: boolean;
    properties: {
        ext: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                myCustomRenativeProp: string;
            }[];
        };
        engines: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: ({
                '@rnv/engine-rn': string;
                '@rnv/engine-rn-tvos': string;
                '@rnv/engine-rn-web': string;
                '@rnv/engine-rn-next': string;
                '@rnv/engine-lightning': string;
                '@rnv/engine-rn-electron': string;
                '@rnv/engine-rn-windows': string;
                'custom-engine'?: undefined;
            } | {
                '@rnv/engine-rn': string;
                'custom-engine': {
                    version: string;
                };
                '@rnv/engine-rn-tvos'?: undefined;
                '@rnv/engine-rn-web'?: undefined;
                '@rnv/engine-rn-next'?: undefined;
                '@rnv/engine-lightning'?: undefined;
                '@rnv/engine-rn-electron'?: undefined;
                '@rnv/engine-rn-windows'?: undefined;
            })[];
        };
        platforms: {
            additionalProperties: boolean;
            type: string;
            properties: {
                ios: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        deploymentTarget: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        excludedArchs: {
                            type: string;
                            description: string;
                            default: null;
                            examples: string[][];
                        };
                        teamIdentifier: {
                            type: string;
                        };
                        scheme: {
                            type: string;
                        };
                        appleId: {
                            type: string;
                        };
                        orientationSupport: {
                            type: string;
                            properties: {
                                phone: {
                                    type: string;
                                };
                                tab: {
                                    type: string;
                                };
                            };
                            examples: {
                                phone: string[];
                                tab: string[];
                            }[];
                        };
                        provisioningStyle: {
                            type: string;
                        };
                        codeSignIdentity: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        commandLineArguments: {
                            type: string;
                            description: string;
                            examples: string[][];
                        };
                        provisionProfileSpecifier: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        systemCapabilities: {
                            additionalProperties: boolean;
                            type: string;
                            examples: {
                                'com.apple.SafariKeychain': boolean;
                                'com.apple.Wallet': boolean;
                                'com.apple.HealthKit': boolean;
                                'com.apple.ApplicationGroups.iOS': boolean;
                                'com.apple.iCloud': boolean;
                                'com.apple.DataProtection': boolean;
                                'com.apple.HomeKit': boolean;
                                'com.apple.ClassKit': boolean;
                                'com.apple.VPNLite': boolean;
                                'com.apple.AutoFillCredentialProvider': boolean;
                                'com.apple.AccessWiFi': boolean;
                                'com.apple.InAppPurchase': boolean;
                                'com.apple.HotspotConfiguration': boolean;
                                'com.apple.Multipath': boolean;
                                'com.apple.GameCenter.iOS': boolean;
                                'com.apple.BackgroundModes': boolean;
                                'com.apple.InterAppAudio': boolean;
                                'com.apple.WAC': boolean;
                                'com.apple.Push': boolean;
                                'com.apple.NearFieldCommunicationTagReading': boolean;
                                'com.apple.ApplePay': boolean;
                                'com.apple.Keychain': boolean;
                                'com.apple.Maps.iOS': boolean;
                                'com.apple.Siri': boolean;
                                'com.apple.NetworkExtensions.iOS': boolean;
                            }[];
                        };
                        entitlements: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        runScheme: {
                            type: string;
                        };
                        sdk: {
                            type: string;
                        };
                        testFlightId: {
                            type: string;
                        };
                        firebaseId: {
                            type: string;
                        };
                        exportOptions: {
                            type: string;
                            additionalProperties: boolean;
                            properties: {
                                method: {
                                    type: string;
                                };
                                teamID: {
                                    type: string;
                                };
                                uploadBitcode: {
                                    type: string;
                                };
                                compileBitcode: {
                                    type: string;
                                };
                                uploadSymbols: {
                                    type: string;
                                };
                                signingStyle: {
                                    type: string;
                                };
                                signingCertificate: {
                                    type: string;
                                };
                                provisioningProfiles: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                            };
                        };
                        Podfile: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        xcodeproj: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        plist: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateApplicationMethods: {
                            type: string;
                            properties: {
                                didFinishLaunchingWithOptions: {
                                    type: string;
                                };
                                open: {
                                    type: string;
                                };
                                supportedInterfaceOrientationsFor: {
                                    type: string;
                                };
                                didReceiveRemoteNotification: {
                                    type: string;
                                };
                                didFailToRegisterForRemoteNotificationsWithError: {
                                    type: string;
                                };
                                didReceive: {
                                    type: string;
                                };
                                didRegister: {
                                    type: string;
                                };
                                didRegisterForRemoteNotificationsWithDeviceToken: {
                                    type: string;
                                };
                                didConnectCarInterfaceController: {
                                    type: string;
                                };
                                didDisconnectCarInterfaceController: {
                                    type: string;
                                };
                            };
                        };
                        appDelegateMethods: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateImports: {
                            type: string;
                        };
                        appDelegateExtensions: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                android: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                web: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        pagesDir: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        outputDir: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        exportDir: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        environment: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                webtv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        pagesDir: {
                            type: string;
                        };
                        environment: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                chromecast: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tvos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        deploymentTarget: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        excludedArchs: {
                            type: string;
                            description: string;
                            default: null;
                            examples: string[][];
                        };
                        teamIdentifier: {
                            type: string;
                        };
                        scheme: {
                            type: string;
                        };
                        appleId: {
                            type: string;
                        };
                        orientationSupport: {
                            type: string;
                            properties: {
                                phone: {
                                    type: string;
                                };
                                tab: {
                                    type: string;
                                };
                            };
                            examples: {
                                phone: string[];
                                tab: string[];
                            }[];
                        };
                        provisioningStyle: {
                            type: string;
                        };
                        codeSignIdentity: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        commandLineArguments: {
                            type: string;
                            description: string;
                            examples: string[][];
                        };
                        provisionProfileSpecifier: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        systemCapabilities: {
                            additionalProperties: boolean;
                            type: string;
                            examples: {
                                'com.apple.SafariKeychain': boolean;
                                'com.apple.Wallet': boolean;
                                'com.apple.HealthKit': boolean;
                                'com.apple.ApplicationGroups.iOS': boolean;
                                'com.apple.iCloud': boolean;
                                'com.apple.DataProtection': boolean;
                                'com.apple.HomeKit': boolean;
                                'com.apple.ClassKit': boolean;
                                'com.apple.VPNLite': boolean;
                                'com.apple.AutoFillCredentialProvider': boolean;
                                'com.apple.AccessWiFi': boolean;
                                'com.apple.InAppPurchase': boolean;
                                'com.apple.HotspotConfiguration': boolean;
                                'com.apple.Multipath': boolean;
                                'com.apple.GameCenter.iOS': boolean;
                                'com.apple.BackgroundModes': boolean;
                                'com.apple.InterAppAudio': boolean;
                                'com.apple.WAC': boolean;
                                'com.apple.Push': boolean;
                                'com.apple.NearFieldCommunicationTagReading': boolean;
                                'com.apple.ApplePay': boolean;
                                'com.apple.Keychain': boolean;
                                'com.apple.Maps.iOS': boolean;
                                'com.apple.Siri': boolean;
                                'com.apple.NetworkExtensions.iOS': boolean;
                            }[];
                        };
                        entitlements: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        runScheme: {
                            type: string;
                        };
                        sdk: {
                            type: string;
                        };
                        testFlightId: {
                            type: string;
                        };
                        firebaseId: {
                            type: string;
                        };
                        exportOptions: {
                            type: string;
                            additionalProperties: boolean;
                            properties: {
                                method: {
                                    type: string;
                                };
                                teamID: {
                                    type: string;
                                };
                                uploadBitcode: {
                                    type: string;
                                };
                                compileBitcode: {
                                    type: string;
                                };
                                uploadSymbols: {
                                    type: string;
                                };
                                signingStyle: {
                                    type: string;
                                };
                                signingCertificate: {
                                    type: string;
                                };
                                provisioningProfiles: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                            };
                        };
                        Podfile: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        xcodeproj: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        plist: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateApplicationMethods: {
                            type: string;
                            properties: {
                                didFinishLaunchingWithOptions: {
                                    type: string;
                                };
                                open: {
                                    type: string;
                                };
                                supportedInterfaceOrientationsFor: {
                                    type: string;
                                };
                                didReceiveRemoteNotification: {
                                    type: string;
                                };
                                didFailToRegisterForRemoteNotificationsWithError: {
                                    type: string;
                                };
                                didReceive: {
                                    type: string;
                                };
                                didRegister: {
                                    type: string;
                                };
                                didRegisterForRemoteNotificationsWithDeviceToken: {
                                    type: string;
                                };
                                didConnectCarInterfaceController: {
                                    type: string;
                                };
                                didDisconnectCarInterfaceController: {
                                    type: string;
                                };
                            };
                        };
                        appDelegateMethods: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateImports: {
                            type: string;
                        };
                        appDelegateExtensions: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                androidtv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                firetv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                webos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                macos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        appleId: {
                            type: string;
                        };
                        environment: {
                            type: string;
                        };
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                linux: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tizen: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        package: {
                            type: string;
                        };
                        certificateProfile: {
                            type: string;
                        };
                        appName: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                windows: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        language: {
                            type: string;
                            description: string;
                        };
                        experimentalNuGetDependency: {
                            type: string;
                        };
                        useWinUI3: {
                            type: string;
                        };
                        nuGetTestVersion: {
                            type: string;
                        };
                        reactNativeEngine: {
                            type: string;
                        };
                        nuGetTestFeed: {
                            type: string;
                        };
                        overwrite: {
                            type: string;
                            description: string;
                        };
                        release: {
                            type: string;
                            description: string;
                        };
                        root: {
                            type: string;
                            description: string;
                        };
                        arch: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        singleproc: {
                            type: string;
                            description: string;
                        };
                        emulator: {
                            type: string;
                        };
                        device: {
                            type: string;
                        };
                        target: {
                            type: string;
                        };
                        remoteDebugging: {
                            type: string;
                        };
                        logging: {
                            type: string;
                            description: string;
                        };
                        packager: {
                            type: string;
                        };
                        bundle: {
                            type: string;
                        };
                        launch: {
                            type: string;
                            description: string;
                        };
                        autolink: {
                            type: string;
                            description: string;
                        };
                        build: {
                            type: string;
                            description: string;
                        };
                        deploy: {
                            type: string;
                        };
                        sln: {
                            type: string;
                            description: string;
                        };
                        proj: {
                            type: string;
                            description: string;
                        };
                        appPath: {
                            type: string;
                            description: string;
                        };
                        msbuildprops: {
                            type: string;
                            description: string;
                        };
                        buildLogDirectory: {
                            type: string;
                            description: string;
                        };
                        info: {
                            type: string;
                            description: string;
                        };
                        directDebugging: {
                            type: string;
                        };
                        telemetry: {
                            type: string;
                            description: string;
                        };
                        devPort: {
                            type: string;
                        };
                        additionalMetroOptions: {
                            type: string;
                        };
                        packageExtension: {
                            type: string;
                        };
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                xbox: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        language: {
                            type: string;
                            description: string;
                        };
                        experimentalNuGetDependency: {
                            type: string;
                        };
                        useWinUI3: {
                            type: string;
                        };
                        nuGetTestVersion: {
                            type: string;
                        };
                        reactNativeEngine: {
                            type: string;
                        };
                        nuGetTestFeed: {
                            type: string;
                        };
                        overwrite: {
                            type: string;
                            description: string;
                        };
                        release: {
                            type: string;
                            description: string;
                        };
                        root: {
                            type: string;
                            description: string;
                        };
                        arch: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        singleproc: {
                            type: string;
                            description: string;
                        };
                        emulator: {
                            type: string;
                        };
                        device: {
                            type: string;
                        };
                        target: {
                            type: string;
                        };
                        remoteDebugging: {
                            type: string;
                        };
                        logging: {
                            type: string;
                            description: string;
                        };
                        packager: {
                            type: string;
                        };
                        bundle: {
                            type: string;
                        };
                        launch: {
                            type: string;
                            description: string;
                        };
                        autolink: {
                            type: string;
                            description: string;
                        };
                        build: {
                            type: string;
                            description: string;
                        };
                        deploy: {
                            type: string;
                        };
                        sln: {
                            type: string;
                            description: string;
                        };
                        proj: {
                            type: string;
                            description: string;
                        };
                        appPath: {
                            type: string;
                            description: string;
                        };
                        msbuildprops: {
                            type: string;
                            description: string;
                        };
                        buildLogDirectory: {
                            type: string;
                            description: string;
                        };
                        info: {
                            type: string;
                            description: string;
                        };
                        directDebugging: {
                            type: string;
                        };
                        telemetry: {
                            type: string;
                            description: string;
                        };
                        devPort: {
                            type: string;
                        };
                        additionalMetroOptions: {
                            type: string;
                        };
                        packageExtension: {
                            type: string;
                        };
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                firefoxtv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                firefoxos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tizenmobile: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        package: {
                            type: string;
                        };
                        certificateProfile: {
                            type: string;
                        };
                        appName: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tizenwatch: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        package: {
                            type: string;
                        };
                        certificateProfile: {
                            type: string;
                        };
                        appName: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                androidwear: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                kaios: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
            };
        };
        sdks: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                ANDROID_SDK: string;
                ANDROID_NDK: string;
                TIZEN_SDK: string;
                WEBOS_SDK: string;
                KAIOS_SDK: string;
            }[];
        };
        env: {
            additionalProperties: boolean;
            type: string;
        };
        defaultTargets: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                android: string;
                androidtv: string;
                androidwear: string;
                ios: string;
                tvos: string;
                tizen: string;
                tizenwatch: string;
                tizenmobile: string;
                webos: string;
            }[];
        };
        plugins: {
            type: string;
            description: string;
            examples: {
                renative: string;
                react: string;
                'react-native-cached-image': string;
                'react-native-web-image-loader': string;
                'react-native-gesture-handler': {
                    version: string;
                };
            }[];
            additionalProperties: {
                type: string[];
                additionalProperties: boolean;
                properties: {
                    enabled: {
                        type: string;
                    };
                    props: {
                        additionalProperties: boolean;
                        type: string;
                    };
                    version: {
                        type: string;
                    };
                    source: {
                        type: string;
                        description: string;
                        examples: string[];
                    };
                    'no-npm': {
                        type: string;
                    };
                    skipMerge: {
                        type: string;
                        description: string;
                        examples: boolean[];
                    };
                    npm: {
                        additionalProperties: boolean;
                        type: string;
                    };
                    pluginDependencies: {
                        type: string[];
                    };
                    webpack: {
                        additionalProperties: boolean;
                        type: string;
                        description: string;
                    };
                    webpackConfig: {
                        additionalProperties: boolean;
                        type: string;
                        description: string;
                        properties: {
                            modulePaths: {
                                type: string[];
                                additionalProperties: boolean;
                            };
                            moduleAliases: {
                                type: string[];
                                additionalProperties: boolean;
                            };
                        };
                    };
                    android: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    androidtv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    firetv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    androidwear: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    ios: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            podName: {
                                type: string;
                            };
                            podNames: {
                                type: string;
                            };
                            isStatic: {
                                type: string;
                            };
                            staticPods: {
                                type: string;
                                description: string;
                                examples: string[][];
                            };
                            Podfile: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            xcodeproj: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            plist: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateApplicationMethods: {
                                type: string;
                                properties: {
                                    didFinishLaunchingWithOptions: {
                                        type: string;
                                    };
                                    open: {
                                        type: string;
                                    };
                                    supportedInterfaceOrientationsFor: {
                                        type: string;
                                    };
                                    didReceiveRemoteNotification: {
                                        type: string;
                                    };
                                    didFailToRegisterForRemoteNotificationsWithError: {
                                        type: string;
                                    };
                                    didReceive: {
                                        type: string;
                                    };
                                    didRegister: {
                                        type: string;
                                    };
                                    didRegisterForRemoteNotificationsWithDeviceToken: {
                                        type: string;
                                    };
                                    didConnectCarInterfaceController: {
                                        type: string;
                                    };
                                    didDisconnectCarInterfaceController: {
                                        type: string;
                                    };
                                };
                            };
                            appDelegateMethods: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateImports: {
                                type: string;
                            };
                            appDelegateExtensions: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    tvos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            podName: {
                                type: string;
                            };
                            podNames: {
                                type: string;
                            };
                            isStatic: {
                                type: string;
                            };
                            staticPods: {
                                type: string;
                                description: string;
                                examples: string[][];
                            };
                            Podfile: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            xcodeproj: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            plist: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateApplicationMethods: {
                                type: string;
                                properties: {
                                    didFinishLaunchingWithOptions: {
                                        type: string;
                                    };
                                    open: {
                                        type: string;
                                    };
                                    supportedInterfaceOrientationsFor: {
                                        type: string;
                                    };
                                    didReceiveRemoteNotification: {
                                        type: string;
                                    };
                                    didFailToRegisterForRemoteNotificationsWithError: {
                                        type: string;
                                    };
                                    didReceive: {
                                        type: string;
                                    };
                                    didRegister: {
                                        type: string;
                                    };
                                    didRegisterForRemoteNotificationsWithDeviceToken: {
                                        type: string;
                                    };
                                    didConnectCarInterfaceController: {
                                        type: string;
                                    };
                                    didDisconnectCarInterfaceController: {
                                        type: string;
                                    };
                                };
                            };
                            appDelegateMethods: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateImports: {
                                type: string;
                            };
                            appDelegateExtensions: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    tizen: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    webos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    web: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    webtv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    chromecast: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    firefox: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    macos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    linux: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    windows: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    xbox: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                };
            };
        };
        projectName: {
            type: string;
            description: string;
            examples: string[];
        };
        extend: {
            type: string;
        };
        extendsTemplate: {
            type: string;
            description: string;
            examples: string[];
        };
        projectTemplates: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                'my-custom-template': {};
            }[];
        };
        permissions: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                ios: {};
                android: {};
            }[];
            properties: {
                android: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        INTERNET: {
                            key: string;
                            security: string;
                        };
                        SYSTEM_ALERT_WINDOW: {
                            key: string;
                            security: string;
                        };
                    }[];
                };
                ios: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        NSAppleMusicUsageDescription: {
                            desc: string;
                        };
                        NSBluetoothPeripheralUsageDescription: {
                            desc: string;
                        };
                        NSCalendarsUsageDescription: {
                            desc: string;
                        };
                        NSCameraUsageDescription: {
                            desc: string;
                        };
                        NSLocationWhenInUseUsageDescription: {
                            desc: string;
                        };
                        NSMicrophoneUsageDescription: {
                            desc: string;
                        };
                        NSMotionUsageDescription: {
                            desc: string;
                        };
                        NSPhotoLibraryAddUsageDescription: {
                            desc: string;
                        };
                        NSPhotoLibraryUsageDescription: {
                            desc: string;
                        };
                        NSSpeechRecognitionUsageDescription: {
                            desc: string;
                        };
                        NSContactsUsageDescription: {
                            desc: string;
                        };
                        NSFaceIDUsageDescription: {
                            desc: string;
                        };
                        NSLocationAlwaysUsageDescription: {
                            desc: string;
                        };
                        NSBluetoothAlwaysUsageDescription: {
                            desc: string;
                        };
                        NSLocationAlwaysAndWhenInUseUsageDescription: {
                            desc: string;
                        };
                    }[];
                };
            };
        };
        workspaceID: {
            type: string;
            description: string;
            examples: string[];
        };
        version: {
            type: string;
            description: string;
            examples: string[];
        };
        versionCode: {
            description: string;
            type: string;
            examples: string[];
        };
        versionFormat: {
            description: string;
            examples: string[];
            type: string;
        };
        versionCodeFormat: {
            description: string;
            examples: string[];
            type: string;
        };
        description: {
            type: string;
        };
        id: {
            type: string;
            description: string;
            examples: string[];
        };
        isMonorepo: {
            type: string;
        };
        monoRepo: {
            type: string;
            description: string;
            example: string;
        };
        enableAnalytics: {
            type: string;
            description: string;
            examples: boolean[];
        };
        paths: {
            additionalProperties: boolean;
            type: string;
            description: string;
            properties: {
                appConfigsDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                platformAssetsDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                platformBuildsDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                pluginTemplates: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomScope: {
                            npm: string;
                            path: string;
                        };
                    }[];
                };
            };
        };
        tasks: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                install: {
                    script: string;
                };
            }[];
        };
        pipes: {
            type: string;
            description: string;
            examples: string[][];
        };
        defaults: {
            additionalProperties: boolean;
            type: string;
            description: string;
            properties: {
                supportedPlatforms: {
                    type: string;
                    description: string;
                    examples: string[][];
                };
                schemes: {
                    type: string;
                    description: string;
                    examples: {
                        ios: string;
                        android: string;
                    }[];
                };
                targets: {
                    type: string;
                    description: string;
                    examples: {
                        ios: string;
                        tvos: string;
                    }[];
                };
                ports: {
                    type: string;
                    description: string;
                    examples: {
                        ios: number;
                        android: number;
                        androidtv: number;
                        tvos: number;
                        macos: number;
                        web: number;
                        tizen: number;
                        webos: number;
                        androidwear: number;
                        tizenwatch: number;
                        tizenmobile: number;
                        windows: number;
                        kaios: number;
                        firefoxos: number;
                        firefoxtv: number;
                        webtv: number;
                        linux: number;
                    }[];
                };
            };
        };
        pluginTemplates: {
            additionalProperties: boolean;
            type: string;
        };
        templates: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                '@rnv/template-starter': {
                    version: string;
                };
            }[];
        };
        currentTemplate: {
            type: string;
            description: string;
            examples: string[];
        };
        crypto: {
            additionalProperties: boolean;
            type: string;
            description: string;
            properties: {
                encrypt: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        dest: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                    };
                };
                decrypt: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        source: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                    };
                };
                optional: {
                    type: string;
                };
            };
        };
        integrations: {
            additionalProperties: boolean;
            type: string;
        };
        publish: {
            additionalProperties: boolean;
            type: string;
        };
        private: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                myPrivateKy: string;
            }[];
        };
        hidden: {
            type: string;
            description: string;
            examples: boolean[];
        };
        templateConfig: {
            description: string;
            type: string;
            additionalProperties: boolean;
            properties: {
                includedPaths: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                bootstrapQuestions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: {
                        title: string;
                        type: string;
                        configProp: {
                            key: string;
                            file: string;
                        };
                        options: {
                            title: string;
                            value: {
                                id: string;
                            };
                        }[];
                    }[][];
                };
            };
        };
        enableHookRebuild: {
            type: string;
            description: string;
            examples: boolean[];
        };
        runtime: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                someRuntimeProperty: string;
            }[];
        };
        common: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                author: {
                    name: string;
                    email: string;
                    url: string;
                };
                license: string;
                includedPlugins: string[];
                includedFonts: string[];
                backgroundColor: string;
                runtime: {
                    welcomeMessage: string;
                };
            }[];
            properties: {
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
    };
};
export function validateRenativeJsonSchema(cObj: any): (boolean | Ajv)[];
export function getRuntimeObjectSchema(): {
    $id: string;
    type: string;
    additionalProperties: boolean;
    properties: {
        buildConfig: {
            additionalProperties: boolean;
            type: string;
        };
        cli: {
            additionalProperties: boolean;
            type: string;
        };
        files: {
            additionalProperties: boolean;
            type: string;
            properties: {
                rnv: {
                    additionalProperties: boolean;
                    type: string;
                };
                workspace: {
                    additionalProperties: boolean;
                    type: string;
                };
                defaultWorkspace: {
                    additionalProperties: boolean;
                    type: string;
                };
                project: {
                    additionalProperties: boolean;
                    type: string;
                };
                appConfig: {
                    additionalProperties: boolean;
                    type: string;
                };
                buildHooks: {
                    additionalProperties: boolean;
                    type: string;
                };
                home: {
                    additionalProperties: boolean;
                    type: string;
                };
                template: {
                    additionalProperties: boolean;
                    type: string;
                };
            };
        };
        paths: {
            additionalProperties: boolean;
            type: string;
            properties: {
                GLOBAL_RNV_DIR: {
                    type: string;
                };
                GLOBAL_RNV_CONFIG: {
                    type: string;
                };
                rnv: {
                    additionalProperties: boolean;
                    type: string;
                };
                workspace: {
                    additionalProperties: boolean;
                    type: string;
                };
                defaultWorkspace: {
                    additionalProperties: boolean;
                    type: string;
                };
                project: {
                    additionalProperties: boolean;
                    type: string;
                };
                appConfig: {
                    additionalProperties: boolean;
                    type: string;
                };
                buildHooks: {
                    additionalProperties: boolean;
                    type: string;
                };
                home: {
                    additionalProperties: boolean;
                    type: string;
                };
                template: {
                    additionalProperties: boolean;
                    type: string;
                };
            };
        };
        runtime: {
            additionalProperties: boolean;
            type: string;
            properties: {
                appId: {
                    type: string;
                };
                port: {
                    type: string;
                };
                target: {
                    type: string;
                };
                scheme: {
                    type: string;
                };
                localhost: {
                    type: string;
                };
                timestamp: {
                    type: string;
                };
                forceBundleAssets: {
                    type: string;
                };
                missingEnginePlugins: {
                    additionalProperties: boolean;
                    type: string;
                };
                engine: {
                    additionalProperties: boolean;
                    type: string;
                };
                task: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                };
                hosted: {
                    type: string;
                };
                supportedPlatforms: {
                    type: string;
                };
                enginesById: {
                    additionalProperties: boolean;
                    type: string;
                };
                enginesByPlatform: {
                    additionalProperties: boolean;
                    type: string;
                };
            };
        };
        isBuildHooksReady: {
            type: string;
        };
        program: {
            additionalProperties: boolean;
            type: string;
        };
        process: {
            additionalProperties: boolean;
            type: string;
        };
        command: {
            type: string;
        };
        subCommand: {
            type: string;
        };
        platformDefaults: {
            additionalProperties: boolean;
            type: string;
        };
        platform: {
            additionalProperties: boolean;
            type: string;
        };
        timeStart: {
            additionalProperties: boolean;
            type: string;
        };
        rnvVersion: {
            type: string;
        };
        configPropsInjects: {
            type: string;
        };
        runtimePropsInjects: {
            type: string;
        };
        systemPropsInjects: {
            type: string;
        };
        _currentTask: {};
        buildHooks: {
            additionalProperties: boolean;
            type: string;
        };
        buildPipes: {
            additionalProperties: boolean;
            type: string;
        };
        isBuildHookReady: {
            additionalProperties: boolean;
            type: string;
        };
        rnv: {
            additionalProperties: boolean;
            type: string;
        };
        workspace: {
            additionalProperties: boolean;
            type: string;
        };
        defaultWorkspace: {
            additionalProperties: boolean;
            type: string;
        };
        project: {
            additionalProperties: boolean;
            type: string;
        };
        appConfig: {
            additionalProperties: boolean;
            type: string;
        };
        home: {
            additionalProperties: boolean;
            type: string;
        };
        template: {
            additionalProperties: boolean;
            type: string;
        };
    };
};
export function validateRuntimeObjectSchema(cObj: any): (boolean | Ajv)[];
import Ajv from "ajv";
//# sourceMappingURL=index.d.ts.map