export namespace schemaPlatforms {
    const $id: string;
    namespace definitions {
        namespace android {
            const additionalProperties: boolean;
            const type: string;
            const properties: {
                enableAndroidX: {
                    type: string;
                    default: boolean;
                    examples: boolean[];
                };
                enableHermes: {
                    type: string;
                    default: boolean;
                    description: string;
                    examples: boolean[];
                };
                reactNativeEngine: {
                    type: string;
                    default: string;
                    description: string;
                    examples: boolean[];
                };
                signingConfig: {
                    type: string;
                    default: string;
                    description: string;
                    examples: string[];
                };
                minSdkVersion: {
                    type: string;
                    default: number;
                    examples: number[];
                };
                multipleAPKs: {
                    type: string;
                    default: boolean;
                    description: string;
                    examples: boolean[];
                };
                aab: {
                    type: string;
                    description: string;
                    default: boolean;
                    examples: boolean[];
                };
                extraGradleParams: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                minifyEnabled: {
                    type: string;
                    description: string;
                    default: boolean;
                    examples: boolean[];
                };
                targetSdkVersion: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                compileSdkVersion: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                gradleBuildToolsVersion: {
                    type: string;
                    description: string;
                    default: string;
                    examples: string[];
                };
                gradleWrapperVersion: {
                    type: string;
                    description: string;
                    default: string;
                    examples: string[];
                };
                mainActivity: {
                    type: string;
                    description: string;
                    default: string;
                    properties: {
                        onCreate: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                    };
                };
                storeFile: {
                    type: string;
                };
                storePassword: {
                    type: string;
                    description: string;
                };
                keyAlias: {
                    type: string;
                };
                keyPassword: {
                    type: string;
                    description: string;
                };
                excludedFeatures: {
                    type: string;
                };
                includedFeatures: {
                    type: string;
                };
                'gradle.properties': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        'gradle.properties': {
                            'android.debug.obsoleteApi': boolean;
                            'debug.keystore': string;
                            'org.gradle.daemon': boolean;
                            'org.gradle.parallel': boolean;
                            'org.gradle.configureondemand': boolean;
                        };
                    }[];
                };
                'build.gradle': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        allprojects: {
                            repositories: {
                                'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                            };
                        };
                    }[];
                };
                'app/build.gradle': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        apply: string[];
                    }[];
                };
                AndroidManifest: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: ({
                        children: {
                            tag: string;
                            'android:name': string;
                            children: {
                                tag: string;
                                'android:name': string;
                            }[];
                        }[];
                    } | {
                        children: {
                            tag: string;
                            'android:name': string;
                            'android:allowBackup': boolean;
                            'android:largeHeap': boolean;
                            'android:usesCleartextTraffic': boolean;
                            'tools:targetApi': number;
                        }[];
                    })[];
                };
                applyPlugin: {
                    type: string;
                };
                BuildGradle: {
                    type: string;
                    description: string;
                    properties: {
                        allprojects: {
                            type: string;
                            properties: {
                                repositories: {
                                    type: string;
                                    description: string;
                                    additionalProperties: boolean;
                                    examples: {
                                        "flatDir { dirs 'libs'}": boolean;
                                    }[];
                                };
                            };
                        };
                    };
                };
                implementation: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        }
        namespace ios {
            const additionalProperties_1: boolean;
            export { additionalProperties_1 as additionalProperties };
            const type_1: string;
            export { type_1 as type };
            export namespace properties_1 {
                export namespace deploymentTarget {
                    const type_2: string;
                    export { type_2 as type };
                }
                export namespace teamID {
                    const type_3: string;
                    export { type_3 as type };
                }
                export namespace excludedArchs {
                    const type_4: string;
                    export { type_4 as type };
                    export const description: string;
                    const _default: null;
                    export { _default as default };
                    export const examples: string[][];
                }
                export namespace teamIdentifier {
                    const type_5: string;
                    export { type_5 as type };
                }
                export namespace scheme {
                    const type_6: string;
                    export { type_6 as type };
                }
                export namespace appleId {
                    const type_7: string;
                    export { type_7 as type };
                }
                export namespace orientationSupport {
                    const type_8: string;
                    export { type_8 as type };
                    export namespace properties_2 {
                        namespace phone {
                            const type_9: string;
                            export { type_9 as type };
                        }
                        namespace tab {
                            const type_10: string;
                            export { type_10 as type };
                        }
                    }
                    export { properties_2 as properties };
                    const examples_1: {
                        phone: string[];
                        tab: string[];
                    }[];
                    export { examples_1 as examples };
                }
                export namespace provisioningStyle {
                    const type_11: string;
                    export { type_11 as type };
                }
                export namespace codeSignIdentity {
                    const type_12: string;
                    export { type_12 as type };
                    const description_1: string;
                    export { description_1 as description };
                    const examples_2: string[];
                    export { examples_2 as examples };
                }
                export namespace commandLineArguments {
                    const type_13: string;
                    export { type_13 as type };
                    const description_2: string;
                    export { description_2 as description };
                    const examples_3: string[][];
                    export { examples_3 as examples };
                }
                export namespace provisionProfileSpecifier {
                    const type_14: string;
                    export { type_14 as type };
                }
                export namespace provisioningProfiles {
                    const additionalProperties_2: boolean;
                    export { additionalProperties_2 as additionalProperties };
                    const type_15: string;
                    export { type_15 as type };
                }
                export namespace systemCapabilities {
                    const additionalProperties_3: boolean;
                    export { additionalProperties_3 as additionalProperties };
                    const type_16: string;
                    export { type_16 as type };
                    const examples_4: {
                        'com.apple.SafariKeychain': boolean;
                        'com.apple.Wallet': boolean;
                        'com.apple.HealthKit': boolean;
                        'com.apple.ApplicationGroups.iOS': boolean;
                        'com.apple.iCloud': boolean;
                        'com.apple.DataProtection': boolean;
                        'com.apple.HomeKit': boolean;
                        'com.apple.ClassKit': boolean;
                        'com.apple.VPNLite': boolean;
                        'com.apple.AutoFillCredentialProvider': boolean;
                        'com.apple.AccessWiFi': boolean;
                        'com.apple.InAppPurchase': boolean;
                        'com.apple.HotspotConfiguration': boolean;
                        'com.apple.Multipath': boolean;
                        'com.apple.GameCenter.iOS': boolean;
                        'com.apple.BackgroundModes': boolean;
                        'com.apple.InterAppAudio': boolean;
                        'com.apple.WAC': boolean;
                        'com.apple.Push': boolean;
                        'com.apple.NearFieldCommunicationTagReading': boolean;
                        'com.apple.ApplePay': boolean;
                        'com.apple.Keychain': boolean;
                        'com.apple.Maps.iOS': boolean;
                        'com.apple.Siri': boolean;
                        'com.apple.NetworkExtensions.iOS': boolean;
                    }[];
                    export { examples_4 as examples };
                }
                export namespace entitlements {
                    const additionalProperties_4: boolean;
                    export { additionalProperties_4 as additionalProperties };
                    const type_17: string;
                    export { type_17 as type };
                }
                export namespace runScheme {
                    const type_18: string;
                    export { type_18 as type };
                }
                export namespace sdk {
                    const type_19: string;
                    export { type_19 as type };
                }
                export namespace testFlightId {
                    const type_20: string;
                    export { type_20 as type };
                }
                export namespace firebaseId {
                    const type_21: string;
                    export { type_21 as type };
                }
                export namespace exportOptions {
                    const type_22: string;
                    export { type_22 as type };
                    const additionalProperties_5: boolean;
                    export { additionalProperties_5 as additionalProperties };
                    export namespace properties_3 {
                        export namespace method {
                            const type_23: string;
                            export { type_23 as type };
                        }
                        export namespace teamID_1 {
                            const type_24: string;
                            export { type_24 as type };
                        }
                        export { teamID_1 as teamID };
                        export namespace uploadBitcode {
                            const type_25: string;
                            export { type_25 as type };
                        }
                        export namespace compileBitcode {
                            const type_26: string;
                            export { type_26 as type };
                        }
                        export namespace uploadSymbols {
                            const type_27: string;
                            export { type_27 as type };
                        }
                        export namespace signingStyle {
                            const type_28: string;
                            export { type_28 as type };
                        }
                        export namespace signingCertificate {
                            const type_29: string;
                            export { type_29 as type };
                        }
                        export namespace provisioningProfiles_1 {
                            const additionalProperties_6: boolean;
                            export { additionalProperties_6 as additionalProperties };
                            const type_30: string;
                            export { type_30 as type };
                        }
                        export { provisioningProfiles_1 as provisioningProfiles };
                    }
                    export { properties_3 as properties };
                }
                export namespace Podfile {
                    const additionalProperties_7: boolean;
                    export { additionalProperties_7 as additionalProperties };
                    const type_31: string;
                    export { type_31 as type };
                }
                export namespace xcodeproj {
                    const additionalProperties_8: boolean;
                    export { additionalProperties_8 as additionalProperties };
                    const type_32: string;
                    export { type_32 as type };
                }
                export namespace plist {
                    const additionalProperties_9: boolean;
                    export { additionalProperties_9 as additionalProperties };
                    const type_33: string;
                    export { type_33 as type };
                }
                export namespace appDelegateApplicationMethods {
                    const type_34: string;
                    export { type_34 as type };
                    export namespace properties_4 {
                        namespace didFinishLaunchingWithOptions {
                            const type_35: string;
                            export { type_35 as type };
                        }
                        namespace open {
                            const type_36: string;
                            export { type_36 as type };
                        }
                        namespace supportedInterfaceOrientationsFor {
                            const type_37: string;
                            export { type_37 as type };
                        }
                        namespace didReceiveRemoteNotification {
                            const type_38: string;
                            export { type_38 as type };
                        }
                        namespace didFailToRegisterForRemoteNotificationsWithError {
                            const type_39: string;
                            export { type_39 as type };
                        }
                        namespace didReceive {
                            const type_40: string;
                            export { type_40 as type };
                        }
                        namespace didRegister {
                            const type_41: string;
                            export { type_41 as type };
                        }
                        namespace didRegisterForRemoteNotificationsWithDeviceToken {
                            const type_42: string;
                            export { type_42 as type };
                        }
                        namespace didConnectCarInterfaceController {
                            const type_43: string;
                            export { type_43 as type };
                        }
                        namespace didDisconnectCarInterfaceController {
                            const type_44: string;
                            export { type_44 as type };
                        }
                    }
                    export { properties_4 as properties };
                }
                export namespace appDelegateMethods {
                    const additionalProperties_10: boolean;
                    export { additionalProperties_10 as additionalProperties };
                    const type_45: string;
                    export { type_45 as type };
                }
                export namespace appDelegateImports {
                    const type_46: string;
                    export { type_46 as type };
                }
                export namespace appDelegateExtensions {
                    const type_47: string;
                    export { type_47 as type };
                }
                export namespace buildSchemes {
                    export namespace additionalProperties_11 {
                        const type_48: string;
                        export { type_48 as type };
                        const additionalProperties_12: boolean;
                        export { additionalProperties_12 as additionalProperties };
                        const properties_5: any;
                        export { properties_5 as properties };
                    }
                    export { additionalProperties_11 as additionalProperties };
                    const type_49: string;
                    export { type_49 as type };
                }
                export namespace engine {
                    const type_50: string;
                    export { type_50 as type };
                }
                export namespace entryFile {
                    const type_51: string;
                    export { type_51 as type };
                }
                export namespace bundleAssets {
                    const type_52: string;
                    export { type_52 as type };
                    const description_3: string;
                    export { description_3 as description };
                }
                export namespace enableSourceMaps {
                    const type_53: string;
                    export { type_53 as type };
                    const description_4: string;
                    export { description_4 as description };
                }
                export namespace bundleIsDev {
                    const type_54: string;
                    export { type_54 as type };
                }
                export namespace deploy {
                    const additionalProperties_13: boolean;
                    export { additionalProperties_13 as additionalProperties };
                    const type_55: string;
                    export { type_55 as type };
                    export namespace properties_6 {
                        export namespace type_56 {
                            const type_57: string;
                            export { type_57 as type };
                        }
                        export { type_56 as type };
                    }
                    export { properties_6 as properties };
                }
                export namespace ext {
                    const additionalProperties_14: boolean;
                    export { additionalProperties_14 as additionalProperties };
                    const type_58: string;
                    export { type_58 as type };
                    const description_5: string;
                    export { description_5 as description };
                    const examples_5: {
                        myCustomRenativeProp: string;
                    }[];
                    export { examples_5 as examples };
                }
                export namespace excludedPlugins {
                    const type_59: string;
                    export { type_59 as type };
                    export namespace items {
                        const type_60: string;
                        export { type_60 as type };
                    }
                    const description_6: string;
                    export { description_6 as description };
                    const examples_6: string[][];
                    export { examples_6 as examples };
                }
                export namespace includedPlugins {
                    const type_61: string;
                    export { type_61 as type };
                    export namespace items_1 {
                        const type_62: string;
                        export { type_62 as type };
                    }
                    export { items_1 as items };
                    const description_7: string;
                    export { description_7 as description };
                    const examples_7: string[][];
                    export { examples_7 as examples };
                }
                export namespace includedPermissions {
                    const type_63: string;
                    export { type_63 as type };
                    export namespace items_2 {
                        const type_64: string;
                        export { type_64 as type };
                    }
                    export { items_2 as items };
                    const description_8: string;
                    export { description_8 as description };
                    const examples_8: string[][];
                    export { examples_8 as examples };
                }
                export namespace permissions {
                    const description_9: string;
                    export { description_9 as description };
                    const type_65: string;
                    export { type_65 as type };
                    export namespace items_3 {
                        const type_66: string;
                        export { type_66 as type };
                    }
                    export { items_3 as items };
                }
                export namespace id {
                    const type_67: string;
                    export { type_67 as type };
                }
                export namespace title {
                    const type_68: string;
                    export { type_68 as type };
                    const description_10: string;
                    export { description_10 as description };
                    const examples_9: string[];
                    export { examples_9 as examples };
                }
                export namespace description_11 {
                    const type_69: string;
                    export { type_69 as type };
                    const description_12: string;
                    export { description_12 as description };
                    const examples_10: string[];
                    export { examples_10 as examples };
                }
                export { description_11 as description };
                export namespace author {
                    const additionalProperties_15: boolean;
                    export { additionalProperties_15 as additionalProperties };
                    const type_70: string[];
                    export { type_70 as type };
                }
                export namespace includedFonts {
                    const type_71: string;
                    export { type_71 as type };
                    export namespace items_4 {
                        const type_72: string;
                        export { type_72 as type };
                    }
                    export { items_4 as items };
                    const description_13: string;
                    export { description_13 as description };
                    const examples_11: string[][];
                    export { examples_11 as examples };
                }
                export namespace backgroundColor {
                    const type_73: string;
                    export { type_73 as type };
                    const description_14: string;
                    export { description_14 as description };
                    const examples_12: string[];
                    export { examples_12 as examples };
                }
                export namespace splashScreen {
                    const type_74: string;
                    export { type_74 as type };
                }
                export namespace ignoreWarnings {
                    const type_75: string;
                    export { type_75 as type };
                }
                export namespace ignoreLogs {
                    const type_76: string;
                    export { type_76 as type };
                }
                export namespace license {
                    const type_77: string;
                    export { type_77 as type };
                }
                export namespace timestampAssets {
                    const type_78: string;
                    export { type_78 as type };
                    const description_15: string;
                    export { description_15 as description };
                    const examples_13: boolean[];
                    export { examples_13 as examples };
                }
                export namespace versionedAssets {
                    const type_79: string;
                    export { type_79 as type };
                    const description_16: string;
                    export { description_16 as description };
                    const examples_14: boolean[];
                    export { examples_14 as examples };
                }
                export namespace fontSources {
                    const type_80: string;
                    export { type_80 as type };
                    export namespace items_5 {
                        const type_81: string;
                        export { type_81 as type };
                    }
                    export { items_5 as items };
                    const description_17: string;
                    export { description_17 as description };
                    const examples_15: string[][];
                    export { examples_15 as examples };
                }
                export namespace assetSources {
                    const type_82: string;
                    export { type_82 as type };
                    export namespace items_6 {
                        const type_83: string;
                        export { type_83 as type };
                    }
                    export { items_6 as items };
                    const description_18: string;
                    export { description_18 as description };
                    const examples_16: string[][];
                    export { examples_16 as examples };
                }
                export namespace portOffset {
                    const type_84: string;
                    export { type_84 as type };
                    const description_19: string;
                    export { description_19 as description };
                    const examples_17: number[];
                    export { examples_17 as examples };
                }
                export namespace runtime {
                    const additionalProperties_16: boolean;
                    export { additionalProperties_16 as additionalProperties };
                    const type_85: string;
                    export { type_85 as type };
                    const description_20: string;
                    export { description_20 as description };
                    const examples_18: {
                        someRuntimeProperty: string;
                    }[];
                    export { examples_18 as examples };
                }
            }
            export { properties_1 as properties };
        }
        namespace tvos {
            const additionalProperties_17: boolean;
            export { additionalProperties_17 as additionalProperties };
            const type_86: string;
            export { type_86 as type };
            export namespace properties_7 { }
            export { properties_7 as properties };
        }
        namespace tizen {
            const additionalProperties_18: boolean;
            export { additionalProperties_18 as additionalProperties };
            const type_87: string;
            export { type_87 as type };
            export namespace properties_8 {
                export namespace _package {
                    const type_88: string;
                    export { type_88 as type };
                }
                export { _package as package };
                export namespace certificateProfile {
                    const type_89: string;
                    export { type_89 as type };
                }
                export namespace appName {
                    const type_90: string;
                    export { type_90 as type };
                }
                export namespace devServerHost {
                    const type_91: string;
                    export { type_91 as type };
                }
                export namespace webpackConfig {
                    const additionalProperties_19: boolean;
                    export { additionalProperties_19 as additionalProperties };
                    const type_92: string;
                    export { type_92 as type };
                    export namespace properties_9 {
                        export namespace devServerHost_1 {
                            const type_93: string;
                            export { type_93 as type };
                        }
                        export { devServerHost_1 as devServerHost };
                        export namespace publicUrl {
                            const type_94: string;
                            export { type_94 as type };
                        }
                        export namespace metaTags {
                            const additionalProperties_20: boolean;
                            export { additionalProperties_20 as additionalProperties };
                            const type_95: string;
                            export { type_95 as type };
                        }
                        export namespace customScripts {
                            const type_96: string;
                            export { type_96 as type };
                        }
                        export namespace extend {
                            const additionalProperties_21: boolean;
                            export { additionalProperties_21 as additionalProperties };
                            const type_97: string;
                            export { type_97 as type };
                            const description_21: string;
                            export { description_21 as description };
                            const examples_19: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                            export { examples_19 as examples };
                        }
                    }
                    export { properties_9 as properties };
                }
            }
            export { properties_8 as properties };
        }
        namespace webos {
            const additionalProperties_22: boolean;
            export { additionalProperties_22 as additionalProperties };
            const type_98: string;
            export { type_98 as type };
            export namespace properties_10 { }
            export { properties_10 as properties };
        }
        namespace web {
            const additionalProperties_23: boolean;
            export { additionalProperties_23 as additionalProperties };
            const type_99: string;
            export { type_99 as type };
            export namespace properties_11 {
                namespace pagesDir {
                    const type_100: string;
                    export { type_100 as type };
                    const description_22: string;
                    export { description_22 as description };
                    const examples_20: string[];
                    export { examples_20 as examples };
                }
                namespace outputDir {
                    const type_101: string;
                    export { type_101 as type };
                    const description_23: string;
                    export { description_23 as description };
                    const examples_21: string[];
                    export { examples_21 as examples };
                }
                namespace exportDir {
                    const type_102: string;
                    export { type_102 as type };
                    const description_24: string;
                    export { description_24 as description };
                    const examples_22: string[];
                    export { examples_22 as examples };
                }
                namespace environment {
                    const type_103: string;
                    export { type_103 as type };
                }
            }
            export { properties_11 as properties };
        }
        namespace webtv {
            const additionalProperties_24: boolean;
            export { additionalProperties_24 as additionalProperties };
            const type_104: string;
            export { type_104 as type };
            export namespace properties_12 {
                export namespace pagesDir_1 {
                    const type_105: string;
                    export { type_105 as type };
                }
                export { pagesDir_1 as pagesDir };
                export namespace environment_1 {
                    const type_106: string;
                    export { type_106 as type };
                }
                export { environment_1 as environment };
            }
            export { properties_12 as properties };
        }
        namespace chromecast {
            const additionalProperties_25: boolean;
            export { additionalProperties_25 as additionalProperties };
            const type_107: string;
            export { type_107 as type };
            export namespace properties_13 { }
            export { properties_13 as properties };
        }
        namespace firefox {
            const additionalProperties_26: boolean;
            export { additionalProperties_26 as additionalProperties };
            const type_108: string;
            export { type_108 as type };
            export namespace properties_14 { }
            export { properties_14 as properties };
        }
        namespace macos {
            const additionalProperties_27: boolean;
            export { additionalProperties_27 as additionalProperties };
            const type_109: string;
            export { type_109 as type };
            export namespace properties_15 {
                export namespace appleId_1 {
                    const type_110: string;
                    export { type_110 as type };
                }
                export { appleId_1 as appleId };
                export namespace environment_2 {
                    const type_111: string;
                    export { type_111 as type };
                }
                export { environment_2 as environment };
                export namespace electronConfig {
                    const additionalProperties_28: boolean;
                    export { additionalProperties_28 as additionalProperties };
                    const type_112: string;
                    export { type_112 as type };
                    const description_25: string;
                    export { description_25 as description };
                    const examples_23: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                    export { examples_23 as examples };
                }
                export namespace BrowserWindow {
                    const type_113: string;
                    export { type_113 as type };
                    const additionalProperties_29: boolean;
                    export { additionalProperties_29 as additionalProperties };
                    const description_26: string;
                    export { description_26 as description };
                    const examples_24: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    export { examples_24 as examples };
                    export namespace properties_16 {
                        namespace width {
                            const type_114: string;
                            export { type_114 as type };
                            const description_27: string;
                            export { description_27 as description };
                        }
                        namespace height {
                            const type_115: string;
                            export { type_115 as type };
                            const description_28: string;
                            export { description_28 as description };
                        }
                        namespace webPreferences {
                            const additionalProperties_30: boolean;
                            export { additionalProperties_30 as additionalProperties };
                            const type_116: string;
                            export { type_116 as type };
                            const description_29: string;
                            export { description_29 as description };
                            const examples_25: {
                                devTools: boolean;
                            }[];
                            export { examples_25 as examples };
                        }
                    }
                    export { properties_16 as properties };
                }
            }
            export { properties_15 as properties };
        }
        namespace linux {
            const additionalProperties_31: boolean;
            export { additionalProperties_31 as additionalProperties };
            const type_117: string;
            export { type_117 as type };
            export namespace properties_17 { }
            export { properties_17 as properties };
        }
        namespace windows {
            const additionalProperties_32: boolean;
            export { additionalProperties_32 as additionalProperties };
            const type_118: string;
            export { type_118 as type };
            export namespace properties_18 {
                export namespace language {
                    const type_119: string;
                    export { type_119 as type };
                    const description_30: string;
                    export { description_30 as description };
                }
                export namespace experimentalNuGetDependency {
                    const type_120: string;
                    export { type_120 as type };
                }
                export namespace useWinUI3 {
                    const type_121: string;
                    export { type_121 as type };
                }
                export namespace nuGetTestVersion {
                    const type_122: string;
                    export { type_122 as type };
                }
                export namespace reactNativeEngine {
                    const type_123: string;
                    export { type_123 as type };
                }
                export namespace nuGetTestFeed {
                    const type_124: string;
                    export { type_124 as type };
                }
                export namespace overwrite {
                    const type_125: string;
                    export { type_125 as type };
                    const description_31: string;
                    export { description_31 as description };
                }
                export namespace release {
                    const type_126: string;
                    export { type_126 as type };
                    const description_32: string;
                    export { description_32 as description };
                }
                export namespace root {
                    const type_127: string;
                    export { type_127 as type };
                    const description_33: string;
                    export { description_33 as description };
                }
                export namespace arch {
                    const type_128: string;
                    export { type_128 as type };
                    const description_34: string;
                    export { description_34 as description };
                    const examples_26: string[];
                    export { examples_26 as examples };
                }
                export namespace singleproc {
                    const type_129: string;
                    export { type_129 as type };
                    const description_35: string;
                    export { description_35 as description };
                }
                export namespace emulator {
                    const type_130: string;
                    export { type_130 as type };
                }
                export namespace device {
                    const type_131: string;
                    export { type_131 as type };
                }
                export namespace target {
                    const type_132: string;
                    export { type_132 as type };
                }
                export namespace remoteDebugging {
                    const type_133: string;
                    export { type_133 as type };
                }
                export namespace logging {
                    const type_134: string;
                    export { type_134 as type };
                    const description_36: string;
                    export { description_36 as description };
                }
                export namespace packager {
                    const type_135: string;
                    export { type_135 as type };
                }
                export namespace bundle {
                    const type_136: string;
                    export { type_136 as type };
                }
                export namespace launch {
                    const type_137: string;
                    export { type_137 as type };
                    const description_37: string;
                    export { description_37 as description };
                }
                export namespace autolink {
                    const type_138: string;
                    export { type_138 as type };
                    const description_38: string;
                    export { description_38 as description };
                }
                export namespace build {
                    const type_139: string;
                    export { type_139 as type };
                    const description_39: string;
                    export { description_39 as description };
                }
                export namespace deploy_1 {
                    const type_140: string;
                    export { type_140 as type };
                }
                export { deploy_1 as deploy };
                export namespace sln {
                    const type_141: string;
                    export { type_141 as type };
                    const description_40: string;
                    export { description_40 as description };
                }
                export namespace proj {
                    const type_142: string;
                    export { type_142 as type };
                    const description_41: string;
                    export { description_41 as description };
                }
                export namespace appPath {
                    const type_143: string;
                    export { type_143 as type };
                    const description_42: string;
                    export { description_42 as description };
                }
                export namespace msbuildprops {
                    const type_144: string;
                    export { type_144 as type };
                    const description_43: string;
                    export { description_43 as description };
                }
                export namespace buildLogDirectory {
                    const type_145: string;
                    export { type_145 as type };
                    const description_44: string;
                    export { description_44 as description };
                }
                export namespace info {
                    const type_146: string;
                    export { type_146 as type };
                    const description_45: string;
                    export { description_45 as description };
                }
                export namespace directDebugging {
                    const type_147: string;
                    export { type_147 as type };
                }
                export namespace telemetry {
                    const type_148: string;
                    export { type_148 as type };
                    const description_46: string;
                    export { description_46 as description };
                }
                export namespace devPort {
                    const type_149: string;
                    export { type_149 as type };
                }
                export namespace additionalMetroOptions {
                    const type_150: string;
                    export { type_150 as type };
                }
                export namespace packageExtension {
                    const type_151: string;
                    export { type_151 as type };
                }
            }
            export { properties_18 as properties };
        }
        namespace xbox {
            const additionalProperties_33: boolean;
            export { additionalProperties_33 as additionalProperties };
            const type_152: string;
            export { type_152 as type };
            export namespace properties_19 { }
            export { properties_19 as properties };
        }
    }
}
export namespace schemaRoot {
    const $id_1: string;
    export { $id_1 as $id };
    const type_153: string;
    export { type_153 as type };
    const additionalProperties_34: boolean;
    export { additionalProperties_34 as additionalProperties };
    export namespace properties_20 {
        export namespace engines {
            const additionalProperties_35: boolean;
            export { additionalProperties_35 as additionalProperties };
            const type_154: string;
            export { type_154 as type };
            const description_47: string;
            export { description_47 as description };
            const examples_27: ({
                '@rnv/engine-rn': string;
                '@rnv/engine-rn-tvos': string;
                '@rnv/engine-rn-web': string;
                '@rnv/engine-rn-next': string;
                '@rnv/engine-lightning': string;
                '@rnv/engine-rn-electron': string;
                '@rnv/engine-rn-windows': string;
                'custom-engine'?: undefined;
            } | {
                '@rnv/engine-rn': string;
                'custom-engine': {
                    version: string;
                };
                '@rnv/engine-rn-tvos'?: undefined;
                '@rnv/engine-rn-web'?: undefined;
                '@rnv/engine-rn-next'?: undefined;
                '@rnv/engine-lightning'?: undefined;
                '@rnv/engine-rn-electron'?: undefined;
                '@rnv/engine-rn-windows'?: undefined;
            })[];
            export { examples_27 as examples };
        }
        export namespace platforms {
            const additionalProperties_36: boolean;
            export { additionalProperties_36 as additionalProperties };
            const type_155: string;
            export { type_155 as type };
            export namespace properties_21 {
                export namespace ios_1 { }
                export { ios_1 as ios };
                export namespace android_1 { }
                export { android_1 as android };
                export namespace web_1 { }
                export { web_1 as web };
                export namespace webtv_1 { }
                export { webtv_1 as webtv };
                export namespace chromecast_1 { }
                export { chromecast_1 as chromecast };
                export namespace tvos_1 { }
                export { tvos_1 as tvos };
                export namespace androidtv { }
                export namespace firetv { }
                export namespace webos_1 { }
                export { webos_1 as webos };
                export namespace macos_1 { }
                export { macos_1 as macos };
                export namespace linux_1 { }
                export { linux_1 as linux };
                export namespace tizen_1 { }
                export { tizen_1 as tizen };
                export namespace windows_1 { }
                export { windows_1 as windows };
                export namespace xbox_1 { }
                export { xbox_1 as xbox };
                export namespace firefoxtv { }
                export namespace firefoxos { }
                export namespace tizenmobile { }
                export namespace tizenwatch { }
                export namespace androidwear { }
                export namespace kaios { }
            }
            export { properties_21 as properties };
        }
        export namespace sdks {
            const additionalProperties_37: boolean;
            export { additionalProperties_37 as additionalProperties };
            const type_156: string;
            export { type_156 as type };
            const description_48: string;
            export { description_48 as description };
            const examples_28: {
                ANDROID_SDK: string;
                ANDROID_NDK: string;
                TIZEN_SDK: string;
                WEBOS_SDK: string;
                KAIOS_SDK: string;
            }[];
            export { examples_28 as examples };
        }
        export namespace env {
            const additionalProperties_38: boolean;
            export { additionalProperties_38 as additionalProperties };
            const type_157: string;
            export { type_157 as type };
        }
        export namespace defaultTargets {
            const additionalProperties_39: boolean;
            export { additionalProperties_39 as additionalProperties };
            const type_158: string;
            export { type_158 as type };
            const description_49: string;
            export { description_49 as description };
            const examples_29: {
                android: string;
                androidtv: string;
                androidwear: string;
                ios: string;
                tvos: string;
                tizen: string;
                tizenwatch: string;
                tizenmobile: string;
                webos: string;
            }[];
            export { examples_29 as examples };
        }
        export namespace plugins {
            const type_159: string;
            export { type_159 as type };
            const description_50: string;
            export { description_50 as description };
            const examples_30: {
                renative: string;
                react: string;
                'react-native-cached-image': string;
                'react-native-web-image-loader': string;
                'react-native-gesture-handler': {
                    version: string;
                };
            }[];
            export { examples_30 as examples };
            export namespace additionalProperties_40 {
                const type_160: string[];
                export { type_160 as type };
                const additionalProperties_41: boolean;
                export { additionalProperties_41 as additionalProperties };
                const properties_22: {
                    enabled: {
                        type: string;
                    };
                    props: {
                        additionalProperties: boolean;
                        type: string;
                    };
                    version: {
                        type: string;
                    };
                    source: {
                        type: string;
                        description: string;
                        examples: string[];
                    };
                    'no-npm': {
                        type: string;
                    };
                    skipMerge: {
                        type: string;
                        description: string;
                        examples: boolean[];
                    };
                    npm: {
                        additionalProperties: boolean;
                        type: string;
                    };
                    pluginDependencies: {
                        type: string[];
                    };
                    webpack: {
                        additionalProperties: boolean;
                        type: string;
                        description: string;
                    };
                    webpackConfig: {
                        additionalProperties: boolean;
                        type: string;
                        description: string;
                        properties: {
                            modulePaths: {
                                type: string[];
                                additionalProperties: boolean;
                            };
                            moduleAliases: {
                                type: string[];
                                additionalProperties: boolean;
                            };
                        };
                    };
                    android: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    androidtv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    firetv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    androidwear: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    ios: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            podName: {
                                type: string;
                            };
                            podNames: {
                                type: string;
                            };
                            isStatic: {
                                type: string;
                            };
                            staticPods: {
                                type: string;
                                description: string;
                                examples: string[][];
                            };
                            Podfile: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            xcodeproj: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            plist: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateApplicationMethods: {
                                type: string;
                                properties: {
                                    didFinishLaunchingWithOptions: {
                                        type: string;
                                    };
                                    open: {
                                        type: string;
                                    };
                                    supportedInterfaceOrientationsFor: {
                                        type: string;
                                    };
                                    didReceiveRemoteNotification: {
                                        type: string;
                                    };
                                    didFailToRegisterForRemoteNotificationsWithError: {
                                        type: string;
                                    };
                                    didReceive: {
                                        type: string;
                                    };
                                    didRegister: {
                                        type: string;
                                    };
                                    didRegisterForRemoteNotificationsWithDeviceToken: {
                                        type: string;
                                    };
                                    didConnectCarInterfaceController: {
                                        type: string;
                                    };
                                    didDisconnectCarInterfaceController: {
                                        type: string;
                                    };
                                };
                            };
                            appDelegateMethods: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateImports: {
                                type: string;
                            };
                            appDelegateExtensions: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    tvos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            podName: {
                                type: string;
                            };
                            podNames: {
                                type: string;
                            };
                            isStatic: {
                                type: string;
                            };
                            staticPods: {
                                type: string;
                                description: string;
                                examples: string[][];
                            };
                            Podfile: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            xcodeproj: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            plist: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateApplicationMethods: {
                                type: string;
                                properties: {
                                    didFinishLaunchingWithOptions: {
                                        type: string;
                                    };
                                    open: {
                                        type: string;
                                    };
                                    supportedInterfaceOrientationsFor: {
                                        type: string;
                                    };
                                    didReceiveRemoteNotification: {
                                        type: string;
                                    };
                                    didFailToRegisterForRemoteNotificationsWithError: {
                                        type: string;
                                    };
                                    didReceive: {
                                        type: string;
                                    };
                                    didRegister: {
                                        type: string;
                                    };
                                    didRegisterForRemoteNotificationsWithDeviceToken: {
                                        type: string;
                                    };
                                    didConnectCarInterfaceController: {
                                        type: string;
                                    };
                                    didDisconnectCarInterfaceController: {
                                        type: string;
                                    };
                                };
                            };
                            appDelegateMethods: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateImports: {
                                type: string;
                            };
                            appDelegateExtensions: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    tizen: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    webos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    web: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    webtv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    chromecast: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    firefox: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    macos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    linux: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    windows: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    xbox: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                };
                export { properties_22 as properties };
            }
            export { additionalProperties_40 as additionalProperties };
        }
        export namespace projectName {
            const type_161: string;
            export { type_161 as type };
            const description_51: string;
            export { description_51 as description };
            const examples_31: string[];
            export { examples_31 as examples };
        }
        export namespace extend_1 {
            const type_162: string;
            export { type_162 as type };
        }
        export { extend_1 as extend };
        export namespace extendsTemplate {
            const type_163: string;
            export { type_163 as type };
            const description_52: string;
            export { description_52 as description };
            const examples_32: string[];
            export { examples_32 as examples };
        }
        export namespace projectTemplates {
            const additionalProperties_42: boolean;
            export { additionalProperties_42 as additionalProperties };
            const type_164: string;
            export { type_164 as type };
            const description_53: string;
            export { description_53 as description };
            const examples_33: {
                'my-custom-template': {};
            }[];
            export { examples_33 as examples };
        }
        export namespace permissions_1 {
            const additionalProperties_43: boolean;
            export { additionalProperties_43 as additionalProperties };
            const type_165: string;
            export { type_165 as type };
            const description_54: string;
            export { description_54 as description };
            const examples_34: {
                ios: {};
                android: {};
            }[];
            export { examples_34 as examples };
            export namespace properties_23 {
                export namespace android_2 {
                    const additionalProperties_44: boolean;
                    export { additionalProperties_44 as additionalProperties };
                    const type_166: string;
                    export { type_166 as type };
                    const description_55: string;
                    export { description_55 as description };
                    const examples_35: {
                        INTERNET: {
                            key: string;
                            security: string;
                        };
                        SYSTEM_ALERT_WINDOW: {
                            key: string;
                            security: string;
                        };
                    }[];
                    export { examples_35 as examples };
                }
                export { android_2 as android };
                export namespace ios_2 {
                    const additionalProperties_45: boolean;
                    export { additionalProperties_45 as additionalProperties };
                    const type_167: string;
                    export { type_167 as type };
                    const description_56: string;
                    export { description_56 as description };
                    const examples_36: {
                        NSAppleMusicUsageDescription: {
                            desc: string;
                        };
                        NSBluetoothPeripheralUsageDescription: {
                            desc: string;
                        };
                        NSCalendarsUsageDescription: {
                            desc: string;
                        };
                        NSCameraUsageDescription: {
                            desc: string;
                        };
                        NSLocationWhenInUseUsageDescription: {
                            desc: string;
                        };
                        NSMicrophoneUsageDescription: {
                            desc: string;
                        };
                        NSMotionUsageDescription: {
                            desc: string;
                        };
                        NSPhotoLibraryAddUsageDescription: {
                            desc: string;
                        };
                        NSPhotoLibraryUsageDescription: {
                            desc: string;
                        };
                        NSSpeechRecognitionUsageDescription: {
                            desc: string;
                        };
                        NSContactsUsageDescription: {
                            desc: string;
                        };
                        NSFaceIDUsageDescription: {
                            desc: string;
                        };
                        NSLocationAlwaysUsageDescription: {
                            desc: string;
                        };
                        NSBluetoothAlwaysUsageDescription: {
                            desc: string;
                        };
                        NSLocationAlwaysAndWhenInUseUsageDescription: {
                            desc: string;
                        };
                    }[];
                    export { examples_36 as examples };
                }
                export { ios_2 as ios };
            }
            export { properties_23 as properties };
        }
        export { permissions_1 as permissions };
        export namespace workspaceID {
            const type_168: string;
            export { type_168 as type };
            const description_57: string;
            export { description_57 as description };
            const examples_37: string[];
            export { examples_37 as examples };
        }
        export namespace version {
            const type_169: string;
            export { type_169 as type };
            const description_58: string;
            export { description_58 as description };
            const examples_38: string[];
            export { examples_38 as examples };
        }
        export namespace versionCode {
            const description_59: string;
            export { description_59 as description };
            const type_170: string;
            export { type_170 as type };
            const examples_39: string[];
            export { examples_39 as examples };
        }
        export namespace versionFormat {
            const description_60: string;
            export { description_60 as description };
            const examples_40: string[];
            export { examples_40 as examples };
            const type_171: string;
            export { type_171 as type };
        }
        export namespace versionCodeFormat {
            const description_61: string;
            export { description_61 as description };
            const examples_41: string[];
            export { examples_41 as examples };
            const type_172: string;
            export { type_172 as type };
        }
        export namespace description_62 {
            const type_173: string;
            export { type_173 as type };
        }
        export { description_62 as description };
        export namespace id_1 {
            const type_174: string;
            export { type_174 as type };
            const description_63: string;
            export { description_63 as description };
            const examples_42: string[];
            export { examples_42 as examples };
        }
        export { id_1 as id };
        export namespace isMonorepo {
            const type_175: string;
            export { type_175 as type };
        }
        export namespace monoRepo {
            const type_176: string;
            export { type_176 as type };
            const description_64: string;
            export { description_64 as description };
            export const example: string;
        }
        export namespace enableAnalytics {
            const type_177: string;
            export { type_177 as type };
            const description_65: string;
            export { description_65 as description };
            const examples_43: boolean[];
            export { examples_43 as examples };
        }
        export namespace paths {
            const additionalProperties_46: boolean;
            export { additionalProperties_46 as additionalProperties };
            const type_178: string;
            export { type_178 as type };
            const description_66: string;
            export { description_66 as description };
            export namespace properties_24 {
                namespace appConfigsDir {
                    const type_179: string;
                    export { type_179 as type };
                    const description_67: string;
                    export { description_67 as description };
                    const examples_44: string[];
                    export { examples_44 as examples };
                }
                namespace platformAssetsDir {
                    const type_180: string;
                    export { type_180 as type };
                    const description_68: string;
                    export { description_68 as description };
                    const examples_45: string[];
                    export { examples_45 as examples };
                }
                namespace platformBuildsDir {
                    const type_181: string;
                    export { type_181 as type };
                    const description_69: string;
                    export { description_69 as description };
                    const examples_46: string[];
                    export { examples_46 as examples };
                }
                namespace pluginTemplates {
                    const additionalProperties_47: boolean;
                    export { additionalProperties_47 as additionalProperties };
                    const type_182: string;
                    export { type_182 as type };
                    const description_70: string;
                    export { description_70 as description };
                    const examples_47: {
                        myCustomScope: {
                            npm: string;
                            path: string;
                        };
                    }[];
                    export { examples_47 as examples };
                }
            }
            export { properties_24 as properties };
        }
        export namespace tasks {
            const additionalProperties_48: boolean;
            export { additionalProperties_48 as additionalProperties };
            const type_183: string;
            export { type_183 as type };
            const description_71: string;
            export { description_71 as description };
            const examples_48: {
                install: {
                    script: string;
                };
            }[];
            export { examples_48 as examples };
        }
        export namespace pipes {
            const type_184: string;
            export { type_184 as type };
            const description_72: string;
            export { description_72 as description };
            const examples_49: string[][];
            export { examples_49 as examples };
        }
        export namespace defaults {
            const additionalProperties_49: boolean;
            export { additionalProperties_49 as additionalProperties };
            const type_185: string;
            export { type_185 as type };
            const description_73: string;
            export { description_73 as description };
            export namespace properties_25 {
                namespace supportedPlatforms {
                    const type_186: string;
                    export { type_186 as type };
                    const description_74: string;
                    export { description_74 as description };
                    const examples_50: string[][];
                    export { examples_50 as examples };
                }
                namespace schemes {
                    const type_187: string;
                    export { type_187 as type };
                    const description_75: string;
                    export { description_75 as description };
                    const examples_51: {
                        ios: string;
                        android: string;
                    }[];
                    export { examples_51 as examples };
                }
                namespace targets {
                    const type_188: string;
                    export { type_188 as type };
                    const description_76: string;
                    export { description_76 as description };
                    const examples_52: {
                        ios: string;
                        tvos: string;
                    }[];
                    export { examples_52 as examples };
                }
                namespace ports {
                    const type_189: string;
                    export { type_189 as type };
                    const description_77: string;
                    export { description_77 as description };
                    const examples_53: {
                        ios: number;
                        android: number;
                        androidtv: number;
                        tvos: number;
                        macos: number;
                        web: number;
                        tizen: number;
                        webos: number;
                        androidwear: number;
                        tizenwatch: number;
                        tizenmobile: number;
                        windows: number;
                        kaios: number;
                        firefoxos: number;
                        firefoxtv: number;
                        webtv: number;
                        linux: number;
                    }[];
                    export { examples_53 as examples };
                }
            }
            export { properties_25 as properties };
        }
        export namespace pluginTemplates_1 {
            const additionalProperties_50: boolean;
            export { additionalProperties_50 as additionalProperties };
            const type_190: string;
            export { type_190 as type };
        }
        export { pluginTemplates_1 as pluginTemplates };
        export namespace templates {
            const additionalProperties_51: boolean;
            export { additionalProperties_51 as additionalProperties };
            const type_191: string;
            export { type_191 as type };
            const description_78: string;
            export { description_78 as description };
            const examples_54: {
                '@rnv/template-starter': {
                    version: string;
                };
            }[];
            export { examples_54 as examples };
        }
        export namespace currentTemplate {
            const type_192: string;
            export { type_192 as type };
            const description_79: string;
            export { description_79 as description };
            const examples_55: string[];
            export { examples_55 as examples };
        }
        export namespace crypto {
            const additionalProperties_52: boolean;
            export { additionalProperties_52 as additionalProperties };
            const type_193: string;
            export { type_193 as type };
            const description_80: string;
            export { description_80 as description };
            export namespace properties_26 {
                namespace encrypt {
                    const additionalProperties_53: boolean;
                    export { additionalProperties_53 as additionalProperties };
                    const type_194: string;
                    export { type_194 as type };
                    export namespace properties_27 {
                        namespace dest {
                            const type_195: string;
                            export { type_195 as type };
                            const description_81: string;
                            export { description_81 as description };
                            const examples_56: string[];
                            export { examples_56 as examples };
                        }
                    }
                    export { properties_27 as properties };
                }
                namespace decrypt {
                    const additionalProperties_54: boolean;
                    export { additionalProperties_54 as additionalProperties };
                    const type_196: string;
                    export { type_196 as type };
                    export namespace properties_28 {
                        namespace source {
                            const type_197: string;
                            export { type_197 as type };
                            const description_82: string;
                            export { description_82 as description };
                            const examples_57: string[];
                            export { examples_57 as examples };
                        }
                    }
                    export { properties_28 as properties };
                }
                namespace optional {
                    const type_198: string;
                    export { type_198 as type };
                }
            }
            export { properties_26 as properties };
        }
        export namespace integrations {
            const additionalProperties_55: boolean;
            export { additionalProperties_55 as additionalProperties };
            const type_199: string;
            export { type_199 as type };
        }
        export namespace publish {
            const additionalProperties_56: boolean;
            export { additionalProperties_56 as additionalProperties };
            const type_200: string;
            export { type_200 as type };
        }
        export namespace _private {
            const additionalProperties_57: boolean;
            export { additionalProperties_57 as additionalProperties };
            const type_201: string;
            export { type_201 as type };
            const description_83: string;
            export { description_83 as description };
            const examples_58: {
                myPrivateKy: string;
            }[];
            export { examples_58 as examples };
        }
        export { _private as private };
        export namespace hidden {
            const type_202: string;
            export { type_202 as type };
            const description_84: string;
            export { description_84 as description };
            const examples_59: boolean[];
            export { examples_59 as examples };
        }
        export namespace templateConfig {
            const description_85: string;
            export { description_85 as description };
            const type_203: string;
            export { type_203 as type };
            const additionalProperties_58: boolean;
            export { additionalProperties_58 as additionalProperties };
            export namespace properties_29 {
                namespace includedPaths {
                    const type_204: string;
                    export { type_204 as type };
                    export namespace items_7 {
                        const type_205: string;
                        export { type_205 as type };
                    }
                    export { items_7 as items };
                    const description_86: string;
                    export { description_86 as description };
                    const examples_60: string[][];
                    export { examples_60 as examples };
                }
                namespace bootstrapQuestions {
                    const type_206: string;
                    export { type_206 as type };
                    export namespace items_8 {
                        const type_207: string;
                        export { type_207 as type };
                    }
                    export { items_8 as items };
                    const description_87: string;
                    export { description_87 as description };
                    const examples_61: {
                        title: string;
                        type: string;
                        configProp: {
                            key: string;
                            file: string;
                        };
                        options: {
                            title: string;
                            value: {
                                id: string;
                            };
                        }[];
                    }[][];
                    export { examples_61 as examples };
                }
            }
            export { properties_29 as properties };
        }
        export namespace enableHookRebuild {
            const type_208: string;
            export { type_208 as type };
            const description_88: string;
            export { description_88 as description };
            const examples_62: boolean[];
            export { examples_62 as examples };
        }
        export namespace common {
            const additionalProperties_59: boolean;
            export { additionalProperties_59 as additionalProperties };
            const type_209: string;
            export { type_209 as type };
            const description_89: string;
            export { description_89 as description };
            const examples_63: {
                author: {
                    name: string;
                    email: string;
                    url: string;
                };
                license: string;
                includedPlugins: string[];
                includedFonts: string[];
                backgroundColor: string;
                runtime: {
                    welcomeMessage: string;
                };
            }[];
            export { examples_63 as examples };
            export namespace properties_30 { }
            export { properties_30 as properties };
        }
    }
    export { properties_20 as properties };
}
export const SCHEMAS_RENATIVE_JSON: ({
    $id: string;
    definitions: {
        android: {
            additionalProperties: boolean;
            type: string;
            properties: {
                enableAndroidX: {
                    type: string;
                    default: boolean;
                    examples: boolean[];
                };
                enableHermes: {
                    type: string;
                    default: boolean;
                    description: string;
                    examples: boolean[];
                };
                reactNativeEngine: {
                    type: string;
                    default: string;
                    description: string;
                    examples: boolean[];
                };
                signingConfig: {
                    type: string;
                    default: string;
                    description: string;
                    examples: string[];
                };
                minSdkVersion: {
                    type: string;
                    default: number;
                    examples: number[];
                };
                multipleAPKs: {
                    type: string;
                    default: boolean;
                    description: string;
                    examples: boolean[];
                };
                aab: {
                    type: string;
                    description: string;
                    default: boolean;
                    examples: boolean[];
                };
                extraGradleParams: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                minifyEnabled: {
                    type: string;
                    description: string;
                    default: boolean;
                    examples: boolean[];
                };
                targetSdkVersion: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                compileSdkVersion: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                gradleBuildToolsVersion: {
                    type: string;
                    description: string;
                    default: string;
                    examples: string[];
                };
                gradleWrapperVersion: {
                    type: string;
                    description: string;
                    default: string;
                    examples: string[];
                };
                mainActivity: {
                    type: string;
                    description: string;
                    default: string;
                    properties: {
                        onCreate: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                    };
                };
                storeFile: {
                    type: string;
                };
                storePassword: {
                    type: string;
                    description: string;
                };
                keyAlias: {
                    type: string;
                };
                keyPassword: {
                    type: string;
                    description: string;
                };
                excludedFeatures: {
                    type: string;
                };
                includedFeatures: {
                    type: string;
                };
                'gradle.properties': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        'gradle.properties': {
                            'android.debug.obsoleteApi': boolean;
                            'debug.keystore': string;
                            'org.gradle.daemon': boolean;
                            'org.gradle.parallel': boolean;
                            'org.gradle.configureondemand': boolean;
                        };
                    }[];
                };
                'build.gradle': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        allprojects: {
                            repositories: {
                                'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                            };
                        };
                    }[];
                };
                'app/build.gradle': {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        apply: string[];
                    }[];
                };
                AndroidManifest: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: ({
                        children: {
                            tag: string;
                            'android:name': string;
                            children: {
                                tag: string;
                                'android:name': string;
                            }[];
                        }[];
                    } | {
                        children: {
                            tag: string;
                            'android:name': string;
                            'android:allowBackup': boolean;
                            'android:largeHeap': boolean;
                            'android:usesCleartextTraffic': boolean;
                            'tools:targetApi': number;
                        }[];
                    })[];
                };
                applyPlugin: {
                    type: string;
                };
                BuildGradle: {
                    type: string;
                    description: string;
                    properties: {
                        allprojects: {
                            type: string;
                            properties: {
                                repositories: {
                                    type: string;
                                    description: string;
                                    additionalProperties: boolean;
                                    examples: {
                                        "flatDir { dirs 'libs'}": boolean;
                                    }[];
                                };
                            };
                        };
                    };
                };
                implementation: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        ios: {
            additionalProperties: boolean;
            type: string;
            properties: {
                deploymentTarget: {
                    type: string;
                };
                teamID: {
                    type: string;
                };
                excludedArchs: {
                    type: string;
                    description: string;
                    default: null;
                    examples: string[][];
                };
                teamIdentifier: {
                    type: string;
                };
                scheme: {
                    type: string;
                };
                appleId: {
                    type: string;
                };
                orientationSupport: {
                    type: string;
                    properties: {
                        phone: {
                            type: string;
                        };
                        tab: {
                            type: string;
                        };
                    };
                    examples: {
                        phone: string[];
                        tab: string[];
                    }[];
                };
                provisioningStyle: {
                    type: string;
                };
                codeSignIdentity: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                commandLineArguments: {
                    type: string;
                    description: string;
                    examples: string[][];
                };
                provisionProfileSpecifier: {
                    type: string;
                };
                provisioningProfiles: {
                    additionalProperties: boolean;
                    type: string;
                };
                systemCapabilities: {
                    additionalProperties: boolean;
                    type: string;
                    examples: {
                        'com.apple.SafariKeychain': boolean;
                        'com.apple.Wallet': boolean;
                        'com.apple.HealthKit': boolean;
                        'com.apple.ApplicationGroups.iOS': boolean;
                        'com.apple.iCloud': boolean;
                        'com.apple.DataProtection': boolean;
                        'com.apple.HomeKit': boolean;
                        'com.apple.ClassKit': boolean;
                        'com.apple.VPNLite': boolean;
                        'com.apple.AutoFillCredentialProvider': boolean;
                        'com.apple.AccessWiFi': boolean;
                        'com.apple.InAppPurchase': boolean;
                        'com.apple.HotspotConfiguration': boolean;
                        'com.apple.Multipath': boolean;
                        'com.apple.GameCenter.iOS': boolean;
                        'com.apple.BackgroundModes': boolean;
                        'com.apple.InterAppAudio': boolean;
                        'com.apple.WAC': boolean;
                        'com.apple.Push': boolean;
                        'com.apple.NearFieldCommunicationTagReading': boolean;
                        'com.apple.ApplePay': boolean;
                        'com.apple.Keychain': boolean;
                        'com.apple.Maps.iOS': boolean;
                        'com.apple.Siri': boolean;
                        'com.apple.NetworkExtensions.iOS': boolean;
                    }[];
                };
                entitlements: {
                    additionalProperties: boolean;
                    type: string;
                };
                runScheme: {
                    type: string;
                };
                sdk: {
                    type: string;
                };
                testFlightId: {
                    type: string;
                };
                firebaseId: {
                    type: string;
                };
                exportOptions: {
                    type: string;
                    additionalProperties: boolean;
                    properties: {
                        method: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        uploadBitcode: {
                            type: string;
                        };
                        compileBitcode: {
                            type: string;
                        };
                        uploadSymbols: {
                            type: string;
                        };
                        signingStyle: {
                            type: string;
                        };
                        signingCertificate: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                    };
                };
                Podfile: {
                    additionalProperties: boolean;
                    type: string;
                };
                xcodeproj: {
                    additionalProperties: boolean;
                    type: string;
                };
                plist: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateApplicationMethods: {
                    type: string;
                    properties: {
                        didFinishLaunchingWithOptions: {
                            type: string;
                        };
                        open: {
                            type: string;
                        };
                        supportedInterfaceOrientationsFor: {
                            type: string;
                        };
                        didReceiveRemoteNotification: {
                            type: string;
                        };
                        didFailToRegisterForRemoteNotificationsWithError: {
                            type: string;
                        };
                        didReceive: {
                            type: string;
                        };
                        didRegister: {
                            type: string;
                        };
                        didRegisterForRemoteNotificationsWithDeviceToken: {
                            type: string;
                        };
                        didConnectCarInterfaceController: {
                            type: string;
                        };
                        didDisconnectCarInterfaceController: {
                            type: string;
                        };
                    };
                };
                appDelegateMethods: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateImports: {
                    type: string;
                };
                appDelegateExtensions: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        tvos: {
            additionalProperties: boolean;
            type: string;
            properties: {
                deploymentTarget: {
                    type: string;
                };
                teamID: {
                    type: string;
                };
                excludedArchs: {
                    type: string;
                    description: string;
                    default: null;
                    examples: string[][];
                };
                teamIdentifier: {
                    type: string;
                };
                scheme: {
                    type: string;
                };
                appleId: {
                    type: string;
                };
                orientationSupport: {
                    type: string;
                    properties: {
                        phone: {
                            type: string;
                        };
                        tab: {
                            type: string;
                        };
                    };
                    examples: {
                        phone: string[];
                        tab: string[];
                    }[];
                };
                provisioningStyle: {
                    type: string;
                };
                codeSignIdentity: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                commandLineArguments: {
                    type: string;
                    description: string;
                    examples: string[][];
                };
                provisionProfileSpecifier: {
                    type: string;
                };
                provisioningProfiles: {
                    additionalProperties: boolean;
                    type: string;
                };
                systemCapabilities: {
                    additionalProperties: boolean;
                    type: string;
                    examples: {
                        'com.apple.SafariKeychain': boolean;
                        'com.apple.Wallet': boolean;
                        'com.apple.HealthKit': boolean;
                        'com.apple.ApplicationGroups.iOS': boolean;
                        'com.apple.iCloud': boolean;
                        'com.apple.DataProtection': boolean;
                        'com.apple.HomeKit': boolean;
                        'com.apple.ClassKit': boolean;
                        'com.apple.VPNLite': boolean;
                        'com.apple.AutoFillCredentialProvider': boolean;
                        'com.apple.AccessWiFi': boolean;
                        'com.apple.InAppPurchase': boolean;
                        'com.apple.HotspotConfiguration': boolean;
                        'com.apple.Multipath': boolean;
                        'com.apple.GameCenter.iOS': boolean;
                        'com.apple.BackgroundModes': boolean;
                        'com.apple.InterAppAudio': boolean;
                        'com.apple.WAC': boolean;
                        'com.apple.Push': boolean;
                        'com.apple.NearFieldCommunicationTagReading': boolean;
                        'com.apple.ApplePay': boolean;
                        'com.apple.Keychain': boolean;
                        'com.apple.Maps.iOS': boolean;
                        'com.apple.Siri': boolean;
                        'com.apple.NetworkExtensions.iOS': boolean;
                    }[];
                };
                entitlements: {
                    additionalProperties: boolean;
                    type: string;
                };
                runScheme: {
                    type: string;
                };
                sdk: {
                    type: string;
                };
                testFlightId: {
                    type: string;
                };
                firebaseId: {
                    type: string;
                };
                exportOptions: {
                    type: string;
                    additionalProperties: boolean;
                    properties: {
                        method: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        uploadBitcode: {
                            type: string;
                        };
                        compileBitcode: {
                            type: string;
                        };
                        uploadSymbols: {
                            type: string;
                        };
                        signingStyle: {
                            type: string;
                        };
                        signingCertificate: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                    };
                };
                Podfile: {
                    additionalProperties: boolean;
                    type: string;
                };
                xcodeproj: {
                    additionalProperties: boolean;
                    type: string;
                };
                plist: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateApplicationMethods: {
                    type: string;
                    properties: {
                        didFinishLaunchingWithOptions: {
                            type: string;
                        };
                        open: {
                            type: string;
                        };
                        supportedInterfaceOrientationsFor: {
                            type: string;
                        };
                        didReceiveRemoteNotification: {
                            type: string;
                        };
                        didFailToRegisterForRemoteNotificationsWithError: {
                            type: string;
                        };
                        didReceive: {
                            type: string;
                        };
                        didRegister: {
                            type: string;
                        };
                        didRegisterForRemoteNotificationsWithDeviceToken: {
                            type: string;
                        };
                        didConnectCarInterfaceController: {
                            type: string;
                        };
                        didDisconnectCarInterfaceController: {
                            type: string;
                        };
                    };
                };
                appDelegateMethods: {
                    additionalProperties: boolean;
                    type: string;
                };
                appDelegateImports: {
                    type: string;
                };
                appDelegateExtensions: {
                    type: string;
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        tizen: {
            additionalProperties: boolean;
            type: string;
            properties: {
                package: {
                    type: string;
                };
                certificateProfile: {
                    type: string;
                };
                appName: {
                    type: string;
                };
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        webos: {
            additionalProperties: boolean;
            type: string;
            properties: {
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        web: {
            additionalProperties: boolean;
            type: string;
            properties: {
                pagesDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                outputDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                exportDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                environment: {
                    type: string;
                };
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        webtv: {
            additionalProperties: boolean;
            type: string;
            properties: {
                pagesDir: {
                    type: string;
                };
                environment: {
                    type: string;
                };
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        chromecast: {
            additionalProperties: boolean;
            type: string;
            properties: {
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        firefox: {
            additionalProperties: boolean;
            type: string;
            properties: {
                devServerHost: {
                    type: string;
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        macos: {
            additionalProperties: boolean;
            type: string;
            properties: {
                appleId: {
                    type: string;
                };
                environment: {
                    type: string;
                };
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        linux: {
            additionalProperties: boolean;
            type: string;
            properties: {
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                deploy: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        type: {
                            type: string;
                        };
                    };
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        windows: {
            additionalProperties: boolean;
            type: string;
            properties: {
                language: {
                    type: string;
                    description: string;
                };
                experimentalNuGetDependency: {
                    type: string;
                };
                useWinUI3: {
                    type: string;
                };
                nuGetTestVersion: {
                    type: string;
                };
                reactNativeEngine: {
                    type: string;
                };
                nuGetTestFeed: {
                    type: string;
                };
                overwrite: {
                    type: string;
                    description: string;
                };
                release: {
                    type: string;
                    description: string;
                };
                root: {
                    type: string;
                    description: string;
                };
                arch: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                singleproc: {
                    type: string;
                    description: string;
                };
                emulator: {
                    type: string;
                };
                device: {
                    type: string;
                };
                target: {
                    type: string;
                };
                remoteDebugging: {
                    type: string;
                };
                logging: {
                    type: string;
                    description: string;
                };
                packager: {
                    type: string;
                };
                bundle: {
                    type: string;
                };
                launch: {
                    type: string;
                    description: string;
                };
                autolink: {
                    type: string;
                    description: string;
                };
                build: {
                    type: string;
                    description: string;
                };
                deploy: {
                    type: string;
                };
                sln: {
                    type: string;
                    description: string;
                };
                proj: {
                    type: string;
                    description: string;
                };
                appPath: {
                    type: string;
                    description: string;
                };
                msbuildprops: {
                    type: string;
                    description: string;
                };
                buildLogDirectory: {
                    type: string;
                    description: string;
                };
                info: {
                    type: string;
                    description: string;
                };
                directDebugging: {
                    type: string;
                };
                telemetry: {
                    type: string;
                    description: string;
                };
                devPort: {
                    type: string;
                };
                additionalMetroOptions: {
                    type: string;
                };
                packageExtension: {
                    type: string;
                };
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
        xbox: {
            additionalProperties: boolean;
            type: string;
            properties: {
                language: {
                    type: string;
                    description: string;
                };
                experimentalNuGetDependency: {
                    type: string;
                };
                useWinUI3: {
                    type: string;
                };
                nuGetTestVersion: {
                    type: string;
                };
                reactNativeEngine: {
                    type: string;
                };
                nuGetTestFeed: {
                    type: string;
                };
                overwrite: {
                    type: string;
                    description: string;
                };
                release: {
                    type: string;
                    description: string;
                };
                root: {
                    type: string;
                    description: string;
                };
                arch: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                singleproc: {
                    type: string;
                    description: string;
                };
                emulator: {
                    type: string;
                };
                device: {
                    type: string;
                };
                target: {
                    type: string;
                };
                remoteDebugging: {
                    type: string;
                };
                logging: {
                    type: string;
                    description: string;
                };
                packager: {
                    type: string;
                };
                bundle: {
                    type: string;
                };
                launch: {
                    type: string;
                    description: string;
                };
                autolink: {
                    type: string;
                    description: string;
                };
                build: {
                    type: string;
                    description: string;
                };
                deploy: {
                    type: string;
                };
                sln: {
                    type: string;
                    description: string;
                };
                proj: {
                    type: string;
                    description: string;
                };
                appPath: {
                    type: string;
                    description: string;
                };
                msbuildprops: {
                    type: string;
                    description: string;
                };
                buildLogDirectory: {
                    type: string;
                    description: string;
                };
                info: {
                    type: string;
                    description: string;
                };
                directDebugging: {
                    type: string;
                };
                telemetry: {
                    type: string;
                    description: string;
                };
                devPort: {
                    type: string;
                };
                additionalMetroOptions: {
                    type: string;
                };
                packageExtension: {
                    type: string;
                };
                electronConfig: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        mac: {
                            target: string[];
                            hardenedRuntime: boolean;
                        };
                        dmg: {
                            sign: boolean;
                        };
                        mas: {
                            type: string;
                            hardenedRuntime: boolean;
                        };
                        mainInjection: string;
                    }[];
                };
                BrowserWindow: {
                    type: string;
                    additionalProperties: boolean;
                    description: string;
                    examples: {
                        width: number;
                        height: number;
                        webPreferences: {
                            devTools: boolean;
                        };
                    }[];
                    properties: {
                        width: {
                            type: string;
                            description: string;
                        };
                        height: {
                            type: string;
                            description: string;
                        };
                        webPreferences: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                devTools: boolean;
                            }[];
                        };
                    };
                };
                webpackConfig: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        publicUrl: {
                            type: string;
                        };
                        metaTags: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        customScripts: {
                            type: string;
                        };
                        extend: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                devtool: string;
                                module?: undefined;
                            } | {
                                module: {
                                    rules: {
                                        test: RegExp;
                                        use: string[];
                                        enforce: string;
                                    }[];
                                };
                                devtool?: undefined;
                            })[];
                        };
                    };
                };
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                engine: {
                    type: string;
                };
                entryFile: {
                    type: string;
                };
                bundleAssets: {
                    type: string;
                    description: string;
                };
                enableSourceMaps: {
                    type: string;
                    description: string;
                };
                bundleIsDev: {
                    type: string;
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
    };
} | {
    $id: string;
    type: string;
    additionalProperties: boolean;
    properties: {
        ext: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                myCustomRenativeProp: string;
            }[];
        };
        engines: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: ({
                '@rnv/engine-rn': string;
                '@rnv/engine-rn-tvos': string;
                '@rnv/engine-rn-web': string;
                '@rnv/engine-rn-next': string;
                '@rnv/engine-lightning': string;
                '@rnv/engine-rn-electron': string;
                '@rnv/engine-rn-windows': string;
                'custom-engine'?: undefined;
            } | {
                '@rnv/engine-rn': string;
                'custom-engine': {
                    version: string;
                };
                '@rnv/engine-rn-tvos'?: undefined;
                '@rnv/engine-rn-web'?: undefined;
                '@rnv/engine-rn-next'?: undefined;
                '@rnv/engine-lightning'?: undefined;
                '@rnv/engine-rn-electron'?: undefined;
                '@rnv/engine-rn-windows'?: undefined;
            })[];
        };
        platforms: {
            additionalProperties: boolean;
            type: string;
            properties: {
                ios: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        deploymentTarget: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        excludedArchs: {
                            type: string;
                            description: string;
                            default: null;
                            examples: string[][];
                        };
                        teamIdentifier: {
                            type: string;
                        };
                        scheme: {
                            type: string;
                        };
                        appleId: {
                            type: string;
                        };
                        orientationSupport: {
                            type: string;
                            properties: {
                                phone: {
                                    type: string;
                                };
                                tab: {
                                    type: string;
                                };
                            };
                            examples: {
                                phone: string[];
                                tab: string[];
                            }[];
                        };
                        provisioningStyle: {
                            type: string;
                        };
                        codeSignIdentity: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        commandLineArguments: {
                            type: string;
                            description: string;
                            examples: string[][];
                        };
                        provisionProfileSpecifier: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        systemCapabilities: {
                            additionalProperties: boolean;
                            type: string;
                            examples: {
                                'com.apple.SafariKeychain': boolean;
                                'com.apple.Wallet': boolean;
                                'com.apple.HealthKit': boolean;
                                'com.apple.ApplicationGroups.iOS': boolean;
                                'com.apple.iCloud': boolean;
                                'com.apple.DataProtection': boolean;
                                'com.apple.HomeKit': boolean;
                                'com.apple.ClassKit': boolean;
                                'com.apple.VPNLite': boolean;
                                'com.apple.AutoFillCredentialProvider': boolean;
                                'com.apple.AccessWiFi': boolean;
                                'com.apple.InAppPurchase': boolean;
                                'com.apple.HotspotConfiguration': boolean;
                                'com.apple.Multipath': boolean;
                                'com.apple.GameCenter.iOS': boolean;
                                'com.apple.BackgroundModes': boolean;
                                'com.apple.InterAppAudio': boolean;
                                'com.apple.WAC': boolean;
                                'com.apple.Push': boolean;
                                'com.apple.NearFieldCommunicationTagReading': boolean;
                                'com.apple.ApplePay': boolean;
                                'com.apple.Keychain': boolean;
                                'com.apple.Maps.iOS': boolean;
                                'com.apple.Siri': boolean;
                                'com.apple.NetworkExtensions.iOS': boolean;
                            }[];
                        };
                        entitlements: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        runScheme: {
                            type: string;
                        };
                        sdk: {
                            type: string;
                        };
                        testFlightId: {
                            type: string;
                        };
                        firebaseId: {
                            type: string;
                        };
                        exportOptions: {
                            type: string;
                            additionalProperties: boolean;
                            properties: {
                                method: {
                                    type: string;
                                };
                                teamID: {
                                    type: string;
                                };
                                uploadBitcode: {
                                    type: string;
                                };
                                compileBitcode: {
                                    type: string;
                                };
                                uploadSymbols: {
                                    type: string;
                                };
                                signingStyle: {
                                    type: string;
                                };
                                signingCertificate: {
                                    type: string;
                                };
                                provisioningProfiles: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                            };
                        };
                        Podfile: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        xcodeproj: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        plist: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateApplicationMethods: {
                            type: string;
                            properties: {
                                didFinishLaunchingWithOptions: {
                                    type: string;
                                };
                                open: {
                                    type: string;
                                };
                                supportedInterfaceOrientationsFor: {
                                    type: string;
                                };
                                didReceiveRemoteNotification: {
                                    type: string;
                                };
                                didFailToRegisterForRemoteNotificationsWithError: {
                                    type: string;
                                };
                                didReceive: {
                                    type: string;
                                };
                                didRegister: {
                                    type: string;
                                };
                                didRegisterForRemoteNotificationsWithDeviceToken: {
                                    type: string;
                                };
                                didConnectCarInterfaceController: {
                                    type: string;
                                };
                                didDisconnectCarInterfaceController: {
                                    type: string;
                                };
                            };
                        };
                        appDelegateMethods: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateImports: {
                            type: string;
                        };
                        appDelegateExtensions: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                android: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                web: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        pagesDir: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        outputDir: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        exportDir: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        environment: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                webtv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        pagesDir: {
                            type: string;
                        };
                        environment: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                chromecast: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tvos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        deploymentTarget: {
                            type: string;
                        };
                        teamID: {
                            type: string;
                        };
                        excludedArchs: {
                            type: string;
                            description: string;
                            default: null;
                            examples: string[][];
                        };
                        teamIdentifier: {
                            type: string;
                        };
                        scheme: {
                            type: string;
                        };
                        appleId: {
                            type: string;
                        };
                        orientationSupport: {
                            type: string;
                            properties: {
                                phone: {
                                    type: string;
                                };
                                tab: {
                                    type: string;
                                };
                            };
                            examples: {
                                phone: string[];
                                tab: string[];
                            }[];
                        };
                        provisioningStyle: {
                            type: string;
                        };
                        codeSignIdentity: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        commandLineArguments: {
                            type: string;
                            description: string;
                            examples: string[][];
                        };
                        provisionProfileSpecifier: {
                            type: string;
                        };
                        provisioningProfiles: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        systemCapabilities: {
                            additionalProperties: boolean;
                            type: string;
                            examples: {
                                'com.apple.SafariKeychain': boolean;
                                'com.apple.Wallet': boolean;
                                'com.apple.HealthKit': boolean;
                                'com.apple.ApplicationGroups.iOS': boolean;
                                'com.apple.iCloud': boolean;
                                'com.apple.DataProtection': boolean;
                                'com.apple.HomeKit': boolean;
                                'com.apple.ClassKit': boolean;
                                'com.apple.VPNLite': boolean;
                                'com.apple.AutoFillCredentialProvider': boolean;
                                'com.apple.AccessWiFi': boolean;
                                'com.apple.InAppPurchase': boolean;
                                'com.apple.HotspotConfiguration': boolean;
                                'com.apple.Multipath': boolean;
                                'com.apple.GameCenter.iOS': boolean;
                                'com.apple.BackgroundModes': boolean;
                                'com.apple.InterAppAudio': boolean;
                                'com.apple.WAC': boolean;
                                'com.apple.Push': boolean;
                                'com.apple.NearFieldCommunicationTagReading': boolean;
                                'com.apple.ApplePay': boolean;
                                'com.apple.Keychain': boolean;
                                'com.apple.Maps.iOS': boolean;
                                'com.apple.Siri': boolean;
                                'com.apple.NetworkExtensions.iOS': boolean;
                            }[];
                        };
                        entitlements: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        runScheme: {
                            type: string;
                        };
                        sdk: {
                            type: string;
                        };
                        testFlightId: {
                            type: string;
                        };
                        firebaseId: {
                            type: string;
                        };
                        exportOptions: {
                            type: string;
                            additionalProperties: boolean;
                            properties: {
                                method: {
                                    type: string;
                                };
                                teamID: {
                                    type: string;
                                };
                                uploadBitcode: {
                                    type: string;
                                };
                                compileBitcode: {
                                    type: string;
                                };
                                uploadSymbols: {
                                    type: string;
                                };
                                signingStyle: {
                                    type: string;
                                };
                                signingCertificate: {
                                    type: string;
                                };
                                provisioningProfiles: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                            };
                        };
                        Podfile: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        xcodeproj: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        plist: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateApplicationMethods: {
                            type: string;
                            properties: {
                                didFinishLaunchingWithOptions: {
                                    type: string;
                                };
                                open: {
                                    type: string;
                                };
                                supportedInterfaceOrientationsFor: {
                                    type: string;
                                };
                                didReceiveRemoteNotification: {
                                    type: string;
                                };
                                didFailToRegisterForRemoteNotificationsWithError: {
                                    type: string;
                                };
                                didReceive: {
                                    type: string;
                                };
                                didRegister: {
                                    type: string;
                                };
                                didRegisterForRemoteNotificationsWithDeviceToken: {
                                    type: string;
                                };
                                didConnectCarInterfaceController: {
                                    type: string;
                                };
                                didDisconnectCarInterfaceController: {
                                    type: string;
                                };
                            };
                        };
                        appDelegateMethods: {
                            additionalProperties: boolean;
                            type: string;
                        };
                        appDelegateImports: {
                            type: string;
                        };
                        appDelegateExtensions: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                androidtv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                firetv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                webos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                macos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        appleId: {
                            type: string;
                        };
                        environment: {
                            type: string;
                        };
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                linux: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tizen: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        package: {
                            type: string;
                        };
                        certificateProfile: {
                            type: string;
                        };
                        appName: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                windows: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        language: {
                            type: string;
                            description: string;
                        };
                        experimentalNuGetDependency: {
                            type: string;
                        };
                        useWinUI3: {
                            type: string;
                        };
                        nuGetTestVersion: {
                            type: string;
                        };
                        reactNativeEngine: {
                            type: string;
                        };
                        nuGetTestFeed: {
                            type: string;
                        };
                        overwrite: {
                            type: string;
                            description: string;
                        };
                        release: {
                            type: string;
                            description: string;
                        };
                        root: {
                            type: string;
                            description: string;
                        };
                        arch: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        singleproc: {
                            type: string;
                            description: string;
                        };
                        emulator: {
                            type: string;
                        };
                        device: {
                            type: string;
                        };
                        target: {
                            type: string;
                        };
                        remoteDebugging: {
                            type: string;
                        };
                        logging: {
                            type: string;
                            description: string;
                        };
                        packager: {
                            type: string;
                        };
                        bundle: {
                            type: string;
                        };
                        launch: {
                            type: string;
                            description: string;
                        };
                        autolink: {
                            type: string;
                            description: string;
                        };
                        build: {
                            type: string;
                            description: string;
                        };
                        deploy: {
                            type: string;
                        };
                        sln: {
                            type: string;
                            description: string;
                        };
                        proj: {
                            type: string;
                            description: string;
                        };
                        appPath: {
                            type: string;
                            description: string;
                        };
                        msbuildprops: {
                            type: string;
                            description: string;
                        };
                        buildLogDirectory: {
                            type: string;
                            description: string;
                        };
                        info: {
                            type: string;
                            description: string;
                        };
                        directDebugging: {
                            type: string;
                        };
                        telemetry: {
                            type: string;
                            description: string;
                        };
                        devPort: {
                            type: string;
                        };
                        additionalMetroOptions: {
                            type: string;
                        };
                        packageExtension: {
                            type: string;
                        };
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                xbox: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        language: {
                            type: string;
                            description: string;
                        };
                        experimentalNuGetDependency: {
                            type: string;
                        };
                        useWinUI3: {
                            type: string;
                        };
                        nuGetTestVersion: {
                            type: string;
                        };
                        reactNativeEngine: {
                            type: string;
                        };
                        nuGetTestFeed: {
                            type: string;
                        };
                        overwrite: {
                            type: string;
                            description: string;
                        };
                        release: {
                            type: string;
                            description: string;
                        };
                        root: {
                            type: string;
                            description: string;
                        };
                        arch: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        singleproc: {
                            type: string;
                            description: string;
                        };
                        emulator: {
                            type: string;
                        };
                        device: {
                            type: string;
                        };
                        target: {
                            type: string;
                        };
                        remoteDebugging: {
                            type: string;
                        };
                        logging: {
                            type: string;
                            description: string;
                        };
                        packager: {
                            type: string;
                        };
                        bundle: {
                            type: string;
                        };
                        launch: {
                            type: string;
                            description: string;
                        };
                        autolink: {
                            type: string;
                            description: string;
                        };
                        build: {
                            type: string;
                            description: string;
                        };
                        deploy: {
                            type: string;
                        };
                        sln: {
                            type: string;
                            description: string;
                        };
                        proj: {
                            type: string;
                            description: string;
                        };
                        appPath: {
                            type: string;
                            description: string;
                        };
                        msbuildprops: {
                            type: string;
                            description: string;
                        };
                        buildLogDirectory: {
                            type: string;
                            description: string;
                        };
                        info: {
                            type: string;
                            description: string;
                        };
                        directDebugging: {
                            type: string;
                        };
                        telemetry: {
                            type: string;
                            description: string;
                        };
                        devPort: {
                            type: string;
                        };
                        additionalMetroOptions: {
                            type: string;
                        };
                        packageExtension: {
                            type: string;
                        };
                        electronConfig: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                mac: {
                                    target: string[];
                                    hardenedRuntime: boolean;
                                };
                                dmg: {
                                    sign: boolean;
                                };
                                mas: {
                                    type: string;
                                    hardenedRuntime: boolean;
                                };
                                mainInjection: string;
                            }[];
                        };
                        BrowserWindow: {
                            type: string;
                            additionalProperties: boolean;
                            description: string;
                            examples: {
                                width: number;
                                height: number;
                                webPreferences: {
                                    devTools: boolean;
                                };
                            }[];
                            properties: {
                                width: {
                                    type: string;
                                    description: string;
                                };
                                height: {
                                    type: string;
                                    description: string;
                                };
                                webPreferences: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: {
                                        devTools: boolean;
                                    }[];
                                };
                            };
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                firefoxtv: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                firefoxos: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tizenmobile: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        package: {
                            type: string;
                        };
                        certificateProfile: {
                            type: string;
                        };
                        appName: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                tizenwatch: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        package: {
                            type: string;
                        };
                        certificateProfile: {
                            type: string;
                        };
                        appName: {
                            type: string;
                        };
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                androidwear: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        enableAndroidX: {
                            type: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        enableHermes: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        reactNativeEngine: {
                            type: string;
                            default: string;
                            description: string;
                            examples: boolean[];
                        };
                        signingConfig: {
                            type: string;
                            default: string;
                            description: string;
                            examples: string[];
                        };
                        minSdkVersion: {
                            type: string;
                            default: number;
                            examples: number[];
                        };
                        multipleAPKs: {
                            type: string;
                            default: boolean;
                            description: string;
                            examples: boolean[];
                        };
                        aab: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        extraGradleParams: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        minifyEnabled: {
                            type: string;
                            description: string;
                            default: boolean;
                            examples: boolean[];
                        };
                        targetSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        compileSdkVersion: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        gradleBuildToolsVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        gradleWrapperVersion: {
                            type: string;
                            description: string;
                            default: string;
                            examples: string[];
                        };
                        mainActivity: {
                            type: string;
                            description: string;
                            default: string;
                            properties: {
                                onCreate: {
                                    type: string;
                                    description: string;
                                    default: string;
                                    examples: string[];
                                };
                            };
                        };
                        storeFile: {
                            type: string;
                        };
                        storePassword: {
                            type: string;
                            description: string;
                        };
                        keyAlias: {
                            type: string;
                        };
                        keyPassword: {
                            type: string;
                            description: string;
                        };
                        excludedFeatures: {
                            type: string;
                        };
                        includedFeatures: {
                            type: string;
                        };
                        'gradle.properties': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                'gradle.properties': {
                                    'android.debug.obsoleteApi': boolean;
                                    'debug.keystore': string;
                                    'org.gradle.daemon': boolean;
                                    'org.gradle.parallel': boolean;
                                    'org.gradle.configureondemand': boolean;
                                };
                            }[];
                        };
                        'build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                allprojects: {
                                    repositories: {
                                        'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                    };
                                };
                            }[];
                        };
                        'app/build.gradle': {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                apply: string[];
                            }[];
                        };
                        AndroidManifest: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: ({
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                    }[];
                                }[];
                            } | {
                                children: {
                                    tag: string;
                                    'android:name': string;
                                    'android:allowBackup': boolean;
                                    'android:largeHeap': boolean;
                                    'android:usesCleartextTraffic': boolean;
                                    'tools:targetApi': number;
                                }[];
                            })[];
                        };
                        applyPlugin: {
                            type: string;
                        };
                        BuildGradle: {
                            type: string;
                            description: string;
                            properties: {
                                allprojects: {
                                    type: string;
                                    properties: {
                                        repositories: {
                                            type: string;
                                            description: string;
                                            additionalProperties: boolean;
                                            examples: {
                                                "flatDir { dirs 'libs'}": boolean;
                                            }[];
                                        };
                                    };
                                };
                            };
                        };
                        implementation: {
                            type: string;
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
                kaios: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        devServerHost: {
                            type: string;
                        };
                        webpackConfig: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                devServerHost: {
                                    type: string;
                                };
                                publicUrl: {
                                    type: string;
                                };
                                metaTags: {
                                    additionalProperties: boolean;
                                    type: string;
                                };
                                customScripts: {
                                    type: string;
                                };
                                extend: {
                                    additionalProperties: boolean;
                                    type: string;
                                    description: string;
                                    examples: ({
                                        devtool: string;
                                        module?: undefined;
                                    } | {
                                        module: {
                                            rules: {
                                                test: RegExp;
                                                use: string[];
                                                enforce: string;
                                            }[];
                                        };
                                        devtool?: undefined;
                                    })[];
                                };
                            };
                        };
                        buildSchemes: {
                            additionalProperties: {
                                type: string;
                                additionalProperties: boolean;
                                properties: any;
                            };
                            type: string;
                        };
                        engine: {
                            type: string;
                        };
                        entryFile: {
                            type: string;
                        };
                        bundleAssets: {
                            type: string;
                            description: string;
                        };
                        enableSourceMaps: {
                            type: string;
                            description: string;
                        };
                        bundleIsDev: {
                            type: string;
                        };
                        deploy: {
                            additionalProperties: boolean;
                            type: string;
                            properties: {
                                type: {
                                    type: string;
                                };
                            };
                        };
                        ext: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                myCustomRenativeProp: string;
                            }[];
                        };
                        excludedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPlugins: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        includedPermissions: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        permissions: {
                            description: string;
                            type: string;
                            items: {
                                type: string;
                            };
                        };
                        id: {
                            type: string;
                        };
                        title: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        description: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        author: {
                            additionalProperties: boolean;
                            type: string[];
                        };
                        includedFonts: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        backgroundColor: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                        splashScreen: {
                            type: string;
                        };
                        ignoreWarnings: {
                            type: string;
                        };
                        ignoreLogs: {
                            type: string;
                        };
                        license: {
                            type: string;
                        };
                        timestampAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        versionedAssets: {
                            type: string;
                            description: string;
                            examples: boolean[];
                        };
                        fontSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        assetSources: {
                            type: string;
                            items: {
                                type: string;
                            };
                            description: string;
                            examples: string[][];
                        };
                        portOffset: {
                            type: string;
                            description: string;
                            examples: number[];
                        };
                        runtime: {
                            additionalProperties: boolean;
                            type: string;
                            description: string;
                            examples: {
                                someRuntimeProperty: string;
                            }[];
                        };
                    };
                };
            };
        };
        sdks: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                ANDROID_SDK: string;
                ANDROID_NDK: string;
                TIZEN_SDK: string;
                WEBOS_SDK: string;
                KAIOS_SDK: string;
            }[];
        };
        env: {
            additionalProperties: boolean;
            type: string;
        };
        defaultTargets: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                android: string;
                androidtv: string;
                androidwear: string;
                ios: string;
                tvos: string;
                tizen: string;
                tizenwatch: string;
                tizenmobile: string;
                webos: string;
            }[];
        };
        plugins: {
            type: string;
            description: string;
            examples: {
                renative: string;
                react: string;
                'react-native-cached-image': string;
                'react-native-web-image-loader': string;
                'react-native-gesture-handler': {
                    version: string;
                };
            }[];
            additionalProperties: {
                type: string[];
                additionalProperties: boolean;
                properties: {
                    enabled: {
                        type: string;
                    };
                    props: {
                        additionalProperties: boolean;
                        type: string;
                    };
                    version: {
                        type: string;
                    };
                    source: {
                        type: string;
                        description: string;
                        examples: string[];
                    };
                    'no-npm': {
                        type: string;
                    };
                    skipMerge: {
                        type: string;
                        description: string;
                        examples: boolean[];
                    };
                    npm: {
                        additionalProperties: boolean;
                        type: string;
                    };
                    pluginDependencies: {
                        type: string[];
                    };
                    webpack: {
                        additionalProperties: boolean;
                        type: string;
                        description: string;
                    };
                    webpackConfig: {
                        additionalProperties: boolean;
                        type: string;
                        description: string;
                        properties: {
                            modulePaths: {
                                type: string[];
                                additionalProperties: boolean;
                            };
                            moduleAliases: {
                                type: string[];
                                additionalProperties: boolean;
                            };
                        };
                    };
                    android: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    androidtv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    firetv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    androidwear: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            package: {
                                type: string;
                            };
                            projectName: {
                                type: string;
                            };
                            skipImplementation: {
                                type: string;
                            };
                            afterEvaluate: {
                                type: string;
                            };
                            'gradle.properties': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    'gradle.properties': {
                                        'android.debug.obsoleteApi': boolean;
                                        'debug.keystore': string;
                                        'org.gradle.daemon': boolean;
                                        'org.gradle.parallel': boolean;
                                        'org.gradle.configureondemand': boolean;
                                    };
                                }[];
                            };
                            'build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    allprojects: {
                                        repositories: {
                                            'maven { url "https://dl.bintray.com/onfido/maven" }': boolean;
                                        };
                                    };
                                }[];
                            };
                            'app/build.gradle': {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: {
                                    apply: string[];
                                }[];
                            };
                            AndroidManifest: {
                                additionalProperties: boolean;
                                type: string;
                                description: string;
                                examples: ({
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        children: {
                                            tag: string;
                                            'android:name': string;
                                        }[];
                                    }[];
                                } | {
                                    children: {
                                        tag: string;
                                        'android:name': string;
                                        'android:allowBackup': boolean;
                                        'android:largeHeap': boolean;
                                        'android:usesCleartextTraffic': boolean;
                                        'tools:targetApi': number;
                                    }[];
                                })[];
                            };
                            applyPlugin: {
                                type: string;
                            };
                            BuildGradle: {
                                type: string;
                                description: string;
                                properties: {
                                    allprojects: {
                                        type: string;
                                        properties: {
                                            repositories: {
                                                type: string;
                                                description: string;
                                                additionalProperties: boolean;
                                                examples: {
                                                    "flatDir { dirs 'libs'}": boolean;
                                                }[];
                                            };
                                        };
                                    };
                                };
                            };
                            implementation: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    ios: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            podName: {
                                type: string;
                            };
                            podNames: {
                                type: string;
                            };
                            isStatic: {
                                type: string;
                            };
                            staticPods: {
                                type: string;
                                description: string;
                                examples: string[][];
                            };
                            Podfile: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            xcodeproj: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            plist: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateApplicationMethods: {
                                type: string;
                                properties: {
                                    didFinishLaunchingWithOptions: {
                                        type: string;
                                    };
                                    open: {
                                        type: string;
                                    };
                                    supportedInterfaceOrientationsFor: {
                                        type: string;
                                    };
                                    didReceiveRemoteNotification: {
                                        type: string;
                                    };
                                    didFailToRegisterForRemoteNotificationsWithError: {
                                        type: string;
                                    };
                                    didReceive: {
                                        type: string;
                                    };
                                    didRegister: {
                                        type: string;
                                    };
                                    didRegisterForRemoteNotificationsWithDeviceToken: {
                                        type: string;
                                    };
                                    didConnectCarInterfaceController: {
                                        type: string;
                                    };
                                    didDisconnectCarInterfaceController: {
                                        type: string;
                                    };
                                };
                            };
                            appDelegateMethods: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateImports: {
                                type: string;
                            };
                            appDelegateExtensions: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    tvos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            podName: {
                                type: string;
                            };
                            podNames: {
                                type: string;
                            };
                            isStatic: {
                                type: string;
                            };
                            staticPods: {
                                type: string;
                                description: string;
                                examples: string[][];
                            };
                            Podfile: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            xcodeproj: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            plist: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateApplicationMethods: {
                                type: string;
                                properties: {
                                    didFinishLaunchingWithOptions: {
                                        type: string;
                                    };
                                    open: {
                                        type: string;
                                    };
                                    supportedInterfaceOrientationsFor: {
                                        type: string;
                                    };
                                    didReceiveRemoteNotification: {
                                        type: string;
                                    };
                                    didFailToRegisterForRemoteNotificationsWithError: {
                                        type: string;
                                    };
                                    didReceive: {
                                        type: string;
                                    };
                                    didRegister: {
                                        type: string;
                                    };
                                    didRegisterForRemoteNotificationsWithDeviceToken: {
                                        type: string;
                                    };
                                    didConnectCarInterfaceController: {
                                        type: string;
                                    };
                                    didDisconnectCarInterfaceController: {
                                        type: string;
                                    };
                                };
                            };
                            appDelegateMethods: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            appDelegateImports: {
                                type: string;
                            };
                            appDelegateExtensions: {
                                type: string;
                            };
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    tizen: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    webos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    web: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    webtv: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    chromecast: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    firefox: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    macos: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    linux: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    windows: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                    xbox: {
                        additionalProperties: boolean;
                        type: string;
                        properties: {
                            webpackConfig: {
                                additionalProperties: boolean;
                                type: string;
                            };
                            enabled: {
                                type: string;
                            };
                            path: {
                                type: string;
                                description: string;
                                examples: string[];
                            };
                        };
                    };
                };
            };
        };
        projectName: {
            type: string;
            description: string;
            examples: string[];
        };
        extend: {
            type: string;
        };
        extendsTemplate: {
            type: string;
            description: string;
            examples: string[];
        };
        projectTemplates: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                'my-custom-template': {};
            }[];
        };
        permissions: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                ios: {};
                android: {};
            }[];
            properties: {
                android: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        INTERNET: {
                            key: string;
                            security: string;
                        };
                        SYSTEM_ALERT_WINDOW: {
                            key: string;
                            security: string;
                        };
                    }[];
                };
                ios: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        NSAppleMusicUsageDescription: {
                            desc: string;
                        };
                        NSBluetoothPeripheralUsageDescription: {
                            desc: string;
                        };
                        NSCalendarsUsageDescription: {
                            desc: string;
                        };
                        NSCameraUsageDescription: {
                            desc: string;
                        };
                        NSLocationWhenInUseUsageDescription: {
                            desc: string;
                        };
                        NSMicrophoneUsageDescription: {
                            desc: string;
                        };
                        NSMotionUsageDescription: {
                            desc: string;
                        };
                        NSPhotoLibraryAddUsageDescription: {
                            desc: string;
                        };
                        NSPhotoLibraryUsageDescription: {
                            desc: string;
                        };
                        NSSpeechRecognitionUsageDescription: {
                            desc: string;
                        };
                        NSContactsUsageDescription: {
                            desc: string;
                        };
                        NSFaceIDUsageDescription: {
                            desc: string;
                        };
                        NSLocationAlwaysUsageDescription: {
                            desc: string;
                        };
                        NSBluetoothAlwaysUsageDescription: {
                            desc: string;
                        };
                        NSLocationAlwaysAndWhenInUseUsageDescription: {
                            desc: string;
                        };
                    }[];
                };
            };
        };
        workspaceID: {
            type: string;
            description: string;
            examples: string[];
        };
        version: {
            type: string;
            description: string;
            examples: string[];
        };
        versionCode: {
            description: string;
            type: string;
            examples: string[];
        };
        versionFormat: {
            description: string;
            examples: string[];
            type: string;
        };
        versionCodeFormat: {
            description: string;
            examples: string[];
            type: string;
        };
        description: {
            type: string;
        };
        id: {
            type: string;
            description: string;
            examples: string[];
        };
        isMonorepo: {
            type: string;
        };
        monoRepo: {
            type: string;
            description: string;
            example: string;
        };
        enableAnalytics: {
            type: string;
            description: string;
            examples: boolean[];
        };
        paths: {
            additionalProperties: boolean;
            type: string;
            description: string;
            properties: {
                appConfigsDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                platformAssetsDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                platformBuildsDir: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                pluginTemplates: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomScope: {
                            npm: string;
                            path: string;
                        };
                    }[];
                };
            };
        };
        tasks: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                install: {
                    script: string;
                };
            }[];
        };
        pipes: {
            type: string;
            description: string;
            examples: string[][];
        };
        defaults: {
            additionalProperties: boolean;
            type: string;
            description: string;
            properties: {
                supportedPlatforms: {
                    type: string;
                    description: string;
                    examples: string[][];
                };
                schemes: {
                    type: string;
                    description: string;
                    examples: {
                        ios: string;
                        android: string;
                    }[];
                };
                targets: {
                    type: string;
                    description: string;
                    examples: {
                        ios: string;
                        tvos: string;
                    }[];
                };
                ports: {
                    type: string;
                    description: string;
                    examples: {
                        ios: number;
                        android: number;
                        androidtv: number;
                        tvos: number;
                        macos: number;
                        web: number;
                        tizen: number;
                        webos: number;
                        androidwear: number;
                        tizenwatch: number;
                        tizenmobile: number;
                        windows: number;
                        kaios: number;
                        firefoxos: number;
                        firefoxtv: number;
                        webtv: number;
                        linux: number;
                    }[];
                };
            };
        };
        pluginTemplates: {
            additionalProperties: boolean;
            type: string;
        };
        templates: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                '@rnv/template-starter': {
                    version: string;
                };
            }[];
        };
        currentTemplate: {
            type: string;
            description: string;
            examples: string[];
        };
        crypto: {
            additionalProperties: boolean;
            type: string;
            description: string;
            properties: {
                encrypt: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        dest: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                    };
                };
                decrypt: {
                    additionalProperties: boolean;
                    type: string;
                    properties: {
                        source: {
                            type: string;
                            description: string;
                            examples: string[];
                        };
                    };
                };
                optional: {
                    type: string;
                };
            };
        };
        integrations: {
            additionalProperties: boolean;
            type: string;
        };
        publish: {
            additionalProperties: boolean;
            type: string;
        };
        private: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                myPrivateKy: string;
            }[];
        };
        hidden: {
            type: string;
            description: string;
            examples: boolean[];
        };
        templateConfig: {
            description: string;
            type: string;
            additionalProperties: boolean;
            properties: {
                includedPaths: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                bootstrapQuestions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: {
                        title: string;
                        type: string;
                        configProp: {
                            key: string;
                            file: string;
                        };
                        options: {
                            title: string;
                            value: {
                                id: string;
                            };
                        }[];
                    }[][];
                };
            };
        };
        enableHookRebuild: {
            type: string;
            description: string;
            examples: boolean[];
        };
        runtime: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                someRuntimeProperty: string;
            }[];
        };
        common: {
            additionalProperties: boolean;
            type: string;
            description: string;
            examples: {
                author: {
                    name: string;
                    email: string;
                    url: string;
                };
                license: string;
                includedPlugins: string[];
                includedFonts: string[];
                backgroundColor: string;
                runtime: {
                    welcomeMessage: string;
                };
            }[];
            properties: {
                buildSchemes: {
                    additionalProperties: {
                        type: string;
                        additionalProperties: boolean;
                        properties: any;
                    };
                    type: string;
                };
                ext: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        myCustomRenativeProp: string;
                    }[];
                };
                excludedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPlugins: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                includedPermissions: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                permissions: {
                    description: string;
                    type: string;
                    items: {
                        type: string;
                    };
                };
                id: {
                    type: string;
                };
                title: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                description: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                author: {
                    additionalProperties: boolean;
                    type: string[];
                };
                includedFonts: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                backgroundColor: {
                    type: string;
                    description: string;
                    examples: string[];
                };
                splashScreen: {
                    type: string;
                };
                ignoreWarnings: {
                    type: string;
                };
                ignoreLogs: {
                    type: string;
                };
                license: {
                    type: string;
                };
                timestampAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                versionedAssets: {
                    type: string;
                    description: string;
                    examples: boolean[];
                };
                fontSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                assetSources: {
                    type: string;
                    items: {
                        type: string;
                    };
                    description: string;
                    examples: string[][];
                };
                portOffset: {
                    type: string;
                    description: string;
                    examples: number[];
                };
                runtime: {
                    additionalProperties: boolean;
                    type: string;
                    description: string;
                    examples: {
                        someRuntimeProperty: string;
                    }[];
                };
            };
        };
    };
})[];
//# sourceMappingURL=schemaRenativeJson.d.ts.map